import sgqlc.types
import sgqlc.types.datetime


schema = sgqlc.types.Schema()



########################################################################
# Scalars and Enumerations
########################################################################
class BigInt(sgqlc.types.Scalar):
    __schema__ = schema


Boolean = sgqlc.types.Boolean

class CircuitsCircuitStatusChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('ACTIVE', 'DECOMMISSIONED', 'DEPROVISIONING', 'OFFLINE', 'PLANNED', 'PROVISIONING')


class CircuitsCircuitTerminationCableEndChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('A', 'B')


class CircuitsCircuitTerminationTermSideChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('A', 'Z')


Date = sgqlc.types.datetime.Date

DateTime = sgqlc.types.datetime.DateTime

class DcimCableLengthUnitChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('CM', 'FT', 'IN', 'KM', 'M', 'MI')


class DcimCableStatusChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('CONNECTED', 'DECOMMISSIONING', 'PLANNED')


class DcimCableTerminationCableEndChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('A', 'B')


class DcimCableTypeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('AOC', 'CAT3', 'CAT5', 'CAT5E', 'CAT6', 'CAT6A', 'CAT7', 'CAT7A', 'CAT8', 'COAXIAL', 'DAC_ACTIVE', 'DAC_PASSIVE', 'MMF', 'MMF_OM1', 'MMF_OM2', 'MMF_OM3', 'MMF_OM4', 'MMF_OM5', 'MRJ21_TRUNK', 'POWER', 'SMF', 'SMF_OS1', 'SMF_OS2')


class DcimConsolePortCableEndChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('A', 'B')


class DcimConsolePortSpeedChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('A_115200', 'A_1200', 'A_19200', 'A_2400', 'A_38400', 'A_4800', 'A_57600', 'A_9600')


class DcimConsolePortTemplateTypeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('DB_25', 'DE_9', 'MINI_DIN_8', 'OTHER', 'RJ_11', 'RJ_12', 'RJ_45', 'USB_A', 'USB_B', 'USB_C', 'USB_MICRO_A', 'USB_MICRO_AB', 'USB_MICRO_B', 'USB_MINI_A', 'USB_MINI_B')


class DcimConsolePortTypeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('DB_25', 'DE_9', 'MINI_DIN_8', 'OTHER', 'RJ_11', 'RJ_12', 'RJ_45', 'USB_A', 'USB_B', 'USB_C', 'USB_MICRO_A', 'USB_MICRO_AB', 'USB_MICRO_B', 'USB_MINI_A', 'USB_MINI_B')


class DcimConsoleServerPortCableEndChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('A', 'B')


class DcimConsoleServerPortSpeedChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('A_115200', 'A_1200', 'A_19200', 'A_2400', 'A_38400', 'A_4800', 'A_57600', 'A_9600')


class DcimConsoleServerPortTemplateTypeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('DB_25', 'DE_9', 'MINI_DIN_8', 'OTHER', 'RJ_11', 'RJ_12', 'RJ_45', 'USB_A', 'USB_B', 'USB_C', 'USB_MICRO_A', 'USB_MICRO_AB', 'USB_MICRO_B', 'USB_MINI_A', 'USB_MINI_B')


class DcimConsoleServerPortTypeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('DB_25', 'DE_9', 'MINI_DIN_8', 'OTHER', 'RJ_11', 'RJ_12', 'RJ_45', 'USB_A', 'USB_B', 'USB_C', 'USB_MICRO_A', 'USB_MICRO_AB', 'USB_MICRO_B', 'USB_MINI_A', 'USB_MINI_B')


class DcimDeviceAirflowChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('FRONT_TO_REAR', 'LEFT_TO_RIGHT', 'MIXED', 'PASSIVE', 'REAR_TO_FRONT', 'RIGHT_TO_LEFT', 'SIDE_TO_REAR')


class DcimDeviceFaceChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('FRONT', 'REAR')


class DcimDeviceStatusChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('ACTIVE', 'DECOMMISSIONING', 'FAILED', 'INVENTORY', 'OFFLINE', 'PLANNED', 'STAGED')


class DcimDeviceTypeAirflowChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('FRONT_TO_REAR', 'LEFT_TO_RIGHT', 'MIXED', 'PASSIVE', 'REAR_TO_FRONT', 'RIGHT_TO_LEFT', 'SIDE_TO_REAR')


class DcimDeviceTypeSubdeviceRoleChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('CHILD', 'PARENT')


class DcimDeviceTypeWeightUnitChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('G', 'KG', 'LB', 'OZ')


class DcimFrontPortCableEndChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('A', 'B')


class DcimFrontPortTemplateTypeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('A_110_PUNCH', 'A_4P2C', 'A_4P4C', 'A_6P2C', 'A_6P4C', 'A_6P6C', 'A_8P2C', 'A_8P4C', 'A_8P6C', 'A_8P8C', 'BNC', 'CS', 'F', 'FC', 'GG45', 'LC', 'LC_APC', 'LC_PC', 'LC_UPC', 'LSH', 'LSH_APC', 'LSH_PC', 'LSH_UPC', 'MPO', 'MRJ21', 'MTRJ', 'N', 'OTHER', 'SC', 'SC_APC', 'SC_PC', 'SC_UPC', 'SMA_905', 'SMA_906', 'SN', 'SPLICE', 'ST', 'TERA_1P', 'TERA_2P', 'TERA_4P', 'URM_P2', 'URM_P4', 'URM_P8')


class DcimFrontPortTypeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('A_110_PUNCH', 'A_4P2C', 'A_4P4C', 'A_6P2C', 'A_6P4C', 'A_6P6C', 'A_8P2C', 'A_8P4C', 'A_8P6C', 'A_8P8C', 'BNC', 'CS', 'F', 'FC', 'GG45', 'LC', 'LC_APC', 'LC_PC', 'LC_UPC', 'LSH', 'LSH_APC', 'LSH_PC', 'LSH_UPC', 'MPO', 'MRJ21', 'MTRJ', 'N', 'OTHER', 'SC', 'SC_APC', 'SC_PC', 'SC_UPC', 'SMA_905', 'SMA_906', 'SN', 'SPLICE', 'ST', 'TERA_1P', 'TERA_2P', 'TERA_4P', 'URM_P2', 'URM_P4', 'URM_P8')


class DcimInterfaceCableEndChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('A', 'B')


class DcimInterfaceDuplexChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('AUTO', 'FULL', 'HALF')


class DcimInterfaceModeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('ACCESS', 'TAGGED', 'TAGGED_ALL')


class DcimInterfacePoeModeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('PD', 'PSE')


class DcimInterfacePoeTypeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('PASSIVE_24V_2PAIR', 'PASSIVE_24V_4PAIR', 'PASSIVE_48V_2PAIR', 'PASSIVE_48V_4PAIR', 'TYPE1_IEEE802_3AF', 'TYPE2_IEEE802_3AT', 'TYPE2_IEEE802_3AZ', 'TYPE3_IEEE802_3BT', 'TYPE4_IEEE802_3BT')


class DcimInterfaceRfChannelChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('A_2_4G_10_2457_22', 'A_2_4G_11_2462_22', 'A_2_4G_12_2467_22', 'A_2_4G_13_2472_22', 'A_2_4G_1_2412_22', 'A_2_4G_2_2417_22', 'A_2_4G_3_2422_22', 'A_2_4G_4_2427_22', 'A_2_4G_5_2432_22', 'A_2_4G_6_2437_22', 'A_2_4G_7_2442_22', 'A_2_4G_8_2447_22', 'A_2_4G_9_2452_22', 'A_5G_100_5500_20', 'A_5G_102_5510_40', 'A_5G_104_5520_20', 'A_5G_106_5530_80', 'A_5G_108_5540_20', 'A_5G_110_5550_40', 'A_5G_112_5560_20', 'A_5G_114_5570_160', 'A_5G_116_5580_20', 'A_5G_118_5590_40', 'A_5G_120_5600_20', 'A_5G_122_5610_80', 'A_5G_124_5620_20', 'A_5G_126_5630_40', 'A_5G_128_5640_20', 'A_5G_132_5660_20', 'A_5G_134_5670_40', 'A_5G_136_5680_20', 'A_5G_138_5690_80', 'A_5G_140_5700_20', 'A_5G_142_5710_40', 'A_5G_144_5720_20', 'A_5G_149_5745_20', 'A_5G_151_5755_40', 'A_5G_153_5765_20', 'A_5G_155_5775_80', 'A_5G_157_5785_20', 'A_5G_159_5795_40', 'A_5G_161_5805_20', 'A_5G_163_5815_160', 'A_5G_165_5825_20', 'A_5G_167_5835_40', 'A_5G_169_5845_20', 'A_5G_171_5855_80', 'A_5G_173_5865_20', 'A_5G_175_5875_40', 'A_5G_177_5885_20', 'A_5G_32_5160_20', 'A_5G_34_5170_40', 'A_5G_36_5180_20', 'A_5G_38_5190_40', 'A_5G_40_5200_20', 'A_5G_42_5210_80', 'A_5G_44_5220_20', 'A_5G_46_5230_40', 'A_5G_48_5240_20', 'A_5G_50_5250_160', 'A_5G_52_5260_20', 'A_5G_54_5270_40', 'A_5G_56_5280_20', 'A_5G_58_5290_80', 'A_5G_60_5300_20', 'A_5G_62_5310_40', 'A_5G_64_5320_20', 'A_60G_10_61560_4320', 'A_60G_11_63720_4320', 'A_60G_12_65880_4320', 'A_60G_13_68040_4320', 'A_60G_17_60480_6480', 'A_60G_18_62640_6480', 'A_60G_19_64800_6480', 'A_60G_1_58320_2160', 'A_60G_20_66960_6480', 'A_60G_25_61560_6480', 'A_60G_26_63720_6480', 'A_60G_27_65880_6480', 'A_60G_2_60480_2160', 'A_60G_3_62640_2160', 'A_60G_4_64800_2160', 'A_60G_5_66960_2160', 'A_60G_6_69120_2160', 'A_60G_9_59400_4320', 'A_6G_101_6455_20', 'A_6G_103_6465_80', 'A_6G_105_6475_20', 'A_6G_107_6485_40', 'A_6G_109_6495_20', 'A_6G_111_6505_160', 'A_6G_113_6515_20', 'A_6G_115_6525_40', 'A_6G_117_6535_20', 'A_6G_119_6545_80', 'A_6G_11_6005_40', 'A_6G_121_6555_20', 'A_6G_123_6565_40', 'A_6G_125_6575_20', 'A_6G_129_6595_20', 'A_6G_131_6605_40', 'A_6G_133_6615_20', 'A_6G_135_6625_80', 'A_6G_137_6635_20', 'A_6G_139_6645_40', 'A_6G_13_6015_20', 'A_6G_141_6655_20', 'A_6G_143_6665_160', 'A_6G_145_6675_20', 'A_6G_147_6685_40', 'A_6G_149_6695_20', 'A_6G_151_6705_80', 'A_6G_153_6715_20', 'A_6G_155_6725_40', 'A_6G_157_6735_20', 'A_6G_159_6745_320', 'A_6G_15_6025_160', 'A_6G_161_6755_20', 'A_6G_163_6765_40', 'A_6G_165_6775_20', 'A_6G_167_6785_80', 'A_6G_169_6795_20', 'A_6G_171_6805_40', 'A_6G_173_6815_20', 'A_6G_175_6825_160', 'A_6G_177_6835_20', 'A_6G_179_6845_40', 'A_6G_17_6035_20', 'A_6G_181_6855_20', 'A_6G_183_6865_80', 'A_6G_185_6875_20', 'A_6G_187_6885_40', 'A_6G_189_6895_20', 'A_6G_193_6915_20', 'A_6G_195_6925_40', 'A_6G_197_6935_20', 'A_6G_199_6945_80', 'A_6G_19_6045_40', 'A_6G_1_5955_20', 'A_6G_201_6955_20', 'A_6G_203_6965_40', 'A_6G_205_6975_20', 'A_6G_207_6985_160', 'A_6G_209_6995_20', 'A_6G_211_7005_40', 'A_6G_213_7015_20', 'A_6G_215_7025_80', 'A_6G_217_7035_20', 'A_6G_219_7045_40', 'A_6G_21_6055_20', 'A_6G_221_7055_20', 'A_6G_225_7075_20', 'A_6G_227_7085_40', 'A_6G_229_7095_20', 'A_6G_233_7115_20', 'A_6G_23_6065_80', 'A_6G_25_6075_20', 'A_6G_27_6085_40', 'A_6G_29_6095_20', 'A_6G_31_6105_320', 'A_6G_33_6115_20', 'A_6G_35_6125_40', 'A_6G_37_6135_20', 'A_6G_39_6145_80', 'A_6G_3_5965_40', 'A_6G_41_6155_20', 'A_6G_43_6165_40', 'A_6G_45_6175_20', 'A_6G_47_6185_160', 'A_6G_49_6195_20', 'A_6G_51_6205_40', 'A_6G_53_6215_20', 'A_6G_55_6225_80', 'A_6G_57_6235_20', 'A_6G_59_6245_40', 'A_6G_5_5975_20', 'A_6G_61_6255_20', 'A_6G_65_6275_20', 'A_6G_67_6285_40', 'A_6G_69_6295_20', 'A_6G_71_6305_80', 'A_6G_73_6315_20', 'A_6G_75_6325_40', 'A_6G_77_6335_20', 'A_6G_79_6345_160', 'A_6G_7_5985_80', 'A_6G_81_6355_20', 'A_6G_83_6365_40', 'A_6G_85_6375_20', 'A_6G_87_6385_80', 'A_6G_89_6395_20', 'A_6G_91_6405_40', 'A_6G_93_6415_20', 'A_6G_95_6425_320', 'A_6G_97_6435_20', 'A_6G_99_6445_40', 'A_6G_9_5995_20')


class DcimInterfaceRfRoleChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('AP', 'STATION')


class DcimInterfaceTemplatePoeModeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('PD', 'PSE')


class DcimInterfaceTemplatePoeTypeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('PASSIVE_24V_2PAIR', 'PASSIVE_24V_4PAIR', 'PASSIVE_48V_2PAIR', 'PASSIVE_48V_4PAIR', 'TYPE1_IEEE802_3AF', 'TYPE2_IEEE802_3AT', 'TYPE2_IEEE802_3AZ', 'TYPE3_IEEE802_3BT', 'TYPE4_IEEE802_3BT')


class DcimInterfaceTemplateTypeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('A_1000BASE_KX', 'A_1000BASE_T', 'A_1000BASE_X_GBIC', 'A_1000BASE_X_SFP', 'A_100BASE_FX', 'A_100BASE_LFX', 'A_100BASE_T1', 'A_100BASE_TX', 'A_100GBASE_KP4', 'A_100GBASE_KR2', 'A_100GBASE_KR4', 'A_100GBASE_X_CFP', 'A_100GBASE_X_CFP2', 'A_100GBASE_X_CFP4', 'A_100GBASE_X_CPAK', 'A_100GBASE_X_QSFP28', 'A_10GBASE_CX4', 'A_10GBASE_KR', 'A_10GBASE_KX4', 'A_10GBASE_T', 'A_10GBASE_X_SFPP', 'A_10GBASE_X_X2', 'A_10GBASE_X_XENPAK', 'A_10GBASE_X_XFP', 'A_10G_EPON', 'A_128GFC_QSFP28', 'A_16GFC_SFPP', 'A_1GFC_SFP', 'A_200GBASE_X_CFP2', 'A_200GBASE_X_QSFP56', 'A_25GBASE_KR', 'A_25GBASE_X_SFP28', 'A_2GFC_SFP', 'A_2_5GBASE_T', 'A_32GFC_SFP28', 'A_400GBASE_X_OSFP', 'A_400GBASE_X_QSFPDD', 'A_40GBASE_KR4', 'A_40GBASE_X_QSFPP', 'A_4GFC_SFP', 'A_50GBASE_KR', 'A_50GBASE_X_SFP28', 'A_50GBASE_X_SFP56', 'A_5GBASE_T', 'A_64GFC_QSFPP', 'A_800GBASE_X_OSFP', 'A_800GBASE_X_QSFPDD', 'A_8GFC_SFPP', 'BRIDGE', 'CDMA', 'CISCO_FLEXSTACK', 'CISCO_FLEXSTACK_PLUS', 'CISCO_STACKWISE', 'CISCO_STACKWISE_160', 'CISCO_STACKWISE_1T', 'CISCO_STACKWISE_320', 'CISCO_STACKWISE_480', 'CISCO_STACKWISE_80', 'CISCO_STACKWISE_PLUS', 'DOCSIS', 'E1', 'E3', 'EPON', 'EXTREME_SUMMITSTACK', 'EXTREME_SUMMITSTACK_128', 'EXTREME_SUMMITSTACK_256', 'EXTREME_SUMMITSTACK_512', 'GPON', 'GSM', 'IEEE802_11A', 'IEEE802_11AC', 'IEEE802_11AD', 'IEEE802_11AX', 'IEEE802_11AY', 'IEEE802_11G', 'IEEE802_11N', 'IEEE802_15_1', 'INFINIBAND_DDR', 'INFINIBAND_EDR', 'INFINIBAND_FDR', 'INFINIBAND_FDR10', 'INFINIBAND_HDR', 'INFINIBAND_NDR', 'INFINIBAND_QDR', 'INFINIBAND_SDR', 'INFINIBAND_XDR', 'JUNIPER_VCP', 'LAG', 'LTE', 'NG_PON2', 'OTHER', 'OTHER_WIRELESS', 'SONET_OC12', 'SONET_OC192', 'SONET_OC1920', 'SONET_OC3', 'SONET_OC3840', 'SONET_OC48', 'SONET_OC768', 'T1', 'T3', 'VIRTUAL', 'XDSL', 'XGS_PON', 'XG_PON')


class DcimInterfaceTypeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('A_1000BASE_KX', 'A_1000BASE_T', 'A_1000BASE_X_GBIC', 'A_1000BASE_X_SFP', 'A_100BASE_FX', 'A_100BASE_LFX', 'A_100BASE_T1', 'A_100BASE_TX', 'A_100GBASE_KP4', 'A_100GBASE_KR2', 'A_100GBASE_KR4', 'A_100GBASE_X_CFP', 'A_100GBASE_X_CFP2', 'A_100GBASE_X_CFP4', 'A_100GBASE_X_CPAK', 'A_100GBASE_X_QSFP28', 'A_10GBASE_CX4', 'A_10GBASE_KR', 'A_10GBASE_KX4', 'A_10GBASE_T', 'A_10GBASE_X_SFPP', 'A_10GBASE_X_X2', 'A_10GBASE_X_XENPAK', 'A_10GBASE_X_XFP', 'A_10G_EPON', 'A_128GFC_QSFP28', 'A_16GFC_SFPP', 'A_1GFC_SFP', 'A_200GBASE_X_CFP2', 'A_200GBASE_X_QSFP56', 'A_25GBASE_KR', 'A_25GBASE_X_SFP28', 'A_2GFC_SFP', 'A_2_5GBASE_T', 'A_32GFC_SFP28', 'A_400GBASE_X_OSFP', 'A_400GBASE_X_QSFPDD', 'A_40GBASE_KR4', 'A_40GBASE_X_QSFPP', 'A_4GFC_SFP', 'A_50GBASE_KR', 'A_50GBASE_X_SFP28', 'A_50GBASE_X_SFP56', 'A_5GBASE_T', 'A_64GFC_QSFPP', 'A_800GBASE_X_OSFP', 'A_800GBASE_X_QSFPDD', 'A_8GFC_SFPP', 'BRIDGE', 'CDMA', 'CISCO_FLEXSTACK', 'CISCO_FLEXSTACK_PLUS', 'CISCO_STACKWISE', 'CISCO_STACKWISE_160', 'CISCO_STACKWISE_1T', 'CISCO_STACKWISE_320', 'CISCO_STACKWISE_480', 'CISCO_STACKWISE_80', 'CISCO_STACKWISE_PLUS', 'DOCSIS', 'E1', 'E3', 'EPON', 'EXTREME_SUMMITSTACK', 'EXTREME_SUMMITSTACK_128', 'EXTREME_SUMMITSTACK_256', 'EXTREME_SUMMITSTACK_512', 'GPON', 'GSM', 'IEEE802_11A', 'IEEE802_11AC', 'IEEE802_11AD', 'IEEE802_11AX', 'IEEE802_11AY', 'IEEE802_11G', 'IEEE802_11N', 'IEEE802_15_1', 'INFINIBAND_DDR', 'INFINIBAND_EDR', 'INFINIBAND_FDR', 'INFINIBAND_FDR10', 'INFINIBAND_HDR', 'INFINIBAND_NDR', 'INFINIBAND_QDR', 'INFINIBAND_SDR', 'INFINIBAND_XDR', 'JUNIPER_VCP', 'LAG', 'LTE', 'NG_PON2', 'OTHER', 'OTHER_WIRELESS', 'SONET_OC12', 'SONET_OC192', 'SONET_OC1920', 'SONET_OC3', 'SONET_OC3840', 'SONET_OC48', 'SONET_OC768', 'T1', 'T3', 'VIRTUAL', 'XDSL', 'XGS_PON', 'XG_PON')


class DcimLocationStatusChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('ACTIVE', 'DECOMMISSIONING', 'PLANNED', 'RETIRED', 'STAGING')


class DcimModuleStatusChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('ACTIVE', 'DECOMMISSIONING', 'FAILED', 'OFFLINE', 'PLANNED', 'STAGED')


class DcimModuleTypeWeightUnitChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('G', 'KG', 'LB', 'OZ')


class DcimPowerFeedCableEndChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('A', 'B')


class DcimPowerFeedPhaseChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('SINGLE_PHASE', 'THREE_PHASE')


class DcimPowerFeedStatusChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('ACTIVE', 'FAILED', 'OFFLINE', 'PLANNED')


class DcimPowerFeedSupplyChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('AC', 'DC')


class DcimPowerFeedTypeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('PRIMARY', 'REDUNDANT')


class DcimPowerOutletCableEndChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('A', 'B')


class DcimPowerOutletFeedLegChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('A', 'B', 'C')


class DcimPowerOutletTemplateFeedLegChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('A', 'B', 'C')


class DcimPowerOutletTemplateTypeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('CS6360C', 'CS6364C', 'CS8164C', 'CS8264C', 'CS8364C', 'CS8464C', 'DC_TERMINAL', 'HARDWIRED', 'HDOT_CX', 'IEC_60309_2P_E_4H', 'IEC_60309_2P_E_6H', 'IEC_60309_2P_E_9H', 'IEC_60309_3P_E_4H', 'IEC_60309_3P_E_6H', 'IEC_60309_3P_E_9H', 'IEC_60309_3P_N_E_4H', 'IEC_60309_3P_N_E_6H', 'IEC_60309_3P_N_E_9H', 'IEC_60309_P_N_E_4H', 'IEC_60309_P_N_E_6H', 'IEC_60309_P_N_E_9H', 'IEC_60320_C13', 'IEC_60320_C15', 'IEC_60320_C19', 'IEC_60320_C21', 'IEC_60320_C5', 'IEC_60320_C7', 'ITA_E', 'ITA_F', 'ITA_G', 'ITA_H', 'ITA_I', 'ITA_J', 'ITA_K', 'ITA_L', 'ITA_M', 'ITA_MULTISTANDARD', 'ITA_N', 'ITA_O', 'NEMA_10_30R', 'NEMA_10_50R', 'NEMA_14_20R', 'NEMA_14_30R', 'NEMA_14_50R', 'NEMA_14_60R', 'NEMA_15_15R', 'NEMA_15_20R', 'NEMA_15_30R', 'NEMA_15_50R', 'NEMA_15_60R', 'NEMA_1_15R', 'NEMA_5_15R', 'NEMA_5_20R', 'NEMA_5_30R', 'NEMA_5_50R', 'NEMA_6_15R', 'NEMA_6_20R', 'NEMA_6_30R', 'NEMA_6_50R', 'NEMA_L10_30R', 'NEMA_L14_20R', 'NEMA_L14_30R', 'NEMA_L14_50R', 'NEMA_L14_60R', 'NEMA_L15_20R', 'NEMA_L15_30R', 'NEMA_L15_50R', 'NEMA_L15_60R', 'NEMA_L1_15R', 'NEMA_L21_20R', 'NEMA_L21_30R', 'NEMA_L22_30R', 'NEMA_L5_15R', 'NEMA_L5_20R', 'NEMA_L5_30R', 'NEMA_L5_50R', 'NEMA_L6_15R', 'NEMA_L6_20R', 'NEMA_L6_30R', 'NEMA_L6_50R', 'NEUTRIK_POWERCON_20A', 'NEUTRIK_POWERCON_32A', 'NEUTRIK_POWERCON_TRUE1', 'NEUTRIK_POWERCON_TRUE1_TOP', 'OTHER', 'SAF_D_GRID', 'UBIQUITI_SMARTPOWER', 'USB_A', 'USB_C', 'USB_MICRO_B')


class DcimPowerOutletTypeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('CS6360C', 'CS6364C', 'CS8164C', 'CS8264C', 'CS8364C', 'CS8464C', 'DC_TERMINAL', 'HARDWIRED', 'HDOT_CX', 'IEC_60309_2P_E_4H', 'IEC_60309_2P_E_6H', 'IEC_60309_2P_E_9H', 'IEC_60309_3P_E_4H', 'IEC_60309_3P_E_6H', 'IEC_60309_3P_E_9H', 'IEC_60309_3P_N_E_4H', 'IEC_60309_3P_N_E_6H', 'IEC_60309_3P_N_E_9H', 'IEC_60309_P_N_E_4H', 'IEC_60309_P_N_E_6H', 'IEC_60309_P_N_E_9H', 'IEC_60320_C13', 'IEC_60320_C15', 'IEC_60320_C19', 'IEC_60320_C21', 'IEC_60320_C5', 'IEC_60320_C7', 'ITA_E', 'ITA_F', 'ITA_G', 'ITA_H', 'ITA_I', 'ITA_J', 'ITA_K', 'ITA_L', 'ITA_M', 'ITA_MULTISTANDARD', 'ITA_N', 'ITA_O', 'NEMA_10_30R', 'NEMA_10_50R', 'NEMA_14_20R', 'NEMA_14_30R', 'NEMA_14_50R', 'NEMA_14_60R', 'NEMA_15_15R', 'NEMA_15_20R', 'NEMA_15_30R', 'NEMA_15_50R', 'NEMA_15_60R', 'NEMA_1_15R', 'NEMA_5_15R', 'NEMA_5_20R', 'NEMA_5_30R', 'NEMA_5_50R', 'NEMA_6_15R', 'NEMA_6_20R', 'NEMA_6_30R', 'NEMA_6_50R', 'NEMA_L10_30R', 'NEMA_L14_20R', 'NEMA_L14_30R', 'NEMA_L14_50R', 'NEMA_L14_60R', 'NEMA_L15_20R', 'NEMA_L15_30R', 'NEMA_L15_50R', 'NEMA_L15_60R', 'NEMA_L1_15R', 'NEMA_L21_20R', 'NEMA_L21_30R', 'NEMA_L22_30R', 'NEMA_L5_15R', 'NEMA_L5_20R', 'NEMA_L5_30R', 'NEMA_L5_50R', 'NEMA_L6_15R', 'NEMA_L6_20R', 'NEMA_L6_30R', 'NEMA_L6_50R', 'NEUTRIK_POWERCON_20A', 'NEUTRIK_POWERCON_32A', 'NEUTRIK_POWERCON_TRUE1', 'NEUTRIK_POWERCON_TRUE1_TOP', 'OTHER', 'SAF_D_GRID', 'UBIQUITI_SMARTPOWER', 'USB_A', 'USB_C', 'USB_MICRO_B')


class DcimPowerPortCableEndChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('A', 'B')


class DcimPowerPortTemplateTypeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('CS6361C', 'CS6365C', 'CS8165C', 'CS8265C', 'CS8365C', 'CS8465C', 'DC_TERMINAL', 'HARDWIRED', 'IEC_60309_2P_E_4H', 'IEC_60309_2P_E_6H', 'IEC_60309_2P_E_9H', 'IEC_60309_3P_E_4H', 'IEC_60309_3P_E_6H', 'IEC_60309_3P_E_9H', 'IEC_60309_3P_N_E_4H', 'IEC_60309_3P_N_E_6H', 'IEC_60309_3P_N_E_9H', 'IEC_60309_P_N_E_4H', 'IEC_60309_P_N_E_6H', 'IEC_60309_P_N_E_9H', 'IEC_60320_C14', 'IEC_60320_C16', 'IEC_60320_C20', 'IEC_60320_C22', 'IEC_60320_C6', 'IEC_60320_C8', 'ITA_C', 'ITA_E', 'ITA_EF', 'ITA_F', 'ITA_G', 'ITA_H', 'ITA_I', 'ITA_J', 'ITA_K', 'ITA_L', 'ITA_M', 'ITA_N', 'ITA_O', 'NEMA_10_30P', 'NEMA_10_50P', 'NEMA_14_20P', 'NEMA_14_30P', 'NEMA_14_50P', 'NEMA_14_60P', 'NEMA_15_15P', 'NEMA_15_20P', 'NEMA_15_30P', 'NEMA_15_50P', 'NEMA_15_60P', 'NEMA_1_15P', 'NEMA_5_15P', 'NEMA_5_20P', 'NEMA_5_30P', 'NEMA_5_50P', 'NEMA_6_15P', 'NEMA_6_20P', 'NEMA_6_30P', 'NEMA_6_50P', 'NEMA_L10_30P', 'NEMA_L14_20P', 'NEMA_L14_30P', 'NEMA_L14_50P', 'NEMA_L14_60P', 'NEMA_L15_20P', 'NEMA_L15_30P', 'NEMA_L15_50P', 'NEMA_L15_60P', 'NEMA_L1_15P', 'NEMA_L21_20P', 'NEMA_L21_30P', 'NEMA_L22_30P', 'NEMA_L5_15P', 'NEMA_L5_20P', 'NEMA_L5_30P', 'NEMA_L5_50P', 'NEMA_L6_15P', 'NEMA_L6_20P', 'NEMA_L6_30P', 'NEMA_L6_50P', 'NEUTRIK_POWERCON_20', 'NEUTRIK_POWERCON_32', 'NEUTRIK_POWERCON_TRUE1', 'NEUTRIK_POWERCON_TRUE1_TOP', 'OTHER', 'SAF_D_GRID', 'UBIQUITI_SMARTPOWER', 'USB_3_B', 'USB_3_MICRO_B', 'USB_A', 'USB_B', 'USB_C', 'USB_MICRO_A', 'USB_MICRO_AB', 'USB_MICRO_B', 'USB_MINI_A', 'USB_MINI_B')


class DcimPowerPortTypeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('CS6361C', 'CS6365C', 'CS8165C', 'CS8265C', 'CS8365C', 'CS8465C', 'DC_TERMINAL', 'HARDWIRED', 'IEC_60309_2P_E_4H', 'IEC_60309_2P_E_6H', 'IEC_60309_2P_E_9H', 'IEC_60309_3P_E_4H', 'IEC_60309_3P_E_6H', 'IEC_60309_3P_E_9H', 'IEC_60309_3P_N_E_4H', 'IEC_60309_3P_N_E_6H', 'IEC_60309_3P_N_E_9H', 'IEC_60309_P_N_E_4H', 'IEC_60309_P_N_E_6H', 'IEC_60309_P_N_E_9H', 'IEC_60320_C14', 'IEC_60320_C16', 'IEC_60320_C20', 'IEC_60320_C22', 'IEC_60320_C6', 'IEC_60320_C8', 'ITA_C', 'ITA_E', 'ITA_EF', 'ITA_F', 'ITA_G', 'ITA_H', 'ITA_I', 'ITA_J', 'ITA_K', 'ITA_L', 'ITA_M', 'ITA_N', 'ITA_O', 'NEMA_10_30P', 'NEMA_10_50P', 'NEMA_14_20P', 'NEMA_14_30P', 'NEMA_14_50P', 'NEMA_14_60P', 'NEMA_15_15P', 'NEMA_15_20P', 'NEMA_15_30P', 'NEMA_15_50P', 'NEMA_15_60P', 'NEMA_1_15P', 'NEMA_5_15P', 'NEMA_5_20P', 'NEMA_5_30P', 'NEMA_5_50P', 'NEMA_6_15P', 'NEMA_6_20P', 'NEMA_6_30P', 'NEMA_6_50P', 'NEMA_L10_30P', 'NEMA_L14_20P', 'NEMA_L14_30P', 'NEMA_L14_50P', 'NEMA_L14_60P', 'NEMA_L15_20P', 'NEMA_L15_30P', 'NEMA_L15_50P', 'NEMA_L15_60P', 'NEMA_L1_15P', 'NEMA_L21_20P', 'NEMA_L21_30P', 'NEMA_L22_30P', 'NEMA_L5_15P', 'NEMA_L5_20P', 'NEMA_L5_30P', 'NEMA_L5_50P', 'NEMA_L6_15P', 'NEMA_L6_20P', 'NEMA_L6_30P', 'NEMA_L6_50P', 'NEUTRIK_POWERCON_20', 'NEUTRIK_POWERCON_32', 'NEUTRIK_POWERCON_TRUE1', 'NEUTRIK_POWERCON_TRUE1_TOP', 'OTHER', 'SAF_D_GRID', 'UBIQUITI_SMARTPOWER', 'USB_3_B', 'USB_3_MICRO_B', 'USB_A', 'USB_B', 'USB_C', 'USB_MICRO_A', 'USB_MICRO_AB', 'USB_MICRO_B', 'USB_MINI_A', 'USB_MINI_B')


class DcimRackOuterUnitChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('IN', 'MM')


class DcimRackStatusChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('ACTIVE', 'AVAILABLE', 'DEPRECATED', 'PLANNED', 'RESERVED')


class DcimRackTypeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('A_2_POST_FRAME', 'A_4_POST_CABINET', 'A_4_POST_FRAME', 'WALL_CABINET', 'WALL_CABINET_VERTICAL', 'WALL_FRAME', 'WALL_FRAME_VERTICAL')


class DcimRackWeightUnitChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('G', 'KG', 'LB', 'OZ')


class DcimRackWidthChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('A_10', 'A_19', 'A_21', 'A_23')


class DcimRearPortCableEndChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('A', 'B')


class DcimRearPortTemplateTypeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('A_110_PUNCH', 'A_4P2C', 'A_4P4C', 'A_6P2C', 'A_6P4C', 'A_6P6C', 'A_8P2C', 'A_8P4C', 'A_8P6C', 'A_8P8C', 'BNC', 'CS', 'F', 'FC', 'GG45', 'LC', 'LC_APC', 'LC_PC', 'LC_UPC', 'LSH', 'LSH_APC', 'LSH_PC', 'LSH_UPC', 'MPO', 'MRJ21', 'MTRJ', 'N', 'OTHER', 'SC', 'SC_APC', 'SC_PC', 'SC_UPC', 'SMA_905', 'SMA_906', 'SN', 'SPLICE', 'ST', 'TERA_1P', 'TERA_2P', 'TERA_4P', 'URM_P2', 'URM_P4', 'URM_P8')


class DcimRearPortTypeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('A_110_PUNCH', 'A_4P2C', 'A_4P4C', 'A_6P2C', 'A_6P4C', 'A_6P6C', 'A_8P2C', 'A_8P4C', 'A_8P6C', 'A_8P8C', 'BNC', 'CS', 'F', 'FC', 'GG45', 'LC', 'LC_APC', 'LC_PC', 'LC_UPC', 'LSH', 'LSH_APC', 'LSH_PC', 'LSH_UPC', 'MPO', 'MRJ21', 'MTRJ', 'N', 'OTHER', 'SC', 'SC_APC', 'SC_PC', 'SC_UPC', 'SMA_905', 'SMA_906', 'SN', 'SPLICE', 'ST', 'TERA_1P', 'TERA_2P', 'TERA_4P', 'URM_P2', 'URM_P4', 'URM_P8')


class DcimSiteStatusChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('ACTIVE', 'DECOMMISSIONING', 'PLANNED', 'RETIRED', 'STAGING')


class DcimSiteTimeZoneChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('AFRICA_ABIDJAN', 'AFRICA_ACCRA', 'AFRICA_ADDIS_ABABA', 'AFRICA_ALGIERS', 'AFRICA_ASMARA', 'AFRICA_BAMAKO', 'AFRICA_BANGUI', 'AFRICA_BANJUL', 'AFRICA_BISSAU', 'AFRICA_BLANTYRE', 'AFRICA_BRAZZAVILLE', 'AFRICA_BUJUMBURA', 'AFRICA_CAIRO', 'AFRICA_CASABLANCA', 'AFRICA_CEUTA', 'AFRICA_CONAKRY', 'AFRICA_DAKAR', 'AFRICA_DAR_ES_SALAAM', 'AFRICA_DJIBOUTI', 'AFRICA_DOUALA', 'AFRICA_EL_AAIUN', 'AFRICA_FREETOWN', 'AFRICA_GABORONE', 'AFRICA_HARARE', 'AFRICA_JOHANNESBURG', 'AFRICA_JUBA', 'AFRICA_KAMPALA', 'AFRICA_KHARTOUM', 'AFRICA_KIGALI', 'AFRICA_KINSHASA', 'AFRICA_LAGOS', 'AFRICA_LIBREVILLE', 'AFRICA_LOME', 'AFRICA_LUANDA', 'AFRICA_LUBUMBASHI', 'AFRICA_LUSAKA', 'AFRICA_MALABO', 'AFRICA_MAPUTO', 'AFRICA_MASERU', 'AFRICA_MBABANE', 'AFRICA_MOGADISHU', 'AFRICA_MONROVIA', 'AFRICA_NAIROBI', 'AFRICA_NDJAMENA', 'AFRICA_NIAMEY', 'AFRICA_NOUAKCHOTT', 'AFRICA_OUAGADOUGOU', 'AFRICA_PORTO_NOVO', 'AFRICA_SAO_TOME', 'AFRICA_TRIPOLI', 'AFRICA_TUNIS', 'AFRICA_WINDHOEK', 'AMERICA_ADAK', 'AMERICA_ANCHORAGE', 'AMERICA_ANGUILLA', 'AMERICA_ANTIGUA', 'AMERICA_ARAGUAINA', 'AMERICA_ARGENTINA_BUENOS_AIRES', 'AMERICA_ARGENTINA_CATAMARCA', 'AMERICA_ARGENTINA_CORDOBA', 'AMERICA_ARGENTINA_JUJUY', 'AMERICA_ARGENTINA_LA_RIOJA', 'AMERICA_ARGENTINA_MENDOZA', 'AMERICA_ARGENTINA_RIO_GALLEGOS', 'AMERICA_ARGENTINA_SALTA', 'AMERICA_ARGENTINA_SAN_JUAN', 'AMERICA_ARGENTINA_SAN_LUIS', 'AMERICA_ARGENTINA_TUCUMAN', 'AMERICA_ARGENTINA_USHUAIA', 'AMERICA_ARUBA', 'AMERICA_ASUNCION', 'AMERICA_ATIKOKAN', 'AMERICA_BAHIA', 'AMERICA_BAHIA_BANDERAS', 'AMERICA_BARBADOS', 'AMERICA_BELEM', 'AMERICA_BELIZE', 'AMERICA_BLANC_SABLON', 'AMERICA_BOA_VISTA', 'AMERICA_BOGOTA', 'AMERICA_BOISE', 'AMERICA_CAMBRIDGE_BAY', 'AMERICA_CAMPO_GRANDE', 'AMERICA_CANCUN', 'AMERICA_CARACAS', 'AMERICA_CAYENNE', 'AMERICA_CAYMAN', 'AMERICA_CHICAGO', 'AMERICA_CHIHUAHUA', 'AMERICA_CIUDAD_JUAREZ', 'AMERICA_COSTA_RICA', 'AMERICA_CRESTON', 'AMERICA_CUIABA', 'AMERICA_CURACAO', 'AMERICA_DANMARKSHAVN', 'AMERICA_DAWSON', 'AMERICA_DAWSON_CREEK', 'AMERICA_DENVER', 'AMERICA_DETROIT', 'AMERICA_DOMINICA', 'AMERICA_EDMONTON', 'AMERICA_EIRUNEPE', 'AMERICA_EL_SALVADOR', 'AMERICA_FORTALEZA', 'AMERICA_FORT_NELSON', 'AMERICA_GLACE_BAY', 'AMERICA_GOOSE_BAY', 'AMERICA_GRAND_TURK', 'AMERICA_GRENADA', 'AMERICA_GUADELOUPE', 'AMERICA_GUATEMALA', 'AMERICA_GUAYAQUIL', 'AMERICA_GUYANA', 'AMERICA_HALIFAX', 'AMERICA_HAVANA', 'AMERICA_HERMOSILLO', 'AMERICA_INDIANA_INDIANAPOLIS', 'AMERICA_INDIANA_KNOX', 'AMERICA_INDIANA_MARENGO', 'AMERICA_INDIANA_PETERSBURG', 'AMERICA_INDIANA_TELL_CITY', 'AMERICA_INDIANA_VEVAY', 'AMERICA_INDIANA_VINCENNES', 'AMERICA_INDIANA_WINAMAC', 'AMERICA_INUVIK', 'AMERICA_IQALUIT', 'AMERICA_JAMAICA', 'AMERICA_JUNEAU', 'AMERICA_KENTUCKY_LOUISVILLE', 'AMERICA_KENTUCKY_MONTICELLO', 'AMERICA_KRALENDIJK', 'AMERICA_LA_PAZ', 'AMERICA_LIMA', 'AMERICA_LOS_ANGELES', 'AMERICA_LOWER_PRINCES', 'AMERICA_MACEIO', 'AMERICA_MANAGUA', 'AMERICA_MANAUS', 'AMERICA_MARIGOT', 'AMERICA_MARTINIQUE', 'AMERICA_MATAMOROS', 'AMERICA_MAZATLAN', 'AMERICA_MENOMINEE', 'AMERICA_MERIDA', 'AMERICA_METLAKATLA', 'AMERICA_MEXICO_CITY', 'AMERICA_MIQUELON', 'AMERICA_MONCTON', 'AMERICA_MONTERREY', 'AMERICA_MONTEVIDEO', 'AMERICA_MONTSERRAT', 'AMERICA_NASSAU', 'AMERICA_NEW_YORK', 'AMERICA_NOME', 'AMERICA_NORONHA', 'AMERICA_NORTH_DAKOTA_BEULAH', 'AMERICA_NORTH_DAKOTA_CENTER', 'AMERICA_NORTH_DAKOTA_NEW_SALEM', 'AMERICA_NUUK', 'AMERICA_OJINAGA', 'AMERICA_PANAMA', 'AMERICA_PARAMARIBO', 'AMERICA_PHOENIX', 'AMERICA_PORTO_VELHO', 'AMERICA_PORT_AU_PRINCE', 'AMERICA_PORT_OF_SPAIN', 'AMERICA_PUERTO_RICO', 'AMERICA_PUNTA_ARENAS', 'AMERICA_RANKIN_INLET', 'AMERICA_RECIFE', 'AMERICA_REGINA', 'AMERICA_RESOLUTE', 'AMERICA_RIO_BRANCO', 'AMERICA_SANTAREM', 'AMERICA_SANTIAGO', 'AMERICA_SANTO_DOMINGO', 'AMERICA_SAO_PAULO', 'AMERICA_SCORESBYSUND', 'AMERICA_SITKA', 'AMERICA_ST_BARTHELEMY', 'AMERICA_ST_JOHNS', 'AMERICA_ST_KITTS', 'AMERICA_ST_LUCIA', 'AMERICA_ST_THOMAS', 'AMERICA_ST_VINCENT', 'AMERICA_SWIFT_CURRENT', 'AMERICA_TEGUCIGALPA', 'AMERICA_THULE', 'AMERICA_TIJUANA', 'AMERICA_TORONTO', 'AMERICA_TORTOLA', 'AMERICA_VANCOUVER', 'AMERICA_WHITEHORSE', 'AMERICA_WINNIPEG', 'AMERICA_YAKUTAT', 'AMERICA_YELLOWKNIFE', 'ANTARCTICA_CASEY', 'ANTARCTICA_DAVIS', 'ANTARCTICA_DUMONTDURVILLE', 'ANTARCTICA_MACQUARIE', 'ANTARCTICA_MAWSON', 'ANTARCTICA_MCMURDO', 'ANTARCTICA_PALMER', 'ANTARCTICA_ROTHERA', 'ANTARCTICA_SYOWA', 'ANTARCTICA_TROLL', 'ANTARCTICA_VOSTOK', 'ARCTIC_LONGYEARBYEN', 'ASIA_ADEN', 'ASIA_ALMATY', 'ASIA_AMMAN', 'ASIA_ANADYR', 'ASIA_AQTAU', 'ASIA_AQTOBE', 'ASIA_ASHGABAT', 'ASIA_ATYRAU', 'ASIA_BAGHDAD', 'ASIA_BAHRAIN', 'ASIA_BAKU', 'ASIA_BANGKOK', 'ASIA_BARNAUL', 'ASIA_BEIRUT', 'ASIA_BISHKEK', 'ASIA_BRUNEI', 'ASIA_CHITA', 'ASIA_CHOIBALSAN', 'ASIA_COLOMBO', 'ASIA_DAMASCUS', 'ASIA_DHAKA', 'ASIA_DILI', 'ASIA_DUBAI', 'ASIA_DUSHANBE', 'ASIA_FAMAGUSTA', 'ASIA_GAZA', 'ASIA_HEBRON', 'ASIA_HONG_KONG', 'ASIA_HOVD', 'ASIA_HO_CHI_MINH', 'ASIA_IRKUTSK', 'ASIA_JAKARTA', 'ASIA_JAYAPURA', 'ASIA_JERUSALEM', 'ASIA_KABUL', 'ASIA_KAMCHATKA', 'ASIA_KARACHI', 'ASIA_KATHMANDU', 'ASIA_KHANDYGA', 'ASIA_KOLKATA', 'ASIA_KRASNOYARSK', 'ASIA_KUALA_LUMPUR', 'ASIA_KUCHING', 'ASIA_KUWAIT', 'ASIA_MACAU', 'ASIA_MAGADAN', 'ASIA_MAKASSAR', 'ASIA_MANILA', 'ASIA_MUSCAT', 'ASIA_NICOSIA', 'ASIA_NOVOKUZNETSK', 'ASIA_NOVOSIBIRSK', 'ASIA_OMSK', 'ASIA_ORAL', 'ASIA_PHNOM_PENH', 'ASIA_PONTIANAK', 'ASIA_PYONGYANG', 'ASIA_QATAR', 'ASIA_QOSTANAY', 'ASIA_QYZYLORDA', 'ASIA_RIYADH', 'ASIA_SAKHALIN', 'ASIA_SAMARKAND', 'ASIA_SEOUL', 'ASIA_SHANGHAI', 'ASIA_SINGAPORE', 'ASIA_SREDNEKOLYMSK', 'ASIA_TAIPEI', 'ASIA_TASHKENT', 'ASIA_TBILISI', 'ASIA_TEHRAN', 'ASIA_THIMPHU', 'ASIA_TOKYO', 'ASIA_TOMSK', 'ASIA_ULAANBAATAR', 'ASIA_URUMQI', 'ASIA_UST_NERA', 'ASIA_VIENTIANE', 'ASIA_VLADIVOSTOK', 'ASIA_YAKUTSK', 'ASIA_YANGON', 'ASIA_YEKATERINBURG', 'ASIA_YEREVAN', 'ATLANTIC_AZORES', 'ATLANTIC_BERMUDA', 'ATLANTIC_CANARY', 'ATLANTIC_CAPE_VERDE', 'ATLANTIC_FAROE', 'ATLANTIC_MADEIRA', 'ATLANTIC_REYKJAVIK', 'ATLANTIC_SOUTH_GEORGIA', 'ATLANTIC_STANLEY', 'ATLANTIC_ST_HELENA', 'AUSTRALIA_ADELAIDE', 'AUSTRALIA_BRISBANE', 'AUSTRALIA_BROKEN_HILL', 'AUSTRALIA_DARWIN', 'AUSTRALIA_EUCLA', 'AUSTRALIA_HOBART', 'AUSTRALIA_LINDEMAN', 'AUSTRALIA_LORD_HOWE', 'AUSTRALIA_MELBOURNE', 'AUSTRALIA_PERTH', 'AUSTRALIA_SYDNEY', 'CANADA_ATLANTIC', 'CANADA_CENTRAL', 'CANADA_EASTERN', 'CANADA_MOUNTAIN', 'CANADA_NEWFOUNDLAND', 'CANADA_PACIFIC', 'EUROPE_AMSTERDAM', 'EUROPE_ANDORRA', 'EUROPE_ASTRAKHAN', 'EUROPE_ATHENS', 'EUROPE_BELGRADE', 'EUROPE_BERLIN', 'EUROPE_BRATISLAVA', 'EUROPE_BRUSSELS', 'EUROPE_BUCHAREST', 'EUROPE_BUDAPEST', 'EUROPE_BUSINGEN', 'EUROPE_CHISINAU', 'EUROPE_COPENHAGEN', 'EUROPE_DUBLIN', 'EUROPE_GIBRALTAR', 'EUROPE_GUERNSEY', 'EUROPE_HELSINKI', 'EUROPE_ISLE_OF_MAN', 'EUROPE_ISTANBUL', 'EUROPE_JERSEY', 'EUROPE_KALININGRAD', 'EUROPE_KIROV', 'EUROPE_KYIV', 'EUROPE_LISBON', 'EUROPE_LJUBLJANA', 'EUROPE_LONDON', 'EUROPE_LUXEMBOURG', 'EUROPE_MADRID', 'EUROPE_MALTA', 'EUROPE_MARIEHAMN', 'EUROPE_MINSK', 'EUROPE_MONACO', 'EUROPE_MOSCOW', 'EUROPE_OSLO', 'EUROPE_PARIS', 'EUROPE_PODGORICA', 'EUROPE_PRAGUE', 'EUROPE_RIGA', 'EUROPE_ROME', 'EUROPE_SAMARA', 'EUROPE_SAN_MARINO', 'EUROPE_SARAJEVO', 'EUROPE_SARATOV', 'EUROPE_SIMFEROPOL', 'EUROPE_SKOPJE', 'EUROPE_SOFIA', 'EUROPE_STOCKHOLM', 'EUROPE_TALLINN', 'EUROPE_TIRANE', 'EUROPE_ULYANOVSK', 'EUROPE_VADUZ', 'EUROPE_VATICAN', 'EUROPE_VIENNA', 'EUROPE_VILNIUS', 'EUROPE_VOLGOGRAD', 'EUROPE_WARSAW', 'EUROPE_ZAGREB', 'EUROPE_ZURICH', 'GMT', 'INDIAN_ANTANANARIVO', 'INDIAN_CHAGOS', 'INDIAN_CHRISTMAS', 'INDIAN_COCOS', 'INDIAN_COMORO', 'INDIAN_KERGUELEN', 'INDIAN_MAHE', 'INDIAN_MALDIVES', 'INDIAN_MAURITIUS', 'INDIAN_MAYOTTE', 'INDIAN_REUNION', 'PACIFIC_APIA', 'PACIFIC_AUCKLAND', 'PACIFIC_BOUGAINVILLE', 'PACIFIC_CHATHAM', 'PACIFIC_CHUUK', 'PACIFIC_EASTER', 'PACIFIC_EFATE', 'PACIFIC_FAKAOFO', 'PACIFIC_FIJI', 'PACIFIC_FUNAFUTI', 'PACIFIC_GALAPAGOS', 'PACIFIC_GAMBIER', 'PACIFIC_GUADALCANAL', 'PACIFIC_GUAM', 'PACIFIC_HONOLULU', 'PACIFIC_KANTON', 'PACIFIC_KIRITIMATI', 'PACIFIC_KOSRAE', 'PACIFIC_KWAJALEIN', 'PACIFIC_MAJURO', 'PACIFIC_MARQUESAS', 'PACIFIC_MIDWAY', 'PACIFIC_NAURU', 'PACIFIC_NIUE', 'PACIFIC_NORFOLK', 'PACIFIC_NOUMEA', 'PACIFIC_PAGO_PAGO', 'PACIFIC_PALAU', 'PACIFIC_PITCAIRN', 'PACIFIC_POHNPEI', 'PACIFIC_PORT_MORESBY', 'PACIFIC_RAROTONGA', 'PACIFIC_SAIPAN', 'PACIFIC_TAHITI', 'PACIFIC_TARAWA', 'PACIFIC_TONGATAPU', 'PACIFIC_WAKE', 'PACIFIC_WALLIS', 'US_ALASKA', 'US_ARIZONA', 'US_CENTRAL', 'US_EASTERN', 'US_HAWAII', 'US_MOUNTAIN', 'US_PACIFIC', 'UTC')


class DcimVirtualDeviceContextStatusChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('ACTIVE', 'OFFLINE', 'PLANNED')


class Decimal(sgqlc.types.Scalar):
    __schema__ = schema


class ExtrasCustomFieldFilterLogicChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('DISABLED', 'EXACT', 'LOOSE')


class ExtrasCustomFieldTypeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('BOOLEAN', 'DATE', 'DECIMAL', 'INTEGER', 'JSON', 'LONGTEXT', 'MULTIOBJECT', 'MULTISELECT', 'OBJECT', 'SELECT', 'TEXT', 'URL')


class ExtrasCustomFieldUiVisibilityChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('HIDDEN', 'READ_ONLY', 'READ_WRITE')


class ExtrasCustomLinkButtonClassChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('BLACK', 'BLUE', 'CYAN', 'GHOST_DARK', 'GRAY', 'GREEN', 'INDIGO', 'ORANGE', 'OUTLINE_DARK', 'PINK', 'PURPLE', 'RED', 'TEAL', 'WHITE', 'YELLOW')


class ExtrasJournalEntryKindChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('DANGER', 'INFO', 'SUCCESS', 'WARNING')


class ExtrasObjectChangeActionChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('CREATE', 'DELETE', 'UPDATE')


class ExtrasWebhookHttpMethodChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('DELETE', 'GET', 'PATCH', 'POST', 'PUT')


class GenericScalar(sgqlc.types.Scalar):
    __schema__ = schema


ID = sgqlc.types.ID

Int = sgqlc.types.Int

class IpamFHRPGroupAuthTypeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('MD5', 'PLAINTEXT')


class IpamFHRPGroupProtocolChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('CARP', 'CLUSTERXL', 'GLBP', 'HSRP', 'OTHER', 'VRRP2', 'VRRP3')


class IpamIPAddressRoleChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('ANYCAST', 'CARP', 'GLBP', 'HSRP', 'LOOPBACK', 'SECONDARY', 'VIP', 'VRRP')


class IpamIPAddressStatusChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('ACTIVE', 'DEPRECATED', 'DHCP', 'RESERVED', 'SLAAC')


class IpamIPRangeStatusChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('ACTIVE', 'DEPRECATED', 'RESERVED')


class IpamL2VPNTypeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('EPL', 'EP_LAN', 'EP_TREE', 'EVPL', 'EVP_LAN', 'EVP_TREE', 'MPLS_EVPN', 'PBB_EVPN', 'VPLS', 'VPWS', 'VXLAN', 'VXLAN_EVPN')


class IpamPrefixStatusChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('ACTIVE', 'CONTAINER', 'DEPRECATED', 'RESERVED')


class IpamServiceProtocolChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('SCTP', 'TCP', 'UDP')


class IpamServiceTemplateProtocolChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('SCTP', 'TCP', 'UDP')


class IpamVLANStatusChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('ACTIVE', 'DEPRECATED', 'RESERVED')


class JSONString(sgqlc.types.Scalar):
    __schema__ = schema


String = sgqlc.types.String

class TenancyContactAssignmentPriorityChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('INACTIVE', 'PRIMARY', 'SECONDARY', 'TERTIARY')


class UUID(sgqlc.types.Scalar):
    __schema__ = schema


class VirtualizationClusterStatusChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('ACTIVE', 'DECOMMISSIONING', 'OFFLINE', 'PLANNED', 'STAGING')


class VirtualizationVMInterfaceModeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('ACCESS', 'TAGGED', 'TAGGED_ALL')


class VirtualizationVirtualMachineStatusChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('ACTIVE', 'DECOMMISSIONING', 'FAILED', 'OFFLINE', 'PLANNED', 'STAGED')


class WirelessWirelessLANAuthCipherChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('AES', 'AUTO', 'TKIP')


class WirelessWirelessLANAuthTypeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('OPEN', 'WEP', 'WPA_ENTERPRISE', 'WPA_PERSONAL')


class WirelessWirelessLANStatusChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('ACTIVE', 'DEPRECATED', 'DISABLED', 'RESERVED')


class WirelessWirelessLinkAuthCipherChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('AES', 'AUTO', 'TKIP')


class WirelessWirelessLinkAuthTypeChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('OPEN', 'WEP', 'WPA_ENTERPRISE', 'WPA_PERSONAL')


class WirelessWirelessLinkStatusChoices(sgqlc.types.Enum):
    __schema__ = schema
    __choices__ = ('CONNECTED', 'DECOMMISSIONING', 'PLANNED')



########################################################################
# Input Objects
########################################################################

########################################################################
# Output Objects and Interfaces
########################################################################
class ASNType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'asn', 'rir', 'tenant', 'sites', 'providers', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    asn = sgqlc.types.Field(BigInt, graphql_name='asn')
    rir = sgqlc.types.Field(sgqlc.types.non_null('RIRType'), graphql_name='rir')
    tenant = sgqlc.types.Field('TenantType', graphql_name='tenant')
    sites = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('SiteType'))), graphql_name='sites', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('facility', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility', default=None)),
        ('latitude', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude', default=None)),
        ('longitude', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('asn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn', default=None)),
        ('asn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('facility__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__n', default=None)),
        ('facility__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__ic', default=None)),
        ('facility__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__nic', default=None)),
        ('facility__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__iew', default=None)),
        ('facility__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__niew', default=None)),
        ('facility__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__isw', default=None)),
        ('facility__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__nisw', default=None)),
        ('facility__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__ie', default=None)),
        ('facility__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__nie', default=None)),
        ('facility__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__empty', default=None)),
        ('latitude__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__n', default=None)),
        ('latitude__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__lte', default=None)),
        ('latitude__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__lt', default=None)),
        ('latitude__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__gte', default=None)),
        ('latitude__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__gt', default=None)),
        ('longitude__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__n', default=None)),
        ('longitude__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__lte', default=None)),
        ('longitude__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__lt', default=None)),
        ('longitude__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__gte', default=None)),
        ('longitude__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
        ('asn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__n', default=None)),
        ('asn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn_id__n', default=None)),
))
    )
    providers = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('ProviderType'))), graphql_name='providers', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('account', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='account', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('asn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('account__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='account__n', default=None)),
        ('account__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='account__ic', default=None)),
        ('account__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='account__nic', default=None)),
        ('account__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='account__iew', default=None)),
        ('account__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='account__niew', default=None)),
        ('account__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='account__isw', default=None)),
        ('account__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='account__nisw', default=None)),
        ('account__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='account__ie', default=None)),
        ('account__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='account__nie', default=None)),
        ('account__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='account__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('asn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn_id__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of('JournalEntryType'), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class AggregateType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'prefix', 'rir', 'tenant', 'date_added', 'family', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    prefix = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='prefix')
    rir = sgqlc.types.Field(sgqlc.types.non_null('RIRType'), graphql_name='rir')
    tenant = sgqlc.types.Field('TenantType', graphql_name='tenant')
    date_added = sgqlc.types.Field(Date, graphql_name='date_added')
    family = sgqlc.types.Field('IPAddressFamilyType', graphql_name='family')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of('JournalEntryType'), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class CableTerminationType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'cable', 'cable_end', '_device', '_rack', '_location', '_site', 'display', 'class_type', 'tags', 'journal_entries', 'custom_fields', 'changelog', 'termination')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    cable = sgqlc.types.Field(sgqlc.types.non_null('CableType'), graphql_name='cable')
    cable_end = sgqlc.types.Field(sgqlc.types.non_null(DcimCableTerminationCableEndChoices), graphql_name='cable_end')
    _device = sgqlc.types.Field('DeviceType', graphql_name='_device')
    _rack = sgqlc.types.Field('RackType', graphql_name='_rack')
    _location = sgqlc.types.Field('LocationType', graphql_name='_location')
    _site = sgqlc.types.Field('SiteType', graphql_name='_site')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of('JournalEntryType'), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')
    termination = sgqlc.types.Field('CableTerminationTerminationType', graphql_name='termination')


class CableType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'type', 'status', 'tenant', 'label', 'color', 'length', 'length_unit', '_abs_length', 'terminations', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog', 'a_terminations', 'b_terminations')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    type = sgqlc.types.Field(DcimCableTypeChoices, graphql_name='type')
    status = sgqlc.types.Field(sgqlc.types.non_null(DcimCableStatusChoices), graphql_name='status')
    tenant = sgqlc.types.Field('TenantType', graphql_name='tenant')
    label = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='label')
    color = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='color')
    length = sgqlc.types.Field(Decimal, graphql_name='length')
    length_unit = sgqlc.types.Field(DcimCableLengthUnitChoices, graphql_name='length_unit')
    _abs_length = sgqlc.types.Field(Decimal, graphql_name='_abs_length')
    terminations = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(CableTerminationType))), graphql_name='terminations', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('cable', sgqlc.types.Arg(String, graphql_name='cable', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('termination_type', sgqlc.types.Arg(String, graphql_name='termination_type', default=None)),
        ('termination_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('cable__n', sgqlc.types.Arg(String, graphql_name='cable__n', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('termination_type__n', sgqlc.types.Arg(String, graphql_name='termination_type__n', default=None)),
        ('termination_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id__n', default=None)),
        ('termination_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id__lte', default=None)),
        ('termination_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id__lt', default=None)),
        ('termination_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id__gte', default=None)),
        ('termination_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id__gt', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of('JournalEntryType'), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')
    a_terminations = sgqlc.types.Field(sgqlc.types.list_of('CableTerminationTerminationType'), graphql_name='a_terminations')
    b_terminations = sgqlc.types.Field(sgqlc.types.list_of('CableTerminationTerminationType'), graphql_name='b_terminations')


class CircuitTerminationType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'cable', 'cable_end', 'mark_connected', 'circuit', 'term_side', 'site', 'provider_network', 'port_speed', 'upstream_speed', 'xconnect_id', 'pp_info', 'description', 'display', 'class_type', 'changelog', 'link_peers', 'custom_fields')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    cable = sgqlc.types.Field(CableType, graphql_name='cable')
    cable_end = sgqlc.types.Field(CircuitsCircuitTerminationCableEndChoices, graphql_name='cable_end')
    mark_connected = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='mark_connected')
    circuit = sgqlc.types.Field(sgqlc.types.non_null('CircuitType'), graphql_name='circuit')
    term_side = sgqlc.types.Field(sgqlc.types.non_null(CircuitsCircuitTerminationTermSideChoices), graphql_name='term_side')
    site = sgqlc.types.Field('SiteType', graphql_name='site')
    provider_network = sgqlc.types.Field('ProviderNetworkType', graphql_name='provider_network')
    port_speed = sgqlc.types.Field(Int, graphql_name='port_speed')
    upstream_speed = sgqlc.types.Field(Int, graphql_name='upstream_speed')
    xconnect_id = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='xconnect_id')
    pp_info = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='pp_info')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')
    link_peers = sgqlc.types.Field(sgqlc.types.list_of('LinkPeerType'), graphql_name='link_peers')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')


class CircuitType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'cid', 'provider', 'type', 'status', 'tenant', 'install_date', 'termination_date', 'commit_rate', 'termination_a', 'termination_z', 'terminations', 'contacts', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    cid = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='cid')
    provider = sgqlc.types.Field(sgqlc.types.non_null('ProviderType'), graphql_name='provider')
    type = sgqlc.types.Field(sgqlc.types.non_null('CircuitTypeType'), graphql_name='type')
    status = sgqlc.types.Field(sgqlc.types.non_null(CircuitsCircuitStatusChoices), graphql_name='status')
    tenant = sgqlc.types.Field('TenantType', graphql_name='tenant')
    install_date = sgqlc.types.Field(Date, graphql_name='install_date')
    termination_date = sgqlc.types.Field(Date, graphql_name='termination_date')
    commit_rate = sgqlc.types.Field(Int, graphql_name='commit_rate')
    termination_a = sgqlc.types.Field(CircuitTerminationType, graphql_name='termination_a')
    termination_z = sgqlc.types.Field(CircuitTerminationType, graphql_name='termination_z')
    terminations = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(CircuitTerminationType))), graphql_name='terminations', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('term_side', sgqlc.types.Arg(String, graphql_name='term_side', default=None)),
        ('port_speed', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='port_speed', default=None)),
        ('upstream_speed', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='upstream_speed', default=None)),
        ('xconnect_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('circuit_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='circuit_id', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('provider_network_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_network_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('term_side__n', sgqlc.types.Arg(String, graphql_name='term_side__n', default=None)),
        ('port_speed__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='port_speed__n', default=None)),
        ('port_speed__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='port_speed__lte', default=None)),
        ('port_speed__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='port_speed__lt', default=None)),
        ('port_speed__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='port_speed__gte', default=None)),
        ('port_speed__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='port_speed__gt', default=None)),
        ('upstream_speed__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='upstream_speed__n', default=None)),
        ('upstream_speed__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='upstream_speed__lte', default=None)),
        ('upstream_speed__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='upstream_speed__lt', default=None)),
        ('upstream_speed__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='upstream_speed__gte', default=None)),
        ('upstream_speed__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='upstream_speed__gt', default=None)),
        ('xconnect_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__n', default=None)),
        ('xconnect_id__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__ic', default=None)),
        ('xconnect_id__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__nic', default=None)),
        ('xconnect_id__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__iew', default=None)),
        ('xconnect_id__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__niew', default=None)),
        ('xconnect_id__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__isw', default=None)),
        ('xconnect_id__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__nisw', default=None)),
        ('xconnect_id__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__ie', default=None)),
        ('xconnect_id__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__nie', default=None)),
        ('xconnect_id__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('circuit_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='circuit_id__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('provider_network_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_network_id__n', default=None)),
))
    )
    contacts = sgqlc.types.Field(sgqlc.types.list_of('ContactAssignmentType'), graphql_name='contacts')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of('JournalEntryType'), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class CircuitTypeType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'name', 'slug', 'description', 'circuits', 'display', 'class_type', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    slug = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='slug')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    circuits = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(CircuitType))), graphql_name='circuits', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('cid', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('install_date', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='install_date', default=None)),
        ('termination_date', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_date', default=None)),
        ('commit_rate', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='commit_rate', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('provider_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_id', default=None)),
        ('provider', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider', default=None)),
        ('provider_network_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_network_id', default=None)),
        ('type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type_id', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('cid__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__n', default=None)),
        ('cid__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__ic', default=None)),
        ('cid__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__nic', default=None)),
        ('cid__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__iew', default=None)),
        ('cid__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__niew', default=None)),
        ('cid__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__isw', default=None)),
        ('cid__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__nisw', default=None)),
        ('cid__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__ie', default=None)),
        ('cid__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__nie', default=None)),
        ('cid__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('install_date__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='install_date__n', default=None)),
        ('install_date__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='install_date__lte', default=None)),
        ('install_date__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='install_date__lt', default=None)),
        ('install_date__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='install_date__gte', default=None)),
        ('install_date__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='install_date__gt', default=None)),
        ('termination_date__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_date__n', default=None)),
        ('termination_date__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_date__lte', default=None)),
        ('termination_date__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_date__lt', default=None)),
        ('termination_date__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_date__gte', default=None)),
        ('termination_date__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_date__gt', default=None)),
        ('commit_rate__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='commit_rate__n', default=None)),
        ('commit_rate__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='commit_rate__lte', default=None)),
        ('commit_rate__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='commit_rate__lt', default=None)),
        ('commit_rate__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='commit_rate__gte', default=None)),
        ('commit_rate__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='commit_rate__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('provider_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_id__n', default=None)),
        ('provider__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider__n', default=None)),
        ('provider_network_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_network_id__n', default=None)),
        ('type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type_id__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class ClusterGroupType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'name', 'slug', 'description', 'clusters', 'display', 'class_type', 'custom_fields', 'changelog', 'vlan_groups')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    slug = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='slug')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    clusters = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('ClusterType'))), graphql_name='clusters', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type_id', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
        ('type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type_id__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')
    vlan_groups = sgqlc.types.Field(sgqlc.types.list_of('VLANGroupType'), graphql_name='vlan_groups')


class ClusterType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'name', 'type', 'group', 'status', 'tenant', 'site', 'devices', 'virtual_machines', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog', 'vlan_groups')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    type = sgqlc.types.Field(sgqlc.types.non_null('ClusterTypeType'), graphql_name='type')
    group = sgqlc.types.Field(ClusterGroupType, graphql_name='group')
    status = sgqlc.types.Field(sgqlc.types.non_null(VirtualizationClusterStatusChoices), graphql_name='status')
    tenant = sgqlc.types.Field('TenantType', graphql_name='tenant')
    site = sgqlc.types.Field('SiteType', graphql_name='site')
    devices = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('DeviceType'))), graphql_name='devices', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('asset_tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag', default=None)),
        ('face', sgqlc.types.Arg(String, graphql_name='face', default=None)),
        ('position', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position', default=None)),
        ('airflow', sgqlc.types.Arg(String, graphql_name='airflow', default=None)),
        ('vc_position', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position', default=None)),
        ('vc_priority', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('local_context_data', sgqlc.types.Arg(Boolean, graphql_name='local_context_data', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('device_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type', default=None)),
        ('device_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type_id', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('parent_device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_device_id', default=None)),
        ('platform_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id', default=None)),
        ('platform', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('cluster_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id', default=None)),
        ('model', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('is_full_depth', sgqlc.types.Arg(Boolean, graphql_name='is_full_depth', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('serial', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial', default=None)),
        ('has_primary_ip', sgqlc.types.Arg(Boolean, graphql_name='has_primary_ip', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis_member', sgqlc.types.Arg(Boolean, graphql_name='virtual_chassis_member', default=None)),
        ('console_ports', sgqlc.types.Arg(Boolean, graphql_name='console_ports', default=None)),
        ('console_server_ports', sgqlc.types.Arg(Boolean, graphql_name='console_server_ports', default=None)),
        ('power_ports', sgqlc.types.Arg(Boolean, graphql_name='power_ports', default=None)),
        ('power_outlets', sgqlc.types.Arg(Boolean, graphql_name='power_outlets', default=None)),
        ('interfaces', sgqlc.types.Arg(Boolean, graphql_name='interfaces', default=None)),
        ('pass_through_ports', sgqlc.types.Arg(Boolean, graphql_name='pass_through_ports', default=None)),
        ('module_bays', sgqlc.types.Arg(Boolean, graphql_name='module_bays', default=None)),
        ('device_bays', sgqlc.types.Arg(Boolean, graphql_name='device_bays', default=None)),
        ('primary_ip4_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip4_id', default=None)),
        ('primary_ip6_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip6_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('asset_tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__n', default=None)),
        ('asset_tag__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ic', default=None)),
        ('asset_tag__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nic', default=None)),
        ('asset_tag__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__iew', default=None)),
        ('asset_tag__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__niew', default=None)),
        ('asset_tag__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__isw', default=None)),
        ('asset_tag__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nisw', default=None)),
        ('asset_tag__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ie', default=None)),
        ('asset_tag__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nie', default=None)),
        ('asset_tag__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__empty', default=None)),
        ('face__n', sgqlc.types.Arg(String, graphql_name='face__n', default=None)),
        ('position__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__n', default=None)),
        ('position__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__lte', default=None)),
        ('position__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__lt', default=None)),
        ('position__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__gte', default=None)),
        ('position__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__gt', default=None)),
        ('airflow__n', sgqlc.types.Arg(String, graphql_name='airflow__n', default=None)),
        ('vc_position__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__n', default=None)),
        ('vc_position__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__lte', default=None)),
        ('vc_position__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__lt', default=None)),
        ('vc_position__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__gte', default=None)),
        ('vc_position__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__gt', default=None)),
        ('vc_priority__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__n', default=None)),
        ('vc_priority__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__lte', default=None)),
        ('vc_priority__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__lt', default=None)),
        ('vc_priority__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__gte', default=None)),
        ('vc_priority__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
        ('device_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type__n', default=None)),
        ('device_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type_id__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('parent_device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_device_id__n', default=None)),
        ('platform_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id__n', default=None)),
        ('platform__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('cluster_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id__n', default=None)),
        ('model__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__n', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('serial__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__n', default=None)),
        ('serial__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ic', default=None)),
        ('serial__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nic', default=None)),
        ('serial__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__iew', default=None)),
        ('serial__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__niew', default=None)),
        ('serial__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__isw', default=None)),
        ('serial__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nisw', default=None)),
        ('serial__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ie', default=None)),
        ('serial__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nie', default=None)),
        ('serial__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__empty', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('primary_ip4_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip4_id__n', default=None)),
        ('primary_ip6_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip6_id__n', default=None)),
))
    )
    virtual_machines = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('VirtualMachineType'))), graphql_name='virtual_machines', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('cluster', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster', default=None)),
        ('vcpus', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus', default=None)),
        ('memory', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory', default=None)),
        ('disk', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('local_context_data', sgqlc.types.Arg(Boolean, graphql_name='local_context_data', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('cluster_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group_id', default=None)),
        ('cluster_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group', default=None)),
        ('cluster_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type_id', default=None)),
        ('cluster_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type', default=None)),
        ('cluster_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('platform_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id', default=None)),
        ('platform', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('has_primary_ip', sgqlc.types.Arg(Boolean, graphql_name='has_primary_ip', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('cluster__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster__n', default=None)),
        ('vcpus__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__n', default=None)),
        ('vcpus__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__lte', default=None)),
        ('vcpus__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__lt', default=None)),
        ('vcpus__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__gte', default=None)),
        ('vcpus__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__gt', default=None)),
        ('memory__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__n', default=None)),
        ('memory__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__lte', default=None)),
        ('memory__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__lt', default=None)),
        ('memory__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__gte', default=None)),
        ('memory__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__gt', default=None)),
        ('disk__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__n', default=None)),
        ('disk__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__lte', default=None)),
        ('disk__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__lt', default=None)),
        ('disk__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__gte', default=None)),
        ('disk__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('cluster_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group_id__n', default=None)),
        ('cluster_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group__n', default=None)),
        ('cluster_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type_id__n', default=None)),
        ('cluster_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type__n', default=None)),
        ('cluster_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('platform_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id__n', default=None)),
        ('platform__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of('JournalEntryType'), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')
    vlan_groups = sgqlc.types.Field(sgqlc.types.list_of('VLANGroupType'), graphql_name='vlan_groups')


class ClusterTypeType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'name', 'slug', 'description', 'clusters', 'display', 'class_type', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    slug = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='slug')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    clusters = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ClusterType))), graphql_name='clusters', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type_id', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
        ('type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type_id__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class ConfigContextType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'name', 'weight', 'description', 'is_active', 'regions', 'site_groups', 'sites', 'locations', 'device_types', 'roles', 'platforms', 'cluster_types', 'cluster_groups', 'clusters', 'tenant_groups', 'tenants', 'tags', 'data', 'display', 'class_type', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    weight = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='weight')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    is_active = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='is_active')
    regions = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('RegionType'))), graphql_name='regions', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('parent', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('parent__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent__n', default=None)),
))
    )
    site_groups = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('SiteGroupType'))), graphql_name='site_groups', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('parent', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('parent__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent__n', default=None)),
))
    )
    sites = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('SiteType'))), graphql_name='sites', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('facility', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility', default=None)),
        ('latitude', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude', default=None)),
        ('longitude', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('asn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn', default=None)),
        ('asn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('facility__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__n', default=None)),
        ('facility__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__ic', default=None)),
        ('facility__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__nic', default=None)),
        ('facility__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__iew', default=None)),
        ('facility__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__niew', default=None)),
        ('facility__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__isw', default=None)),
        ('facility__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__nisw', default=None)),
        ('facility__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__ie', default=None)),
        ('facility__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__nie', default=None)),
        ('facility__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__empty', default=None)),
        ('latitude__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__n', default=None)),
        ('latitude__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__lte', default=None)),
        ('latitude__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__lt', default=None)),
        ('latitude__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__gte', default=None)),
        ('latitude__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__gt', default=None)),
        ('longitude__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__n', default=None)),
        ('longitude__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__lte', default=None)),
        ('longitude__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__lt', default=None)),
        ('longitude__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__gte', default=None)),
        ('longitude__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
        ('asn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__n', default=None)),
        ('asn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn_id__n', default=None)),
))
    )
    locations = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('LocationType'))), graphql_name='locations', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('parent', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('parent__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent__n', default=None)),
))
    )
    device_types = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('DeviceTypeType'))), graphql_name='device_types', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('model', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('part_number', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number', default=None)),
        ('u_height', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height', default=None)),
        ('is_full_depth', sgqlc.types.Arg(Boolean, graphql_name='is_full_depth', default=None)),
        ('subdevice_role', sgqlc.types.Arg(String, graphql_name='subdevice_role', default=None)),
        ('airflow', sgqlc.types.Arg(String, graphql_name='airflow', default=None)),
        ('weight', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight', default=None)),
        ('weight_unit', sgqlc.types.Arg(String, graphql_name='weight_unit', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('has_front_image', sgqlc.types.Arg(Boolean, graphql_name='has_front_image', default=None)),
        ('has_rear_image', sgqlc.types.Arg(Boolean, graphql_name='has_rear_image', default=None)),
        ('console_ports', sgqlc.types.Arg(Boolean, graphql_name='console_ports', default=None)),
        ('console_server_ports', sgqlc.types.Arg(Boolean, graphql_name='console_server_ports', default=None)),
        ('power_ports', sgqlc.types.Arg(Boolean, graphql_name='power_ports', default=None)),
        ('power_outlets', sgqlc.types.Arg(Boolean, graphql_name='power_outlets', default=None)),
        ('interfaces', sgqlc.types.Arg(Boolean, graphql_name='interfaces', default=None)),
        ('pass_through_ports', sgqlc.types.Arg(Boolean, graphql_name='pass_through_ports', default=None)),
        ('module_bays', sgqlc.types.Arg(Boolean, graphql_name='module_bays', default=None)),
        ('device_bays', sgqlc.types.Arg(Boolean, graphql_name='device_bays', default=None)),
        ('inventory_items', sgqlc.types.Arg(Boolean, graphql_name='inventory_items', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('model__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__n', default=None)),
        ('model__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__ic', default=None)),
        ('model__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__nic', default=None)),
        ('model__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__iew', default=None)),
        ('model__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__niew', default=None)),
        ('model__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__isw', default=None)),
        ('model__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__nisw', default=None)),
        ('model__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__ie', default=None)),
        ('model__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__nie', default=None)),
        ('model__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('part_number__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__n', default=None)),
        ('part_number__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__ic', default=None)),
        ('part_number__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__nic', default=None)),
        ('part_number__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__iew', default=None)),
        ('part_number__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__niew', default=None)),
        ('part_number__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__isw', default=None)),
        ('part_number__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__nisw', default=None)),
        ('part_number__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__ie', default=None)),
        ('part_number__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__nie', default=None)),
        ('part_number__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__empty', default=None)),
        ('u_height__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__n', default=None)),
        ('u_height__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__lte', default=None)),
        ('u_height__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__lt', default=None)),
        ('u_height__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__gte', default=None)),
        ('u_height__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__gt', default=None)),
        ('subdevice_role__n', sgqlc.types.Arg(String, graphql_name='subdevice_role__n', default=None)),
        ('airflow__n', sgqlc.types.Arg(String, graphql_name='airflow__n', default=None)),
        ('weight__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__n', default=None)),
        ('weight__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__lte', default=None)),
        ('weight__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__lt', default=None)),
        ('weight__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__gte', default=None)),
        ('weight__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__gt', default=None)),
        ('weight_unit__n', sgqlc.types.Arg(String, graphql_name='weight_unit__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
))
    )
    roles = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('DeviceRoleType'))), graphql_name='roles', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('color', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color', default=None)),
        ('vm_role', sgqlc.types.Arg(Boolean, graphql_name='vm_role', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('color__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__n', default=None)),
        ('color__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ic', default=None)),
        ('color__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nic', default=None)),
        ('color__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__iew', default=None)),
        ('color__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__niew', default=None)),
        ('color__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__isw', default=None)),
        ('color__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nisw', default=None)),
        ('color__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ie', default=None)),
        ('color__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nie', default=None)),
        ('color__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
))
    )
    platforms = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('PlatformType'))), graphql_name='platforms', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('napalm_driver', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('napalm_driver__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__n', default=None)),
        ('napalm_driver__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__ic', default=None)),
        ('napalm_driver__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__nic', default=None)),
        ('napalm_driver__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__iew', default=None)),
        ('napalm_driver__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__niew', default=None)),
        ('napalm_driver__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__isw', default=None)),
        ('napalm_driver__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__nisw', default=None)),
        ('napalm_driver__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__ie', default=None)),
        ('napalm_driver__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__nie', default=None)),
        ('napalm_driver__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
))
    )
    cluster_types = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ClusterTypeType))), graphql_name='cluster_types', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
))
    )
    cluster_groups = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ClusterGroupType))), graphql_name='cluster_groups', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
))
    )
    clusters = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ClusterType))), graphql_name='clusters', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type_id', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
        ('type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type_id__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
))
    )
    tenant_groups = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('TenantGroupType'))), graphql_name='tenant_groups', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('parent', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('parent__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent__n', default=None)),
))
    )
    tenants = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('TenantType'))), graphql_name='tenants', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
))
    )
    tags = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('TagType'))), graphql_name='tags', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('color', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('content_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type', default=None)),
        ('content_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('color__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__n', default=None)),
        ('color__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ic', default=None)),
        ('color__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nic', default=None)),
        ('color__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__iew', default=None)),
        ('color__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__niew', default=None)),
        ('color__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__isw', default=None)),
        ('color__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nisw', default=None)),
        ('color__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ie', default=None)),
        ('color__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nie', default=None)),
        ('color__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
))
    )
    data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='data')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class ConsolePortTemplateType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'name', '_name', 'label', 'description', 'device_type', 'module_type', 'type', 'display', 'class_type', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    _name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='_name')
    label = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='label')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    device_type = sgqlc.types.Field('DeviceTypeType', graphql_name='device_type')
    module_type = sgqlc.types.Field('ModuleTypeType', graphql_name='module_type')
    type = sgqlc.types.Field(DcimConsolePortTemplateTypeChoices, graphql_name='type')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class ConsolePortType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'device', 'name', '_name', 'label', 'description', 'module', 'cable', 'cable_end', 'mark_connected', 'type', 'speed', 'connected_endpoints', 'link_peers', 'display', 'class_type', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    device = sgqlc.types.Field(sgqlc.types.non_null('DeviceType'), graphql_name='device')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    _name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='_name')
    label = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='label')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    module = sgqlc.types.Field('ModuleType', graphql_name='module')
    cable = sgqlc.types.Field(CableType, graphql_name='cable')
    cable_end = sgqlc.types.Field(DcimConsolePortCableEndChoices, graphql_name='cable_end')
    mark_connected = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='mark_connected')
    type = sgqlc.types.Field(DcimConsolePortTypeChoices, graphql_name='type')
    speed = sgqlc.types.Field(DcimConsolePortSpeedChoices, graphql_name='speed')
    connected_endpoints = sgqlc.types.Field(sgqlc.types.list_of('LinkPeerType'), graphql_name='connected_endpoints')
    link_peers = sgqlc.types.Field(sgqlc.types.list_of('LinkPeerType'), graphql_name='link_peers')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class ConsoleServerPortTemplateType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'name', '_name', 'label', 'description', 'device_type', 'module_type', 'type', 'display', 'class_type', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    _name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='_name')
    label = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='label')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    device_type = sgqlc.types.Field('DeviceTypeType', graphql_name='device_type')
    module_type = sgqlc.types.Field('ModuleTypeType', graphql_name='module_type')
    type = sgqlc.types.Field(DcimConsoleServerPortTemplateTypeChoices, graphql_name='type')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class ConsoleServerPortType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'device', 'name', '_name', 'label', 'description', 'module', 'cable', 'cable_end', 'mark_connected', 'type', 'speed', 'connected_endpoints', 'link_peers', 'display', 'class_type', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    device = sgqlc.types.Field(sgqlc.types.non_null('DeviceType'), graphql_name='device')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    _name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='_name')
    label = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='label')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    module = sgqlc.types.Field('ModuleType', graphql_name='module')
    cable = sgqlc.types.Field(CableType, graphql_name='cable')
    cable_end = sgqlc.types.Field(DcimConsoleServerPortCableEndChoices, graphql_name='cable_end')
    mark_connected = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='mark_connected')
    type = sgqlc.types.Field(DcimConsoleServerPortTypeChoices, graphql_name='type')
    speed = sgqlc.types.Field(DcimConsoleServerPortSpeedChoices, graphql_name='speed')
    connected_endpoints = sgqlc.types.Field(sgqlc.types.list_of('LinkPeerType'), graphql_name='connected_endpoints')
    link_peers = sgqlc.types.Field(sgqlc.types.list_of('LinkPeerType'), graphql_name='link_peers')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class ContactAssignmentType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'content_type', 'object_id', 'contact', 'role', 'priority', 'display', 'class_type')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    content_type = sgqlc.types.Field(sgqlc.types.non_null('ContentTypeType'), graphql_name='content_type')
    object_id = sgqlc.types.Field(sgqlc.types.non_null(BigInt), graphql_name='object_id')
    contact = sgqlc.types.Field(sgqlc.types.non_null('ContactType'), graphql_name='contact')
    role = sgqlc.types.Field(sgqlc.types.non_null('ContactRoleType'), graphql_name='role')
    priority = sgqlc.types.Field(TenancyContactAssignmentPriorityChoices, graphql_name='priority')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')


class ContactGroupType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'parent', 'name', 'slug', 'description', 'lft', 'rght', 'tree_id', 'level', 'children', 'contacts', 'display', 'class_type', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    parent = sgqlc.types.Field('ContactGroupType', graphql_name='parent')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    slug = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='slug')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    lft = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='lft')
    rght = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='rght')
    tree_id = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='tree_id')
    level = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='level')
    children = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('ContactGroupType'))), graphql_name='children', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('parent', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('parent__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent__n', default=None)),
))
    )
    contacts = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('ContactType'))), graphql_name='contacts', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('title', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='title', default=None)),
        ('phone', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='phone', default=None)),
        ('email', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email', default=None)),
        ('address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address', default=None)),
        ('link', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='link', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('title__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='title__n', default=None)),
        ('title__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='title__ic', default=None)),
        ('title__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='title__nic', default=None)),
        ('title__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='title__iew', default=None)),
        ('title__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='title__niew', default=None)),
        ('title__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='title__isw', default=None)),
        ('title__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='title__nisw', default=None)),
        ('title__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='title__ie', default=None)),
        ('title__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='title__nie', default=None)),
        ('title__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='title__empty', default=None)),
        ('phone__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='phone__n', default=None)),
        ('phone__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='phone__ic', default=None)),
        ('phone__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='phone__nic', default=None)),
        ('phone__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='phone__iew', default=None)),
        ('phone__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='phone__niew', default=None)),
        ('phone__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='phone__isw', default=None)),
        ('phone__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='phone__nisw', default=None)),
        ('phone__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='phone__ie', default=None)),
        ('phone__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='phone__nie', default=None)),
        ('phone__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='phone__empty', default=None)),
        ('email__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__n', default=None)),
        ('email__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__ic', default=None)),
        ('email__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__nic', default=None)),
        ('email__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__iew', default=None)),
        ('email__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__niew', default=None)),
        ('email__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__isw', default=None)),
        ('email__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__nisw', default=None)),
        ('email__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__ie', default=None)),
        ('email__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__nie', default=None)),
        ('email__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__empty', default=None)),
        ('address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address__n', default=None)),
        ('address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address__ic', default=None)),
        ('address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address__nic', default=None)),
        ('address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address__iew', default=None)),
        ('address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address__niew', default=None)),
        ('address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address__isw', default=None)),
        ('address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address__nisw', default=None)),
        ('address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address__ie', default=None)),
        ('address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address__nie', default=None)),
        ('address__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address__empty', default=None)),
        ('link__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='link__n', default=None)),
        ('link__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='link__ic', default=None)),
        ('link__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='link__nic', default=None)),
        ('link__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='link__iew', default=None)),
        ('link__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='link__niew', default=None)),
        ('link__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='link__isw', default=None)),
        ('link__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='link__nisw', default=None)),
        ('link__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='link__ie', default=None)),
        ('link__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='link__nie', default=None)),
        ('link__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='link__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class ContactRoleType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'name', 'slug', 'description', 'assignments', 'display', 'class_type', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    slug = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='slug')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    assignments = sgqlc.types.Field(sgqlc.types.list_of(ContactAssignmentType), graphql_name='assignments')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class ContactType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'group', 'name', 'title', 'phone', 'email', 'address', 'link', 'assignments', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    group = sgqlc.types.Field(ContactGroupType, graphql_name='group')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    title = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='title')
    phone = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='phone')
    email = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='email')
    address = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='address')
    link = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='link')
    assignments = sgqlc.types.Field(sgqlc.types.list_of(ContactAssignmentType), graphql_name='assignments')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of('JournalEntryType'), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class ContentTypeType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'app_label', 'model')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    app_label = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='app_label')
    model = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='model')


class CustomFieldType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'type', 'object_type', 'name', 'label', 'group_name', 'description', 'required', 'search_weight', 'filter_logic', 'default', 'weight', 'validation_minimum', 'validation_maximum', 'validation_regex', 'choices', 'ui_visibility', 'display', 'class_type', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    type = sgqlc.types.Field(sgqlc.types.non_null(ExtrasCustomFieldTypeChoices), graphql_name='type')
    object_type = sgqlc.types.Field(ContentTypeType, graphql_name='object_type')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    label = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='label')
    group_name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='group_name')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    required = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='required')
    search_weight = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='search_weight')
    filter_logic = sgqlc.types.Field(sgqlc.types.non_null(ExtrasCustomFieldFilterLogicChoices), graphql_name='filter_logic')
    default = sgqlc.types.Field(JSONString, graphql_name='default')
    weight = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='weight')
    validation_minimum = sgqlc.types.Field(Int, graphql_name='validation_minimum')
    validation_maximum = sgqlc.types.Field(Int, graphql_name='validation_maximum')
    validation_regex = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='validation_regex')
    choices = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(String)), graphql_name='choices')
    ui_visibility = sgqlc.types.Field(sgqlc.types.non_null(ExtrasCustomFieldUiVisibilityChoices), graphql_name='ui_visibility')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class CustomLinkType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'name', 'enabled', 'link_text', 'link_url', 'weight', 'group_name', 'button_class', 'new_window', 'display', 'class_type', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    enabled = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='enabled')
    link_text = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='link_text')
    link_url = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='link_url')
    weight = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='weight')
    group_name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='group_name')
    button_class = sgqlc.types.Field(sgqlc.types.non_null(ExtrasCustomLinkButtonClassChoices), graphql_name='button_class')
    new_window = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='new_window')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class DeviceBayTemplateType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'device_type', 'name', '_name', 'label', 'description', 'display', 'class_type', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    device_type = sgqlc.types.Field(sgqlc.types.non_null('DeviceTypeType'), graphql_name='device_type')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    _name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='_name')
    label = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='label')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class DeviceBayType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'device', 'name', '_name', 'label', 'description', 'installed_device', 'display', 'class_type', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    device = sgqlc.types.Field(sgqlc.types.non_null('DeviceType'), graphql_name='device')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    _name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='_name')
    label = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='label')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    installed_device = sgqlc.types.Field('DeviceType', graphql_name='installed_device')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class DeviceRoleType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'name', 'slug', 'description', 'color', 'vm_role', 'devices', 'virtual_machines', 'display', 'class_type', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    slug = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='slug')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    color = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='color')
    vm_role = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='vm_role')
    devices = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('DeviceType'))), graphql_name='devices', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('asset_tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag', default=None)),
        ('face', sgqlc.types.Arg(String, graphql_name='face', default=None)),
        ('position', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position', default=None)),
        ('airflow', sgqlc.types.Arg(String, graphql_name='airflow', default=None)),
        ('vc_position', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position', default=None)),
        ('vc_priority', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('local_context_data', sgqlc.types.Arg(Boolean, graphql_name='local_context_data', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('device_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type', default=None)),
        ('device_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type_id', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('parent_device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_device_id', default=None)),
        ('platform_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id', default=None)),
        ('platform', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('cluster_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id', default=None)),
        ('model', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('is_full_depth', sgqlc.types.Arg(Boolean, graphql_name='is_full_depth', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('serial', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial', default=None)),
        ('has_primary_ip', sgqlc.types.Arg(Boolean, graphql_name='has_primary_ip', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis_member', sgqlc.types.Arg(Boolean, graphql_name='virtual_chassis_member', default=None)),
        ('console_ports', sgqlc.types.Arg(Boolean, graphql_name='console_ports', default=None)),
        ('console_server_ports', sgqlc.types.Arg(Boolean, graphql_name='console_server_ports', default=None)),
        ('power_ports', sgqlc.types.Arg(Boolean, graphql_name='power_ports', default=None)),
        ('power_outlets', sgqlc.types.Arg(Boolean, graphql_name='power_outlets', default=None)),
        ('interfaces', sgqlc.types.Arg(Boolean, graphql_name='interfaces', default=None)),
        ('pass_through_ports', sgqlc.types.Arg(Boolean, graphql_name='pass_through_ports', default=None)),
        ('module_bays', sgqlc.types.Arg(Boolean, graphql_name='module_bays', default=None)),
        ('device_bays', sgqlc.types.Arg(Boolean, graphql_name='device_bays', default=None)),
        ('primary_ip4_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip4_id', default=None)),
        ('primary_ip6_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip6_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('asset_tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__n', default=None)),
        ('asset_tag__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ic', default=None)),
        ('asset_tag__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nic', default=None)),
        ('asset_tag__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__iew', default=None)),
        ('asset_tag__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__niew', default=None)),
        ('asset_tag__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__isw', default=None)),
        ('asset_tag__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nisw', default=None)),
        ('asset_tag__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ie', default=None)),
        ('asset_tag__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nie', default=None)),
        ('asset_tag__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__empty', default=None)),
        ('face__n', sgqlc.types.Arg(String, graphql_name='face__n', default=None)),
        ('position__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__n', default=None)),
        ('position__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__lte', default=None)),
        ('position__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__lt', default=None)),
        ('position__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__gte', default=None)),
        ('position__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__gt', default=None)),
        ('airflow__n', sgqlc.types.Arg(String, graphql_name='airflow__n', default=None)),
        ('vc_position__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__n', default=None)),
        ('vc_position__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__lte', default=None)),
        ('vc_position__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__lt', default=None)),
        ('vc_position__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__gte', default=None)),
        ('vc_position__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__gt', default=None)),
        ('vc_priority__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__n', default=None)),
        ('vc_priority__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__lte', default=None)),
        ('vc_priority__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__lt', default=None)),
        ('vc_priority__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__gte', default=None)),
        ('vc_priority__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
        ('device_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type__n', default=None)),
        ('device_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type_id__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('parent_device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_device_id__n', default=None)),
        ('platform_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id__n', default=None)),
        ('platform__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('cluster_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id__n', default=None)),
        ('model__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__n', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('serial__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__n', default=None)),
        ('serial__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ic', default=None)),
        ('serial__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nic', default=None)),
        ('serial__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__iew', default=None)),
        ('serial__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__niew', default=None)),
        ('serial__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__isw', default=None)),
        ('serial__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nisw', default=None)),
        ('serial__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ie', default=None)),
        ('serial__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nie', default=None)),
        ('serial__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__empty', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('primary_ip4_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip4_id__n', default=None)),
        ('primary_ip6_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip6_id__n', default=None)),
))
    )
    virtual_machines = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('VirtualMachineType'))), graphql_name='virtual_machines', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('cluster', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster', default=None)),
        ('vcpus', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus', default=None)),
        ('memory', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory', default=None)),
        ('disk', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('local_context_data', sgqlc.types.Arg(Boolean, graphql_name='local_context_data', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('cluster_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group_id', default=None)),
        ('cluster_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group', default=None)),
        ('cluster_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type_id', default=None)),
        ('cluster_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type', default=None)),
        ('cluster_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('platform_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id', default=None)),
        ('platform', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('has_primary_ip', sgqlc.types.Arg(Boolean, graphql_name='has_primary_ip', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('cluster__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster__n', default=None)),
        ('vcpus__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__n', default=None)),
        ('vcpus__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__lte', default=None)),
        ('vcpus__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__lt', default=None)),
        ('vcpus__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__gte', default=None)),
        ('vcpus__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__gt', default=None)),
        ('memory__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__n', default=None)),
        ('memory__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__lte', default=None)),
        ('memory__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__lt', default=None)),
        ('memory__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__gte', default=None)),
        ('memory__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__gt', default=None)),
        ('disk__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__n', default=None)),
        ('disk__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__lte', default=None)),
        ('disk__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__lt', default=None)),
        ('disk__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__gte', default=None)),
        ('disk__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('cluster_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group_id__n', default=None)),
        ('cluster_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group__n', default=None)),
        ('cluster_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type_id__n', default=None)),
        ('cluster_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type__n', default=None)),
        ('cluster_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('platform_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id__n', default=None)),
        ('platform__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class DeviceType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'local_context_data', 'device_type', 'device_role', 'tenant', 'platform', 'name', '_name', 'serial', 'asset_tag', 'site', 'location', 'rack', 'position', 'face', 'status', 'airflow', 'primary_ip4', 'primary_ip6', 'cluster', 'virtual_chassis', 'vc_position', 'vc_priority', 'consoleports', 'consoleserverports', 'powerports', 'poweroutlets', 'interfaces', 'frontports', 'rearports', 'modulebays', 'parent_bay', 'devicebays', 'inventoryitems', 'cabletermination_set', 'modules', 'vc_master_for', 'vdcs', 'services', 'virtual_machines', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog', 'contacts', 'image_attachments', 'config_context')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    local_context_data = sgqlc.types.Field(JSONString, graphql_name='local_context_data')
    device_type = sgqlc.types.Field(sgqlc.types.non_null('DeviceTypeType'), graphql_name='device_type')
    device_role = sgqlc.types.Field(sgqlc.types.non_null(DeviceRoleType), graphql_name='device_role')
    tenant = sgqlc.types.Field('TenantType', graphql_name='tenant')
    platform = sgqlc.types.Field('PlatformType', graphql_name='platform')
    name = sgqlc.types.Field(String, graphql_name='name')
    _name = sgqlc.types.Field(String, graphql_name='_name')
    serial = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='serial')
    asset_tag = sgqlc.types.Field(String, graphql_name='asset_tag')
    site = sgqlc.types.Field(sgqlc.types.non_null('SiteType'), graphql_name='site')
    location = sgqlc.types.Field('LocationType', graphql_name='location')
    rack = sgqlc.types.Field('RackType', graphql_name='rack')
    position = sgqlc.types.Field(Decimal, graphql_name='position')
    face = sgqlc.types.Field(DcimDeviceFaceChoices, graphql_name='face')
    status = sgqlc.types.Field(sgqlc.types.non_null(DcimDeviceStatusChoices), graphql_name='status')
    airflow = sgqlc.types.Field(DcimDeviceAirflowChoices, graphql_name='airflow')
    primary_ip4 = sgqlc.types.Field('IPAddressType', graphql_name='primary_ip4')
    primary_ip6 = sgqlc.types.Field('IPAddressType', graphql_name='primary_ip6')
    cluster = sgqlc.types.Field(ClusterType, graphql_name='cluster')
    virtual_chassis = sgqlc.types.Field('VirtualChassisType', graphql_name='virtual_chassis')
    vc_position = sgqlc.types.Field(Int, graphql_name='vc_position')
    vc_priority = sgqlc.types.Field(Int, graphql_name='vc_priority')
    consoleports = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ConsolePortType))), graphql_name='consoleports', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
))
    )
    consoleserverports = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ConsoleServerPortType))), graphql_name='consoleserverports', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
))
    )
    powerports = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('PowerPortType'))), graphql_name='powerports', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('maximum_draw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw', default=None)),
        ('allocated_draw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('maximum_draw__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__n', default=None)),
        ('maximum_draw__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__lte', default=None)),
        ('maximum_draw__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__lt', default=None)),
        ('maximum_draw__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__gte', default=None)),
        ('maximum_draw__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__gt', default=None)),
        ('allocated_draw__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__n', default=None)),
        ('allocated_draw__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__lte', default=None)),
        ('allocated_draw__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__lt', default=None)),
        ('allocated_draw__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__gte', default=None)),
        ('allocated_draw__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
))
    )
    poweroutlets = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('PowerOutletType'))), graphql_name='poweroutlets', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('feed_leg', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='feed_leg', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('feed_leg__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='feed_leg__n', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
))
    )
    interfaces = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('InterfaceType'))), graphql_name='interfaces', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('enabled', sgqlc.types.Arg(Boolean, graphql_name='enabled', default=None)),
        ('mtu', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu', default=None)),
        ('mgmt_only', sgqlc.types.Arg(Boolean, graphql_name='mgmt_only', default=None)),
        ('poe_mode', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode', default=None)),
        ('poe_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type', default=None)),
        ('mode', sgqlc.types.Arg(String, graphql_name='mode', default=None)),
        ('rf_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_role', default=None)),
        ('rf_channel', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel', default=None)),
        ('rf_channel_frequency', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency', default=None)),
        ('rf_channel_width', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width', default=None)),
        ('tx_power', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('kind', sgqlc.types.Arg(String, graphql_name='kind', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('bridge_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id', default=None)),
        ('lag_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='lag_id', default=None)),
        ('speed', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed', default=None)),
        ('duplex', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='duplex', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('wwn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn', default=None)),
        ('vlan_id', sgqlc.types.Arg(String, graphql_name='vlan_id', default=None)),
        ('vlan', sgqlc.types.Arg(String, graphql_name='vlan', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('vdc_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_id', default=None)),
        ('vdc_identifier', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_identifier', default=None)),
        ('vdc', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc', default=None)),
        ('l2vpn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id', default=None)),
        ('l2vpn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('mtu__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__n', default=None)),
        ('mtu__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lte', default=None)),
        ('mtu__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lt', default=None)),
        ('mtu__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gte', default=None)),
        ('mtu__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gt', default=None)),
        ('poe_mode__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode__n', default=None)),
        ('poe_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type__n', default=None)),
        ('mode__n', sgqlc.types.Arg(String, graphql_name='mode__n', default=None)),
        ('rf_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_role__n', default=None)),
        ('rf_channel__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel__n', default=None)),
        ('rf_channel_frequency__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__n', default=None)),
        ('rf_channel_frequency__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__lte', default=None)),
        ('rf_channel_frequency__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__lt', default=None)),
        ('rf_channel_frequency__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__gte', default=None)),
        ('rf_channel_frequency__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__gt', default=None)),
        ('rf_channel_width__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__n', default=None)),
        ('rf_channel_width__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__lte', default=None)),
        ('rf_channel_width__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__lt', default=None)),
        ('rf_channel_width__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__gte', default=None)),
        ('rf_channel_width__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__gt', default=None)),
        ('tx_power__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__n', default=None)),
        ('tx_power__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__lte', default=None)),
        ('tx_power__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__lt', default=None)),
        ('tx_power__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__gte', default=None)),
        ('tx_power__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('bridge_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id__n', default=None)),
        ('lag_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='lag_id__n', default=None)),
        ('speed__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__n', default=None)),
        ('speed__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__lte', default=None)),
        ('speed__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__lt', default=None)),
        ('speed__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__gte', default=None)),
        ('speed__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__gt', default=None)),
        ('duplex__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='duplex__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('wwn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__n', default=None)),
        ('wwn__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__ic', default=None)),
        ('wwn__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nic', default=None)),
        ('wwn__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__iew', default=None)),
        ('wwn__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__niew', default=None)),
        ('wwn__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__isw', default=None)),
        ('wwn__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nisw', default=None)),
        ('wwn__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__ie', default=None)),
        ('wwn__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nie', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('vdc_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_id__n', default=None)),
        ('vdc_identifier__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_identifier__n', default=None)),
        ('vdc__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc__n', default=None)),
        ('l2vpn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id__n', default=None)),
        ('l2vpn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn__n', default=None)),
))
    )
    frontports = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('FrontPortType'))), graphql_name='frontports', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('color', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('color__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__n', default=None)),
        ('color__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ic', default=None)),
        ('color__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nic', default=None)),
        ('color__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__iew', default=None)),
        ('color__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__niew', default=None)),
        ('color__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__isw', default=None)),
        ('color__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nisw', default=None)),
        ('color__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ie', default=None)),
        ('color__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nie', default=None)),
        ('color__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
))
    )
    rearports = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('RearPortType'))), graphql_name='rearports', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('color', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color', default=None)),
        ('positions', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('color__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__n', default=None)),
        ('color__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ic', default=None)),
        ('color__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nic', default=None)),
        ('color__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__iew', default=None)),
        ('color__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__niew', default=None)),
        ('color__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__isw', default=None)),
        ('color__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nisw', default=None)),
        ('color__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ie', default=None)),
        ('color__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nie', default=None)),
        ('color__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__empty', default=None)),
        ('positions__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__n', default=None)),
        ('positions__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__lte', default=None)),
        ('positions__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__lt', default=None)),
        ('positions__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__gte', default=None)),
        ('positions__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
))
    )
    modulebays = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('ModuleBayType'))), graphql_name='modulebays', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
))
    )
    parent_bay = sgqlc.types.Field(DeviceBayType, graphql_name='parent_bay')
    devicebays = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(DeviceBayType))), graphql_name='devicebays', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
))
    )
    inventoryitems = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('InventoryItemType'))), graphql_name='inventoryitems', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('part_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id', default=None)),
        ('asset_tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag', default=None)),
        ('discovered', sgqlc.types.Arg(Boolean, graphql_name='discovered', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('component_type', sgqlc.types.Arg(String, graphql_name='component_type', default=None)),
        ('component_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id', default=None)),
        ('serial', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('part_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__n', default=None)),
        ('part_id__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__ic', default=None)),
        ('part_id__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nic', default=None)),
        ('part_id__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__iew', default=None)),
        ('part_id__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__niew', default=None)),
        ('part_id__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__isw', default=None)),
        ('part_id__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nisw', default=None)),
        ('part_id__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__ie', default=None)),
        ('part_id__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nie', default=None)),
        ('part_id__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__empty', default=None)),
        ('asset_tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__n', default=None)),
        ('asset_tag__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ic', default=None)),
        ('asset_tag__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nic', default=None)),
        ('asset_tag__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__iew', default=None)),
        ('asset_tag__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__niew', default=None)),
        ('asset_tag__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__isw', default=None)),
        ('asset_tag__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nisw', default=None)),
        ('asset_tag__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ie', default=None)),
        ('asset_tag__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nie', default=None)),
        ('asset_tag__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__empty', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('component_type__n', sgqlc.types.Arg(String, graphql_name='component_type__n', default=None)),
        ('component_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__n', default=None)),
        ('component_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__lte', default=None)),
        ('component_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__lt', default=None)),
        ('component_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__gte', default=None)),
        ('component_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__gt', default=None)),
        ('serial__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__n', default=None)),
        ('serial__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ic', default=None)),
        ('serial__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nic', default=None)),
        ('serial__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__iew', default=None)),
        ('serial__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__niew', default=None)),
        ('serial__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__isw', default=None)),
        ('serial__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nisw', default=None)),
        ('serial__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ie', default=None)),
        ('serial__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nie', default=None)),
        ('serial__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__empty', default=None)),
))
    )
    cabletermination_set = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(CableTerminationType))), graphql_name='cabletermination_set', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('cable', sgqlc.types.Arg(String, graphql_name='cable', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('termination_type', sgqlc.types.Arg(String, graphql_name='termination_type', default=None)),
        ('termination_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('cable__n', sgqlc.types.Arg(String, graphql_name='cable__n', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('termination_type__n', sgqlc.types.Arg(String, graphql_name='termination_type__n', default=None)),
        ('termination_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id__n', default=None)),
        ('termination_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id__lte', default=None)),
        ('termination_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id__lt', default=None)),
        ('termination_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id__gte', default=None)),
        ('termination_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id__gt', default=None)),
))
    )
    modules = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('ModuleType'))), graphql_name='modules', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('asset_tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('module_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_type_id', default=None)),
        ('module_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_type', default=None)),
        ('module_bay_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_bay_id', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('serial', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('asset_tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__n', default=None)),
        ('asset_tag__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ic', default=None)),
        ('asset_tag__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nic', default=None)),
        ('asset_tag__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__iew', default=None)),
        ('asset_tag__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__niew', default=None)),
        ('asset_tag__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__isw', default=None)),
        ('asset_tag__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nisw', default=None)),
        ('asset_tag__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ie', default=None)),
        ('asset_tag__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nie', default=None)),
        ('asset_tag__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
        ('module_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_type_id__n', default=None)),
        ('module_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_type__n', default=None)),
        ('module_bay_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_bay_id__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('serial__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__n', default=None)),
        ('serial__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ic', default=None)),
        ('serial__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nic', default=None)),
        ('serial__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__iew', default=None)),
        ('serial__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__niew', default=None)),
        ('serial__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__isw', default=None)),
        ('serial__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nisw', default=None)),
        ('serial__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ie', default=None)),
        ('serial__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nie', default=None)),
        ('serial__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__empty', default=None)),
))
    )
    vc_master_for = sgqlc.types.Field('VirtualChassisType', graphql_name='vc_master_for')
    vdcs = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('VirtualDeviceContextType'))), graphql_name='vdcs', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('has_primary_ip', sgqlc.types.Arg(Boolean, graphql_name='has_primary_ip', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
))
    )
    services = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('ServiceType'))), graphql_name='services', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('protocol', sgqlc.types.Arg(String, graphql_name='protocol', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_machine_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id', default=None)),
        ('virtual_machine', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine', default=None)),
        ('ipaddress_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ipaddress_id', default=None)),
        ('ipaddress', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ipaddress', default=None)),
        ('port', sgqlc.types.Arg(Int, graphql_name='port', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('protocol__n', sgqlc.types.Arg(String, graphql_name='protocol__n', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_machine_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id__n', default=None)),
        ('virtual_machine__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine__n', default=None)),
        ('ipaddress_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ipaddress_id__n', default=None)),
        ('ipaddress__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ipaddress__n', default=None)),
))
    )
    virtual_machines = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('VirtualMachineType'))), graphql_name='virtual_machines', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('cluster', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster', default=None)),
        ('vcpus', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus', default=None)),
        ('memory', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory', default=None)),
        ('disk', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('local_context_data', sgqlc.types.Arg(Boolean, graphql_name='local_context_data', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('cluster_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group_id', default=None)),
        ('cluster_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group', default=None)),
        ('cluster_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type_id', default=None)),
        ('cluster_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type', default=None)),
        ('cluster_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('platform_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id', default=None)),
        ('platform', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('has_primary_ip', sgqlc.types.Arg(Boolean, graphql_name='has_primary_ip', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('cluster__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster__n', default=None)),
        ('vcpus__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__n', default=None)),
        ('vcpus__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__lte', default=None)),
        ('vcpus__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__lt', default=None)),
        ('vcpus__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__gte', default=None)),
        ('vcpus__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__gt', default=None)),
        ('memory__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__n', default=None)),
        ('memory__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__lte', default=None)),
        ('memory__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__lt', default=None)),
        ('memory__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__gte', default=None)),
        ('memory__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__gt', default=None)),
        ('disk__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__n', default=None)),
        ('disk__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__lte', default=None)),
        ('disk__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__lt', default=None)),
        ('disk__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__gte', default=None)),
        ('disk__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('cluster_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group_id__n', default=None)),
        ('cluster_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group__n', default=None)),
        ('cluster_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type_id__n', default=None)),
        ('cluster_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type__n', default=None)),
        ('cluster_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('platform_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id__n', default=None)),
        ('platform__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of('JournalEntryType'), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')
    contacts = sgqlc.types.Field(sgqlc.types.list_of(ContactAssignmentType), graphql_name='contacts')
    image_attachments = sgqlc.types.Field(sgqlc.types.list_of('ImageAttachmentType'), graphql_name='image_attachments')
    config_context = sgqlc.types.Field(GenericScalar, graphql_name='config_context')


class DeviceTypeType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'weight', 'weight_unit', '_abs_weight', 'manufacturer', 'model', 'slug', 'part_number', 'u_height', 'is_full_depth', 'subdevice_role', 'airflow', 'front_image', 'rear_image', 'consoleporttemplates', 'consoleserverporttemplates', 'powerporttemplates', 'poweroutlettemplates', 'interfacetemplates', 'frontporttemplates', 'rearporttemplates', 'modulebaytemplates', 'devicebaytemplates', 'inventoryitemtemplates', 'instances', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    weight = sgqlc.types.Field(Decimal, graphql_name='weight')
    weight_unit = sgqlc.types.Field(DcimDeviceTypeWeightUnitChoices, graphql_name='weight_unit')
    _abs_weight = sgqlc.types.Field(BigInt, graphql_name='_abs_weight')
    manufacturer = sgqlc.types.Field(sgqlc.types.non_null('ManufacturerType'), graphql_name='manufacturer')
    model = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='model')
    slug = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='slug')
    part_number = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='part_number')
    u_height = sgqlc.types.Field(sgqlc.types.non_null(Decimal), graphql_name='u_height')
    is_full_depth = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='is_full_depth')
    subdevice_role = sgqlc.types.Field(DcimDeviceTypeSubdeviceRoleChoices, graphql_name='subdevice_role')
    airflow = sgqlc.types.Field(DcimDeviceTypeAirflowChoices, graphql_name='airflow')
    front_image = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='front_image')
    rear_image = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='rear_image')
    consoleporttemplates = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ConsolePortTemplateType))), graphql_name='consoleporttemplates', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('type', sgqlc.types.Arg(String, graphql_name='type', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('moduletype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('type__n', sgqlc.types.Arg(String, graphql_name='type__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('moduletype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id__n', default=None)),
))
    )
    consoleserverporttemplates = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ConsoleServerPortTemplateType))), graphql_name='consoleserverporttemplates', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('type', sgqlc.types.Arg(String, graphql_name='type', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('moduletype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('type__n', sgqlc.types.Arg(String, graphql_name='type__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('moduletype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id__n', default=None)),
))
    )
    powerporttemplates = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('PowerPortTemplateType'))), graphql_name='powerporttemplates', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('type', sgqlc.types.Arg(String, graphql_name='type', default=None)),
        ('maximum_draw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw', default=None)),
        ('allocated_draw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('moduletype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('type__n', sgqlc.types.Arg(String, graphql_name='type__n', default=None)),
        ('maximum_draw__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__n', default=None)),
        ('maximum_draw__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__lte', default=None)),
        ('maximum_draw__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__lt', default=None)),
        ('maximum_draw__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__gte', default=None)),
        ('maximum_draw__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__gt', default=None)),
        ('allocated_draw__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__n', default=None)),
        ('allocated_draw__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__lte', default=None)),
        ('allocated_draw__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__lt', default=None)),
        ('allocated_draw__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__gte', default=None)),
        ('allocated_draw__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('moduletype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id__n', default=None)),
))
    )
    poweroutlettemplates = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('PowerOutletTemplateType'))), graphql_name='poweroutlettemplates', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('type', sgqlc.types.Arg(String, graphql_name='type', default=None)),
        ('feed_leg', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='feed_leg', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('moduletype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('type__n', sgqlc.types.Arg(String, graphql_name='type__n', default=None)),
        ('feed_leg__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='feed_leg__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('moduletype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id__n', default=None)),
))
    )
    interfacetemplates = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('InterfaceTemplateType'))), graphql_name='interfacetemplates', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('mgmt_only', sgqlc.types.Arg(Boolean, graphql_name='mgmt_only', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('moduletype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id', default=None)),
        ('poe_mode', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode', default=None)),
        ('poe_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('moduletype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id__n', default=None)),
        ('poe_mode__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode__n', default=None)),
        ('poe_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type__n', default=None)),
))
    )
    frontporttemplates = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('FrontPortTemplateType'))), graphql_name='frontporttemplates', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('color', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('moduletype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('color__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__n', default=None)),
        ('color__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ic', default=None)),
        ('color__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nic', default=None)),
        ('color__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__iew', default=None)),
        ('color__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__niew', default=None)),
        ('color__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__isw', default=None)),
        ('color__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nisw', default=None)),
        ('color__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ie', default=None)),
        ('color__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nie', default=None)),
        ('color__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('moduletype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id__n', default=None)),
))
    )
    rearporttemplates = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('RearPortTemplateType'))), graphql_name='rearporttemplates', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('color', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color', default=None)),
        ('positions', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('moduletype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('color__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__n', default=None)),
        ('color__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ic', default=None)),
        ('color__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nic', default=None)),
        ('color__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__iew', default=None)),
        ('color__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__niew', default=None)),
        ('color__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__isw', default=None)),
        ('color__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nisw', default=None)),
        ('color__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ie', default=None)),
        ('color__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nie', default=None)),
        ('color__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__empty', default=None)),
        ('positions__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__n', default=None)),
        ('positions__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__lte', default=None)),
        ('positions__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__lt', default=None)),
        ('positions__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__gte', default=None)),
        ('positions__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('moduletype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id__n', default=None)),
))
    )
    modulebaytemplates = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('ModuleBayTemplateType'))), graphql_name='modulebaytemplates', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
))
    )
    devicebaytemplates = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(DeviceBayTemplateType))), graphql_name='devicebaytemplates', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
))
    )
    inventoryitemtemplates = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('InventoryItemTemplateType'))), graphql_name='inventoryitemtemplates', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('part_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('component_type', sgqlc.types.Arg(String, graphql_name='component_type', default=None)),
        ('component_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('part_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__n', default=None)),
        ('part_id__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__ic', default=None)),
        ('part_id__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nic', default=None)),
        ('part_id__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__iew', default=None)),
        ('part_id__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__niew', default=None)),
        ('part_id__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__isw', default=None)),
        ('part_id__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nisw', default=None)),
        ('part_id__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__ie', default=None)),
        ('part_id__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nie', default=None)),
        ('part_id__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('component_type__n', sgqlc.types.Arg(String, graphql_name='component_type__n', default=None)),
        ('component_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__n', default=None)),
        ('component_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__lte', default=None)),
        ('component_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__lt', default=None)),
        ('component_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__gte', default=None)),
        ('component_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__gt', default=None)),
))
    )
    instances = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(DeviceType))), graphql_name='instances', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('asset_tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag', default=None)),
        ('face', sgqlc.types.Arg(String, graphql_name='face', default=None)),
        ('position', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position', default=None)),
        ('airflow', sgqlc.types.Arg(String, graphql_name='airflow', default=None)),
        ('vc_position', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position', default=None)),
        ('vc_priority', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('local_context_data', sgqlc.types.Arg(Boolean, graphql_name='local_context_data', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('device_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type', default=None)),
        ('device_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type_id', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('parent_device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_device_id', default=None)),
        ('platform_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id', default=None)),
        ('platform', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('cluster_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id', default=None)),
        ('model', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('is_full_depth', sgqlc.types.Arg(Boolean, graphql_name='is_full_depth', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('serial', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial', default=None)),
        ('has_primary_ip', sgqlc.types.Arg(Boolean, graphql_name='has_primary_ip', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis_member', sgqlc.types.Arg(Boolean, graphql_name='virtual_chassis_member', default=None)),
        ('console_ports', sgqlc.types.Arg(Boolean, graphql_name='console_ports', default=None)),
        ('console_server_ports', sgqlc.types.Arg(Boolean, graphql_name='console_server_ports', default=None)),
        ('power_ports', sgqlc.types.Arg(Boolean, graphql_name='power_ports', default=None)),
        ('power_outlets', sgqlc.types.Arg(Boolean, graphql_name='power_outlets', default=None)),
        ('interfaces', sgqlc.types.Arg(Boolean, graphql_name='interfaces', default=None)),
        ('pass_through_ports', sgqlc.types.Arg(Boolean, graphql_name='pass_through_ports', default=None)),
        ('module_bays', sgqlc.types.Arg(Boolean, graphql_name='module_bays', default=None)),
        ('device_bays', sgqlc.types.Arg(Boolean, graphql_name='device_bays', default=None)),
        ('primary_ip4_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip4_id', default=None)),
        ('primary_ip6_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip6_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('asset_tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__n', default=None)),
        ('asset_tag__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ic', default=None)),
        ('asset_tag__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nic', default=None)),
        ('asset_tag__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__iew', default=None)),
        ('asset_tag__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__niew', default=None)),
        ('asset_tag__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__isw', default=None)),
        ('asset_tag__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nisw', default=None)),
        ('asset_tag__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ie', default=None)),
        ('asset_tag__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nie', default=None)),
        ('asset_tag__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__empty', default=None)),
        ('face__n', sgqlc.types.Arg(String, graphql_name='face__n', default=None)),
        ('position__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__n', default=None)),
        ('position__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__lte', default=None)),
        ('position__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__lt', default=None)),
        ('position__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__gte', default=None)),
        ('position__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__gt', default=None)),
        ('airflow__n', sgqlc.types.Arg(String, graphql_name='airflow__n', default=None)),
        ('vc_position__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__n', default=None)),
        ('vc_position__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__lte', default=None)),
        ('vc_position__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__lt', default=None)),
        ('vc_position__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__gte', default=None)),
        ('vc_position__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__gt', default=None)),
        ('vc_priority__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__n', default=None)),
        ('vc_priority__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__lte', default=None)),
        ('vc_priority__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__lt', default=None)),
        ('vc_priority__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__gte', default=None)),
        ('vc_priority__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
        ('device_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type__n', default=None)),
        ('device_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type_id__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('parent_device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_device_id__n', default=None)),
        ('platform_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id__n', default=None)),
        ('platform__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('cluster_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id__n', default=None)),
        ('model__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__n', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('serial__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__n', default=None)),
        ('serial__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ic', default=None)),
        ('serial__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nic', default=None)),
        ('serial__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__iew', default=None)),
        ('serial__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__niew', default=None)),
        ('serial__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__isw', default=None)),
        ('serial__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nisw', default=None)),
        ('serial__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ie', default=None)),
        ('serial__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nie', default=None)),
        ('serial__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__empty', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('primary_ip4_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip4_id__n', default=None)),
        ('primary_ip6_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip6_id__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of('JournalEntryType'), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class ExportTemplateType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'name', 'description', 'template_code', 'mime_type', 'file_extension', 'as_attachment', 'display', 'class_type', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    template_code = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='template_code')
    mime_type = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='mime_type')
    file_extension = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='file_extension')
    as_attachment = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='as_attachment')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class FHRPGroupAssignmentType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'group', 'priority', 'display', 'class_type', 'interface')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    group = sgqlc.types.Field(sgqlc.types.non_null('FHRPGroupType'), graphql_name='group')
    priority = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='priority')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    interface = sgqlc.types.Field('FHRPGroupInterfaceType', graphql_name='interface')


class FHRPGroupType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'group_id', 'name', 'protocol', 'auth_type', 'auth_key', 'fhrpgroupassignment_set', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    group_id = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='group_id')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    protocol = sgqlc.types.Field(sgqlc.types.non_null(IpamFHRPGroupProtocolChoices), graphql_name='protocol')
    auth_type = sgqlc.types.Field(IpamFHRPGroupAuthTypeChoices, graphql_name='auth_type')
    auth_key = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='auth_key')
    fhrpgroupassignment_set = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(FHRPGroupAssignmentType))), graphql_name='fhrpgroupassignment_set', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('interface_type', sgqlc.types.Arg(String, graphql_name='interface_type', default=None)),
        ('interface_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_id', default=None)),
        ('priority', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='priority', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('virtual_machine', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine', default=None)),
        ('virtual_machine_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('interface_type__n', sgqlc.types.Arg(String, graphql_name='interface_type__n', default=None)),
        ('interface_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_id__n', default=None)),
        ('interface_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_id__lte', default=None)),
        ('interface_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_id__lt', default=None)),
        ('interface_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_id__gte', default=None)),
        ('interface_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_id__gt', default=None)),
        ('priority__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='priority__n', default=None)),
        ('priority__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='priority__lte', default=None)),
        ('priority__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='priority__lt', default=None)),
        ('priority__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='priority__gte', default=None)),
        ('priority__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='priority__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of('JournalEntryType'), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class FrontPortTemplateType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'name', '_name', 'label', 'description', 'device_type', 'module_type', 'type', 'color', 'rear_port', 'rear_port_position', 'display', 'class_type', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    _name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='_name')
    label = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='label')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    device_type = sgqlc.types.Field(DeviceTypeType, graphql_name='device_type')
    module_type = sgqlc.types.Field('ModuleTypeType', graphql_name='module_type')
    type = sgqlc.types.Field(sgqlc.types.non_null(DcimFrontPortTemplateTypeChoices), graphql_name='type')
    color = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='color')
    rear_port = sgqlc.types.Field(sgqlc.types.non_null('RearPortTemplateType'), graphql_name='rear_port')
    rear_port_position = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='rear_port_position')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class FrontPortType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'device', 'name', '_name', 'label', 'description', 'module', 'cable', 'cable_end', 'mark_connected', 'type', 'color', 'rear_port', 'rear_port_position', 'link_peers', 'display', 'class_type', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    device = sgqlc.types.Field(sgqlc.types.non_null(DeviceType), graphql_name='device')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    _name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='_name')
    label = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='label')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    module = sgqlc.types.Field('ModuleType', graphql_name='module')
    cable = sgqlc.types.Field(CableType, graphql_name='cable')
    cable_end = sgqlc.types.Field(DcimFrontPortCableEndChoices, graphql_name='cable_end')
    mark_connected = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='mark_connected')
    type = sgqlc.types.Field(sgqlc.types.non_null(DcimFrontPortTypeChoices), graphql_name='type')
    color = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='color')
    rear_port = sgqlc.types.Field(sgqlc.types.non_null('RearPortType'), graphql_name='rear_port')
    rear_port_position = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='rear_port_position')
    link_peers = sgqlc.types.Field(sgqlc.types.list_of('LinkPeerType'), graphql_name='link_peers')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class GroupType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'name')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')


class IPAddressFamilyType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('value', 'label')
    value = sgqlc.types.Field(Int, graphql_name='value')
    label = sgqlc.types.Field(String, graphql_name='label')


class IPAddressType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'address', 'vrf', 'tenant', 'status', 'role', 'nat_inside', 'dns_name', 'nat_outside', 'services', 'family', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog', 'assigned_object')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    address = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='address')
    vrf = sgqlc.types.Field('VRFType', graphql_name='vrf')
    tenant = sgqlc.types.Field('TenantType', graphql_name='tenant')
    status = sgqlc.types.Field(sgqlc.types.non_null(IpamIPAddressStatusChoices), graphql_name='status')
    role = sgqlc.types.Field(IpamIPAddressRoleChoices, graphql_name='role')
    nat_inside = sgqlc.types.Field('IPAddressType', graphql_name='nat_inside')
    dns_name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='dns_name')
    nat_outside = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('IPAddressType'))), graphql_name='nat_outside', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('dns_name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('family', sgqlc.types.Arg(Int, graphql_name='family', default=None)),
        ('parent', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent', default=None)),
        ('address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address', default=None)),
        ('mask_length', sgqlc.types.Arg(Int, graphql_name='mask_length', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('present_in_vrf_id', sgqlc.types.Arg(String, graphql_name='present_in_vrf_id', default=None)),
        ('present_in_vrf', sgqlc.types.Arg(String, graphql_name='present_in_vrf', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('virtual_machine', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine', default=None)),
        ('virtual_machine_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id', default=None)),
        ('interface', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface', default=None)),
        ('interface_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_id', default=None)),
        ('vminterface', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vminterface', default=None)),
        ('vminterface_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vminterface_id', default=None)),
        ('fhrpgroup_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='fhrpgroup_id', default=None)),
        ('assigned_to_interface', sgqlc.types.Arg(Boolean, graphql_name='assigned_to_interface', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('dns_name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__n', default=None)),
        ('dns_name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__ic', default=None)),
        ('dns_name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__nic', default=None)),
        ('dns_name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__iew', default=None)),
        ('dns_name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__niew', default=None)),
        ('dns_name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__isw', default=None)),
        ('dns_name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__nisw', default=None)),
        ('dns_name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__ie', default=None)),
        ('dns_name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__nie', default=None)),
        ('dns_name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('interface__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface__n', default=None)),
        ('interface_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_id__n', default=None)),
        ('vminterface__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vminterface__n', default=None)),
        ('vminterface_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vminterface_id__n', default=None)),
        ('fhrpgroup_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='fhrpgroup_id__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
))
    )
    services = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('ServiceType'))), graphql_name='services', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('protocol', sgqlc.types.Arg(String, graphql_name='protocol', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_machine_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id', default=None)),
        ('virtual_machine', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine', default=None)),
        ('ipaddress_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ipaddress_id', default=None)),
        ('ipaddress', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ipaddress', default=None)),
        ('port', sgqlc.types.Arg(Int, graphql_name='port', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('protocol__n', sgqlc.types.Arg(String, graphql_name='protocol__n', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_machine_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id__n', default=None)),
        ('virtual_machine__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine__n', default=None)),
        ('ipaddress_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ipaddress_id__n', default=None)),
        ('ipaddress__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ipaddress__n', default=None)),
))
    )
    family = sgqlc.types.Field(IPAddressFamilyType, graphql_name='family')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of('JournalEntryType'), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')
    assigned_object = sgqlc.types.Field('IPAddressAssignmentType', graphql_name='assigned_object')


class IPRangeType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'start_address', 'end_address', 'size', 'vrf', 'tenant', 'status', 'role', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    start_address = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='start_address')
    end_address = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='end_address')
    size = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='size')
    vrf = sgqlc.types.Field('VRFType', graphql_name='vrf')
    tenant = sgqlc.types.Field('TenantType', graphql_name='tenant')
    status = sgqlc.types.Field(sgqlc.types.non_null(IpamIPRangeStatusChoices), graphql_name='status')
    role = sgqlc.types.Field('RoleType', graphql_name='role')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of('JournalEntryType'), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class ImageAttachmentType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'content_type', 'object_id', 'image', 'image_height', 'image_width', 'name', 'display', 'class_type')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    content_type = sgqlc.types.Field(sgqlc.types.non_null(ContentTypeType), graphql_name='content_type')
    object_id = sgqlc.types.Field(sgqlc.types.non_null(BigInt), graphql_name='object_id')
    image = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='image')
    image_height = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='image_height')
    image_width = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='image_width')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')


class InterfaceTemplateType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'name', 'label', 'description', 'device_type', 'module_type', '_name', 'type', 'mgmt_only', 'poe_mode', 'poe_type', 'display', 'class_type', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    label = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='label')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    device_type = sgqlc.types.Field(DeviceTypeType, graphql_name='device_type')
    module_type = sgqlc.types.Field('ModuleTypeType', graphql_name='module_type')
    _name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='_name')
    type = sgqlc.types.Field(sgqlc.types.non_null(DcimInterfaceTemplateTypeChoices), graphql_name='type')
    mgmt_only = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='mgmt_only')
    poe_mode = sgqlc.types.Field(DcimInterfaceTemplatePoeModeChoices, graphql_name='poe_mode')
    poe_type = sgqlc.types.Field(DcimInterfaceTemplatePoeTypeChoices, graphql_name='poe_type')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class InterfaceType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'device', 'name', 'label', 'description', 'module', 'cable', 'cable_end', 'mark_connected', 'enabled', 'mac_address', 'mtu', 'mode', 'parent', 'bridge', '_name', 'vdcs', 'lag', 'type', 'mgmt_only', 'speed', 'duplex', 'wwn', 'rf_role', 'rf_channel', 'rf_channel_frequency', 'rf_channel_width', 'tx_power', 'poe_mode', 'poe_type', 'wireless_link', 'wireless_lans', 'untagged_vlan', 'tagged_vlans', 'vrf', 'member_interfaces', 'child_interfaces', 'bridge_interfaces', 'connected_endpoints', 'link_peers', 'display', 'class_type', 'custom_fields', 'changelog', 'ip_addresses')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    device = sgqlc.types.Field(sgqlc.types.non_null(DeviceType), graphql_name='device')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    label = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='label')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    module = sgqlc.types.Field('ModuleType', graphql_name='module')
    cable = sgqlc.types.Field(CableType, graphql_name='cable')
    cable_end = sgqlc.types.Field(DcimInterfaceCableEndChoices, graphql_name='cable_end')
    mark_connected = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='mark_connected')
    enabled = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='enabled')
    mac_address = sgqlc.types.Field(String, graphql_name='mac_address')
    mtu = sgqlc.types.Field(Int, graphql_name='mtu')
    mode = sgqlc.types.Field(DcimInterfaceModeChoices, graphql_name='mode')
    parent = sgqlc.types.Field('InterfaceType', graphql_name='parent')
    bridge = sgqlc.types.Field('InterfaceType', graphql_name='bridge')
    _name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='_name')
    vdcs = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('VirtualDeviceContextType'))), graphql_name='vdcs', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('has_primary_ip', sgqlc.types.Arg(Boolean, graphql_name='has_primary_ip', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
))
    )
    lag = sgqlc.types.Field('InterfaceType', graphql_name='lag')
    type = sgqlc.types.Field(sgqlc.types.non_null(DcimInterfaceTypeChoices), graphql_name='type')
    mgmt_only = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='mgmt_only')
    speed = sgqlc.types.Field(Int, graphql_name='speed')
    duplex = sgqlc.types.Field(DcimInterfaceDuplexChoices, graphql_name='duplex')
    wwn = sgqlc.types.Field(String, graphql_name='wwn')
    rf_role = sgqlc.types.Field(DcimInterfaceRfRoleChoices, graphql_name='rf_role')
    rf_channel = sgqlc.types.Field(DcimInterfaceRfChannelChoices, graphql_name='rf_channel')
    rf_channel_frequency = sgqlc.types.Field(Decimal, graphql_name='rf_channel_frequency')
    rf_channel_width = sgqlc.types.Field(Decimal, graphql_name='rf_channel_width')
    tx_power = sgqlc.types.Field(Int, graphql_name='tx_power')
    poe_mode = sgqlc.types.Field(DcimInterfacePoeModeChoices, graphql_name='poe_mode')
    poe_type = sgqlc.types.Field(DcimInterfacePoeTypeChoices, graphql_name='poe_type')
    wireless_link = sgqlc.types.Field('WirelessLinkType', graphql_name='wireless_link')
    wireless_lans = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('WirelessLANType'))), graphql_name='wireless_lans', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('ssid', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid', default=None)),
        ('auth_psk', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('vlan_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vlan_id', default=None)),
        ('auth_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_type', default=None)),
        ('auth_cipher', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_cipher', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('ssid__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__n', default=None)),
        ('ssid__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__ic', default=None)),
        ('ssid__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__nic', default=None)),
        ('ssid__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__iew', default=None)),
        ('ssid__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__niew', default=None)),
        ('ssid__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__isw', default=None)),
        ('ssid__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__nisw', default=None)),
        ('ssid__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__ie', default=None)),
        ('ssid__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__nie', default=None)),
        ('ssid__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__empty', default=None)),
        ('auth_psk__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__n', default=None)),
        ('auth_psk__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__ic', default=None)),
        ('auth_psk__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__nic', default=None)),
        ('auth_psk__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__iew', default=None)),
        ('auth_psk__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__niew', default=None)),
        ('auth_psk__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__isw', default=None)),
        ('auth_psk__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__nisw', default=None)),
        ('auth_psk__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__ie', default=None)),
        ('auth_psk__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__nie', default=None)),
        ('auth_psk__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('vlan_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vlan_id__n', default=None)),
        ('auth_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_type__n', default=None)),
        ('auth_cipher__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_cipher__n', default=None)),
))
    )
    untagged_vlan = sgqlc.types.Field('VLANType', graphql_name='untagged_vlan')
    tagged_vlans = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('VLANType'))), graphql_name='tagged_vlans', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('vid', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('available_on_device', sgqlc.types.Arg(String, graphql_name='available_on_device', default=None)),
        ('available_on_virtualmachine', sgqlc.types.Arg(String, graphql_name='available_on_virtualmachine', default=None)),
        ('l2vpn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id', default=None)),
        ('l2vpn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('vid__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__n', default=None)),
        ('vid__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__lte', default=None)),
        ('vid__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__lt', default=None)),
        ('vid__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__gte', default=None)),
        ('vid__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('l2vpn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id__n', default=None)),
        ('l2vpn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn__n', default=None)),
))
    )
    vrf = sgqlc.types.Field('VRFType', graphql_name='vrf')
    member_interfaces = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('InterfaceType'))), graphql_name='member_interfaces', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('enabled', sgqlc.types.Arg(Boolean, graphql_name='enabled', default=None)),
        ('mtu', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu', default=None)),
        ('mgmt_only', sgqlc.types.Arg(Boolean, graphql_name='mgmt_only', default=None)),
        ('poe_mode', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode', default=None)),
        ('poe_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type', default=None)),
        ('mode', sgqlc.types.Arg(String, graphql_name='mode', default=None)),
        ('rf_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_role', default=None)),
        ('rf_channel', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel', default=None)),
        ('rf_channel_frequency', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency', default=None)),
        ('rf_channel_width', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width', default=None)),
        ('tx_power', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('kind', sgqlc.types.Arg(String, graphql_name='kind', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('bridge_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id', default=None)),
        ('lag_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='lag_id', default=None)),
        ('speed', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed', default=None)),
        ('duplex', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='duplex', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('wwn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn', default=None)),
        ('vlan_id', sgqlc.types.Arg(String, graphql_name='vlan_id', default=None)),
        ('vlan', sgqlc.types.Arg(String, graphql_name='vlan', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('vdc_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_id', default=None)),
        ('vdc_identifier', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_identifier', default=None)),
        ('vdc', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc', default=None)),
        ('l2vpn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id', default=None)),
        ('l2vpn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('mtu__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__n', default=None)),
        ('mtu__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lte', default=None)),
        ('mtu__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lt', default=None)),
        ('mtu__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gte', default=None)),
        ('mtu__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gt', default=None)),
        ('poe_mode__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode__n', default=None)),
        ('poe_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type__n', default=None)),
        ('mode__n', sgqlc.types.Arg(String, graphql_name='mode__n', default=None)),
        ('rf_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_role__n', default=None)),
        ('rf_channel__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel__n', default=None)),
        ('rf_channel_frequency__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__n', default=None)),
        ('rf_channel_frequency__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__lte', default=None)),
        ('rf_channel_frequency__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__lt', default=None)),
        ('rf_channel_frequency__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__gte', default=None)),
        ('rf_channel_frequency__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__gt', default=None)),
        ('rf_channel_width__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__n', default=None)),
        ('rf_channel_width__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__lte', default=None)),
        ('rf_channel_width__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__lt', default=None)),
        ('rf_channel_width__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__gte', default=None)),
        ('rf_channel_width__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__gt', default=None)),
        ('tx_power__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__n', default=None)),
        ('tx_power__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__lte', default=None)),
        ('tx_power__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__lt', default=None)),
        ('tx_power__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__gte', default=None)),
        ('tx_power__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('bridge_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id__n', default=None)),
        ('lag_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='lag_id__n', default=None)),
        ('speed__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__n', default=None)),
        ('speed__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__lte', default=None)),
        ('speed__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__lt', default=None)),
        ('speed__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__gte', default=None)),
        ('speed__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__gt', default=None)),
        ('duplex__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='duplex__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('wwn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__n', default=None)),
        ('wwn__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__ic', default=None)),
        ('wwn__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nic', default=None)),
        ('wwn__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__iew', default=None)),
        ('wwn__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__niew', default=None)),
        ('wwn__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__isw', default=None)),
        ('wwn__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nisw', default=None)),
        ('wwn__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__ie', default=None)),
        ('wwn__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nie', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('vdc_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_id__n', default=None)),
        ('vdc_identifier__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_identifier__n', default=None)),
        ('vdc__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc__n', default=None)),
        ('l2vpn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id__n', default=None)),
        ('l2vpn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn__n', default=None)),
))
    )
    child_interfaces = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('InterfaceType'))), graphql_name='child_interfaces', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('enabled', sgqlc.types.Arg(Boolean, graphql_name='enabled', default=None)),
        ('mtu', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu', default=None)),
        ('mgmt_only', sgqlc.types.Arg(Boolean, graphql_name='mgmt_only', default=None)),
        ('poe_mode', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode', default=None)),
        ('poe_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type', default=None)),
        ('mode', sgqlc.types.Arg(String, graphql_name='mode', default=None)),
        ('rf_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_role', default=None)),
        ('rf_channel', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel', default=None)),
        ('rf_channel_frequency', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency', default=None)),
        ('rf_channel_width', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width', default=None)),
        ('tx_power', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('kind', sgqlc.types.Arg(String, graphql_name='kind', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('bridge_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id', default=None)),
        ('lag_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='lag_id', default=None)),
        ('speed', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed', default=None)),
        ('duplex', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='duplex', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('wwn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn', default=None)),
        ('vlan_id', sgqlc.types.Arg(String, graphql_name='vlan_id', default=None)),
        ('vlan', sgqlc.types.Arg(String, graphql_name='vlan', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('vdc_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_id', default=None)),
        ('vdc_identifier', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_identifier', default=None)),
        ('vdc', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc', default=None)),
        ('l2vpn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id', default=None)),
        ('l2vpn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('mtu__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__n', default=None)),
        ('mtu__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lte', default=None)),
        ('mtu__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lt', default=None)),
        ('mtu__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gte', default=None)),
        ('mtu__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gt', default=None)),
        ('poe_mode__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode__n', default=None)),
        ('poe_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type__n', default=None)),
        ('mode__n', sgqlc.types.Arg(String, graphql_name='mode__n', default=None)),
        ('rf_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_role__n', default=None)),
        ('rf_channel__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel__n', default=None)),
        ('rf_channel_frequency__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__n', default=None)),
        ('rf_channel_frequency__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__lte', default=None)),
        ('rf_channel_frequency__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__lt', default=None)),
        ('rf_channel_frequency__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__gte', default=None)),
        ('rf_channel_frequency__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__gt', default=None)),
        ('rf_channel_width__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__n', default=None)),
        ('rf_channel_width__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__lte', default=None)),
        ('rf_channel_width__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__lt', default=None)),
        ('rf_channel_width__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__gte', default=None)),
        ('rf_channel_width__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__gt', default=None)),
        ('tx_power__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__n', default=None)),
        ('tx_power__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__lte', default=None)),
        ('tx_power__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__lt', default=None)),
        ('tx_power__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__gte', default=None)),
        ('tx_power__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('bridge_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id__n', default=None)),
        ('lag_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='lag_id__n', default=None)),
        ('speed__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__n', default=None)),
        ('speed__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__lte', default=None)),
        ('speed__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__lt', default=None)),
        ('speed__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__gte', default=None)),
        ('speed__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__gt', default=None)),
        ('duplex__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='duplex__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('wwn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__n', default=None)),
        ('wwn__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__ic', default=None)),
        ('wwn__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nic', default=None)),
        ('wwn__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__iew', default=None)),
        ('wwn__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__niew', default=None)),
        ('wwn__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__isw', default=None)),
        ('wwn__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nisw', default=None)),
        ('wwn__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__ie', default=None)),
        ('wwn__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nie', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('vdc_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_id__n', default=None)),
        ('vdc_identifier__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_identifier__n', default=None)),
        ('vdc__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc__n', default=None)),
        ('l2vpn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id__n', default=None)),
        ('l2vpn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn__n', default=None)),
))
    )
    bridge_interfaces = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('InterfaceType'))), graphql_name='bridge_interfaces', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('enabled', sgqlc.types.Arg(Boolean, graphql_name='enabled', default=None)),
        ('mtu', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu', default=None)),
        ('mgmt_only', sgqlc.types.Arg(Boolean, graphql_name='mgmt_only', default=None)),
        ('poe_mode', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode', default=None)),
        ('poe_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type', default=None)),
        ('mode', sgqlc.types.Arg(String, graphql_name='mode', default=None)),
        ('rf_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_role', default=None)),
        ('rf_channel', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel', default=None)),
        ('rf_channel_frequency', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency', default=None)),
        ('rf_channel_width', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width', default=None)),
        ('tx_power', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('kind', sgqlc.types.Arg(String, graphql_name='kind', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('bridge_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id', default=None)),
        ('lag_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='lag_id', default=None)),
        ('speed', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed', default=None)),
        ('duplex', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='duplex', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('wwn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn', default=None)),
        ('vlan_id', sgqlc.types.Arg(String, graphql_name='vlan_id', default=None)),
        ('vlan', sgqlc.types.Arg(String, graphql_name='vlan', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('vdc_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_id', default=None)),
        ('vdc_identifier', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_identifier', default=None)),
        ('vdc', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc', default=None)),
        ('l2vpn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id', default=None)),
        ('l2vpn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('mtu__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__n', default=None)),
        ('mtu__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lte', default=None)),
        ('mtu__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lt', default=None)),
        ('mtu__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gte', default=None)),
        ('mtu__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gt', default=None)),
        ('poe_mode__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode__n', default=None)),
        ('poe_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type__n', default=None)),
        ('mode__n', sgqlc.types.Arg(String, graphql_name='mode__n', default=None)),
        ('rf_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_role__n', default=None)),
        ('rf_channel__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel__n', default=None)),
        ('rf_channel_frequency__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__n', default=None)),
        ('rf_channel_frequency__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__lte', default=None)),
        ('rf_channel_frequency__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__lt', default=None)),
        ('rf_channel_frequency__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__gte', default=None)),
        ('rf_channel_frequency__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__gt', default=None)),
        ('rf_channel_width__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__n', default=None)),
        ('rf_channel_width__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__lte', default=None)),
        ('rf_channel_width__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__lt', default=None)),
        ('rf_channel_width__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__gte', default=None)),
        ('rf_channel_width__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__gt', default=None)),
        ('tx_power__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__n', default=None)),
        ('tx_power__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__lte', default=None)),
        ('tx_power__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__lt', default=None)),
        ('tx_power__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__gte', default=None)),
        ('tx_power__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('bridge_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id__n', default=None)),
        ('lag_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='lag_id__n', default=None)),
        ('speed__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__n', default=None)),
        ('speed__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__lte', default=None)),
        ('speed__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__lt', default=None)),
        ('speed__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__gte', default=None)),
        ('speed__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__gt', default=None)),
        ('duplex__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='duplex__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('wwn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__n', default=None)),
        ('wwn__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__ic', default=None)),
        ('wwn__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nic', default=None)),
        ('wwn__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__iew', default=None)),
        ('wwn__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__niew', default=None)),
        ('wwn__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__isw', default=None)),
        ('wwn__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nisw', default=None)),
        ('wwn__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__ie', default=None)),
        ('wwn__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nie', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('vdc_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_id__n', default=None)),
        ('vdc_identifier__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_identifier__n', default=None)),
        ('vdc__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc__n', default=None)),
        ('l2vpn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id__n', default=None)),
        ('l2vpn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn__n', default=None)),
))
    )
    connected_endpoints = sgqlc.types.Field(sgqlc.types.list_of('LinkPeerType'), graphql_name='connected_endpoints')
    link_peers = sgqlc.types.Field(sgqlc.types.list_of('LinkPeerType'), graphql_name='link_peers')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')
    ip_addresses = sgqlc.types.Field(sgqlc.types.list_of(IPAddressType), graphql_name='ip_addresses')


class InventoryItemRoleType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'name', 'slug', 'description', 'color', 'inventory_items', 'inventory_item_templates', 'display', 'class_type', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    slug = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='slug')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    color = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='color')
    inventory_items = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('InventoryItemType'))), graphql_name='inventory_items', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('part_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id', default=None)),
        ('asset_tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag', default=None)),
        ('discovered', sgqlc.types.Arg(Boolean, graphql_name='discovered', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('component_type', sgqlc.types.Arg(String, graphql_name='component_type', default=None)),
        ('component_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id', default=None)),
        ('serial', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('part_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__n', default=None)),
        ('part_id__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__ic', default=None)),
        ('part_id__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nic', default=None)),
        ('part_id__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__iew', default=None)),
        ('part_id__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__niew', default=None)),
        ('part_id__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__isw', default=None)),
        ('part_id__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nisw', default=None)),
        ('part_id__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__ie', default=None)),
        ('part_id__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nie', default=None)),
        ('part_id__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__empty', default=None)),
        ('asset_tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__n', default=None)),
        ('asset_tag__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ic', default=None)),
        ('asset_tag__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nic', default=None)),
        ('asset_tag__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__iew', default=None)),
        ('asset_tag__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__niew', default=None)),
        ('asset_tag__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__isw', default=None)),
        ('asset_tag__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nisw', default=None)),
        ('asset_tag__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ie', default=None)),
        ('asset_tag__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nie', default=None)),
        ('asset_tag__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__empty', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('component_type__n', sgqlc.types.Arg(String, graphql_name='component_type__n', default=None)),
        ('component_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__n', default=None)),
        ('component_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__lte', default=None)),
        ('component_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__lt', default=None)),
        ('component_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__gte', default=None)),
        ('component_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__gt', default=None)),
        ('serial__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__n', default=None)),
        ('serial__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ic', default=None)),
        ('serial__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nic', default=None)),
        ('serial__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__iew', default=None)),
        ('serial__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__niew', default=None)),
        ('serial__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__isw', default=None)),
        ('serial__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nisw', default=None)),
        ('serial__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ie', default=None)),
        ('serial__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nie', default=None)),
        ('serial__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__empty', default=None)),
))
    )
    inventory_item_templates = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('InventoryItemTemplateType'))), graphql_name='inventory_item_templates', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('part_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('component_type', sgqlc.types.Arg(String, graphql_name='component_type', default=None)),
        ('component_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('part_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__n', default=None)),
        ('part_id__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__ic', default=None)),
        ('part_id__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nic', default=None)),
        ('part_id__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__iew', default=None)),
        ('part_id__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__niew', default=None)),
        ('part_id__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__isw', default=None)),
        ('part_id__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nisw', default=None)),
        ('part_id__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__ie', default=None)),
        ('part_id__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nie', default=None)),
        ('part_id__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('component_type__n', sgqlc.types.Arg(String, graphql_name='component_type__n', default=None)),
        ('component_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__n', default=None)),
        ('component_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__lte', default=None)),
        ('component_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__lt', default=None)),
        ('component_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__gte', default=None)),
        ('component_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__gt', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class InventoryItemTemplateType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'device_type', 'name', '_name', 'label', 'description', 'parent', 'role', 'manufacturer', 'part_id', 'lft', 'rght', 'tree_id', 'level', 'child_items', 'display', 'class_type', 'changelog', 'component')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    device_type = sgqlc.types.Field(sgqlc.types.non_null(DeviceTypeType), graphql_name='device_type')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    _name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='_name')
    label = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='label')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    parent = sgqlc.types.Field('InventoryItemTemplateType', graphql_name='parent')
    role = sgqlc.types.Field(InventoryItemRoleType, graphql_name='role')
    manufacturer = sgqlc.types.Field('ManufacturerType', graphql_name='manufacturer')
    part_id = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='part_id')
    lft = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='lft')
    rght = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='rght')
    tree_id = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='tree_id')
    level = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='level')
    child_items = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('InventoryItemTemplateType'))), graphql_name='child_items', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('part_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('component_type', sgqlc.types.Arg(String, graphql_name='component_type', default=None)),
        ('component_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('part_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__n', default=None)),
        ('part_id__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__ic', default=None)),
        ('part_id__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nic', default=None)),
        ('part_id__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__iew', default=None)),
        ('part_id__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__niew', default=None)),
        ('part_id__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__isw', default=None)),
        ('part_id__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nisw', default=None)),
        ('part_id__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__ie', default=None)),
        ('part_id__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nie', default=None)),
        ('part_id__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('component_type__n', sgqlc.types.Arg(String, graphql_name='component_type__n', default=None)),
        ('component_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__n', default=None)),
        ('component_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__lte', default=None)),
        ('component_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__lt', default=None)),
        ('component_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__gte', default=None)),
        ('component_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__gt', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')
    component = sgqlc.types.Field('InventoryItemTemplateComponentType', graphql_name='component')


class InventoryItemType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'device', 'name', '_name', 'label', 'description', 'parent', 'role', 'manufacturer', 'part_id', 'serial', 'asset_tag', 'discovered', 'lft', 'rght', 'tree_id', 'level', 'child_items', 'display', 'class_type', 'custom_fields', 'changelog', 'component')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    device = sgqlc.types.Field(sgqlc.types.non_null(DeviceType), graphql_name='device')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    _name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='_name')
    label = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='label')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    parent = sgqlc.types.Field('InventoryItemType', graphql_name='parent')
    role = sgqlc.types.Field(InventoryItemRoleType, graphql_name='role')
    manufacturer = sgqlc.types.Field('ManufacturerType', graphql_name='manufacturer')
    part_id = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='part_id')
    serial = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='serial')
    asset_tag = sgqlc.types.Field(String, graphql_name='asset_tag')
    discovered = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='discovered')
    lft = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='lft')
    rght = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='rght')
    tree_id = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='tree_id')
    level = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='level')
    child_items = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('InventoryItemType'))), graphql_name='child_items', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('part_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id', default=None)),
        ('asset_tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag', default=None)),
        ('discovered', sgqlc.types.Arg(Boolean, graphql_name='discovered', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('component_type', sgqlc.types.Arg(String, graphql_name='component_type', default=None)),
        ('component_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id', default=None)),
        ('serial', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('part_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__n', default=None)),
        ('part_id__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__ic', default=None)),
        ('part_id__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nic', default=None)),
        ('part_id__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__iew', default=None)),
        ('part_id__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__niew', default=None)),
        ('part_id__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__isw', default=None)),
        ('part_id__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nisw', default=None)),
        ('part_id__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__ie', default=None)),
        ('part_id__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nie', default=None)),
        ('part_id__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__empty', default=None)),
        ('asset_tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__n', default=None)),
        ('asset_tag__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ic', default=None)),
        ('asset_tag__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nic', default=None)),
        ('asset_tag__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__iew', default=None)),
        ('asset_tag__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__niew', default=None)),
        ('asset_tag__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__isw', default=None)),
        ('asset_tag__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nisw', default=None)),
        ('asset_tag__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ie', default=None)),
        ('asset_tag__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nie', default=None)),
        ('asset_tag__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__empty', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('component_type__n', sgqlc.types.Arg(String, graphql_name='component_type__n', default=None)),
        ('component_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__n', default=None)),
        ('component_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__lte', default=None)),
        ('component_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__lt', default=None)),
        ('component_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__gte', default=None)),
        ('component_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__gt', default=None)),
        ('serial__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__n', default=None)),
        ('serial__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ic', default=None)),
        ('serial__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nic', default=None)),
        ('serial__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__iew', default=None)),
        ('serial__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__niew', default=None)),
        ('serial__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__isw', default=None)),
        ('serial__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nisw', default=None)),
        ('serial__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ie', default=None)),
        ('serial__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nie', default=None)),
        ('serial__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__empty', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')
    component = sgqlc.types.Field('InventoryItemComponentType', graphql_name='component')


class JournalEntryType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'assigned_object_type', 'assigned_object_id', 'created_by', 'kind', 'comments', 'display', 'class_type', 'changelog', 'custom_fields')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    assigned_object_type = sgqlc.types.Field(sgqlc.types.non_null(ContentTypeType), graphql_name='assigned_object_type')
    assigned_object_id = sgqlc.types.Field(sgqlc.types.non_null(BigInt), graphql_name='assigned_object_id')
    created_by = sgqlc.types.Field('UserType', graphql_name='created_by')
    kind = sgqlc.types.Field(sgqlc.types.non_null(ExtrasJournalEntryKindChoices), graphql_name='kind')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')


class L2VPNTerminationType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'l2vpn', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog', 'assigned_object')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    l2vpn = sgqlc.types.Field(sgqlc.types.non_null('L2VPNType'), graphql_name='l2vpn')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of(JournalEntryType), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')
    assigned_object = sgqlc.types.Field('L2VPNAssignmentType', graphql_name='assigned_object')


class L2VPNType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'name', 'slug', 'type', 'identifier', 'import_targets', 'export_targets', 'tenant', 'terminations', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog', 'contacts')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    slug = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='slug')
    type = sgqlc.types.Field(sgqlc.types.non_null(IpamL2VPNTypeChoices), graphql_name='type')
    identifier = sgqlc.types.Field(BigInt, graphql_name='identifier')
    import_targets = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('RouteTargetType'))), graphql_name='import_targets', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('importing_vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='importing_vrf_id', default=None)),
        ('importing_vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='importing_vrf', default=None)),
        ('exporting_vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='exporting_vrf_id', default=None)),
        ('exporting_vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='exporting_vrf', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('importing_vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='importing_vrf_id__n', default=None)),
        ('importing_vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='importing_vrf__n', default=None)),
        ('exporting_vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='exporting_vrf_id__n', default=None)),
        ('exporting_vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='exporting_vrf__n', default=None)),
))
    )
    export_targets = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('RouteTargetType'))), graphql_name='export_targets', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('importing_vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='importing_vrf_id', default=None)),
        ('importing_vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='importing_vrf', default=None)),
        ('exporting_vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='exporting_vrf_id', default=None)),
        ('exporting_vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='exporting_vrf', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('importing_vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='importing_vrf_id__n', default=None)),
        ('importing_vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='importing_vrf__n', default=None)),
        ('exporting_vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='exporting_vrf_id__n', default=None)),
        ('exporting_vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='exporting_vrf__n', default=None)),
))
    )
    tenant = sgqlc.types.Field('TenantType', graphql_name='tenant')
    terminations = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(L2VPNTerminationType))), graphql_name='terminations')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of(JournalEntryType), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')
    contacts = sgqlc.types.Field(sgqlc.types.list_of(ContactAssignmentType), graphql_name='contacts')


class LocationType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'parent', 'name', 'slug', 'description', 'site', 'status', 'tenant', 'lft', 'rght', 'tree_id', 'level', 'cabletermination_set', 'devices', 'powerpanel_set', 'racks', 'children', 'display', 'class_type', 'custom_fields', 'changelog', 'contacts', 'image_attachments', 'vlan_groups')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    parent = sgqlc.types.Field('LocationType', graphql_name='parent')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    slug = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='slug')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    site = sgqlc.types.Field(sgqlc.types.non_null('SiteType'), graphql_name='site')
    status = sgqlc.types.Field(sgqlc.types.non_null(DcimLocationStatusChoices), graphql_name='status')
    tenant = sgqlc.types.Field('TenantType', graphql_name='tenant')
    lft = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='lft')
    rght = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='rght')
    tree_id = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='tree_id')
    level = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='level')
    cabletermination_set = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(CableTerminationType))), graphql_name='cabletermination_set', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('cable', sgqlc.types.Arg(String, graphql_name='cable', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('termination_type', sgqlc.types.Arg(String, graphql_name='termination_type', default=None)),
        ('termination_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('cable__n', sgqlc.types.Arg(String, graphql_name='cable__n', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('termination_type__n', sgqlc.types.Arg(String, graphql_name='termination_type__n', default=None)),
        ('termination_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id__n', default=None)),
        ('termination_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id__lte', default=None)),
        ('termination_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id__lt', default=None)),
        ('termination_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id__gte', default=None)),
        ('termination_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id__gt', default=None)),
))
    )
    devices = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(DeviceType))), graphql_name='devices', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('asset_tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag', default=None)),
        ('face', sgqlc.types.Arg(String, graphql_name='face', default=None)),
        ('position', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position', default=None)),
        ('airflow', sgqlc.types.Arg(String, graphql_name='airflow', default=None)),
        ('vc_position', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position', default=None)),
        ('vc_priority', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('local_context_data', sgqlc.types.Arg(Boolean, graphql_name='local_context_data', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('device_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type', default=None)),
        ('device_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type_id', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('parent_device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_device_id', default=None)),
        ('platform_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id', default=None)),
        ('platform', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('cluster_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id', default=None)),
        ('model', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('is_full_depth', sgqlc.types.Arg(Boolean, graphql_name='is_full_depth', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('serial', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial', default=None)),
        ('has_primary_ip', sgqlc.types.Arg(Boolean, graphql_name='has_primary_ip', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis_member', sgqlc.types.Arg(Boolean, graphql_name='virtual_chassis_member', default=None)),
        ('console_ports', sgqlc.types.Arg(Boolean, graphql_name='console_ports', default=None)),
        ('console_server_ports', sgqlc.types.Arg(Boolean, graphql_name='console_server_ports', default=None)),
        ('power_ports', sgqlc.types.Arg(Boolean, graphql_name='power_ports', default=None)),
        ('power_outlets', sgqlc.types.Arg(Boolean, graphql_name='power_outlets', default=None)),
        ('interfaces', sgqlc.types.Arg(Boolean, graphql_name='interfaces', default=None)),
        ('pass_through_ports', sgqlc.types.Arg(Boolean, graphql_name='pass_through_ports', default=None)),
        ('module_bays', sgqlc.types.Arg(Boolean, graphql_name='module_bays', default=None)),
        ('device_bays', sgqlc.types.Arg(Boolean, graphql_name='device_bays', default=None)),
        ('primary_ip4_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip4_id', default=None)),
        ('primary_ip6_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip6_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('asset_tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__n', default=None)),
        ('asset_tag__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ic', default=None)),
        ('asset_tag__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nic', default=None)),
        ('asset_tag__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__iew', default=None)),
        ('asset_tag__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__niew', default=None)),
        ('asset_tag__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__isw', default=None)),
        ('asset_tag__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nisw', default=None)),
        ('asset_tag__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ie', default=None)),
        ('asset_tag__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nie', default=None)),
        ('asset_tag__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__empty', default=None)),
        ('face__n', sgqlc.types.Arg(String, graphql_name='face__n', default=None)),
        ('position__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__n', default=None)),
        ('position__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__lte', default=None)),
        ('position__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__lt', default=None)),
        ('position__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__gte', default=None)),
        ('position__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__gt', default=None)),
        ('airflow__n', sgqlc.types.Arg(String, graphql_name='airflow__n', default=None)),
        ('vc_position__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__n', default=None)),
        ('vc_position__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__lte', default=None)),
        ('vc_position__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__lt', default=None)),
        ('vc_position__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__gte', default=None)),
        ('vc_position__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__gt', default=None)),
        ('vc_priority__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__n', default=None)),
        ('vc_priority__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__lte', default=None)),
        ('vc_priority__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__lt', default=None)),
        ('vc_priority__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__gte', default=None)),
        ('vc_priority__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
        ('device_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type__n', default=None)),
        ('device_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type_id__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('parent_device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_device_id__n', default=None)),
        ('platform_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id__n', default=None)),
        ('platform__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('cluster_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id__n', default=None)),
        ('model__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__n', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('serial__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__n', default=None)),
        ('serial__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ic', default=None)),
        ('serial__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nic', default=None)),
        ('serial__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__iew', default=None)),
        ('serial__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__niew', default=None)),
        ('serial__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__isw', default=None)),
        ('serial__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nisw', default=None)),
        ('serial__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ie', default=None)),
        ('serial__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nie', default=None)),
        ('serial__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__empty', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('primary_ip4_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip4_id__n', default=None)),
        ('primary_ip6_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip6_id__n', default=None)),
))
    )
    powerpanel_set = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('PowerPanelType'))), graphql_name='powerpanel_set', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
))
    )
    racks = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('RackType'))), graphql_name='racks', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('facility_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id', default=None)),
        ('asset_tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag', default=None)),
        ('u_height', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height', default=None)),
        ('desc_units', sgqlc.types.Arg(Boolean, graphql_name='desc_units', default=None)),
        ('outer_width', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width', default=None)),
        ('outer_depth', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth', default=None)),
        ('outer_unit', sgqlc.types.Arg(String, graphql_name='outer_unit', default=None)),
        ('mounting_depth', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth', default=None)),
        ('weight', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight', default=None)),
        ('max_weight', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight', default=None)),
        ('weight_unit', sgqlc.types.Arg(String, graphql_name='weight_unit', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('width', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='width', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('serial', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('facility_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__n', default=None)),
        ('facility_id__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__ic', default=None)),
        ('facility_id__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__nic', default=None)),
        ('facility_id__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__iew', default=None)),
        ('facility_id__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__niew', default=None)),
        ('facility_id__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__isw', default=None)),
        ('facility_id__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__nisw', default=None)),
        ('facility_id__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__ie', default=None)),
        ('facility_id__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__nie', default=None)),
        ('facility_id__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__empty', default=None)),
        ('asset_tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__n', default=None)),
        ('asset_tag__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ic', default=None)),
        ('asset_tag__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nic', default=None)),
        ('asset_tag__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__iew', default=None)),
        ('asset_tag__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__niew', default=None)),
        ('asset_tag__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__isw', default=None)),
        ('asset_tag__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nisw', default=None)),
        ('asset_tag__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ie', default=None)),
        ('asset_tag__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nie', default=None)),
        ('asset_tag__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__empty', default=None)),
        ('u_height__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__n', default=None)),
        ('u_height__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__lte', default=None)),
        ('u_height__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__lt', default=None)),
        ('u_height__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__gte', default=None)),
        ('u_height__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__gt', default=None)),
        ('outer_width__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width__n', default=None)),
        ('outer_width__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width__lte', default=None)),
        ('outer_width__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width__lt', default=None)),
        ('outer_width__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width__gte', default=None)),
        ('outer_width__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width__gt', default=None)),
        ('outer_depth__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth__n', default=None)),
        ('outer_depth__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth__lte', default=None)),
        ('outer_depth__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth__lt', default=None)),
        ('outer_depth__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth__gte', default=None)),
        ('outer_depth__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth__gt', default=None)),
        ('outer_unit__n', sgqlc.types.Arg(String, graphql_name='outer_unit__n', default=None)),
        ('mounting_depth__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth__n', default=None)),
        ('mounting_depth__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth__lte', default=None)),
        ('mounting_depth__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth__lt', default=None)),
        ('mounting_depth__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth__gte', default=None)),
        ('mounting_depth__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth__gt', default=None)),
        ('weight__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__n', default=None)),
        ('weight__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__lte', default=None)),
        ('weight__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__lt', default=None)),
        ('weight__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__gte', default=None)),
        ('weight__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__gt', default=None)),
        ('max_weight__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight__n', default=None)),
        ('max_weight__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight__lte', default=None)),
        ('max_weight__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight__lt', default=None)),
        ('max_weight__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight__gte', default=None)),
        ('max_weight__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight__gt', default=None)),
        ('weight_unit__n', sgqlc.types.Arg(String, graphql_name='weight_unit__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('width__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='width__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('serial__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__n', default=None)),
        ('serial__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ic', default=None)),
        ('serial__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nic', default=None)),
        ('serial__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__iew', default=None)),
        ('serial__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__niew', default=None)),
        ('serial__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__isw', default=None)),
        ('serial__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nisw', default=None)),
        ('serial__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ie', default=None)),
        ('serial__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nie', default=None)),
        ('serial__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__empty', default=None)),
))
    )
    children = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('LocationType'))), graphql_name='children', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('parent', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('parent__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')
    contacts = sgqlc.types.Field(sgqlc.types.list_of(ContactAssignmentType), graphql_name='contacts')
    image_attachments = sgqlc.types.Field(sgqlc.types.list_of(ImageAttachmentType), graphql_name='image_attachments')
    vlan_groups = sgqlc.types.Field(sgqlc.types.list_of('VLANGroupType'), graphql_name='vlan_groups')


class ManufacturerType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'name', 'slug', 'description', 'inventory_items', 'inventory_item_templates', 'device_types', 'module_types', 'platforms', 'contacts', 'display', 'class_type', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    slug = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='slug')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    inventory_items = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(InventoryItemType))), graphql_name='inventory_items', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('part_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id', default=None)),
        ('asset_tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag', default=None)),
        ('discovered', sgqlc.types.Arg(Boolean, graphql_name='discovered', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('component_type', sgqlc.types.Arg(String, graphql_name='component_type', default=None)),
        ('component_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id', default=None)),
        ('serial', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('part_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__n', default=None)),
        ('part_id__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__ic', default=None)),
        ('part_id__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nic', default=None)),
        ('part_id__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__iew', default=None)),
        ('part_id__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__niew', default=None)),
        ('part_id__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__isw', default=None)),
        ('part_id__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nisw', default=None)),
        ('part_id__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__ie', default=None)),
        ('part_id__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nie', default=None)),
        ('part_id__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__empty', default=None)),
        ('asset_tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__n', default=None)),
        ('asset_tag__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ic', default=None)),
        ('asset_tag__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nic', default=None)),
        ('asset_tag__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__iew', default=None)),
        ('asset_tag__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__niew', default=None)),
        ('asset_tag__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__isw', default=None)),
        ('asset_tag__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nisw', default=None)),
        ('asset_tag__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ie', default=None)),
        ('asset_tag__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nie', default=None)),
        ('asset_tag__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__empty', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('component_type__n', sgqlc.types.Arg(String, graphql_name='component_type__n', default=None)),
        ('component_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__n', default=None)),
        ('component_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__lte', default=None)),
        ('component_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__lt', default=None)),
        ('component_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__gte', default=None)),
        ('component_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__gt', default=None)),
        ('serial__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__n', default=None)),
        ('serial__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ic', default=None)),
        ('serial__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nic', default=None)),
        ('serial__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__iew', default=None)),
        ('serial__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__niew', default=None)),
        ('serial__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__isw', default=None)),
        ('serial__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nisw', default=None)),
        ('serial__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ie', default=None)),
        ('serial__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nie', default=None)),
        ('serial__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__empty', default=None)),
))
    )
    inventory_item_templates = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(InventoryItemTemplateType))), graphql_name='inventory_item_templates', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('part_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('component_type', sgqlc.types.Arg(String, graphql_name='component_type', default=None)),
        ('component_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('part_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__n', default=None)),
        ('part_id__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__ic', default=None)),
        ('part_id__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nic', default=None)),
        ('part_id__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__iew', default=None)),
        ('part_id__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__niew', default=None)),
        ('part_id__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__isw', default=None)),
        ('part_id__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nisw', default=None)),
        ('part_id__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__ie', default=None)),
        ('part_id__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nie', default=None)),
        ('part_id__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('component_type__n', sgqlc.types.Arg(String, graphql_name='component_type__n', default=None)),
        ('component_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__n', default=None)),
        ('component_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__lte', default=None)),
        ('component_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__lt', default=None)),
        ('component_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__gte', default=None)),
        ('component_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__gt', default=None)),
))
    )
    device_types = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(DeviceTypeType))), graphql_name='device_types', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('model', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('part_number', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number', default=None)),
        ('u_height', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height', default=None)),
        ('is_full_depth', sgqlc.types.Arg(Boolean, graphql_name='is_full_depth', default=None)),
        ('subdevice_role', sgqlc.types.Arg(String, graphql_name='subdevice_role', default=None)),
        ('airflow', sgqlc.types.Arg(String, graphql_name='airflow', default=None)),
        ('weight', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight', default=None)),
        ('weight_unit', sgqlc.types.Arg(String, graphql_name='weight_unit', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('has_front_image', sgqlc.types.Arg(Boolean, graphql_name='has_front_image', default=None)),
        ('has_rear_image', sgqlc.types.Arg(Boolean, graphql_name='has_rear_image', default=None)),
        ('console_ports', sgqlc.types.Arg(Boolean, graphql_name='console_ports', default=None)),
        ('console_server_ports', sgqlc.types.Arg(Boolean, graphql_name='console_server_ports', default=None)),
        ('power_ports', sgqlc.types.Arg(Boolean, graphql_name='power_ports', default=None)),
        ('power_outlets', sgqlc.types.Arg(Boolean, graphql_name='power_outlets', default=None)),
        ('interfaces', sgqlc.types.Arg(Boolean, graphql_name='interfaces', default=None)),
        ('pass_through_ports', sgqlc.types.Arg(Boolean, graphql_name='pass_through_ports', default=None)),
        ('module_bays', sgqlc.types.Arg(Boolean, graphql_name='module_bays', default=None)),
        ('device_bays', sgqlc.types.Arg(Boolean, graphql_name='device_bays', default=None)),
        ('inventory_items', sgqlc.types.Arg(Boolean, graphql_name='inventory_items', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('model__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__n', default=None)),
        ('model__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__ic', default=None)),
        ('model__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__nic', default=None)),
        ('model__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__iew', default=None)),
        ('model__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__niew', default=None)),
        ('model__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__isw', default=None)),
        ('model__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__nisw', default=None)),
        ('model__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__ie', default=None)),
        ('model__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__nie', default=None)),
        ('model__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('part_number__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__n', default=None)),
        ('part_number__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__ic', default=None)),
        ('part_number__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__nic', default=None)),
        ('part_number__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__iew', default=None)),
        ('part_number__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__niew', default=None)),
        ('part_number__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__isw', default=None)),
        ('part_number__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__nisw', default=None)),
        ('part_number__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__ie', default=None)),
        ('part_number__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__nie', default=None)),
        ('part_number__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__empty', default=None)),
        ('u_height__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__n', default=None)),
        ('u_height__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__lte', default=None)),
        ('u_height__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__lt', default=None)),
        ('u_height__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__gte', default=None)),
        ('u_height__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__gt', default=None)),
        ('subdevice_role__n', sgqlc.types.Arg(String, graphql_name='subdevice_role__n', default=None)),
        ('airflow__n', sgqlc.types.Arg(String, graphql_name='airflow__n', default=None)),
        ('weight__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__n', default=None)),
        ('weight__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__lte', default=None)),
        ('weight__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__lt', default=None)),
        ('weight__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__gte', default=None)),
        ('weight__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__gt', default=None)),
        ('weight_unit__n', sgqlc.types.Arg(String, graphql_name='weight_unit__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
))
    )
    module_types = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('ModuleTypeType'))), graphql_name='module_types', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('model', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model', default=None)),
        ('part_number', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number', default=None)),
        ('weight', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight', default=None)),
        ('weight_unit', sgqlc.types.Arg(String, graphql_name='weight_unit', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('console_ports', sgqlc.types.Arg(Boolean, graphql_name='console_ports', default=None)),
        ('console_server_ports', sgqlc.types.Arg(Boolean, graphql_name='console_server_ports', default=None)),
        ('power_ports', sgqlc.types.Arg(Boolean, graphql_name='power_ports', default=None)),
        ('power_outlets', sgqlc.types.Arg(Boolean, graphql_name='power_outlets', default=None)),
        ('interfaces', sgqlc.types.Arg(Boolean, graphql_name='interfaces', default=None)),
        ('pass_through_ports', sgqlc.types.Arg(Boolean, graphql_name='pass_through_ports', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('model__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__n', default=None)),
        ('model__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__ic', default=None)),
        ('model__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__nic', default=None)),
        ('model__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__iew', default=None)),
        ('model__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__niew', default=None)),
        ('model__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__isw', default=None)),
        ('model__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__nisw', default=None)),
        ('model__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__ie', default=None)),
        ('model__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__nie', default=None)),
        ('model__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__empty', default=None)),
        ('part_number__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__n', default=None)),
        ('part_number__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__ic', default=None)),
        ('part_number__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__nic', default=None)),
        ('part_number__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__iew', default=None)),
        ('part_number__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__niew', default=None)),
        ('part_number__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__isw', default=None)),
        ('part_number__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__nisw', default=None)),
        ('part_number__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__ie', default=None)),
        ('part_number__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__nie', default=None)),
        ('part_number__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__empty', default=None)),
        ('weight__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__n', default=None)),
        ('weight__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__lte', default=None)),
        ('weight__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__lt', default=None)),
        ('weight__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__gte', default=None)),
        ('weight__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__gt', default=None)),
        ('weight_unit__n', sgqlc.types.Arg(String, graphql_name='weight_unit__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
))
    )
    platforms = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('PlatformType'))), graphql_name='platforms', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('napalm_driver', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('napalm_driver__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__n', default=None)),
        ('napalm_driver__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__ic', default=None)),
        ('napalm_driver__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__nic', default=None)),
        ('napalm_driver__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__iew', default=None)),
        ('napalm_driver__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__niew', default=None)),
        ('napalm_driver__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__isw', default=None)),
        ('napalm_driver__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__nisw', default=None)),
        ('napalm_driver__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__ie', default=None)),
        ('napalm_driver__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__nie', default=None)),
        ('napalm_driver__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
))
    )
    contacts = sgqlc.types.Field(sgqlc.types.list_of(ContactAssignmentType), graphql_name='contacts')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class ModuleBayTemplateType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'device_type', 'name', '_name', 'label', 'description', 'position', 'display', 'class_type', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    device_type = sgqlc.types.Field(sgqlc.types.non_null(DeviceTypeType), graphql_name='device_type')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    _name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='_name')
    label = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='label')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    position = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='position')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class ModuleBayType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'device', 'name', '_name', 'label', 'description', 'position', 'installed_module', 'display', 'class_type', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    device = sgqlc.types.Field(sgqlc.types.non_null(DeviceType), graphql_name='device')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    _name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='_name')
    label = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='label')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    position = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='position')
    installed_module = sgqlc.types.Field('ModuleType', graphql_name='installed_module')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class ModuleType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'local_context_data', 'device', 'module_bay', 'module_type', 'status', 'serial', 'asset_tag', 'consoleports', 'consoleserverports', 'powerports', 'poweroutlets', 'interfaces', 'frontports', 'rearports', 'display', 'class_type', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    local_context_data = sgqlc.types.Field(JSONString, graphql_name='local_context_data')
    device = sgqlc.types.Field(sgqlc.types.non_null(DeviceType), graphql_name='device')
    module_bay = sgqlc.types.Field(sgqlc.types.non_null(ModuleBayType), graphql_name='module_bay')
    module_type = sgqlc.types.Field(sgqlc.types.non_null('ModuleTypeType'), graphql_name='module_type')
    status = sgqlc.types.Field(sgqlc.types.non_null(DcimModuleStatusChoices), graphql_name='status')
    serial = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='serial')
    asset_tag = sgqlc.types.Field(String, graphql_name='asset_tag')
    consoleports = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ConsolePortType))), graphql_name='consoleports', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
))
    )
    consoleserverports = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ConsoleServerPortType))), graphql_name='consoleserverports', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
))
    )
    powerports = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('PowerPortType'))), graphql_name='powerports', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('maximum_draw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw', default=None)),
        ('allocated_draw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('maximum_draw__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__n', default=None)),
        ('maximum_draw__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__lte', default=None)),
        ('maximum_draw__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__lt', default=None)),
        ('maximum_draw__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__gte', default=None)),
        ('maximum_draw__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__gt', default=None)),
        ('allocated_draw__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__n', default=None)),
        ('allocated_draw__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__lte', default=None)),
        ('allocated_draw__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__lt', default=None)),
        ('allocated_draw__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__gte', default=None)),
        ('allocated_draw__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
))
    )
    poweroutlets = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('PowerOutletType'))), graphql_name='poweroutlets', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('feed_leg', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='feed_leg', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('feed_leg__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='feed_leg__n', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
))
    )
    interfaces = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(InterfaceType))), graphql_name='interfaces', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('enabled', sgqlc.types.Arg(Boolean, graphql_name='enabled', default=None)),
        ('mtu', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu', default=None)),
        ('mgmt_only', sgqlc.types.Arg(Boolean, graphql_name='mgmt_only', default=None)),
        ('poe_mode', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode', default=None)),
        ('poe_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type', default=None)),
        ('mode', sgqlc.types.Arg(String, graphql_name='mode', default=None)),
        ('rf_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_role', default=None)),
        ('rf_channel', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel', default=None)),
        ('rf_channel_frequency', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency', default=None)),
        ('rf_channel_width', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width', default=None)),
        ('tx_power', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('kind', sgqlc.types.Arg(String, graphql_name='kind', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('bridge_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id', default=None)),
        ('lag_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='lag_id', default=None)),
        ('speed', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed', default=None)),
        ('duplex', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='duplex', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('wwn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn', default=None)),
        ('vlan_id', sgqlc.types.Arg(String, graphql_name='vlan_id', default=None)),
        ('vlan', sgqlc.types.Arg(String, graphql_name='vlan', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('vdc_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_id', default=None)),
        ('vdc_identifier', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_identifier', default=None)),
        ('vdc', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc', default=None)),
        ('l2vpn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id', default=None)),
        ('l2vpn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('mtu__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__n', default=None)),
        ('mtu__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lte', default=None)),
        ('mtu__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lt', default=None)),
        ('mtu__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gte', default=None)),
        ('mtu__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gt', default=None)),
        ('poe_mode__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode__n', default=None)),
        ('poe_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type__n', default=None)),
        ('mode__n', sgqlc.types.Arg(String, graphql_name='mode__n', default=None)),
        ('rf_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_role__n', default=None)),
        ('rf_channel__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel__n', default=None)),
        ('rf_channel_frequency__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__n', default=None)),
        ('rf_channel_frequency__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__lte', default=None)),
        ('rf_channel_frequency__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__lt', default=None)),
        ('rf_channel_frequency__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__gte', default=None)),
        ('rf_channel_frequency__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__gt', default=None)),
        ('rf_channel_width__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__n', default=None)),
        ('rf_channel_width__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__lte', default=None)),
        ('rf_channel_width__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__lt', default=None)),
        ('rf_channel_width__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__gte', default=None)),
        ('rf_channel_width__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__gt', default=None)),
        ('tx_power__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__n', default=None)),
        ('tx_power__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__lte', default=None)),
        ('tx_power__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__lt', default=None)),
        ('tx_power__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__gte', default=None)),
        ('tx_power__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('bridge_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id__n', default=None)),
        ('lag_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='lag_id__n', default=None)),
        ('speed__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__n', default=None)),
        ('speed__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__lte', default=None)),
        ('speed__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__lt', default=None)),
        ('speed__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__gte', default=None)),
        ('speed__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__gt', default=None)),
        ('duplex__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='duplex__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('wwn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__n', default=None)),
        ('wwn__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__ic', default=None)),
        ('wwn__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nic', default=None)),
        ('wwn__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__iew', default=None)),
        ('wwn__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__niew', default=None)),
        ('wwn__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__isw', default=None)),
        ('wwn__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nisw', default=None)),
        ('wwn__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__ie', default=None)),
        ('wwn__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nie', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('vdc_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_id__n', default=None)),
        ('vdc_identifier__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_identifier__n', default=None)),
        ('vdc__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc__n', default=None)),
        ('l2vpn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id__n', default=None)),
        ('l2vpn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn__n', default=None)),
))
    )
    frontports = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(FrontPortType))), graphql_name='frontports', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('color', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('color__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__n', default=None)),
        ('color__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ic', default=None)),
        ('color__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nic', default=None)),
        ('color__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__iew', default=None)),
        ('color__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__niew', default=None)),
        ('color__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__isw', default=None)),
        ('color__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nisw', default=None)),
        ('color__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ie', default=None)),
        ('color__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nie', default=None)),
        ('color__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
))
    )
    rearports = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('RearPortType'))), graphql_name='rearports', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('color', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color', default=None)),
        ('positions', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('color__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__n', default=None)),
        ('color__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ic', default=None)),
        ('color__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nic', default=None)),
        ('color__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__iew', default=None)),
        ('color__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__niew', default=None)),
        ('color__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__isw', default=None)),
        ('color__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nisw', default=None)),
        ('color__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ie', default=None)),
        ('color__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nie', default=None)),
        ('color__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__empty', default=None)),
        ('positions__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__n', default=None)),
        ('positions__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__lte', default=None)),
        ('positions__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__lt', default=None)),
        ('positions__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__gte', default=None)),
        ('positions__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class ModuleTypeType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'weight', 'weight_unit', '_abs_weight', 'manufacturer', 'model', 'part_number', 'consoleporttemplates', 'consoleserverporttemplates', 'powerporttemplates', 'poweroutlettemplates', 'interfacetemplates', 'frontporttemplates', 'rearporttemplates', 'instances', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    weight = sgqlc.types.Field(Decimal, graphql_name='weight')
    weight_unit = sgqlc.types.Field(DcimModuleTypeWeightUnitChoices, graphql_name='weight_unit')
    _abs_weight = sgqlc.types.Field(BigInt, graphql_name='_abs_weight')
    manufacturer = sgqlc.types.Field(sgqlc.types.non_null(ManufacturerType), graphql_name='manufacturer')
    model = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='model')
    part_number = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='part_number')
    consoleporttemplates = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ConsolePortTemplateType))), graphql_name='consoleporttemplates', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('type', sgqlc.types.Arg(String, graphql_name='type', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('moduletype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('type__n', sgqlc.types.Arg(String, graphql_name='type__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('moduletype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id__n', default=None)),
))
    )
    consoleserverporttemplates = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ConsoleServerPortTemplateType))), graphql_name='consoleserverporttemplates', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('type', sgqlc.types.Arg(String, graphql_name='type', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('moduletype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('type__n', sgqlc.types.Arg(String, graphql_name='type__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('moduletype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id__n', default=None)),
))
    )
    powerporttemplates = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('PowerPortTemplateType'))), graphql_name='powerporttemplates', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('type', sgqlc.types.Arg(String, graphql_name='type', default=None)),
        ('maximum_draw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw', default=None)),
        ('allocated_draw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('moduletype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('type__n', sgqlc.types.Arg(String, graphql_name='type__n', default=None)),
        ('maximum_draw__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__n', default=None)),
        ('maximum_draw__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__lte', default=None)),
        ('maximum_draw__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__lt', default=None)),
        ('maximum_draw__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__gte', default=None)),
        ('maximum_draw__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__gt', default=None)),
        ('allocated_draw__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__n', default=None)),
        ('allocated_draw__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__lte', default=None)),
        ('allocated_draw__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__lt', default=None)),
        ('allocated_draw__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__gte', default=None)),
        ('allocated_draw__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('moduletype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id__n', default=None)),
))
    )
    poweroutlettemplates = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('PowerOutletTemplateType'))), graphql_name='poweroutlettemplates', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('type', sgqlc.types.Arg(String, graphql_name='type', default=None)),
        ('feed_leg', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='feed_leg', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('moduletype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('type__n', sgqlc.types.Arg(String, graphql_name='type__n', default=None)),
        ('feed_leg__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='feed_leg__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('moduletype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id__n', default=None)),
))
    )
    interfacetemplates = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(InterfaceTemplateType))), graphql_name='interfacetemplates', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('mgmt_only', sgqlc.types.Arg(Boolean, graphql_name='mgmt_only', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('moduletype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id', default=None)),
        ('poe_mode', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode', default=None)),
        ('poe_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('moduletype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id__n', default=None)),
        ('poe_mode__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode__n', default=None)),
        ('poe_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type__n', default=None)),
))
    )
    frontporttemplates = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(FrontPortTemplateType))), graphql_name='frontporttemplates', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('color', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('moduletype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('color__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__n', default=None)),
        ('color__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ic', default=None)),
        ('color__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nic', default=None)),
        ('color__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__iew', default=None)),
        ('color__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__niew', default=None)),
        ('color__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__isw', default=None)),
        ('color__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nisw', default=None)),
        ('color__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ie', default=None)),
        ('color__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nie', default=None)),
        ('color__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('moduletype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id__n', default=None)),
))
    )
    rearporttemplates = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('RearPortTemplateType'))), graphql_name='rearporttemplates', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('color', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color', default=None)),
        ('positions', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('moduletype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('color__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__n', default=None)),
        ('color__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ic', default=None)),
        ('color__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nic', default=None)),
        ('color__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__iew', default=None)),
        ('color__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__niew', default=None)),
        ('color__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__isw', default=None)),
        ('color__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nisw', default=None)),
        ('color__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ie', default=None)),
        ('color__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nie', default=None)),
        ('color__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__empty', default=None)),
        ('positions__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__n', default=None)),
        ('positions__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__lte', default=None)),
        ('positions__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__lt', default=None)),
        ('positions__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__gte', default=None)),
        ('positions__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('moduletype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id__n', default=None)),
))
    )
    instances = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ModuleType))), graphql_name='instances', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('asset_tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('module_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_type_id', default=None)),
        ('module_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_type', default=None)),
        ('module_bay_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_bay_id', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('serial', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('asset_tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__n', default=None)),
        ('asset_tag__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ic', default=None)),
        ('asset_tag__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nic', default=None)),
        ('asset_tag__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__iew', default=None)),
        ('asset_tag__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__niew', default=None)),
        ('asset_tag__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__isw', default=None)),
        ('asset_tag__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nisw', default=None)),
        ('asset_tag__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ie', default=None)),
        ('asset_tag__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nie', default=None)),
        ('asset_tag__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
        ('module_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_type_id__n', default=None)),
        ('module_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_type__n', default=None)),
        ('module_bay_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_bay_id__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('serial__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__n', default=None)),
        ('serial__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ic', default=None)),
        ('serial__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nic', default=None)),
        ('serial__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__iew', default=None)),
        ('serial__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__niew', default=None)),
        ('serial__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__isw', default=None)),
        ('serial__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nisw', default=None)),
        ('serial__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ie', default=None)),
        ('serial__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nie', default=None)),
        ('serial__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__empty', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of(JournalEntryType), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of('ObjectChangeType'), graphql_name='changelog')


class ObjectChangeType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'time', 'user', 'user_name', 'request_id', 'action', 'changed_object_type', 'changed_object_id', 'related_object_type', 'related_object_id', 'object_repr', 'prechange_data', 'postchange_data', 'display', 'class_type')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    time = sgqlc.types.Field(sgqlc.types.non_null(DateTime), graphql_name='time')
    user = sgqlc.types.Field('UserType', graphql_name='user')
    user_name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='user_name')
    request_id = sgqlc.types.Field(sgqlc.types.non_null(UUID), graphql_name='request_id')
    action = sgqlc.types.Field(sgqlc.types.non_null(ExtrasObjectChangeActionChoices), graphql_name='action')
    changed_object_type = sgqlc.types.Field(sgqlc.types.non_null(ContentTypeType), graphql_name='changed_object_type')
    changed_object_id = sgqlc.types.Field(sgqlc.types.non_null(BigInt), graphql_name='changed_object_id')
    related_object_type = sgqlc.types.Field(ContentTypeType, graphql_name='related_object_type')
    related_object_id = sgqlc.types.Field(BigInt, graphql_name='related_object_id')
    object_repr = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='object_repr')
    prechange_data = sgqlc.types.Field(JSONString, graphql_name='prechange_data')
    postchange_data = sgqlc.types.Field(JSONString, graphql_name='postchange_data')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')


class PlatformType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'name', 'slug', 'description', 'manufacturer', 'napalm_driver', 'napalm_args', 'devices', 'virtual_machines', 'display', 'class_type', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    slug = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='slug')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    manufacturer = sgqlc.types.Field(ManufacturerType, graphql_name='manufacturer')
    napalm_driver = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='napalm_driver')
    napalm_args = sgqlc.types.Field(JSONString, graphql_name='napalm_args')
    devices = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(DeviceType))), graphql_name='devices', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('asset_tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag', default=None)),
        ('face', sgqlc.types.Arg(String, graphql_name='face', default=None)),
        ('position', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position', default=None)),
        ('airflow', sgqlc.types.Arg(String, graphql_name='airflow', default=None)),
        ('vc_position', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position', default=None)),
        ('vc_priority', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('local_context_data', sgqlc.types.Arg(Boolean, graphql_name='local_context_data', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('device_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type', default=None)),
        ('device_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type_id', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('parent_device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_device_id', default=None)),
        ('platform_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id', default=None)),
        ('platform', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('cluster_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id', default=None)),
        ('model', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('is_full_depth', sgqlc.types.Arg(Boolean, graphql_name='is_full_depth', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('serial', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial', default=None)),
        ('has_primary_ip', sgqlc.types.Arg(Boolean, graphql_name='has_primary_ip', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis_member', sgqlc.types.Arg(Boolean, graphql_name='virtual_chassis_member', default=None)),
        ('console_ports', sgqlc.types.Arg(Boolean, graphql_name='console_ports', default=None)),
        ('console_server_ports', sgqlc.types.Arg(Boolean, graphql_name='console_server_ports', default=None)),
        ('power_ports', sgqlc.types.Arg(Boolean, graphql_name='power_ports', default=None)),
        ('power_outlets', sgqlc.types.Arg(Boolean, graphql_name='power_outlets', default=None)),
        ('interfaces', sgqlc.types.Arg(Boolean, graphql_name='interfaces', default=None)),
        ('pass_through_ports', sgqlc.types.Arg(Boolean, graphql_name='pass_through_ports', default=None)),
        ('module_bays', sgqlc.types.Arg(Boolean, graphql_name='module_bays', default=None)),
        ('device_bays', sgqlc.types.Arg(Boolean, graphql_name='device_bays', default=None)),
        ('primary_ip4_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip4_id', default=None)),
        ('primary_ip6_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip6_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('asset_tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__n', default=None)),
        ('asset_tag__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ic', default=None)),
        ('asset_tag__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nic', default=None)),
        ('asset_tag__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__iew', default=None)),
        ('asset_tag__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__niew', default=None)),
        ('asset_tag__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__isw', default=None)),
        ('asset_tag__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nisw', default=None)),
        ('asset_tag__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ie', default=None)),
        ('asset_tag__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nie', default=None)),
        ('asset_tag__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__empty', default=None)),
        ('face__n', sgqlc.types.Arg(String, graphql_name='face__n', default=None)),
        ('position__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__n', default=None)),
        ('position__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__lte', default=None)),
        ('position__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__lt', default=None)),
        ('position__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__gte', default=None)),
        ('position__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__gt', default=None)),
        ('airflow__n', sgqlc.types.Arg(String, graphql_name='airflow__n', default=None)),
        ('vc_position__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__n', default=None)),
        ('vc_position__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__lte', default=None)),
        ('vc_position__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__lt', default=None)),
        ('vc_position__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__gte', default=None)),
        ('vc_position__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__gt', default=None)),
        ('vc_priority__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__n', default=None)),
        ('vc_priority__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__lte', default=None)),
        ('vc_priority__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__lt', default=None)),
        ('vc_priority__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__gte', default=None)),
        ('vc_priority__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
        ('device_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type__n', default=None)),
        ('device_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type_id__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('parent_device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_device_id__n', default=None)),
        ('platform_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id__n', default=None)),
        ('platform__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('cluster_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id__n', default=None)),
        ('model__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__n', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('serial__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__n', default=None)),
        ('serial__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ic', default=None)),
        ('serial__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nic', default=None)),
        ('serial__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__iew', default=None)),
        ('serial__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__niew', default=None)),
        ('serial__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__isw', default=None)),
        ('serial__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nisw', default=None)),
        ('serial__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ie', default=None)),
        ('serial__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nie', default=None)),
        ('serial__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__empty', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('primary_ip4_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip4_id__n', default=None)),
        ('primary_ip6_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip6_id__n', default=None)),
))
    )
    virtual_machines = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('VirtualMachineType'))), graphql_name='virtual_machines', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('cluster', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster', default=None)),
        ('vcpus', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus', default=None)),
        ('memory', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory', default=None)),
        ('disk', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('local_context_data', sgqlc.types.Arg(Boolean, graphql_name='local_context_data', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('cluster_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group_id', default=None)),
        ('cluster_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group', default=None)),
        ('cluster_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type_id', default=None)),
        ('cluster_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type', default=None)),
        ('cluster_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('platform_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id', default=None)),
        ('platform', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('has_primary_ip', sgqlc.types.Arg(Boolean, graphql_name='has_primary_ip', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('cluster__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster__n', default=None)),
        ('vcpus__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__n', default=None)),
        ('vcpus__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__lte', default=None)),
        ('vcpus__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__lt', default=None)),
        ('vcpus__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__gte', default=None)),
        ('vcpus__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__gt', default=None)),
        ('memory__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__n', default=None)),
        ('memory__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__lte', default=None)),
        ('memory__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__lt', default=None)),
        ('memory__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__gte', default=None)),
        ('memory__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__gt', default=None)),
        ('disk__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__n', default=None)),
        ('disk__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__lte', default=None)),
        ('disk__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__lt', default=None)),
        ('disk__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__gte', default=None)),
        ('disk__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('cluster_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group_id__n', default=None)),
        ('cluster_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group__n', default=None)),
        ('cluster_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type_id__n', default=None)),
        ('cluster_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type__n', default=None)),
        ('cluster_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('platform_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id__n', default=None)),
        ('platform__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class PowerFeedType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'cable', 'cable_end', 'mark_connected', 'power_panel', 'rack', 'name', 'status', 'type', 'supply', 'phase', 'voltage', 'amperage', 'max_utilization', 'available_power', 'connected_endpoints', 'link_peers', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    cable = sgqlc.types.Field(CableType, graphql_name='cable')
    cable_end = sgqlc.types.Field(DcimPowerFeedCableEndChoices, graphql_name='cable_end')
    mark_connected = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='mark_connected')
    power_panel = sgqlc.types.Field(sgqlc.types.non_null('PowerPanelType'), graphql_name='power_panel')
    rack = sgqlc.types.Field('RackType', graphql_name='rack')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    status = sgqlc.types.Field(sgqlc.types.non_null(DcimPowerFeedStatusChoices), graphql_name='status')
    type = sgqlc.types.Field(sgqlc.types.non_null(DcimPowerFeedTypeChoices), graphql_name='type')
    supply = sgqlc.types.Field(sgqlc.types.non_null(DcimPowerFeedSupplyChoices), graphql_name='supply')
    phase = sgqlc.types.Field(sgqlc.types.non_null(DcimPowerFeedPhaseChoices), graphql_name='phase')
    voltage = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='voltage')
    amperage = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='amperage')
    max_utilization = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='max_utilization')
    available_power = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='available_power')
    connected_endpoints = sgqlc.types.Field(sgqlc.types.list_of('LinkPeerType'), graphql_name='connected_endpoints')
    link_peers = sgqlc.types.Field(sgqlc.types.list_of('LinkPeerType'), graphql_name='link_peers')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of(JournalEntryType), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class PowerOutletTemplateType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'name', '_name', 'label', 'description', 'device_type', 'module_type', 'type', 'power_port', 'feed_leg', 'display', 'class_type', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    _name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='_name')
    label = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='label')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    device_type = sgqlc.types.Field(DeviceTypeType, graphql_name='device_type')
    module_type = sgqlc.types.Field(ModuleTypeType, graphql_name='module_type')
    type = sgqlc.types.Field(DcimPowerOutletTemplateTypeChoices, graphql_name='type')
    power_port = sgqlc.types.Field('PowerPortTemplateType', graphql_name='power_port')
    feed_leg = sgqlc.types.Field(DcimPowerOutletTemplateFeedLegChoices, graphql_name='feed_leg')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class PowerOutletType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'device', 'name', '_name', 'label', 'description', 'module', 'cable', 'cable_end', 'mark_connected', 'type', 'power_port', 'feed_leg', 'connected_endpoints', 'link_peers', 'display', 'class_type', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    device = sgqlc.types.Field(sgqlc.types.non_null(DeviceType), graphql_name='device')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    _name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='_name')
    label = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='label')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    module = sgqlc.types.Field(ModuleType, graphql_name='module')
    cable = sgqlc.types.Field(CableType, graphql_name='cable')
    cable_end = sgqlc.types.Field(DcimPowerOutletCableEndChoices, graphql_name='cable_end')
    mark_connected = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='mark_connected')
    type = sgqlc.types.Field(DcimPowerOutletTypeChoices, graphql_name='type')
    power_port = sgqlc.types.Field('PowerPortType', graphql_name='power_port')
    feed_leg = sgqlc.types.Field(DcimPowerOutletFeedLegChoices, graphql_name='feed_leg')
    connected_endpoints = sgqlc.types.Field(sgqlc.types.list_of('LinkPeerType'), graphql_name='connected_endpoints')
    link_peers = sgqlc.types.Field(sgqlc.types.list_of('LinkPeerType'), graphql_name='link_peers')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class PowerPanelType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'site', 'location', 'name', 'powerfeeds', 'contacts', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    site = sgqlc.types.Field(sgqlc.types.non_null('SiteType'), graphql_name='site')
    location = sgqlc.types.Field(LocationType, graphql_name='location')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    powerfeeds = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(PowerFeedType))), graphql_name='powerfeeds', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('type', sgqlc.types.Arg(String, graphql_name='type', default=None)),
        ('supply', sgqlc.types.Arg(String, graphql_name='supply', default=None)),
        ('phase', sgqlc.types.Arg(String, graphql_name='phase', default=None)),
        ('voltage', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='voltage', default=None)),
        ('amperage', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='amperage', default=None)),
        ('max_utilization', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_utilization', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('power_panel_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='power_panel_id', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('type__n', sgqlc.types.Arg(String, graphql_name='type__n', default=None)),
        ('supply__n', sgqlc.types.Arg(String, graphql_name='supply__n', default=None)),
        ('phase__n', sgqlc.types.Arg(String, graphql_name='phase__n', default=None)),
        ('voltage__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='voltage__n', default=None)),
        ('voltage__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='voltage__lte', default=None)),
        ('voltage__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='voltage__lt', default=None)),
        ('voltage__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='voltage__gte', default=None)),
        ('voltage__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='voltage__gt', default=None)),
        ('amperage__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='amperage__n', default=None)),
        ('amperage__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='amperage__lte', default=None)),
        ('amperage__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='amperage__lt', default=None)),
        ('amperage__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='amperage__gte', default=None)),
        ('amperage__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='amperage__gt', default=None)),
        ('max_utilization__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_utilization__n', default=None)),
        ('max_utilization__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_utilization__lte', default=None)),
        ('max_utilization__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_utilization__lt', default=None)),
        ('max_utilization__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_utilization__gte', default=None)),
        ('max_utilization__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_utilization__gt', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('power_panel_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='power_panel_id__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
))
    )
    contacts = sgqlc.types.Field(sgqlc.types.list_of(ContactAssignmentType), graphql_name='contacts')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of(JournalEntryType), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class PowerPortTemplateType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'name', '_name', 'label', 'description', 'device_type', 'module_type', 'type', 'maximum_draw', 'allocated_draw', 'poweroutlet_templates', 'display', 'class_type', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    _name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='_name')
    label = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='label')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    device_type = sgqlc.types.Field(DeviceTypeType, graphql_name='device_type')
    module_type = sgqlc.types.Field(ModuleTypeType, graphql_name='module_type')
    type = sgqlc.types.Field(DcimPowerPortTemplateTypeChoices, graphql_name='type')
    maximum_draw = sgqlc.types.Field(Int, graphql_name='maximum_draw')
    allocated_draw = sgqlc.types.Field(Int, graphql_name='allocated_draw')
    poweroutlet_templates = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(PowerOutletTemplateType))), graphql_name='poweroutlet_templates', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('type', sgqlc.types.Arg(String, graphql_name='type', default=None)),
        ('feed_leg', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='feed_leg', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('moduletype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('type__n', sgqlc.types.Arg(String, graphql_name='type__n', default=None)),
        ('feed_leg__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='feed_leg__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('moduletype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class PowerPortType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'device', 'name', '_name', 'label', 'description', 'module', 'cable', 'cable_end', 'mark_connected', 'type', 'maximum_draw', 'allocated_draw', 'poweroutlets', 'connected_endpoints', 'link_peers', 'display', 'class_type', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    device = sgqlc.types.Field(sgqlc.types.non_null(DeviceType), graphql_name='device')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    _name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='_name')
    label = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='label')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    module = sgqlc.types.Field(ModuleType, graphql_name='module')
    cable = sgqlc.types.Field(CableType, graphql_name='cable')
    cable_end = sgqlc.types.Field(DcimPowerPortCableEndChoices, graphql_name='cable_end')
    mark_connected = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='mark_connected')
    type = sgqlc.types.Field(DcimPowerPortTypeChoices, graphql_name='type')
    maximum_draw = sgqlc.types.Field(Int, graphql_name='maximum_draw')
    allocated_draw = sgqlc.types.Field(Int, graphql_name='allocated_draw')
    poweroutlets = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(PowerOutletType))), graphql_name='poweroutlets', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('feed_leg', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='feed_leg', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('feed_leg__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='feed_leg__n', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
))
    )
    connected_endpoints = sgqlc.types.Field(sgqlc.types.list_of('LinkPeerType'), graphql_name='connected_endpoints')
    link_peers = sgqlc.types.Field(sgqlc.types.list_of('LinkPeerType'), graphql_name='link_peers')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class PrefixType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'prefix', 'site', 'vrf', 'tenant', 'vlan', 'status', 'role', 'is_pool', 'mark_utilized', '_depth', '_children', 'family', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    prefix = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='prefix')
    site = sgqlc.types.Field('SiteType', graphql_name='site')
    vrf = sgqlc.types.Field('VRFType', graphql_name='vrf')
    tenant = sgqlc.types.Field('TenantType', graphql_name='tenant')
    vlan = sgqlc.types.Field('VLANType', graphql_name='vlan')
    status = sgqlc.types.Field(sgqlc.types.non_null(IpamPrefixStatusChoices), graphql_name='status')
    role = sgqlc.types.Field('RoleType', graphql_name='role')
    is_pool = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='is_pool')
    mark_utilized = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='mark_utilized')
    _depth = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='_depth')
    _children = sgqlc.types.Field(sgqlc.types.non_null(BigInt), graphql_name='_children')
    family = sgqlc.types.Field(IPAddressFamilyType, graphql_name='family')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of(JournalEntryType), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class ProviderNetworkType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'name', 'provider', 'service_id', 'circuit_terminations', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    provider = sgqlc.types.Field(sgqlc.types.non_null('ProviderType'), graphql_name='provider')
    service_id = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='service_id')
    circuit_terminations = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(CircuitTerminationType))), graphql_name='circuit_terminations', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('term_side', sgqlc.types.Arg(String, graphql_name='term_side', default=None)),
        ('port_speed', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='port_speed', default=None)),
        ('upstream_speed', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='upstream_speed', default=None)),
        ('xconnect_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('circuit_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='circuit_id', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('provider_network_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_network_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('term_side__n', sgqlc.types.Arg(String, graphql_name='term_side__n', default=None)),
        ('port_speed__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='port_speed__n', default=None)),
        ('port_speed__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='port_speed__lte', default=None)),
        ('port_speed__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='port_speed__lt', default=None)),
        ('port_speed__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='port_speed__gte', default=None)),
        ('port_speed__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='port_speed__gt', default=None)),
        ('upstream_speed__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='upstream_speed__n', default=None)),
        ('upstream_speed__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='upstream_speed__lte', default=None)),
        ('upstream_speed__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='upstream_speed__lt', default=None)),
        ('upstream_speed__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='upstream_speed__gte', default=None)),
        ('upstream_speed__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='upstream_speed__gt', default=None)),
        ('xconnect_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__n', default=None)),
        ('xconnect_id__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__ic', default=None)),
        ('xconnect_id__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__nic', default=None)),
        ('xconnect_id__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__iew', default=None)),
        ('xconnect_id__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__niew', default=None)),
        ('xconnect_id__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__isw', default=None)),
        ('xconnect_id__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__nisw', default=None)),
        ('xconnect_id__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__ie', default=None)),
        ('xconnect_id__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__nie', default=None)),
        ('xconnect_id__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('circuit_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='circuit_id__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('provider_network_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_network_id__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of(JournalEntryType), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class ProviderType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'name', 'slug', 'asns', 'account', 'circuits', 'networks', 'contacts', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    slug = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='slug')
    asns = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ASNType))), graphql_name='asns', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('asn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('rir_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir_id', default=None)),
        ('rir', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('asn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__n', default=None)),
        ('asn__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__lte', default=None)),
        ('asn__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__lt', default=None)),
        ('asn__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__gte', default=None)),
        ('asn__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('rir_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir_id__n', default=None)),
        ('rir__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
))
    )
    account = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='account')
    circuits = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(CircuitType))), graphql_name='circuits', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('cid', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('install_date', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='install_date', default=None)),
        ('termination_date', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_date', default=None)),
        ('commit_rate', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='commit_rate', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('provider_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_id', default=None)),
        ('provider', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider', default=None)),
        ('provider_network_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_network_id', default=None)),
        ('type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type_id', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('cid__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__n', default=None)),
        ('cid__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__ic', default=None)),
        ('cid__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__nic', default=None)),
        ('cid__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__iew', default=None)),
        ('cid__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__niew', default=None)),
        ('cid__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__isw', default=None)),
        ('cid__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__nisw', default=None)),
        ('cid__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__ie', default=None)),
        ('cid__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__nie', default=None)),
        ('cid__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('install_date__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='install_date__n', default=None)),
        ('install_date__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='install_date__lte', default=None)),
        ('install_date__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='install_date__lt', default=None)),
        ('install_date__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='install_date__gte', default=None)),
        ('install_date__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='install_date__gt', default=None)),
        ('termination_date__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_date__n', default=None)),
        ('termination_date__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_date__lte', default=None)),
        ('termination_date__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_date__lt', default=None)),
        ('termination_date__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_date__gte', default=None)),
        ('termination_date__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_date__gt', default=None)),
        ('commit_rate__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='commit_rate__n', default=None)),
        ('commit_rate__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='commit_rate__lte', default=None)),
        ('commit_rate__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='commit_rate__lt', default=None)),
        ('commit_rate__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='commit_rate__gte', default=None)),
        ('commit_rate__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='commit_rate__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('provider_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_id__n', default=None)),
        ('provider__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider__n', default=None)),
        ('provider_network_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_network_id__n', default=None)),
        ('type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type_id__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
))
    )
    networks = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ProviderNetworkType))), graphql_name='networks', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('service_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='service_id', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('provider_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_id', default=None)),
        ('provider', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('service_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='service_id__n', default=None)),
        ('service_id__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='service_id__ic', default=None)),
        ('service_id__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='service_id__nic', default=None)),
        ('service_id__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='service_id__iew', default=None)),
        ('service_id__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='service_id__niew', default=None)),
        ('service_id__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='service_id__isw', default=None)),
        ('service_id__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='service_id__nisw', default=None)),
        ('service_id__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='service_id__ie', default=None)),
        ('service_id__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='service_id__nie', default=None)),
        ('service_id__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='service_id__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('provider_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_id__n', default=None)),
        ('provider__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider__n', default=None)),
))
    )
    contacts = sgqlc.types.Field(sgqlc.types.list_of(ContactAssignmentType), graphql_name='contacts')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of(JournalEntryType), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class Query(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('wireless_lan', 'wireless_lan_list', 'wireless_lan_group', 'wireless_lan_group_list', 'wireless_link', 'wireless_link_list', 'cluster', 'cluster_list', 'cluster_group', 'cluster_group_list', 'cluster_type', 'cluster_type_list', 'virtual_machine', 'virtual_machine_list', 'vm_interface', 'vm_interface_list', 'tenant', 'tenant_list', 'tenant_group', 'tenant_group_list', 'contact', 'contact_list', 'contact_role', 'contact_role_list', 'contact_group', 'contact_group_list', 'contact_assignment', 'contact_assignment_list', 'asn', 'asn_list', 'aggregate', 'aggregate_list', 'ip_address', 'ip_address_list', 'ip_range', 'ip_range_list', 'l2vpn', 'l2vpn_list', 'l2vpn_termination', 'l2vpn_termination_list', 'prefix', 'prefix_list', 'rir', 'rir_list', 'role', 'role_list', 'route_target', 'route_target_list', 'service', 'service_list', 'service_template', 'service_template_list', 'fhrp_group', 'fhrp_group_list', 'fhrp_group_assignment', 'fhrp_group_assignment_list', 'vlan', 'vlan_list', 'vlan_group', 'vlan_group_list', 'vrf', 'vrf_list', 'config_context', 'config_context_list', 'custom_field', 'custom_field_list', 'custom_link', 'custom_link_list', 'export_template', 'export_template_list', 'image_attachment', 'image_attachment_list', 'saved_filter', 'saved_filter_list', 'journal_entry', 'journal_entry_list', 'tag', 'tag_list', 'webhook', 'webhook_list', 'cable', 'cable_list', 'console_port', 'console_port_list', 'console_port_template', 'console_port_template_list', 'console_server_port', 'console_server_port_list', 'console_server_port_template', 'console_server_port_template_list', 'device', 'device_list', 'device_bay', 'device_bay_list', 'device_bay_template', 'device_bay_template_list', 'device_role', 'device_role_list', 'device_type', 'device_type_list', 'front_port', 'front_port_list', 'front_port_template', 'front_port_template_list', 'interface', 'interface_list', 'interface_template', 'interface_template_list', 'inventory_item', 'inventory_item_list', 'inventory_item_role', 'inventory_item_role_list', 'inventory_item_template', 'inventory_item_template_list', 'location', 'location_list', 'manufacturer', 'manufacturer_list', 'module', 'module_list', 'module_bay', 'module_bay_list', 'module_bay_template', 'module_bay_template_list', 'module_type', 'module_type_list', 'platform', 'platform_list', 'power_feed', 'power_feed_list', 'power_outlet', 'power_outlet_list', 'power_outlet_template', 'power_outlet_template_list', 'power_panel', 'power_panel_list', 'power_port', 'power_port_list', 'power_port_template', 'power_port_template_list', 'rack', 'rack_list', 'rack_reservation', 'rack_reservation_list', 'rack_role', 'rack_role_list', 'rear_port', 'rear_port_list', 'rear_port_template', 'rear_port_template_list', 'region', 'region_list', 'site', 'site_list', 'site_group', 'site_group_list', 'virtual_chassis', 'virtual_chassis_list', 'virtual_device_context', 'virtual_device_context_list', 'circuit', 'circuit_list', 'circuit_termination', 'circuit_termination_list', 'circuit_type', 'circuit_type_list', 'provider', 'provider_list', 'provider_network', 'provider_network_list', 'group', 'group_list', 'user', 'user_list')
    wireless_lan = sgqlc.types.Field('WirelessLANType', graphql_name='wireless_lan', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    wireless_lan_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('WirelessLANType')), graphql_name='wireless_lan_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('ssid', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid', default=None)),
        ('auth_psk', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('vlan_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vlan_id', default=None)),
        ('auth_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_type', default=None)),
        ('auth_cipher', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_cipher', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('ssid__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__n', default=None)),
        ('ssid__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__ic', default=None)),
        ('ssid__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__nic', default=None)),
        ('ssid__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__iew', default=None)),
        ('ssid__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__niew', default=None)),
        ('ssid__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__isw', default=None)),
        ('ssid__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__nisw', default=None)),
        ('ssid__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__ie', default=None)),
        ('ssid__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__nie', default=None)),
        ('ssid__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__empty', default=None)),
        ('auth_psk__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__n', default=None)),
        ('auth_psk__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__ic', default=None)),
        ('auth_psk__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__nic', default=None)),
        ('auth_psk__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__iew', default=None)),
        ('auth_psk__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__niew', default=None)),
        ('auth_psk__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__isw', default=None)),
        ('auth_psk__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__nisw', default=None)),
        ('auth_psk__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__ie', default=None)),
        ('auth_psk__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__nie', default=None)),
        ('auth_psk__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('vlan_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vlan_id__n', default=None)),
        ('auth_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_type__n', default=None)),
        ('auth_cipher__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_cipher__n', default=None)),
))
    )
    wireless_lan_group = sgqlc.types.Field('WirelessLANGroupType', graphql_name='wireless_lan_group', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    wireless_lan_group_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('WirelessLANGroupType')), graphql_name='wireless_lan_group_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('parent', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('parent__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent__n', default=None)),
))
    )
    wireless_link = sgqlc.types.Field('WirelessLinkType', graphql_name='wireless_link', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    wireless_link_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('WirelessLinkType')), graphql_name='wireless_link_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('ssid', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid', default=None)),
        ('auth_psk', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('interface_a_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_a_id', default=None)),
        ('interface_b_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_b_id', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('auth_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_type', default=None)),
        ('auth_cipher', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_cipher', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('ssid__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__n', default=None)),
        ('ssid__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__ic', default=None)),
        ('ssid__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__nic', default=None)),
        ('ssid__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__iew', default=None)),
        ('ssid__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__niew', default=None)),
        ('ssid__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__isw', default=None)),
        ('ssid__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__nisw', default=None)),
        ('ssid__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__ie', default=None)),
        ('ssid__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__nie', default=None)),
        ('ssid__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__empty', default=None)),
        ('auth_psk__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__n', default=None)),
        ('auth_psk__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__ic', default=None)),
        ('auth_psk__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__nic', default=None)),
        ('auth_psk__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__iew', default=None)),
        ('auth_psk__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__niew', default=None)),
        ('auth_psk__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__isw', default=None)),
        ('auth_psk__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__nisw', default=None)),
        ('auth_psk__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__ie', default=None)),
        ('auth_psk__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__nie', default=None)),
        ('auth_psk__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('interface_a_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_a_id__n', default=None)),
        ('interface_a_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_a_id__lte', default=None)),
        ('interface_a_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_a_id__lt', default=None)),
        ('interface_a_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_a_id__gte', default=None)),
        ('interface_a_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_a_id__gt', default=None)),
        ('interface_b_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_b_id__n', default=None)),
        ('interface_b_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_b_id__lte', default=None)),
        ('interface_b_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_b_id__lt', default=None)),
        ('interface_b_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_b_id__gte', default=None)),
        ('interface_b_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_b_id__gt', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('auth_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_type__n', default=None)),
        ('auth_cipher__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_cipher__n', default=None)),
))
    )
    cluster = sgqlc.types.Field(ClusterType, graphql_name='cluster', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    cluster_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(ClusterType)), graphql_name='cluster_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type_id', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
        ('type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type_id__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
))
    )
    cluster_group = sgqlc.types.Field(ClusterGroupType, graphql_name='cluster_group', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    cluster_group_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(ClusterGroupType)), graphql_name='cluster_group_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
))
    )
    cluster_type = sgqlc.types.Field(ClusterTypeType, graphql_name='cluster_type', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    cluster_type_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(ClusterTypeType)), graphql_name='cluster_type_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
))
    )
    virtual_machine = sgqlc.types.Field('VirtualMachineType', graphql_name='virtual_machine', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    virtual_machine_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('VirtualMachineType')), graphql_name='virtual_machine_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('cluster', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster', default=None)),
        ('vcpus', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus', default=None)),
        ('memory', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory', default=None)),
        ('disk', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('local_context_data', sgqlc.types.Arg(Boolean, graphql_name='local_context_data', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('cluster_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group_id', default=None)),
        ('cluster_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group', default=None)),
        ('cluster_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type_id', default=None)),
        ('cluster_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type', default=None)),
        ('cluster_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('platform_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id', default=None)),
        ('platform', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('has_primary_ip', sgqlc.types.Arg(Boolean, graphql_name='has_primary_ip', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('cluster__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster__n', default=None)),
        ('vcpus__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__n', default=None)),
        ('vcpus__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__lte', default=None)),
        ('vcpus__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__lt', default=None)),
        ('vcpus__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__gte', default=None)),
        ('vcpus__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__gt', default=None)),
        ('memory__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__n', default=None)),
        ('memory__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__lte', default=None)),
        ('memory__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__lt', default=None)),
        ('memory__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__gte', default=None)),
        ('memory__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__gt', default=None)),
        ('disk__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__n', default=None)),
        ('disk__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__lte', default=None)),
        ('disk__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__lt', default=None)),
        ('disk__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__gte', default=None)),
        ('disk__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('cluster_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group_id__n', default=None)),
        ('cluster_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group__n', default=None)),
        ('cluster_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type_id__n', default=None)),
        ('cluster_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type__n', default=None)),
        ('cluster_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('platform_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id__n', default=None)),
        ('platform__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
))
    )
    vm_interface = sgqlc.types.Field('VMInterfaceType', graphql_name='vm_interface', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    vm_interface_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('VMInterfaceType')), graphql_name='vm_interface_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('enabled', sgqlc.types.Arg(Boolean, graphql_name='enabled', default=None)),
        ('mtu', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cluster_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id', default=None)),
        ('cluster', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster', default=None)),
        ('virtual_machine_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id', default=None)),
        ('virtual_machine', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('bridge_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('l2vpn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id', default=None)),
        ('l2vpn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('mtu__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__n', default=None)),
        ('mtu__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lte', default=None)),
        ('mtu__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lt', default=None)),
        ('mtu__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gte', default=None)),
        ('mtu__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('cluster_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id__n', default=None)),
        ('cluster__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster__n', default=None)),
        ('virtual_machine_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id__n', default=None)),
        ('virtual_machine__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('bridge_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('l2vpn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id__n', default=None)),
        ('l2vpn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn__n', default=None)),
))
    )
    tenant = sgqlc.types.Field('TenantType', graphql_name='tenant', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    tenant_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('TenantType')), graphql_name='tenant_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
))
    )
    tenant_group = sgqlc.types.Field('TenantGroupType', graphql_name='tenant_group', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    tenant_group_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('TenantGroupType')), graphql_name='tenant_group_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('parent', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('parent__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent__n', default=None)),
))
    )
    contact = sgqlc.types.Field(ContactType, graphql_name='contact', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    contact_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(ContactType)), graphql_name='contact_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('title', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='title', default=None)),
        ('phone', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='phone', default=None)),
        ('email', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email', default=None)),
        ('address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address', default=None)),
        ('link', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='link', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('title__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='title__n', default=None)),
        ('title__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='title__ic', default=None)),
        ('title__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='title__nic', default=None)),
        ('title__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='title__iew', default=None)),
        ('title__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='title__niew', default=None)),
        ('title__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='title__isw', default=None)),
        ('title__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='title__nisw', default=None)),
        ('title__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='title__ie', default=None)),
        ('title__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='title__nie', default=None)),
        ('title__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='title__empty', default=None)),
        ('phone__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='phone__n', default=None)),
        ('phone__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='phone__ic', default=None)),
        ('phone__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='phone__nic', default=None)),
        ('phone__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='phone__iew', default=None)),
        ('phone__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='phone__niew', default=None)),
        ('phone__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='phone__isw', default=None)),
        ('phone__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='phone__nisw', default=None)),
        ('phone__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='phone__ie', default=None)),
        ('phone__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='phone__nie', default=None)),
        ('phone__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='phone__empty', default=None)),
        ('email__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__n', default=None)),
        ('email__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__ic', default=None)),
        ('email__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__nic', default=None)),
        ('email__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__iew', default=None)),
        ('email__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__niew', default=None)),
        ('email__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__isw', default=None)),
        ('email__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__nisw', default=None)),
        ('email__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__ie', default=None)),
        ('email__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__nie', default=None)),
        ('email__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__empty', default=None)),
        ('address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address__n', default=None)),
        ('address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address__ic', default=None)),
        ('address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address__nic', default=None)),
        ('address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address__iew', default=None)),
        ('address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address__niew', default=None)),
        ('address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address__isw', default=None)),
        ('address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address__nisw', default=None)),
        ('address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address__ie', default=None)),
        ('address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address__nie', default=None)),
        ('address__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address__empty', default=None)),
        ('link__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='link__n', default=None)),
        ('link__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='link__ic', default=None)),
        ('link__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='link__nic', default=None)),
        ('link__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='link__iew', default=None)),
        ('link__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='link__niew', default=None)),
        ('link__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='link__isw', default=None)),
        ('link__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='link__nisw', default=None)),
        ('link__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='link__ie', default=None)),
        ('link__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='link__nie', default=None)),
        ('link__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='link__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
))
    )
    contact_role = sgqlc.types.Field(ContactRoleType, graphql_name='contact_role', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    contact_role_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(ContactRoleType)), graphql_name='contact_role_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
))
    )
    contact_group = sgqlc.types.Field(ContactGroupType, graphql_name='contact_group', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    contact_group_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(ContactGroupType)), graphql_name='contact_group_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('parent', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('parent__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent__n', default=None)),
))
    )
    contact_assignment = sgqlc.types.Field(ContactAssignmentType, graphql_name='contact_assignment', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    contact_assignment_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(ContactAssignmentType)), graphql_name='contact_assignment_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('content_type_id', sgqlc.types.Arg(String, graphql_name='content_type_id', default=None)),
        ('object_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='object_id', default=None)),
        ('priority', sgqlc.types.Arg(String, graphql_name='priority', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('content_type', sgqlc.types.Arg(String, graphql_name='content_type', default=None)),
        ('contact_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_id', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('content_type_id__n', sgqlc.types.Arg(String, graphql_name='content_type_id__n', default=None)),
        ('object_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='object_id__n', default=None)),
        ('object_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='object_id__lte', default=None)),
        ('object_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='object_id__lt', default=None)),
        ('object_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='object_id__gte', default=None)),
        ('object_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='object_id__gt', default=None)),
        ('priority__n', sgqlc.types.Arg(String, graphql_name='priority__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('content_type__n', sgqlc.types.Arg(String, graphql_name='content_type__n', default=None)),
        ('contact_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_id__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
))
    )
    asn = sgqlc.types.Field(ASNType, graphql_name='asn', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    asn_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(ASNType)), graphql_name='asn_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('asn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('rir_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir_id', default=None)),
        ('rir', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('asn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__n', default=None)),
        ('asn__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__lte', default=None)),
        ('asn__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__lt', default=None)),
        ('asn__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__gte', default=None)),
        ('asn__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('rir_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir_id__n', default=None)),
        ('rir__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
))
    )
    aggregate = sgqlc.types.Field(AggregateType, graphql_name='aggregate', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    aggregate_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(AggregateType)), graphql_name='aggregate_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('date_added', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='date_added', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('family', sgqlc.types.Arg(Int, graphql_name='family', default=None)),
        ('prefix', sgqlc.types.Arg(String, graphql_name='prefix', default=None)),
        ('rir_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir_id', default=None)),
        ('rir', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('date_added__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='date_added__n', default=None)),
        ('date_added__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='date_added__lte', default=None)),
        ('date_added__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='date_added__lt', default=None)),
        ('date_added__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='date_added__gte', default=None)),
        ('date_added__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='date_added__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('rir_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir_id__n', default=None)),
        ('rir__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir__n', default=None)),
))
    )
    ip_address = sgqlc.types.Field(IPAddressType, graphql_name='ip_address', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    ip_address_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(IPAddressType)), graphql_name='ip_address_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('dns_name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('family', sgqlc.types.Arg(Int, graphql_name='family', default=None)),
        ('parent', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent', default=None)),
        ('address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address', default=None)),
        ('mask_length', sgqlc.types.Arg(Int, graphql_name='mask_length', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('present_in_vrf_id', sgqlc.types.Arg(String, graphql_name='present_in_vrf_id', default=None)),
        ('present_in_vrf', sgqlc.types.Arg(String, graphql_name='present_in_vrf', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('virtual_machine', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine', default=None)),
        ('virtual_machine_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id', default=None)),
        ('interface', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface', default=None)),
        ('interface_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_id', default=None)),
        ('vminterface', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vminterface', default=None)),
        ('vminterface_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vminterface_id', default=None)),
        ('fhrpgroup_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='fhrpgroup_id', default=None)),
        ('assigned_to_interface', sgqlc.types.Arg(Boolean, graphql_name='assigned_to_interface', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('dns_name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__n', default=None)),
        ('dns_name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__ic', default=None)),
        ('dns_name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__nic', default=None)),
        ('dns_name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__iew', default=None)),
        ('dns_name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__niew', default=None)),
        ('dns_name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__isw', default=None)),
        ('dns_name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__nisw', default=None)),
        ('dns_name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__ie', default=None)),
        ('dns_name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__nie', default=None)),
        ('dns_name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('interface__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface__n', default=None)),
        ('interface_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_id__n', default=None)),
        ('vminterface__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vminterface__n', default=None)),
        ('vminterface_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vminterface_id__n', default=None)),
        ('fhrpgroup_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='fhrpgroup_id__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
))
    )
    ip_range = sgqlc.types.Field(IPRangeType, graphql_name='ip_range', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    ip_range_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(IPRangeType)), graphql_name='ip_range_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('family', sgqlc.types.Arg(Int, graphql_name='family', default=None)),
        ('start_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='start_address', default=None)),
        ('end_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='end_address', default=None)),
        ('contains', sgqlc.types.Arg(String, graphql_name='contains', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
))
    )
    l2vpn = sgqlc.types.Field(L2VPNType, graphql_name='l2vpn', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    l2vpn_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(L2VPNType)), graphql_name='l2vpn_list')
    l2vpn_termination = sgqlc.types.Field(L2VPNTerminationType, graphql_name='l2vpn_termination', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    l2vpn_termination_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(L2VPNTerminationType)), graphql_name='l2vpn_termination_list')
    prefix = sgqlc.types.Field(PrefixType, graphql_name='prefix', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    prefix_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(PrefixType)), graphql_name='prefix_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('is_pool', sgqlc.types.Arg(Boolean, graphql_name='is_pool', default=None)),
        ('mark_utilized', sgqlc.types.Arg(Boolean, graphql_name='mark_utilized', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('family', sgqlc.types.Arg(Int, graphql_name='family', default=None)),
        ('prefix', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='prefix', default=None)),
        ('within', sgqlc.types.Arg(String, graphql_name='within', default=None)),
        ('within_include', sgqlc.types.Arg(String, graphql_name='within_include', default=None)),
        ('contains', sgqlc.types.Arg(String, graphql_name='contains', default=None)),
        ('depth', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth', default=None)),
        ('children', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children', default=None)),
        ('mask_length', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mask_length', default=None)),
        ('mask_length__gte', sgqlc.types.Arg(Int, graphql_name='mask_length__gte', default=None)),
        ('mask_length__lte', sgqlc.types.Arg(Int, graphql_name='mask_length__lte', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('present_in_vrf_id', sgqlc.types.Arg(String, graphql_name='present_in_vrf_id', default=None)),
        ('present_in_vrf', sgqlc.types.Arg(String, graphql_name='present_in_vrf', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('vlan_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vlan_id', default=None)),
        ('vlan_vid', sgqlc.types.Arg(Int, graphql_name='vlan_vid', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('depth__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__n', default=None)),
        ('depth__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__lte', default=None)),
        ('depth__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__lt', default=None)),
        ('depth__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__gte', default=None)),
        ('depth__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__gt', default=None)),
        ('children__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__n', default=None)),
        ('children__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__lte', default=None)),
        ('children__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__lt', default=None)),
        ('children__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__gte', default=None)),
        ('children__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__gt', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('vlan_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vlan_id__n', default=None)),
        ('vlan_vid__n', sgqlc.types.Arg(Int, graphql_name='vlan_vid__n', default=None)),
        ('vlan_vid__lte', sgqlc.types.Arg(Int, graphql_name='vlan_vid__lte', default=None)),
        ('vlan_vid__lt', sgqlc.types.Arg(Int, graphql_name='vlan_vid__lt', default=None)),
        ('vlan_vid__gte', sgqlc.types.Arg(Int, graphql_name='vlan_vid__gte', default=None)),
        ('vlan_vid__gt', sgqlc.types.Arg(Int, graphql_name='vlan_vid__gt', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
))
    )
    rir = sgqlc.types.Field('RIRType', graphql_name='rir', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    rir_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('RIRType')), graphql_name='rir_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('is_private', sgqlc.types.Arg(Boolean, graphql_name='is_private', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
))
    )
    role = sgqlc.types.Field('RoleType', graphql_name='role', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    role_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('RoleType')), graphql_name='role_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
))
    )
    route_target = sgqlc.types.Field('RouteTargetType', graphql_name='route_target', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    route_target_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('RouteTargetType')), graphql_name='route_target_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('importing_vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='importing_vrf_id', default=None)),
        ('importing_vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='importing_vrf', default=None)),
        ('exporting_vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='exporting_vrf_id', default=None)),
        ('exporting_vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='exporting_vrf', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('importing_vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='importing_vrf_id__n', default=None)),
        ('importing_vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='importing_vrf__n', default=None)),
        ('exporting_vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='exporting_vrf_id__n', default=None)),
        ('exporting_vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='exporting_vrf__n', default=None)),
))
    )
    service = sgqlc.types.Field('ServiceType', graphql_name='service', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    service_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('ServiceType')), graphql_name='service_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('protocol', sgqlc.types.Arg(String, graphql_name='protocol', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_machine_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id', default=None)),
        ('virtual_machine', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine', default=None)),
        ('ipaddress_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ipaddress_id', default=None)),
        ('ipaddress', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ipaddress', default=None)),
        ('port', sgqlc.types.Arg(Int, graphql_name='port', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('protocol__n', sgqlc.types.Arg(String, graphql_name='protocol__n', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_machine_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id__n', default=None)),
        ('virtual_machine__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine__n', default=None)),
        ('ipaddress_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ipaddress_id__n', default=None)),
        ('ipaddress__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ipaddress__n', default=None)),
))
    )
    service_template = sgqlc.types.Field('ServiceTemplateType', graphql_name='service_template', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    service_template_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('ServiceTemplateType')), graphql_name='service_template_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('protocol', sgqlc.types.Arg(String, graphql_name='protocol', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('port', sgqlc.types.Arg(Int, graphql_name='port', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('protocol__n', sgqlc.types.Arg(String, graphql_name='protocol__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
))
    )
    fhrp_group = sgqlc.types.Field(FHRPGroupType, graphql_name='fhrp_group', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    fhrp_group_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(FHRPGroupType)), graphql_name='fhrp_group_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('auth_key', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_key', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('protocol', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='protocol', default=None)),
        ('auth_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_type', default=None)),
        ('related_ip', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='related_ip', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__lte', default=None)),
        ('group_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__lt', default=None)),
        ('group_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__gte', default=None)),
        ('group_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('auth_key__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_key__n', default=None)),
        ('auth_key__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_key__ic', default=None)),
        ('auth_key__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_key__nic', default=None)),
        ('auth_key__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_key__iew', default=None)),
        ('auth_key__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_key__niew', default=None)),
        ('auth_key__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_key__isw', default=None)),
        ('auth_key__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_key__nisw', default=None)),
        ('auth_key__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_key__ie', default=None)),
        ('auth_key__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_key__nie', default=None)),
        ('auth_key__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_key__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('protocol__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='protocol__n', default=None)),
        ('auth_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_type__n', default=None)),
))
    )
    fhrp_group_assignment = sgqlc.types.Field(FHRPGroupAssignmentType, graphql_name='fhrp_group_assignment', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    fhrp_group_assignment_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(FHRPGroupAssignmentType)), graphql_name='fhrp_group_assignment_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('interface_type', sgqlc.types.Arg(String, graphql_name='interface_type', default=None)),
        ('interface_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_id', default=None)),
        ('priority', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='priority', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('virtual_machine', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine', default=None)),
        ('virtual_machine_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('interface_type__n', sgqlc.types.Arg(String, graphql_name='interface_type__n', default=None)),
        ('interface_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_id__n', default=None)),
        ('interface_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_id__lte', default=None)),
        ('interface_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_id__lt', default=None)),
        ('interface_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_id__gte', default=None)),
        ('interface_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_id__gt', default=None)),
        ('priority__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='priority__n', default=None)),
        ('priority__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='priority__lte', default=None)),
        ('priority__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='priority__lt', default=None)),
        ('priority__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='priority__gte', default=None)),
        ('priority__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='priority__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
))
    )
    vlan = sgqlc.types.Field('VLANType', graphql_name='vlan', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    vlan_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('VLANType')), graphql_name='vlan_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('vid', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('available_on_device', sgqlc.types.Arg(String, graphql_name='available_on_device', default=None)),
        ('available_on_virtualmachine', sgqlc.types.Arg(String, graphql_name='available_on_virtualmachine', default=None)),
        ('l2vpn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id', default=None)),
        ('l2vpn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('vid__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__n', default=None)),
        ('vid__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__lte', default=None)),
        ('vid__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__lt', default=None)),
        ('vid__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__gte', default=None)),
        ('vid__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('l2vpn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id__n', default=None)),
        ('l2vpn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn__n', default=None)),
))
    )
    vlan_group = sgqlc.types.Field('VLANGroupType', graphql_name='vlan_group', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    vlan_group_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('VLANGroupType')), graphql_name='vlan_group_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('min_vid', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='min_vid', default=None)),
        ('max_vid', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_vid', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('scope_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='scope_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('scope_type', sgqlc.types.Arg(String, graphql_name='scope_type', default=None)),
        ('region', sgqlc.types.Arg(Int, graphql_name='region', default=None)),
        ('sitegroup', sgqlc.types.Arg(Int, graphql_name='sitegroup', default=None)),
        ('site', sgqlc.types.Arg(Int, graphql_name='site', default=None)),
        ('location', sgqlc.types.Arg(Int, graphql_name='location', default=None)),
        ('rack', sgqlc.types.Arg(Int, graphql_name='rack', default=None)),
        ('clustergroup', sgqlc.types.Arg(Int, graphql_name='clustergroup', default=None)),
        ('cluster', sgqlc.types.Arg(Int, graphql_name='cluster', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('min_vid__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='min_vid__n', default=None)),
        ('min_vid__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='min_vid__lte', default=None)),
        ('min_vid__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='min_vid__lt', default=None)),
        ('min_vid__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='min_vid__gte', default=None)),
        ('min_vid__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='min_vid__gt', default=None)),
        ('max_vid__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_vid__n', default=None)),
        ('max_vid__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_vid__lte', default=None)),
        ('max_vid__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_vid__lt', default=None)),
        ('max_vid__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_vid__gte', default=None)),
        ('max_vid__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_vid__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('scope_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='scope_id__n', default=None)),
        ('scope_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='scope_id__lte', default=None)),
        ('scope_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='scope_id__lt', default=None)),
        ('scope_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='scope_id__gte', default=None)),
        ('scope_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='scope_id__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('scope_type__n', sgqlc.types.Arg(String, graphql_name='scope_type__n', default=None)),
))
    )
    vrf = sgqlc.types.Field('VRFType', graphql_name='vrf', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    vrf_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('VRFType')), graphql_name='vrf_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('rd', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd', default=None)),
        ('enforce_unique', sgqlc.types.Arg(Boolean, graphql_name='enforce_unique', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('import_target_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='import_target_id', default=None)),
        ('import_target', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='import_target', default=None)),
        ('export_target_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='export_target_id', default=None)),
        ('export_target', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='export_target', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('rd__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__n', default=None)),
        ('rd__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__ic', default=None)),
        ('rd__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__nic', default=None)),
        ('rd__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__iew', default=None)),
        ('rd__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__niew', default=None)),
        ('rd__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__isw', default=None)),
        ('rd__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__nisw', default=None)),
        ('rd__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__ie', default=None)),
        ('rd__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__nie', default=None)),
        ('rd__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('import_target_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='import_target_id__n', default=None)),
        ('import_target__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='import_target__n', default=None)),
        ('export_target_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='export_target_id__n', default=None)),
        ('export_target__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='export_target__n', default=None)),
))
    )
    config_context = sgqlc.types.Field(ConfigContextType, graphql_name='config_context', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    config_context_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(ConfigContextType)), graphql_name='config_context_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('is_active', sgqlc.types.Arg(Boolean, graphql_name='is_active', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('device_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type_id', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('platform_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id', default=None)),
        ('platform', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform', default=None)),
        ('cluster_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type_id', default=None)),
        ('cluster_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type', default=None)),
        ('cluster_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group_id', default=None)),
        ('cluster_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group', default=None)),
        ('cluster_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('tag_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag_id', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('device_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type_id__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('platform_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id__n', default=None)),
        ('platform__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform__n', default=None)),
        ('cluster_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type_id__n', default=None)),
        ('cluster_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type__n', default=None)),
        ('cluster_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group_id__n', default=None)),
        ('cluster_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group__n', default=None)),
        ('cluster_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('tag_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag_id__n', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
))
    )
    custom_field = sgqlc.types.Field(CustomFieldType, graphql_name='custom_field', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    custom_field_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(CustomFieldType)), graphql_name='custom_field_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('content_types', sgqlc.types.Arg(String, graphql_name='content_types', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('group_name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_name', default=None)),
        ('required', sgqlc.types.Arg(Boolean, graphql_name='required', default=None)),
        ('search_weight', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='search_weight', default=None)),
        ('filter_logic', sgqlc.types.Arg(String, graphql_name='filter_logic', default=None)),
        ('ui_visibility', sgqlc.types.Arg(String, graphql_name='ui_visibility', default=None)),
        ('weight', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('content_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('content_types__n', sgqlc.types.Arg(String, graphql_name='content_types__n', default=None)),
        ('content_types__ic', sgqlc.types.Arg(String, graphql_name='content_types__ic', default=None)),
        ('content_types__nic', sgqlc.types.Arg(String, graphql_name='content_types__nic', default=None)),
        ('content_types__iew', sgqlc.types.Arg(String, graphql_name='content_types__iew', default=None)),
        ('content_types__niew', sgqlc.types.Arg(String, graphql_name='content_types__niew', default=None)),
        ('content_types__isw', sgqlc.types.Arg(String, graphql_name='content_types__isw', default=None)),
        ('content_types__nisw', sgqlc.types.Arg(String, graphql_name='content_types__nisw', default=None)),
        ('content_types__ie', sgqlc.types.Arg(String, graphql_name='content_types__ie', default=None)),
        ('content_types__nie', sgqlc.types.Arg(String, graphql_name='content_types__nie', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('group_name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_name__n', default=None)),
        ('group_name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_name__ic', default=None)),
        ('group_name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_name__nic', default=None)),
        ('group_name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_name__iew', default=None)),
        ('group_name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_name__niew', default=None)),
        ('group_name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_name__isw', default=None)),
        ('group_name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_name__nisw', default=None)),
        ('group_name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_name__ie', default=None)),
        ('group_name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_name__nie', default=None)),
        ('group_name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_name__empty', default=None)),
        ('search_weight__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='search_weight__n', default=None)),
        ('search_weight__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='search_weight__lte', default=None)),
        ('search_weight__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='search_weight__lt', default=None)),
        ('search_weight__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='search_weight__gte', default=None)),
        ('search_weight__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='search_weight__gt', default=None)),
        ('filter_logic__n', sgqlc.types.Arg(String, graphql_name='filter_logic__n', default=None)),
        ('ui_visibility__n', sgqlc.types.Arg(String, graphql_name='ui_visibility__n', default=None)),
        ('weight__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__n', default=None)),
        ('weight__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__lte', default=None)),
        ('weight__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__lt', default=None)),
        ('weight__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__gte', default=None)),
        ('weight__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('content_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id__n', default=None)),
        ('content_type_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id__lte', default=None)),
        ('content_type_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id__lt', default=None)),
        ('content_type_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id__gte', default=None)),
        ('content_type_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id__gt', default=None)),
))
    )
    custom_link = sgqlc.types.Field(CustomLinkType, graphql_name='custom_link', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    custom_link_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(CustomLinkType)), graphql_name='custom_link_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('content_types', sgqlc.types.Arg(String, graphql_name='content_types', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('enabled', sgqlc.types.Arg(Boolean, graphql_name='enabled', default=None)),
        ('link_text', sgqlc.types.Arg(String, graphql_name='link_text', default=None)),
        ('link_url', sgqlc.types.Arg(String, graphql_name='link_url', default=None)),
        ('weight', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight', default=None)),
        ('group_name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_name', default=None)),
        ('new_window', sgqlc.types.Arg(Boolean, graphql_name='new_window', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('content_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('content_types__n', sgqlc.types.Arg(String, graphql_name='content_types__n', default=None)),
        ('content_types__ic', sgqlc.types.Arg(String, graphql_name='content_types__ic', default=None)),
        ('content_types__nic', sgqlc.types.Arg(String, graphql_name='content_types__nic', default=None)),
        ('content_types__iew', sgqlc.types.Arg(String, graphql_name='content_types__iew', default=None)),
        ('content_types__niew', sgqlc.types.Arg(String, graphql_name='content_types__niew', default=None)),
        ('content_types__isw', sgqlc.types.Arg(String, graphql_name='content_types__isw', default=None)),
        ('content_types__nisw', sgqlc.types.Arg(String, graphql_name='content_types__nisw', default=None)),
        ('content_types__ie', sgqlc.types.Arg(String, graphql_name='content_types__ie', default=None)),
        ('content_types__nie', sgqlc.types.Arg(String, graphql_name='content_types__nie', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('link_text__n', sgqlc.types.Arg(String, graphql_name='link_text__n', default=None)),
        ('link_text__ic', sgqlc.types.Arg(String, graphql_name='link_text__ic', default=None)),
        ('link_text__nic', sgqlc.types.Arg(String, graphql_name='link_text__nic', default=None)),
        ('link_text__iew', sgqlc.types.Arg(String, graphql_name='link_text__iew', default=None)),
        ('link_text__niew', sgqlc.types.Arg(String, graphql_name='link_text__niew', default=None)),
        ('link_text__isw', sgqlc.types.Arg(String, graphql_name='link_text__isw', default=None)),
        ('link_text__nisw', sgqlc.types.Arg(String, graphql_name='link_text__nisw', default=None)),
        ('link_text__ie', sgqlc.types.Arg(String, graphql_name='link_text__ie', default=None)),
        ('link_text__nie', sgqlc.types.Arg(String, graphql_name='link_text__nie', default=None)),
        ('link_url__n', sgqlc.types.Arg(String, graphql_name='link_url__n', default=None)),
        ('link_url__ic', sgqlc.types.Arg(String, graphql_name='link_url__ic', default=None)),
        ('link_url__nic', sgqlc.types.Arg(String, graphql_name='link_url__nic', default=None)),
        ('link_url__iew', sgqlc.types.Arg(String, graphql_name='link_url__iew', default=None)),
        ('link_url__niew', sgqlc.types.Arg(String, graphql_name='link_url__niew', default=None)),
        ('link_url__isw', sgqlc.types.Arg(String, graphql_name='link_url__isw', default=None)),
        ('link_url__nisw', sgqlc.types.Arg(String, graphql_name='link_url__nisw', default=None)),
        ('link_url__ie', sgqlc.types.Arg(String, graphql_name='link_url__ie', default=None)),
        ('link_url__nie', sgqlc.types.Arg(String, graphql_name='link_url__nie', default=None)),
        ('weight__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__n', default=None)),
        ('weight__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__lte', default=None)),
        ('weight__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__lt', default=None)),
        ('weight__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__gte', default=None)),
        ('weight__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__gt', default=None)),
        ('group_name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_name__n', default=None)),
        ('group_name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_name__ic', default=None)),
        ('group_name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_name__nic', default=None)),
        ('group_name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_name__iew', default=None)),
        ('group_name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_name__niew', default=None)),
        ('group_name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_name__isw', default=None)),
        ('group_name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_name__nisw', default=None)),
        ('group_name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_name__ie', default=None)),
        ('group_name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_name__nie', default=None)),
        ('group_name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_name__empty', default=None)),
        ('content_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id__n', default=None)),
        ('content_type_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id__lte', default=None)),
        ('content_type_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id__lt', default=None)),
        ('content_type_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id__gte', default=None)),
        ('content_type_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id__gt', default=None)),
))
    )
    export_template = sgqlc.types.Field(ExportTemplateType, graphql_name='export_template', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    export_template_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(ExportTemplateType)), graphql_name='export_template_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('content_types', sgqlc.types.Arg(String, graphql_name='content_types', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('content_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('content_types__n', sgqlc.types.Arg(String, graphql_name='content_types__n', default=None)),
        ('content_types__ic', sgqlc.types.Arg(String, graphql_name='content_types__ic', default=None)),
        ('content_types__nic', sgqlc.types.Arg(String, graphql_name='content_types__nic', default=None)),
        ('content_types__iew', sgqlc.types.Arg(String, graphql_name='content_types__iew', default=None)),
        ('content_types__niew', sgqlc.types.Arg(String, graphql_name='content_types__niew', default=None)),
        ('content_types__isw', sgqlc.types.Arg(String, graphql_name='content_types__isw', default=None)),
        ('content_types__nisw', sgqlc.types.Arg(String, graphql_name='content_types__nisw', default=None)),
        ('content_types__ie', sgqlc.types.Arg(String, graphql_name='content_types__ie', default=None)),
        ('content_types__nie', sgqlc.types.Arg(String, graphql_name='content_types__nie', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('content_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id__n', default=None)),
        ('content_type_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id__lte', default=None)),
        ('content_type_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id__lt', default=None)),
        ('content_type_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id__gte', default=None)),
        ('content_type_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id__gt', default=None)),
))
    )
    image_attachment = sgqlc.types.Field(ImageAttachmentType, graphql_name='image_attachment', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    image_attachment_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(ImageAttachmentType)), graphql_name='image_attachment_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('content_type_id', sgqlc.types.Arg(String, graphql_name='content_type_id', default=None)),
        ('object_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='object_id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('created', sgqlc.types.Arg(DateTime, graphql_name='created', default=None)),
        ('content_type', sgqlc.types.Arg(String, graphql_name='content_type', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('content_type_id__n', sgqlc.types.Arg(String, graphql_name='content_type_id__n', default=None)),
        ('object_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='object_id__n', default=None)),
        ('object_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='object_id__lte', default=None)),
        ('object_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='object_id__lt', default=None)),
        ('object_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='object_id__gte', default=None)),
        ('object_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='object_id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('content_type__n', sgqlc.types.Arg(String, graphql_name='content_type__n', default=None)),
))
    )
    saved_filter = sgqlc.types.Field('SavedFilterType', graphql_name='saved_filter', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    saved_filter_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('SavedFilterType')), graphql_name='saved_filter_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('content_types', sgqlc.types.Arg(String, graphql_name='content_types', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('enabled', sgqlc.types.Arg(Boolean, graphql_name='enabled', default=None)),
        ('shared', sgqlc.types.Arg(Boolean, graphql_name='shared', default=None)),
        ('weight', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('content_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id', default=None)),
        ('user_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='user_id', default=None)),
        ('user', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='user', default=None)),
        ('usable', sgqlc.types.Arg(Boolean, graphql_name='usable', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('content_types__n', sgqlc.types.Arg(String, graphql_name='content_types__n', default=None)),
        ('content_types__ic', sgqlc.types.Arg(String, graphql_name='content_types__ic', default=None)),
        ('content_types__nic', sgqlc.types.Arg(String, graphql_name='content_types__nic', default=None)),
        ('content_types__iew', sgqlc.types.Arg(String, graphql_name='content_types__iew', default=None)),
        ('content_types__niew', sgqlc.types.Arg(String, graphql_name='content_types__niew', default=None)),
        ('content_types__isw', sgqlc.types.Arg(String, graphql_name='content_types__isw', default=None)),
        ('content_types__nisw', sgqlc.types.Arg(String, graphql_name='content_types__nisw', default=None)),
        ('content_types__ie', sgqlc.types.Arg(String, graphql_name='content_types__ie', default=None)),
        ('content_types__nie', sgqlc.types.Arg(String, graphql_name='content_types__nie', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('weight__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__n', default=None)),
        ('weight__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__lte', default=None)),
        ('weight__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__lt', default=None)),
        ('weight__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__gte', default=None)),
        ('weight__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__gt', default=None)),
        ('content_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id__n', default=None)),
        ('content_type_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id__lte', default=None)),
        ('content_type_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id__lt', default=None)),
        ('content_type_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id__gte', default=None)),
        ('content_type_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id__gt', default=None)),
        ('user_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='user_id__n', default=None)),
        ('user__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='user__n', default=None)),
))
    )
    journal_entry = sgqlc.types.Field(JournalEntryType, graphql_name='journal_entry', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    journal_entry_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(JournalEntryType)), graphql_name='journal_entry_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('assigned_object_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='assigned_object_type_id', default=None)),
        ('assigned_object_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='assigned_object_id', default=None)),
        ('created', sgqlc.types.Arg(String, graphql_name='created', default=None)),
        ('kind', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='kind', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('assigned_object_type', sgqlc.types.Arg(String, graphql_name='assigned_object_type', default=None)),
        ('created_by_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created_by_id', default=None)),
        ('created_by', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created_by', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('assigned_object_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='assigned_object_type_id__n', default=None)),
        ('assigned_object_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='assigned_object_id__n', default=None)),
        ('assigned_object_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='assigned_object_id__lte', default=None)),
        ('assigned_object_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='assigned_object_id__lt', default=None)),
        ('assigned_object_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='assigned_object_id__gte', default=None)),
        ('assigned_object_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='assigned_object_id__gt', default=None)),
        ('kind__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='kind__n', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('assigned_object_type__n', sgqlc.types.Arg(String, graphql_name='assigned_object_type__n', default=None)),
        ('created_by_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created_by_id__n', default=None)),
        ('created_by__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created_by__n', default=None)),
))
    )
    tag = sgqlc.types.Field('TagType', graphql_name='tag', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    tag_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('TagType')), graphql_name='tag_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('color', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('content_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type', default=None)),
        ('content_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('color__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__n', default=None)),
        ('color__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ic', default=None)),
        ('color__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nic', default=None)),
        ('color__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__iew', default=None)),
        ('color__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__niew', default=None)),
        ('color__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__isw', default=None)),
        ('color__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nisw', default=None)),
        ('color__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ie', default=None)),
        ('color__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nie', default=None)),
        ('color__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
))
    )
    webhook = sgqlc.types.Field('WebhookType', graphql_name='webhook', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    webhook_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('WebhookType')), graphql_name='webhook_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('type_create', sgqlc.types.Arg(Boolean, graphql_name='type_create', default=None)),
        ('type_update', sgqlc.types.Arg(Boolean, graphql_name='type_update', default=None)),
        ('type_delete', sgqlc.types.Arg(Boolean, graphql_name='type_delete', default=None)),
        ('payload_url', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='payload_url', default=None)),
        ('enabled', sgqlc.types.Arg(Boolean, graphql_name='enabled', default=None)),
        ('http_method', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='http_method', default=None)),
        ('http_content_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='http_content_type', default=None)),
        ('secret', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='secret', default=None)),
        ('ssl_verification', sgqlc.types.Arg(Boolean, graphql_name='ssl_verification', default=None)),
        ('ca_file_path', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ca_file_path', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('content_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id', default=None)),
        ('content_types', sgqlc.types.Arg(String, graphql_name='content_types', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('payload_url__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='payload_url__n', default=None)),
        ('payload_url__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='payload_url__ic', default=None)),
        ('payload_url__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='payload_url__nic', default=None)),
        ('payload_url__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='payload_url__iew', default=None)),
        ('payload_url__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='payload_url__niew', default=None)),
        ('payload_url__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='payload_url__isw', default=None)),
        ('payload_url__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='payload_url__nisw', default=None)),
        ('payload_url__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='payload_url__ie', default=None)),
        ('payload_url__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='payload_url__nie', default=None)),
        ('payload_url__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='payload_url__empty', default=None)),
        ('http_method__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='http_method__n', default=None)),
        ('http_content_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='http_content_type__n', default=None)),
        ('http_content_type__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='http_content_type__ic', default=None)),
        ('http_content_type__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='http_content_type__nic', default=None)),
        ('http_content_type__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='http_content_type__iew', default=None)),
        ('http_content_type__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='http_content_type__niew', default=None)),
        ('http_content_type__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='http_content_type__isw', default=None)),
        ('http_content_type__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='http_content_type__nisw', default=None)),
        ('http_content_type__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='http_content_type__ie', default=None)),
        ('http_content_type__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='http_content_type__nie', default=None)),
        ('http_content_type__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='http_content_type__empty', default=None)),
        ('secret__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='secret__n', default=None)),
        ('secret__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='secret__ic', default=None)),
        ('secret__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='secret__nic', default=None)),
        ('secret__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='secret__iew', default=None)),
        ('secret__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='secret__niew', default=None)),
        ('secret__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='secret__isw', default=None)),
        ('secret__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='secret__nisw', default=None)),
        ('secret__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='secret__ie', default=None)),
        ('secret__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='secret__nie', default=None)),
        ('secret__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='secret__empty', default=None)),
        ('ca_file_path__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ca_file_path__n', default=None)),
        ('ca_file_path__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ca_file_path__ic', default=None)),
        ('ca_file_path__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ca_file_path__nic', default=None)),
        ('ca_file_path__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ca_file_path__iew', default=None)),
        ('ca_file_path__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ca_file_path__niew', default=None)),
        ('ca_file_path__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ca_file_path__isw', default=None)),
        ('ca_file_path__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ca_file_path__nisw', default=None)),
        ('ca_file_path__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ca_file_path__ie', default=None)),
        ('ca_file_path__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ca_file_path__nie', default=None)),
        ('ca_file_path__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ca_file_path__empty', default=None)),
        ('content_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id__n', default=None)),
        ('content_type_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id__lte', default=None)),
        ('content_type_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id__lt', default=None)),
        ('content_type_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id__gte', default=None)),
        ('content_type_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='content_type_id__gt', default=None)),
        ('content_types__n', sgqlc.types.Arg(String, graphql_name='content_types__n', default=None)),
        ('content_types__ic', sgqlc.types.Arg(String, graphql_name='content_types__ic', default=None)),
        ('content_types__nic', sgqlc.types.Arg(String, graphql_name='content_types__nic', default=None)),
        ('content_types__iew', sgqlc.types.Arg(String, graphql_name='content_types__iew', default=None)),
        ('content_types__niew', sgqlc.types.Arg(String, graphql_name='content_types__niew', default=None)),
        ('content_types__isw', sgqlc.types.Arg(String, graphql_name='content_types__isw', default=None)),
        ('content_types__nisw', sgqlc.types.Arg(String, graphql_name='content_types__nisw', default=None)),
        ('content_types__ie', sgqlc.types.Arg(String, graphql_name='content_types__ie', default=None)),
        ('content_types__nie', sgqlc.types.Arg(String, graphql_name='content_types__nie', default=None)),
))
    )
    cable = sgqlc.types.Field(CableType, graphql_name='cable', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    cable_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(CableType)), graphql_name='cable_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('length', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='length', default=None)),
        ('length_unit', sgqlc.types.Arg(String, graphql_name='length_unit', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('termination_a_type', sgqlc.types.Arg(String, graphql_name='termination_a_type', default=None)),
        ('termination_a_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_a_id', default=None)),
        ('termination_b_type', sgqlc.types.Arg(String, graphql_name='termination_b_type', default=None)),
        ('termination_b_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_b_id', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('color', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('length__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='length__n', default=None)),
        ('length__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='length__lte', default=None)),
        ('length__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='length__lt', default=None)),
        ('length__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='length__gte', default=None)),
        ('length__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='length__gt', default=None)),
        ('length_unit__n', sgqlc.types.Arg(String, graphql_name='length_unit__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('termination_a_type__n', sgqlc.types.Arg(String, graphql_name='termination_a_type__n', default=None)),
        ('termination_a_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_a_id__n', default=None)),
        ('termination_a_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_a_id__lte', default=None)),
        ('termination_a_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_a_id__lt', default=None)),
        ('termination_a_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_a_id__gte', default=None)),
        ('termination_a_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_a_id__gt', default=None)),
        ('termination_b_type__n', sgqlc.types.Arg(String, graphql_name='termination_b_type__n', default=None)),
        ('termination_b_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_b_id__n', default=None)),
        ('termination_b_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_b_id__lte', default=None)),
        ('termination_b_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_b_id__lt', default=None)),
        ('termination_b_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_b_id__gte', default=None)),
        ('termination_b_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_b_id__gt', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('color__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__n', default=None)),
))
    )
    console_port = sgqlc.types.Field(ConsolePortType, graphql_name='console_port', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    console_port_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(ConsolePortType)), graphql_name='console_port_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
))
    )
    console_port_template = sgqlc.types.Field(ConsolePortTemplateType, graphql_name='console_port_template', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    console_port_template_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(ConsolePortTemplateType)), graphql_name='console_port_template_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('type', sgqlc.types.Arg(String, graphql_name='type', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('moduletype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('type__n', sgqlc.types.Arg(String, graphql_name='type__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('moduletype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id__n', default=None)),
))
    )
    console_server_port = sgqlc.types.Field(ConsoleServerPortType, graphql_name='console_server_port', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    console_server_port_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(ConsoleServerPortType)), graphql_name='console_server_port_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
))
    )
    console_server_port_template = sgqlc.types.Field(ConsoleServerPortTemplateType, graphql_name='console_server_port_template', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    console_server_port_template_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(ConsoleServerPortTemplateType)), graphql_name='console_server_port_template_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('type', sgqlc.types.Arg(String, graphql_name='type', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('moduletype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('type__n', sgqlc.types.Arg(String, graphql_name='type__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('moduletype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id__n', default=None)),
))
    )
    device = sgqlc.types.Field(DeviceType, graphql_name='device', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    device_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(DeviceType)), graphql_name='device_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('asset_tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag', default=None)),
        ('face', sgqlc.types.Arg(String, graphql_name='face', default=None)),
        ('position', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position', default=None)),
        ('airflow', sgqlc.types.Arg(String, graphql_name='airflow', default=None)),
        ('vc_position', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position', default=None)),
        ('vc_priority', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('local_context_data', sgqlc.types.Arg(Boolean, graphql_name='local_context_data', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('device_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type', default=None)),
        ('device_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type_id', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('parent_device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_device_id', default=None)),
        ('platform_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id', default=None)),
        ('platform', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('cluster_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id', default=None)),
        ('model', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('is_full_depth', sgqlc.types.Arg(Boolean, graphql_name='is_full_depth', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('serial', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial', default=None)),
        ('has_primary_ip', sgqlc.types.Arg(Boolean, graphql_name='has_primary_ip', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis_member', sgqlc.types.Arg(Boolean, graphql_name='virtual_chassis_member', default=None)),
        ('console_ports', sgqlc.types.Arg(Boolean, graphql_name='console_ports', default=None)),
        ('console_server_ports', sgqlc.types.Arg(Boolean, graphql_name='console_server_ports', default=None)),
        ('power_ports', sgqlc.types.Arg(Boolean, graphql_name='power_ports', default=None)),
        ('power_outlets', sgqlc.types.Arg(Boolean, graphql_name='power_outlets', default=None)),
        ('interfaces', sgqlc.types.Arg(Boolean, graphql_name='interfaces', default=None)),
        ('pass_through_ports', sgqlc.types.Arg(Boolean, graphql_name='pass_through_ports', default=None)),
        ('module_bays', sgqlc.types.Arg(Boolean, graphql_name='module_bays', default=None)),
        ('device_bays', sgqlc.types.Arg(Boolean, graphql_name='device_bays', default=None)),
        ('primary_ip4_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip4_id', default=None)),
        ('primary_ip6_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip6_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('asset_tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__n', default=None)),
        ('asset_tag__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ic', default=None)),
        ('asset_tag__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nic', default=None)),
        ('asset_tag__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__iew', default=None)),
        ('asset_tag__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__niew', default=None)),
        ('asset_tag__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__isw', default=None)),
        ('asset_tag__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nisw', default=None)),
        ('asset_tag__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ie', default=None)),
        ('asset_tag__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nie', default=None)),
        ('asset_tag__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__empty', default=None)),
        ('face__n', sgqlc.types.Arg(String, graphql_name='face__n', default=None)),
        ('position__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__n', default=None)),
        ('position__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__lte', default=None)),
        ('position__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__lt', default=None)),
        ('position__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__gte', default=None)),
        ('position__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__gt', default=None)),
        ('airflow__n', sgqlc.types.Arg(String, graphql_name='airflow__n', default=None)),
        ('vc_position__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__n', default=None)),
        ('vc_position__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__lte', default=None)),
        ('vc_position__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__lt', default=None)),
        ('vc_position__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__gte', default=None)),
        ('vc_position__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__gt', default=None)),
        ('vc_priority__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__n', default=None)),
        ('vc_priority__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__lte', default=None)),
        ('vc_priority__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__lt', default=None)),
        ('vc_priority__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__gte', default=None)),
        ('vc_priority__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
        ('device_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type__n', default=None)),
        ('device_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type_id__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('parent_device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_device_id__n', default=None)),
        ('platform_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id__n', default=None)),
        ('platform__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('cluster_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id__n', default=None)),
        ('model__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__n', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('serial__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__n', default=None)),
        ('serial__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ic', default=None)),
        ('serial__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nic', default=None)),
        ('serial__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__iew', default=None)),
        ('serial__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__niew', default=None)),
        ('serial__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__isw', default=None)),
        ('serial__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nisw', default=None)),
        ('serial__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ie', default=None)),
        ('serial__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nie', default=None)),
        ('serial__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__empty', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('primary_ip4_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip4_id__n', default=None)),
        ('primary_ip6_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip6_id__n', default=None)),
))
    )
    device_bay = sgqlc.types.Field(DeviceBayType, graphql_name='device_bay', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    device_bay_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(DeviceBayType)), graphql_name='device_bay_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
))
    )
    device_bay_template = sgqlc.types.Field(DeviceBayTemplateType, graphql_name='device_bay_template', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    device_bay_template_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(DeviceBayTemplateType)), graphql_name='device_bay_template_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
))
    )
    device_role = sgqlc.types.Field(DeviceRoleType, graphql_name='device_role', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    device_role_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(DeviceRoleType)), graphql_name='device_role_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('color', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color', default=None)),
        ('vm_role', sgqlc.types.Arg(Boolean, graphql_name='vm_role', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('color__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__n', default=None)),
        ('color__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ic', default=None)),
        ('color__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nic', default=None)),
        ('color__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__iew', default=None)),
        ('color__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__niew', default=None)),
        ('color__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__isw', default=None)),
        ('color__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nisw', default=None)),
        ('color__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ie', default=None)),
        ('color__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nie', default=None)),
        ('color__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
))
    )
    device_type = sgqlc.types.Field(DeviceTypeType, graphql_name='device_type', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    device_type_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(DeviceTypeType)), graphql_name='device_type_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('model', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('part_number', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number', default=None)),
        ('u_height', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height', default=None)),
        ('is_full_depth', sgqlc.types.Arg(Boolean, graphql_name='is_full_depth', default=None)),
        ('subdevice_role', sgqlc.types.Arg(String, graphql_name='subdevice_role', default=None)),
        ('airflow', sgqlc.types.Arg(String, graphql_name='airflow', default=None)),
        ('weight', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight', default=None)),
        ('weight_unit', sgqlc.types.Arg(String, graphql_name='weight_unit', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('has_front_image', sgqlc.types.Arg(Boolean, graphql_name='has_front_image', default=None)),
        ('has_rear_image', sgqlc.types.Arg(Boolean, graphql_name='has_rear_image', default=None)),
        ('console_ports', sgqlc.types.Arg(Boolean, graphql_name='console_ports', default=None)),
        ('console_server_ports', sgqlc.types.Arg(Boolean, graphql_name='console_server_ports', default=None)),
        ('power_ports', sgqlc.types.Arg(Boolean, graphql_name='power_ports', default=None)),
        ('power_outlets', sgqlc.types.Arg(Boolean, graphql_name='power_outlets', default=None)),
        ('interfaces', sgqlc.types.Arg(Boolean, graphql_name='interfaces', default=None)),
        ('pass_through_ports', sgqlc.types.Arg(Boolean, graphql_name='pass_through_ports', default=None)),
        ('module_bays', sgqlc.types.Arg(Boolean, graphql_name='module_bays', default=None)),
        ('device_bays', sgqlc.types.Arg(Boolean, graphql_name='device_bays', default=None)),
        ('inventory_items', sgqlc.types.Arg(Boolean, graphql_name='inventory_items', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('model__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__n', default=None)),
        ('model__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__ic', default=None)),
        ('model__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__nic', default=None)),
        ('model__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__iew', default=None)),
        ('model__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__niew', default=None)),
        ('model__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__isw', default=None)),
        ('model__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__nisw', default=None)),
        ('model__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__ie', default=None)),
        ('model__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__nie', default=None)),
        ('model__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('part_number__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__n', default=None)),
        ('part_number__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__ic', default=None)),
        ('part_number__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__nic', default=None)),
        ('part_number__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__iew', default=None)),
        ('part_number__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__niew', default=None)),
        ('part_number__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__isw', default=None)),
        ('part_number__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__nisw', default=None)),
        ('part_number__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__ie', default=None)),
        ('part_number__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__nie', default=None)),
        ('part_number__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__empty', default=None)),
        ('u_height__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__n', default=None)),
        ('u_height__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__lte', default=None)),
        ('u_height__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__lt', default=None)),
        ('u_height__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__gte', default=None)),
        ('u_height__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__gt', default=None)),
        ('subdevice_role__n', sgqlc.types.Arg(String, graphql_name='subdevice_role__n', default=None)),
        ('airflow__n', sgqlc.types.Arg(String, graphql_name='airflow__n', default=None)),
        ('weight__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__n', default=None)),
        ('weight__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__lte', default=None)),
        ('weight__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__lt', default=None)),
        ('weight__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__gte', default=None)),
        ('weight__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__gt', default=None)),
        ('weight_unit__n', sgqlc.types.Arg(String, graphql_name='weight_unit__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
))
    )
    front_port = sgqlc.types.Field(FrontPortType, graphql_name='front_port', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    front_port_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(FrontPortType)), graphql_name='front_port_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('color', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('color__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__n', default=None)),
        ('color__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ic', default=None)),
        ('color__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nic', default=None)),
        ('color__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__iew', default=None)),
        ('color__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__niew', default=None)),
        ('color__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__isw', default=None)),
        ('color__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nisw', default=None)),
        ('color__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ie', default=None)),
        ('color__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nie', default=None)),
        ('color__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
))
    )
    front_port_template = sgqlc.types.Field(FrontPortTemplateType, graphql_name='front_port_template', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    front_port_template_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(FrontPortTemplateType)), graphql_name='front_port_template_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('color', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('moduletype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('color__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__n', default=None)),
        ('color__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ic', default=None)),
        ('color__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nic', default=None)),
        ('color__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__iew', default=None)),
        ('color__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__niew', default=None)),
        ('color__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__isw', default=None)),
        ('color__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nisw', default=None)),
        ('color__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ie', default=None)),
        ('color__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nie', default=None)),
        ('color__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('moduletype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id__n', default=None)),
))
    )
    interface = sgqlc.types.Field(InterfaceType, graphql_name='interface', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    interface_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(InterfaceType)), graphql_name='interface_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('enabled', sgqlc.types.Arg(Boolean, graphql_name='enabled', default=None)),
        ('mtu', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu', default=None)),
        ('mgmt_only', sgqlc.types.Arg(Boolean, graphql_name='mgmt_only', default=None)),
        ('poe_mode', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode', default=None)),
        ('poe_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type', default=None)),
        ('mode', sgqlc.types.Arg(String, graphql_name='mode', default=None)),
        ('rf_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_role', default=None)),
        ('rf_channel', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel', default=None)),
        ('rf_channel_frequency', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency', default=None)),
        ('rf_channel_width', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width', default=None)),
        ('tx_power', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('kind', sgqlc.types.Arg(String, graphql_name='kind', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('bridge_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id', default=None)),
        ('lag_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='lag_id', default=None)),
        ('speed', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed', default=None)),
        ('duplex', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='duplex', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('wwn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn', default=None)),
        ('vlan_id', sgqlc.types.Arg(String, graphql_name='vlan_id', default=None)),
        ('vlan', sgqlc.types.Arg(String, graphql_name='vlan', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('vdc_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_id', default=None)),
        ('vdc_identifier', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_identifier', default=None)),
        ('vdc', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc', default=None)),
        ('l2vpn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id', default=None)),
        ('l2vpn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('mtu__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__n', default=None)),
        ('mtu__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lte', default=None)),
        ('mtu__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lt', default=None)),
        ('mtu__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gte', default=None)),
        ('mtu__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gt', default=None)),
        ('poe_mode__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode__n', default=None)),
        ('poe_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type__n', default=None)),
        ('mode__n', sgqlc.types.Arg(String, graphql_name='mode__n', default=None)),
        ('rf_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_role__n', default=None)),
        ('rf_channel__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel__n', default=None)),
        ('rf_channel_frequency__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__n', default=None)),
        ('rf_channel_frequency__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__lte', default=None)),
        ('rf_channel_frequency__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__lt', default=None)),
        ('rf_channel_frequency__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__gte', default=None)),
        ('rf_channel_frequency__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__gt', default=None)),
        ('rf_channel_width__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__n', default=None)),
        ('rf_channel_width__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__lte', default=None)),
        ('rf_channel_width__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__lt', default=None)),
        ('rf_channel_width__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__gte', default=None)),
        ('rf_channel_width__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__gt', default=None)),
        ('tx_power__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__n', default=None)),
        ('tx_power__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__lte', default=None)),
        ('tx_power__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__lt', default=None)),
        ('tx_power__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__gte', default=None)),
        ('tx_power__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('bridge_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id__n', default=None)),
        ('lag_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='lag_id__n', default=None)),
        ('speed__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__n', default=None)),
        ('speed__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__lte', default=None)),
        ('speed__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__lt', default=None)),
        ('speed__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__gte', default=None)),
        ('speed__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__gt', default=None)),
        ('duplex__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='duplex__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('wwn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__n', default=None)),
        ('wwn__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__ic', default=None)),
        ('wwn__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nic', default=None)),
        ('wwn__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__iew', default=None)),
        ('wwn__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__niew', default=None)),
        ('wwn__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__isw', default=None)),
        ('wwn__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nisw', default=None)),
        ('wwn__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__ie', default=None)),
        ('wwn__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nie', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('vdc_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_id__n', default=None)),
        ('vdc_identifier__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_identifier__n', default=None)),
        ('vdc__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc__n', default=None)),
        ('l2vpn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id__n', default=None)),
        ('l2vpn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn__n', default=None)),
))
    )
    interface_template = sgqlc.types.Field(InterfaceTemplateType, graphql_name='interface_template', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    interface_template_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(InterfaceTemplateType)), graphql_name='interface_template_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('mgmt_only', sgqlc.types.Arg(Boolean, graphql_name='mgmt_only', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('moduletype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id', default=None)),
        ('poe_mode', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode', default=None)),
        ('poe_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('moduletype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id__n', default=None)),
        ('poe_mode__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode__n', default=None)),
        ('poe_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type__n', default=None)),
))
    )
    inventory_item = sgqlc.types.Field(InventoryItemType, graphql_name='inventory_item', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    inventory_item_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(InventoryItemType)), graphql_name='inventory_item_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('part_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id', default=None)),
        ('asset_tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag', default=None)),
        ('discovered', sgqlc.types.Arg(Boolean, graphql_name='discovered', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('component_type', sgqlc.types.Arg(String, graphql_name='component_type', default=None)),
        ('component_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id', default=None)),
        ('serial', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('part_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__n', default=None)),
        ('part_id__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__ic', default=None)),
        ('part_id__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nic', default=None)),
        ('part_id__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__iew', default=None)),
        ('part_id__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__niew', default=None)),
        ('part_id__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__isw', default=None)),
        ('part_id__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nisw', default=None)),
        ('part_id__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__ie', default=None)),
        ('part_id__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nie', default=None)),
        ('part_id__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__empty', default=None)),
        ('asset_tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__n', default=None)),
        ('asset_tag__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ic', default=None)),
        ('asset_tag__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nic', default=None)),
        ('asset_tag__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__iew', default=None)),
        ('asset_tag__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__niew', default=None)),
        ('asset_tag__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__isw', default=None)),
        ('asset_tag__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nisw', default=None)),
        ('asset_tag__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ie', default=None)),
        ('asset_tag__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nie', default=None)),
        ('asset_tag__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__empty', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('component_type__n', sgqlc.types.Arg(String, graphql_name='component_type__n', default=None)),
        ('component_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__n', default=None)),
        ('component_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__lte', default=None)),
        ('component_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__lt', default=None)),
        ('component_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__gte', default=None)),
        ('component_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__gt', default=None)),
        ('serial__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__n', default=None)),
        ('serial__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ic', default=None)),
        ('serial__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nic', default=None)),
        ('serial__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__iew', default=None)),
        ('serial__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__niew', default=None)),
        ('serial__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__isw', default=None)),
        ('serial__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nisw', default=None)),
        ('serial__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ie', default=None)),
        ('serial__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nie', default=None)),
        ('serial__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__empty', default=None)),
))
    )
    inventory_item_role = sgqlc.types.Field(InventoryItemRoleType, graphql_name='inventory_item_role', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    inventory_item_role_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(InventoryItemRoleType)), graphql_name='inventory_item_role_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('color', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('color__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__n', default=None)),
        ('color__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ic', default=None)),
        ('color__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nic', default=None)),
        ('color__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__iew', default=None)),
        ('color__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__niew', default=None)),
        ('color__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__isw', default=None)),
        ('color__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nisw', default=None)),
        ('color__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ie', default=None)),
        ('color__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nie', default=None)),
        ('color__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
))
    )
    inventory_item_template = sgqlc.types.Field(InventoryItemTemplateType, graphql_name='inventory_item_template', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    inventory_item_template_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(InventoryItemTemplateType)), graphql_name='inventory_item_template_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('part_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('component_type', sgqlc.types.Arg(String, graphql_name='component_type', default=None)),
        ('component_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('part_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__n', default=None)),
        ('part_id__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__ic', default=None)),
        ('part_id__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nic', default=None)),
        ('part_id__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__iew', default=None)),
        ('part_id__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__niew', default=None)),
        ('part_id__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__isw', default=None)),
        ('part_id__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nisw', default=None)),
        ('part_id__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__ie', default=None)),
        ('part_id__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__nie', default=None)),
        ('part_id__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_id__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('component_type__n', sgqlc.types.Arg(String, graphql_name='component_type__n', default=None)),
        ('component_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__n', default=None)),
        ('component_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__lte', default=None)),
        ('component_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__lt', default=None)),
        ('component_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__gte', default=None)),
        ('component_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='component_id__gt', default=None)),
))
    )
    location = sgqlc.types.Field(LocationType, graphql_name='location', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    location_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(LocationType)), graphql_name='location_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('parent', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('parent__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent__n', default=None)),
))
    )
    manufacturer = sgqlc.types.Field(ManufacturerType, graphql_name='manufacturer', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    manufacturer_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(ManufacturerType)), graphql_name='manufacturer_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
))
    )
    module = sgqlc.types.Field(ModuleType, graphql_name='module', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    module_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(ModuleType)), graphql_name='module_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('asset_tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('module_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_type_id', default=None)),
        ('module_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_type', default=None)),
        ('module_bay_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_bay_id', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('serial', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('asset_tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__n', default=None)),
        ('asset_tag__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ic', default=None)),
        ('asset_tag__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nic', default=None)),
        ('asset_tag__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__iew', default=None)),
        ('asset_tag__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__niew', default=None)),
        ('asset_tag__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__isw', default=None)),
        ('asset_tag__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nisw', default=None)),
        ('asset_tag__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ie', default=None)),
        ('asset_tag__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nie', default=None)),
        ('asset_tag__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
        ('module_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_type_id__n', default=None)),
        ('module_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_type__n', default=None)),
        ('module_bay_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_bay_id__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('serial__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__n', default=None)),
        ('serial__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ic', default=None)),
        ('serial__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nic', default=None)),
        ('serial__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__iew', default=None)),
        ('serial__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__niew', default=None)),
        ('serial__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__isw', default=None)),
        ('serial__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nisw', default=None)),
        ('serial__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ie', default=None)),
        ('serial__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nie', default=None)),
        ('serial__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__empty', default=None)),
))
    )
    module_bay = sgqlc.types.Field(ModuleBayType, graphql_name='module_bay', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    module_bay_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(ModuleBayType)), graphql_name='module_bay_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
))
    )
    module_bay_template = sgqlc.types.Field(ModuleBayTemplateType, graphql_name='module_bay_template', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    module_bay_template_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(ModuleBayTemplateType)), graphql_name='module_bay_template_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
))
    )
    module_type = sgqlc.types.Field(ModuleTypeType, graphql_name='module_type', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    module_type_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(ModuleTypeType)), graphql_name='module_type_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('model', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model', default=None)),
        ('part_number', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number', default=None)),
        ('weight', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight', default=None)),
        ('weight_unit', sgqlc.types.Arg(String, graphql_name='weight_unit', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('console_ports', sgqlc.types.Arg(Boolean, graphql_name='console_ports', default=None)),
        ('console_server_ports', sgqlc.types.Arg(Boolean, graphql_name='console_server_ports', default=None)),
        ('power_ports', sgqlc.types.Arg(Boolean, graphql_name='power_ports', default=None)),
        ('power_outlets', sgqlc.types.Arg(Boolean, graphql_name='power_outlets', default=None)),
        ('interfaces', sgqlc.types.Arg(Boolean, graphql_name='interfaces', default=None)),
        ('pass_through_ports', sgqlc.types.Arg(Boolean, graphql_name='pass_through_ports', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('model__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__n', default=None)),
        ('model__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__ic', default=None)),
        ('model__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__nic', default=None)),
        ('model__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__iew', default=None)),
        ('model__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__niew', default=None)),
        ('model__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__isw', default=None)),
        ('model__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__nisw', default=None)),
        ('model__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__ie', default=None)),
        ('model__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__nie', default=None)),
        ('model__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__empty', default=None)),
        ('part_number__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__n', default=None)),
        ('part_number__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__ic', default=None)),
        ('part_number__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__nic', default=None)),
        ('part_number__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__iew', default=None)),
        ('part_number__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__niew', default=None)),
        ('part_number__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__isw', default=None)),
        ('part_number__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__nisw', default=None)),
        ('part_number__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__ie', default=None)),
        ('part_number__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__nie', default=None)),
        ('part_number__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='part_number__empty', default=None)),
        ('weight__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__n', default=None)),
        ('weight__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__lte', default=None)),
        ('weight__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__lt', default=None)),
        ('weight__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__gte', default=None)),
        ('weight__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__gt', default=None)),
        ('weight_unit__n', sgqlc.types.Arg(String, graphql_name='weight_unit__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
))
    )
    platform = sgqlc.types.Field(PlatformType, graphql_name='platform', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    platform_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(PlatformType)), graphql_name='platform_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('napalm_driver', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('napalm_driver__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__n', default=None)),
        ('napalm_driver__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__ic', default=None)),
        ('napalm_driver__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__nic', default=None)),
        ('napalm_driver__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__iew', default=None)),
        ('napalm_driver__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__niew', default=None)),
        ('napalm_driver__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__isw', default=None)),
        ('napalm_driver__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__nisw', default=None)),
        ('napalm_driver__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__ie', default=None)),
        ('napalm_driver__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__nie', default=None)),
        ('napalm_driver__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='napalm_driver__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
))
    )
    power_feed = sgqlc.types.Field(PowerFeedType, graphql_name='power_feed', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    power_feed_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(PowerFeedType)), graphql_name='power_feed_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('type', sgqlc.types.Arg(String, graphql_name='type', default=None)),
        ('supply', sgqlc.types.Arg(String, graphql_name='supply', default=None)),
        ('phase', sgqlc.types.Arg(String, graphql_name='phase', default=None)),
        ('voltage', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='voltage', default=None)),
        ('amperage', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='amperage', default=None)),
        ('max_utilization', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_utilization', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('power_panel_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='power_panel_id', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('type__n', sgqlc.types.Arg(String, graphql_name='type__n', default=None)),
        ('supply__n', sgqlc.types.Arg(String, graphql_name='supply__n', default=None)),
        ('phase__n', sgqlc.types.Arg(String, graphql_name='phase__n', default=None)),
        ('voltage__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='voltage__n', default=None)),
        ('voltage__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='voltage__lte', default=None)),
        ('voltage__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='voltage__lt', default=None)),
        ('voltage__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='voltage__gte', default=None)),
        ('voltage__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='voltage__gt', default=None)),
        ('amperage__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='amperage__n', default=None)),
        ('amperage__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='amperage__lte', default=None)),
        ('amperage__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='amperage__lt', default=None)),
        ('amperage__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='amperage__gte', default=None)),
        ('amperage__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='amperage__gt', default=None)),
        ('max_utilization__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_utilization__n', default=None)),
        ('max_utilization__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_utilization__lte', default=None)),
        ('max_utilization__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_utilization__lt', default=None)),
        ('max_utilization__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_utilization__gte', default=None)),
        ('max_utilization__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_utilization__gt', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('power_panel_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='power_panel_id__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
))
    )
    power_outlet = sgqlc.types.Field(PowerOutletType, graphql_name='power_outlet', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    power_outlet_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(PowerOutletType)), graphql_name='power_outlet_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('feed_leg', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='feed_leg', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('feed_leg__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='feed_leg__n', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
))
    )
    power_outlet_template = sgqlc.types.Field(PowerOutletTemplateType, graphql_name='power_outlet_template', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    power_outlet_template_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(PowerOutletTemplateType)), graphql_name='power_outlet_template_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('type', sgqlc.types.Arg(String, graphql_name='type', default=None)),
        ('feed_leg', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='feed_leg', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('moduletype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('type__n', sgqlc.types.Arg(String, graphql_name='type__n', default=None)),
        ('feed_leg__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='feed_leg__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('moduletype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id__n', default=None)),
))
    )
    power_panel = sgqlc.types.Field(PowerPanelType, graphql_name='power_panel', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    power_panel_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(PowerPanelType)), graphql_name='power_panel_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
))
    )
    power_port = sgqlc.types.Field(PowerPortType, graphql_name='power_port', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    power_port_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(PowerPortType)), graphql_name='power_port_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('maximum_draw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw', default=None)),
        ('allocated_draw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('maximum_draw__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__n', default=None)),
        ('maximum_draw__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__lte', default=None)),
        ('maximum_draw__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__lt', default=None)),
        ('maximum_draw__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__gte', default=None)),
        ('maximum_draw__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__gt', default=None)),
        ('allocated_draw__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__n', default=None)),
        ('allocated_draw__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__lte', default=None)),
        ('allocated_draw__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__lt', default=None)),
        ('allocated_draw__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__gte', default=None)),
        ('allocated_draw__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
))
    )
    power_port_template = sgqlc.types.Field(PowerPortTemplateType, graphql_name='power_port_template', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    power_port_template_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(PowerPortTemplateType)), graphql_name='power_port_template_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('type', sgqlc.types.Arg(String, graphql_name='type', default=None)),
        ('maximum_draw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw', default=None)),
        ('allocated_draw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('moduletype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('type__n', sgqlc.types.Arg(String, graphql_name='type__n', default=None)),
        ('maximum_draw__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__n', default=None)),
        ('maximum_draw__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__lte', default=None)),
        ('maximum_draw__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__lt', default=None)),
        ('maximum_draw__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__gte', default=None)),
        ('maximum_draw__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='maximum_draw__gt', default=None)),
        ('allocated_draw__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__n', default=None)),
        ('allocated_draw__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__lte', default=None)),
        ('allocated_draw__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__lt', default=None)),
        ('allocated_draw__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__gte', default=None)),
        ('allocated_draw__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='allocated_draw__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('moduletype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id__n', default=None)),
))
    )
    rack = sgqlc.types.Field('RackType', graphql_name='rack', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    rack_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('RackType')), graphql_name='rack_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('facility_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id', default=None)),
        ('asset_tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag', default=None)),
        ('u_height', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height', default=None)),
        ('desc_units', sgqlc.types.Arg(Boolean, graphql_name='desc_units', default=None)),
        ('outer_width', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width', default=None)),
        ('outer_depth', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth', default=None)),
        ('outer_unit', sgqlc.types.Arg(String, graphql_name='outer_unit', default=None)),
        ('mounting_depth', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth', default=None)),
        ('weight', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight', default=None)),
        ('max_weight', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight', default=None)),
        ('weight_unit', sgqlc.types.Arg(String, graphql_name='weight_unit', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('width', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='width', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('serial', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('facility_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__n', default=None)),
        ('facility_id__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__ic', default=None)),
        ('facility_id__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__nic', default=None)),
        ('facility_id__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__iew', default=None)),
        ('facility_id__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__niew', default=None)),
        ('facility_id__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__isw', default=None)),
        ('facility_id__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__nisw', default=None)),
        ('facility_id__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__ie', default=None)),
        ('facility_id__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__nie', default=None)),
        ('facility_id__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__empty', default=None)),
        ('asset_tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__n', default=None)),
        ('asset_tag__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ic', default=None)),
        ('asset_tag__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nic', default=None)),
        ('asset_tag__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__iew', default=None)),
        ('asset_tag__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__niew', default=None)),
        ('asset_tag__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__isw', default=None)),
        ('asset_tag__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nisw', default=None)),
        ('asset_tag__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ie', default=None)),
        ('asset_tag__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nie', default=None)),
        ('asset_tag__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__empty', default=None)),
        ('u_height__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__n', default=None)),
        ('u_height__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__lte', default=None)),
        ('u_height__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__lt', default=None)),
        ('u_height__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__gte', default=None)),
        ('u_height__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__gt', default=None)),
        ('outer_width__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width__n', default=None)),
        ('outer_width__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width__lte', default=None)),
        ('outer_width__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width__lt', default=None)),
        ('outer_width__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width__gte', default=None)),
        ('outer_width__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width__gt', default=None)),
        ('outer_depth__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth__n', default=None)),
        ('outer_depth__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth__lte', default=None)),
        ('outer_depth__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth__lt', default=None)),
        ('outer_depth__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth__gte', default=None)),
        ('outer_depth__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth__gt', default=None)),
        ('outer_unit__n', sgqlc.types.Arg(String, graphql_name='outer_unit__n', default=None)),
        ('mounting_depth__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth__n', default=None)),
        ('mounting_depth__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth__lte', default=None)),
        ('mounting_depth__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth__lt', default=None)),
        ('mounting_depth__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth__gte', default=None)),
        ('mounting_depth__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth__gt', default=None)),
        ('weight__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__n', default=None)),
        ('weight__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__lte', default=None)),
        ('weight__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__lt', default=None)),
        ('weight__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__gte', default=None)),
        ('weight__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__gt', default=None)),
        ('max_weight__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight__n', default=None)),
        ('max_weight__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight__lte', default=None)),
        ('max_weight__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight__lt', default=None)),
        ('max_weight__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight__gte', default=None)),
        ('max_weight__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight__gt', default=None)),
        ('weight_unit__n', sgqlc.types.Arg(String, graphql_name='weight_unit__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('width__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='width__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('serial__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__n', default=None)),
        ('serial__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ic', default=None)),
        ('serial__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nic', default=None)),
        ('serial__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__iew', default=None)),
        ('serial__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__niew', default=None)),
        ('serial__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__isw', default=None)),
        ('serial__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nisw', default=None)),
        ('serial__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ie', default=None)),
        ('serial__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nie', default=None)),
        ('serial__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__empty', default=None)),
))
    )
    rack_reservation = sgqlc.types.Field('RackReservationType', graphql_name='rack_reservation', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    rack_reservation_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('RackReservationType')), graphql_name='rack_reservation_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('user_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='user_id', default=None)),
        ('user', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='user', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('user_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='user_id__n', default=None)),
        ('user__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='user__n', default=None)),
))
    )
    rack_role = sgqlc.types.Field('RackRoleType', graphql_name='rack_role', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    rack_role_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('RackRoleType')), graphql_name='rack_role_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('color', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('color__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__n', default=None)),
        ('color__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ic', default=None)),
        ('color__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nic', default=None)),
        ('color__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__iew', default=None)),
        ('color__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__niew', default=None)),
        ('color__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__isw', default=None)),
        ('color__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nisw', default=None)),
        ('color__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ie', default=None)),
        ('color__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nie', default=None)),
        ('color__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
))
    )
    rear_port = sgqlc.types.Field('RearPortType', graphql_name='rear_port', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    rear_port_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('RearPortType')), graphql_name='rear_port_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('color', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color', default=None)),
        ('positions', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('color__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__n', default=None)),
        ('color__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ic', default=None)),
        ('color__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nic', default=None)),
        ('color__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__iew', default=None)),
        ('color__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__niew', default=None)),
        ('color__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__isw', default=None)),
        ('color__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nisw', default=None)),
        ('color__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ie', default=None)),
        ('color__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nie', default=None)),
        ('color__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__empty', default=None)),
        ('positions__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__n', default=None)),
        ('positions__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__lte', default=None)),
        ('positions__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__lt', default=None)),
        ('positions__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__gte', default=None)),
        ('positions__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
))
    )
    rear_port_template = sgqlc.types.Field('RearPortTemplateType', graphql_name='rear_port_template', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    rear_port_template_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('RearPortTemplateType')), graphql_name='rear_port_template_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('color', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color', default=None)),
        ('positions', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('moduletype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('color__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__n', default=None)),
        ('color__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ic', default=None)),
        ('color__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nic', default=None)),
        ('color__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__iew', default=None)),
        ('color__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__niew', default=None)),
        ('color__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__isw', default=None)),
        ('color__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nisw', default=None)),
        ('color__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ie', default=None)),
        ('color__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nie', default=None)),
        ('color__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__empty', default=None)),
        ('positions__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__n', default=None)),
        ('positions__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__lte', default=None)),
        ('positions__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__lt', default=None)),
        ('positions__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__gte', default=None)),
        ('positions__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='positions__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('moduletype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id__n', default=None)),
))
    )
    region = sgqlc.types.Field('RegionType', graphql_name='region', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    region_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('RegionType')), graphql_name='region_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('parent', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('parent__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent__n', default=None)),
))
    )
    site = sgqlc.types.Field('SiteType', graphql_name='site', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    site_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('SiteType')), graphql_name='site_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('facility', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility', default=None)),
        ('latitude', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude', default=None)),
        ('longitude', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('asn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn', default=None)),
        ('asn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('facility__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__n', default=None)),
        ('facility__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__ic', default=None)),
        ('facility__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__nic', default=None)),
        ('facility__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__iew', default=None)),
        ('facility__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__niew', default=None)),
        ('facility__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__isw', default=None)),
        ('facility__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__nisw', default=None)),
        ('facility__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__ie', default=None)),
        ('facility__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__nie', default=None)),
        ('facility__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__empty', default=None)),
        ('latitude__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__n', default=None)),
        ('latitude__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__lte', default=None)),
        ('latitude__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__lt', default=None)),
        ('latitude__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__gte', default=None)),
        ('latitude__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__gt', default=None)),
        ('longitude__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__n', default=None)),
        ('longitude__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__lte', default=None)),
        ('longitude__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__lt', default=None)),
        ('longitude__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__gte', default=None)),
        ('longitude__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
        ('asn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__n', default=None)),
        ('asn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn_id__n', default=None)),
))
    )
    site_group = sgqlc.types.Field('SiteGroupType', graphql_name='site_group', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    site_group_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('SiteGroupType')), graphql_name='site_group_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('parent', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('parent__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent__n', default=None)),
))
    )
    virtual_chassis = sgqlc.types.Field('VirtualChassisType', graphql_name='virtual_chassis', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    virtual_chassis_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('VirtualChassisType')), graphql_name='virtual_chassis_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('domain', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='domain', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('master_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='master_id', default=None)),
        ('master', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='master', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('domain__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='domain__n', default=None)),
        ('domain__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='domain__ic', default=None)),
        ('domain__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='domain__nic', default=None)),
        ('domain__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='domain__iew', default=None)),
        ('domain__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='domain__niew', default=None)),
        ('domain__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='domain__isw', default=None)),
        ('domain__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='domain__nisw', default=None)),
        ('domain__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='domain__ie', default=None)),
        ('domain__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='domain__nie', default=None)),
        ('domain__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='domain__empty', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('master_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='master_id__n', default=None)),
        ('master__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='master__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
))
    )
    virtual_device_context = sgqlc.types.Field('VirtualDeviceContextType', graphql_name='virtual_device_context', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    virtual_device_context_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('VirtualDeviceContextType')), graphql_name='virtual_device_context_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('has_primary_ip', sgqlc.types.Arg(Boolean, graphql_name='has_primary_ip', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
))
    )
    circuit = sgqlc.types.Field(CircuitType, graphql_name='circuit', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    circuit_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(CircuitType)), graphql_name='circuit_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('cid', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('install_date', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='install_date', default=None)),
        ('termination_date', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_date', default=None)),
        ('commit_rate', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='commit_rate', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('provider_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_id', default=None)),
        ('provider', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider', default=None)),
        ('provider_network_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_network_id', default=None)),
        ('type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type_id', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('cid__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__n', default=None)),
        ('cid__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__ic', default=None)),
        ('cid__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__nic', default=None)),
        ('cid__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__iew', default=None)),
        ('cid__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__niew', default=None)),
        ('cid__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__isw', default=None)),
        ('cid__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__nisw', default=None)),
        ('cid__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__ie', default=None)),
        ('cid__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__nie', default=None)),
        ('cid__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('install_date__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='install_date__n', default=None)),
        ('install_date__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='install_date__lte', default=None)),
        ('install_date__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='install_date__lt', default=None)),
        ('install_date__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='install_date__gte', default=None)),
        ('install_date__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='install_date__gt', default=None)),
        ('termination_date__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_date__n', default=None)),
        ('termination_date__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_date__lte', default=None)),
        ('termination_date__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_date__lt', default=None)),
        ('termination_date__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_date__gte', default=None)),
        ('termination_date__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_date__gt', default=None)),
        ('commit_rate__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='commit_rate__n', default=None)),
        ('commit_rate__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='commit_rate__lte', default=None)),
        ('commit_rate__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='commit_rate__lt', default=None)),
        ('commit_rate__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='commit_rate__gte', default=None)),
        ('commit_rate__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='commit_rate__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('provider_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_id__n', default=None)),
        ('provider__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider__n', default=None)),
        ('provider_network_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_network_id__n', default=None)),
        ('type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type_id__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
))
    )
    circuit_termination = sgqlc.types.Field(CircuitTerminationType, graphql_name='circuit_termination', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    circuit_termination_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(CircuitTerminationType)), graphql_name='circuit_termination_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('term_side', sgqlc.types.Arg(String, graphql_name='term_side', default=None)),
        ('port_speed', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='port_speed', default=None)),
        ('upstream_speed', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='upstream_speed', default=None)),
        ('xconnect_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('circuit_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='circuit_id', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('provider_network_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_network_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('term_side__n', sgqlc.types.Arg(String, graphql_name='term_side__n', default=None)),
        ('port_speed__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='port_speed__n', default=None)),
        ('port_speed__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='port_speed__lte', default=None)),
        ('port_speed__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='port_speed__lt', default=None)),
        ('port_speed__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='port_speed__gte', default=None)),
        ('port_speed__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='port_speed__gt', default=None)),
        ('upstream_speed__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='upstream_speed__n', default=None)),
        ('upstream_speed__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='upstream_speed__lte', default=None)),
        ('upstream_speed__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='upstream_speed__lt', default=None)),
        ('upstream_speed__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='upstream_speed__gte', default=None)),
        ('upstream_speed__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='upstream_speed__gt', default=None)),
        ('xconnect_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__n', default=None)),
        ('xconnect_id__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__ic', default=None)),
        ('xconnect_id__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__nic', default=None)),
        ('xconnect_id__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__iew', default=None)),
        ('xconnect_id__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__niew', default=None)),
        ('xconnect_id__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__isw', default=None)),
        ('xconnect_id__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__nisw', default=None)),
        ('xconnect_id__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__ie', default=None)),
        ('xconnect_id__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__nie', default=None)),
        ('xconnect_id__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('circuit_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='circuit_id__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('provider_network_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_network_id__n', default=None)),
))
    )
    circuit_type = sgqlc.types.Field(CircuitTypeType, graphql_name='circuit_type', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    circuit_type_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(CircuitTypeType)), graphql_name='circuit_type_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
))
    )
    provider = sgqlc.types.Field(ProviderType, graphql_name='provider', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    provider_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(ProviderType)), graphql_name='provider_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('account', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='account', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('asn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('account__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='account__n', default=None)),
        ('account__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='account__ic', default=None)),
        ('account__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='account__nic', default=None)),
        ('account__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='account__iew', default=None)),
        ('account__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='account__niew', default=None)),
        ('account__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='account__isw', default=None)),
        ('account__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='account__nisw', default=None)),
        ('account__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='account__ie', default=None)),
        ('account__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='account__nie', default=None)),
        ('account__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='account__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('asn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn_id__n', default=None)),
))
    )
    provider_network = sgqlc.types.Field(ProviderNetworkType, graphql_name='provider_network', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    provider_network_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(ProviderNetworkType)), graphql_name='provider_network_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('service_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='service_id', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('provider_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_id', default=None)),
        ('provider', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('service_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='service_id__n', default=None)),
        ('service_id__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='service_id__ic', default=None)),
        ('service_id__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='service_id__nic', default=None)),
        ('service_id__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='service_id__iew', default=None)),
        ('service_id__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='service_id__niew', default=None)),
        ('service_id__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='service_id__isw', default=None)),
        ('service_id__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='service_id__nisw', default=None)),
        ('service_id__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='service_id__ie', default=None)),
        ('service_id__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='service_id__nie', default=None)),
        ('service_id__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='service_id__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('provider_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_id__n', default=None)),
        ('provider__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider__n', default=None)),
))
    )
    group = sgqlc.types.Field(GroupType, graphql_name='group', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    group_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(GroupType)), graphql_name='group_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
))
    )
    user = sgqlc.types.Field('UserType', graphql_name='user', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='id', default=None)),
))
    )
    user_list = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('UserType')), graphql_name='user_list', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('username', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='username', default=None)),
        ('first_name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='first_name', default=None)),
        ('last_name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_name', default=None)),
        ('email', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email', default=None)),
        ('is_staff', sgqlc.types.Arg(Boolean, graphql_name='is_staff', default=None)),
        ('is_active', sgqlc.types.Arg(Boolean, graphql_name='is_active', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('username__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='username__n', default=None)),
        ('username__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='username__ic', default=None)),
        ('username__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='username__nic', default=None)),
        ('username__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='username__iew', default=None)),
        ('username__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='username__niew', default=None)),
        ('username__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='username__isw', default=None)),
        ('username__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='username__nisw', default=None)),
        ('username__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='username__ie', default=None)),
        ('username__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='username__nie', default=None)),
        ('username__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='username__empty', default=None)),
        ('first_name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='first_name__n', default=None)),
        ('first_name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='first_name__ic', default=None)),
        ('first_name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='first_name__nic', default=None)),
        ('first_name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='first_name__iew', default=None)),
        ('first_name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='first_name__niew', default=None)),
        ('first_name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='first_name__isw', default=None)),
        ('first_name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='first_name__nisw', default=None)),
        ('first_name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='first_name__ie', default=None)),
        ('first_name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='first_name__nie', default=None)),
        ('first_name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='first_name__empty', default=None)),
        ('last_name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_name__n', default=None)),
        ('last_name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_name__ic', default=None)),
        ('last_name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_name__nic', default=None)),
        ('last_name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_name__iew', default=None)),
        ('last_name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_name__niew', default=None)),
        ('last_name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_name__isw', default=None)),
        ('last_name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_name__nisw', default=None)),
        ('last_name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_name__ie', default=None)),
        ('last_name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_name__nie', default=None)),
        ('last_name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_name__empty', default=None)),
        ('email__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__n', default=None)),
        ('email__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__ic', default=None)),
        ('email__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__nic', default=None)),
        ('email__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__iew', default=None)),
        ('email__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__niew', default=None)),
        ('email__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__isw', default=None)),
        ('email__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__nisw', default=None)),
        ('email__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__ie', default=None)),
        ('email__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__nie', default=None)),
        ('email__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='email__empty', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
))
    )


class RIRType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'name', 'slug', 'description', 'is_private', 'asns', 'aggregates', 'display', 'class_type', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    slug = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='slug')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    is_private = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='is_private')
    asns = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ASNType))), graphql_name='asns', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('asn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('rir_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir_id', default=None)),
        ('rir', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('asn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__n', default=None)),
        ('asn__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__lte', default=None)),
        ('asn__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__lt', default=None)),
        ('asn__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__gte', default=None)),
        ('asn__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('rir_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir_id__n', default=None)),
        ('rir__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
))
    )
    aggregates = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(AggregateType))), graphql_name='aggregates', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('date_added', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='date_added', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('family', sgqlc.types.Arg(Int, graphql_name='family', default=None)),
        ('prefix', sgqlc.types.Arg(String, graphql_name='prefix', default=None)),
        ('rir_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir_id', default=None)),
        ('rir', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('date_added__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='date_added__n', default=None)),
        ('date_added__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='date_added__lte', default=None)),
        ('date_added__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='date_added__lt', default=None)),
        ('date_added__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='date_added__gte', default=None)),
        ('date_added__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='date_added__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('rir_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir_id__n', default=None)),
        ('rir__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class RackReservationType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'comments', 'rack', 'units', 'tenant', 'user', 'description', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    rack = sgqlc.types.Field(sgqlc.types.non_null('RackType'), graphql_name='rack')
    units = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(Int))), graphql_name='units')
    tenant = sgqlc.types.Field('TenantType', graphql_name='tenant')
    user = sgqlc.types.Field(sgqlc.types.non_null('UserType'), graphql_name='user')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of(JournalEntryType), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class RackRoleType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'name', 'slug', 'description', 'color', 'racks', 'display', 'class_type', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    slug = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='slug')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    color = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='color')
    racks = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('RackType'))), graphql_name='racks', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('facility_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id', default=None)),
        ('asset_tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag', default=None)),
        ('u_height', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height', default=None)),
        ('desc_units', sgqlc.types.Arg(Boolean, graphql_name='desc_units', default=None)),
        ('outer_width', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width', default=None)),
        ('outer_depth', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth', default=None)),
        ('outer_unit', sgqlc.types.Arg(String, graphql_name='outer_unit', default=None)),
        ('mounting_depth', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth', default=None)),
        ('weight', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight', default=None)),
        ('max_weight', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight', default=None)),
        ('weight_unit', sgqlc.types.Arg(String, graphql_name='weight_unit', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('width', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='width', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('serial', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('facility_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__n', default=None)),
        ('facility_id__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__ic', default=None)),
        ('facility_id__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__nic', default=None)),
        ('facility_id__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__iew', default=None)),
        ('facility_id__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__niew', default=None)),
        ('facility_id__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__isw', default=None)),
        ('facility_id__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__nisw', default=None)),
        ('facility_id__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__ie', default=None)),
        ('facility_id__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__nie', default=None)),
        ('facility_id__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__empty', default=None)),
        ('asset_tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__n', default=None)),
        ('asset_tag__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ic', default=None)),
        ('asset_tag__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nic', default=None)),
        ('asset_tag__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__iew', default=None)),
        ('asset_tag__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__niew', default=None)),
        ('asset_tag__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__isw', default=None)),
        ('asset_tag__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nisw', default=None)),
        ('asset_tag__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ie', default=None)),
        ('asset_tag__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nie', default=None)),
        ('asset_tag__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__empty', default=None)),
        ('u_height__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__n', default=None)),
        ('u_height__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__lte', default=None)),
        ('u_height__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__lt', default=None)),
        ('u_height__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__gte', default=None)),
        ('u_height__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__gt', default=None)),
        ('outer_width__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width__n', default=None)),
        ('outer_width__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width__lte', default=None)),
        ('outer_width__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width__lt', default=None)),
        ('outer_width__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width__gte', default=None)),
        ('outer_width__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width__gt', default=None)),
        ('outer_depth__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth__n', default=None)),
        ('outer_depth__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth__lte', default=None)),
        ('outer_depth__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth__lt', default=None)),
        ('outer_depth__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth__gte', default=None)),
        ('outer_depth__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth__gt', default=None)),
        ('outer_unit__n', sgqlc.types.Arg(String, graphql_name='outer_unit__n', default=None)),
        ('mounting_depth__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth__n', default=None)),
        ('mounting_depth__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth__lte', default=None)),
        ('mounting_depth__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth__lt', default=None)),
        ('mounting_depth__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth__gte', default=None)),
        ('mounting_depth__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth__gt', default=None)),
        ('weight__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__n', default=None)),
        ('weight__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__lte', default=None)),
        ('weight__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__lt', default=None)),
        ('weight__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__gte', default=None)),
        ('weight__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__gt', default=None)),
        ('max_weight__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight__n', default=None)),
        ('max_weight__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight__lte', default=None)),
        ('max_weight__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight__lt', default=None)),
        ('max_weight__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight__gte', default=None)),
        ('max_weight__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight__gt', default=None)),
        ('weight_unit__n', sgqlc.types.Arg(String, graphql_name='weight_unit__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('width__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='width__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('serial__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__n', default=None)),
        ('serial__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ic', default=None)),
        ('serial__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nic', default=None)),
        ('serial__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__iew', default=None)),
        ('serial__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__niew', default=None)),
        ('serial__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__isw', default=None)),
        ('serial__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nisw', default=None)),
        ('serial__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ie', default=None)),
        ('serial__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nie', default=None)),
        ('serial__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__empty', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class RackType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'weight', 'weight_unit', '_abs_weight', 'name', '_name', 'facility_id', 'site', 'location', 'tenant', 'status', 'role', 'serial', 'asset_tag', 'type', 'width', 'u_height', 'desc_units', 'outer_width', 'outer_depth', 'outer_unit', 'max_weight', '_abs_max_weight', 'mounting_depth', 'cabletermination_set', 'devices', 'powerfeed_set', 'reservations', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog', 'contacts', 'image_attachments', 'vlan_groups')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    weight = sgqlc.types.Field(Decimal, graphql_name='weight')
    weight_unit = sgqlc.types.Field(DcimRackWeightUnitChoices, graphql_name='weight_unit')
    _abs_weight = sgqlc.types.Field(BigInt, graphql_name='_abs_weight')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    _name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='_name')
    facility_id = sgqlc.types.Field(String, graphql_name='facility_id')
    site = sgqlc.types.Field(sgqlc.types.non_null('SiteType'), graphql_name='site')
    location = sgqlc.types.Field(LocationType, graphql_name='location')
    tenant = sgqlc.types.Field('TenantType', graphql_name='tenant')
    status = sgqlc.types.Field(sgqlc.types.non_null(DcimRackStatusChoices), graphql_name='status')
    role = sgqlc.types.Field(RackRoleType, graphql_name='role')
    serial = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='serial')
    asset_tag = sgqlc.types.Field(String, graphql_name='asset_tag')
    type = sgqlc.types.Field(DcimRackTypeChoices, graphql_name='type')
    width = sgqlc.types.Field(sgqlc.types.non_null(DcimRackWidthChoices), graphql_name='width')
    u_height = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='u_height')
    desc_units = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='desc_units')
    outer_width = sgqlc.types.Field(Int, graphql_name='outer_width')
    outer_depth = sgqlc.types.Field(Int, graphql_name='outer_depth')
    outer_unit = sgqlc.types.Field(DcimRackOuterUnitChoices, graphql_name='outer_unit')
    max_weight = sgqlc.types.Field(Int, graphql_name='max_weight')
    _abs_max_weight = sgqlc.types.Field(BigInt, graphql_name='_abs_max_weight')
    mounting_depth = sgqlc.types.Field(Int, graphql_name='mounting_depth')
    cabletermination_set = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(CableTerminationType))), graphql_name='cabletermination_set', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('cable', sgqlc.types.Arg(String, graphql_name='cable', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('termination_type', sgqlc.types.Arg(String, graphql_name='termination_type', default=None)),
        ('termination_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('cable__n', sgqlc.types.Arg(String, graphql_name='cable__n', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('termination_type__n', sgqlc.types.Arg(String, graphql_name='termination_type__n', default=None)),
        ('termination_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id__n', default=None)),
        ('termination_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id__lte', default=None)),
        ('termination_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id__lt', default=None)),
        ('termination_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id__gte', default=None)),
        ('termination_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id__gt', default=None)),
))
    )
    devices = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(DeviceType))), graphql_name='devices', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('asset_tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag', default=None)),
        ('face', sgqlc.types.Arg(String, graphql_name='face', default=None)),
        ('position', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position', default=None)),
        ('airflow', sgqlc.types.Arg(String, graphql_name='airflow', default=None)),
        ('vc_position', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position', default=None)),
        ('vc_priority', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('local_context_data', sgqlc.types.Arg(Boolean, graphql_name='local_context_data', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('device_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type', default=None)),
        ('device_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type_id', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('parent_device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_device_id', default=None)),
        ('platform_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id', default=None)),
        ('platform', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('cluster_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id', default=None)),
        ('model', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('is_full_depth', sgqlc.types.Arg(Boolean, graphql_name='is_full_depth', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('serial', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial', default=None)),
        ('has_primary_ip', sgqlc.types.Arg(Boolean, graphql_name='has_primary_ip', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis_member', sgqlc.types.Arg(Boolean, graphql_name='virtual_chassis_member', default=None)),
        ('console_ports', sgqlc.types.Arg(Boolean, graphql_name='console_ports', default=None)),
        ('console_server_ports', sgqlc.types.Arg(Boolean, graphql_name='console_server_ports', default=None)),
        ('power_ports', sgqlc.types.Arg(Boolean, graphql_name='power_ports', default=None)),
        ('power_outlets', sgqlc.types.Arg(Boolean, graphql_name='power_outlets', default=None)),
        ('interfaces', sgqlc.types.Arg(Boolean, graphql_name='interfaces', default=None)),
        ('pass_through_ports', sgqlc.types.Arg(Boolean, graphql_name='pass_through_ports', default=None)),
        ('module_bays', sgqlc.types.Arg(Boolean, graphql_name='module_bays', default=None)),
        ('device_bays', sgqlc.types.Arg(Boolean, graphql_name='device_bays', default=None)),
        ('primary_ip4_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip4_id', default=None)),
        ('primary_ip6_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip6_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('asset_tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__n', default=None)),
        ('asset_tag__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ic', default=None)),
        ('asset_tag__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nic', default=None)),
        ('asset_tag__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__iew', default=None)),
        ('asset_tag__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__niew', default=None)),
        ('asset_tag__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__isw', default=None)),
        ('asset_tag__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nisw', default=None)),
        ('asset_tag__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ie', default=None)),
        ('asset_tag__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nie', default=None)),
        ('asset_tag__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__empty', default=None)),
        ('face__n', sgqlc.types.Arg(String, graphql_name='face__n', default=None)),
        ('position__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__n', default=None)),
        ('position__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__lte', default=None)),
        ('position__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__lt', default=None)),
        ('position__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__gte', default=None)),
        ('position__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__gt', default=None)),
        ('airflow__n', sgqlc.types.Arg(String, graphql_name='airflow__n', default=None)),
        ('vc_position__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__n', default=None)),
        ('vc_position__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__lte', default=None)),
        ('vc_position__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__lt', default=None)),
        ('vc_position__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__gte', default=None)),
        ('vc_position__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__gt', default=None)),
        ('vc_priority__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__n', default=None)),
        ('vc_priority__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__lte', default=None)),
        ('vc_priority__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__lt', default=None)),
        ('vc_priority__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__gte', default=None)),
        ('vc_priority__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
        ('device_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type__n', default=None)),
        ('device_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type_id__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('parent_device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_device_id__n', default=None)),
        ('platform_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id__n', default=None)),
        ('platform__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('cluster_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id__n', default=None)),
        ('model__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__n', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('serial__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__n', default=None)),
        ('serial__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ic', default=None)),
        ('serial__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nic', default=None)),
        ('serial__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__iew', default=None)),
        ('serial__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__niew', default=None)),
        ('serial__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__isw', default=None)),
        ('serial__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nisw', default=None)),
        ('serial__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ie', default=None)),
        ('serial__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nie', default=None)),
        ('serial__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__empty', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('primary_ip4_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip4_id__n', default=None)),
        ('primary_ip6_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip6_id__n', default=None)),
))
    )
    powerfeed_set = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(PowerFeedType))), graphql_name='powerfeed_set', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('type', sgqlc.types.Arg(String, graphql_name='type', default=None)),
        ('supply', sgqlc.types.Arg(String, graphql_name='supply', default=None)),
        ('phase', sgqlc.types.Arg(String, graphql_name='phase', default=None)),
        ('voltage', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='voltage', default=None)),
        ('amperage', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='amperage', default=None)),
        ('max_utilization', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_utilization', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('power_panel_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='power_panel_id', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('type__n', sgqlc.types.Arg(String, graphql_name='type__n', default=None)),
        ('supply__n', sgqlc.types.Arg(String, graphql_name='supply__n', default=None)),
        ('phase__n', sgqlc.types.Arg(String, graphql_name='phase__n', default=None)),
        ('voltage__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='voltage__n', default=None)),
        ('voltage__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='voltage__lte', default=None)),
        ('voltage__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='voltage__lt', default=None)),
        ('voltage__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='voltage__gte', default=None)),
        ('voltage__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='voltage__gt', default=None)),
        ('amperage__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='amperage__n', default=None)),
        ('amperage__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='amperage__lte', default=None)),
        ('amperage__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='amperage__lt', default=None)),
        ('amperage__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='amperage__gte', default=None)),
        ('amperage__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='amperage__gt', default=None)),
        ('max_utilization__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_utilization__n', default=None)),
        ('max_utilization__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_utilization__lte', default=None)),
        ('max_utilization__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_utilization__lt', default=None)),
        ('max_utilization__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_utilization__gte', default=None)),
        ('max_utilization__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_utilization__gt', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('power_panel_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='power_panel_id__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
))
    )
    reservations = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(RackReservationType))), graphql_name='reservations', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('user_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='user_id', default=None)),
        ('user', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='user', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('user_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='user_id__n', default=None)),
        ('user__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='user__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of(JournalEntryType), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')
    contacts = sgqlc.types.Field(sgqlc.types.list_of(ContactAssignmentType), graphql_name='contacts')
    image_attachments = sgqlc.types.Field(sgqlc.types.list_of(ImageAttachmentType), graphql_name='image_attachments')
    vlan_groups = sgqlc.types.Field(sgqlc.types.list_of('VLANGroupType'), graphql_name='vlan_groups')


class RearPortTemplateType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'name', '_name', 'label', 'description', 'device_type', 'module_type', 'type', 'color', 'positions', 'frontport_templates', 'display', 'class_type', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    _name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='_name')
    label = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='label')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    device_type = sgqlc.types.Field(DeviceTypeType, graphql_name='device_type')
    module_type = sgqlc.types.Field(ModuleTypeType, graphql_name='module_type')
    type = sgqlc.types.Field(sgqlc.types.non_null(DcimRearPortTemplateTypeChoices), graphql_name='type')
    color = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='color')
    positions = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='positions')
    frontport_templates = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(FrontPortTemplateType))), graphql_name='frontport_templates', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('color', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('devicetype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id', default=None)),
        ('moduletype_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('color__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__n', default=None)),
        ('color__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ic', default=None)),
        ('color__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nic', default=None)),
        ('color__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__iew', default=None)),
        ('color__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__niew', default=None)),
        ('color__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__isw', default=None)),
        ('color__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nisw', default=None)),
        ('color__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ie', default=None)),
        ('color__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nie', default=None)),
        ('color__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('devicetype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='devicetype_id__n', default=None)),
        ('moduletype_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='moduletype_id__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class RearPortType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'device', 'name', '_name', 'label', 'description', 'module', 'cable', 'cable_end', 'mark_connected', 'type', 'color', 'positions', 'frontports', 'link_peers', 'display', 'class_type', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    device = sgqlc.types.Field(sgqlc.types.non_null(DeviceType), graphql_name='device')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    _name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='_name')
    label = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='label')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    module = sgqlc.types.Field(ModuleType, graphql_name='module')
    cable = sgqlc.types.Field(CableType, graphql_name='cable')
    cable_end = sgqlc.types.Field(DcimRearPortCableEndChoices, graphql_name='cable_end')
    mark_connected = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='mark_connected')
    type = sgqlc.types.Field(sgqlc.types.non_null(DcimRearPortTypeChoices), graphql_name='type')
    color = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='color')
    positions = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='positions')
    frontports = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(FrontPortType))), graphql_name='frontports', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('color', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('color__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__n', default=None)),
        ('color__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ic', default=None)),
        ('color__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nic', default=None)),
        ('color__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__iew', default=None)),
        ('color__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__niew', default=None)),
        ('color__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__isw', default=None)),
        ('color__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nisw', default=None)),
        ('color__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__ie', default=None)),
        ('color__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__nie', default=None)),
        ('color__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
))
    )
    link_peers = sgqlc.types.Field(sgqlc.types.list_of('LinkPeerType'), graphql_name='link_peers')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class RegionType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'parent', 'name', 'slug', 'description', 'lft', 'rght', 'tree_id', 'level', 'children', 'sites', 'display', 'class_type', 'custom_fields', 'changelog', 'contacts', 'vlan_groups')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    parent = sgqlc.types.Field('RegionType', graphql_name='parent')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    slug = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='slug')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    lft = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='lft')
    rght = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='rght')
    tree_id = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='tree_id')
    level = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='level')
    children = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('RegionType'))), graphql_name='children', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('parent', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('parent__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent__n', default=None)),
))
    )
    sites = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('SiteType'))), graphql_name='sites', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('facility', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility', default=None)),
        ('latitude', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude', default=None)),
        ('longitude', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('asn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn', default=None)),
        ('asn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('facility__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__n', default=None)),
        ('facility__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__ic', default=None)),
        ('facility__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__nic', default=None)),
        ('facility__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__iew', default=None)),
        ('facility__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__niew', default=None)),
        ('facility__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__isw', default=None)),
        ('facility__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__nisw', default=None)),
        ('facility__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__ie', default=None)),
        ('facility__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__nie', default=None)),
        ('facility__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__empty', default=None)),
        ('latitude__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__n', default=None)),
        ('latitude__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__lte', default=None)),
        ('latitude__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__lt', default=None)),
        ('latitude__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__gte', default=None)),
        ('latitude__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__gt', default=None)),
        ('longitude__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__n', default=None)),
        ('longitude__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__lte', default=None)),
        ('longitude__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__lt', default=None)),
        ('longitude__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__gte', default=None)),
        ('longitude__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
        ('asn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__n', default=None)),
        ('asn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn_id__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')
    contacts = sgqlc.types.Field(sgqlc.types.list_of(ContactAssignmentType), graphql_name='contacts')
    vlan_groups = sgqlc.types.Field(sgqlc.types.list_of('VLANGroupType'), graphql_name='vlan_groups')


class RoleType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'name', 'slug', 'description', 'weight', 'prefixes', 'ip_ranges', 'vlans', 'display', 'class_type', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    slug = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='slug')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    weight = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='weight')
    prefixes = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(PrefixType))), graphql_name='prefixes', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('is_pool', sgqlc.types.Arg(Boolean, graphql_name='is_pool', default=None)),
        ('mark_utilized', sgqlc.types.Arg(Boolean, graphql_name='mark_utilized', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('family', sgqlc.types.Arg(Int, graphql_name='family', default=None)),
        ('prefix', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='prefix', default=None)),
        ('within', sgqlc.types.Arg(String, graphql_name='within', default=None)),
        ('within_include', sgqlc.types.Arg(String, graphql_name='within_include', default=None)),
        ('contains', sgqlc.types.Arg(String, graphql_name='contains', default=None)),
        ('depth', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth', default=None)),
        ('children', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children', default=None)),
        ('mask_length', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mask_length', default=None)),
        ('mask_length__gte', sgqlc.types.Arg(Int, graphql_name='mask_length__gte', default=None)),
        ('mask_length__lte', sgqlc.types.Arg(Int, graphql_name='mask_length__lte', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('present_in_vrf_id', sgqlc.types.Arg(String, graphql_name='present_in_vrf_id', default=None)),
        ('present_in_vrf', sgqlc.types.Arg(String, graphql_name='present_in_vrf', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('vlan_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vlan_id', default=None)),
        ('vlan_vid', sgqlc.types.Arg(Int, graphql_name='vlan_vid', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('depth__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__n', default=None)),
        ('depth__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__lte', default=None)),
        ('depth__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__lt', default=None)),
        ('depth__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__gte', default=None)),
        ('depth__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__gt', default=None)),
        ('children__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__n', default=None)),
        ('children__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__lte', default=None)),
        ('children__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__lt', default=None)),
        ('children__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__gte', default=None)),
        ('children__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__gt', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('vlan_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vlan_id__n', default=None)),
        ('vlan_vid__n', sgqlc.types.Arg(Int, graphql_name='vlan_vid__n', default=None)),
        ('vlan_vid__lte', sgqlc.types.Arg(Int, graphql_name='vlan_vid__lte', default=None)),
        ('vlan_vid__lt', sgqlc.types.Arg(Int, graphql_name='vlan_vid__lt', default=None)),
        ('vlan_vid__gte', sgqlc.types.Arg(Int, graphql_name='vlan_vid__gte', default=None)),
        ('vlan_vid__gt', sgqlc.types.Arg(Int, graphql_name='vlan_vid__gt', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
))
    )
    ip_ranges = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(IPRangeType))), graphql_name='ip_ranges', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('family', sgqlc.types.Arg(Int, graphql_name='family', default=None)),
        ('start_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='start_address', default=None)),
        ('end_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='end_address', default=None)),
        ('contains', sgqlc.types.Arg(String, graphql_name='contains', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
))
    )
    vlans = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('VLANType'))), graphql_name='vlans', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('vid', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('available_on_device', sgqlc.types.Arg(String, graphql_name='available_on_device', default=None)),
        ('available_on_virtualmachine', sgqlc.types.Arg(String, graphql_name='available_on_virtualmachine', default=None)),
        ('l2vpn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id', default=None)),
        ('l2vpn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('vid__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__n', default=None)),
        ('vid__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__lte', default=None)),
        ('vid__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__lt', default=None)),
        ('vid__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__gte', default=None)),
        ('vid__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('l2vpn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id__n', default=None)),
        ('l2vpn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class RouteTargetType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'name', 'tenant', 'importing_vrfs', 'exporting_vrfs', 'importing_l2vpns', 'exporting_l2vpns', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    tenant = sgqlc.types.Field('TenantType', graphql_name='tenant')
    importing_vrfs = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('VRFType'))), graphql_name='importing_vrfs', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('rd', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd', default=None)),
        ('enforce_unique', sgqlc.types.Arg(Boolean, graphql_name='enforce_unique', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('import_target_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='import_target_id', default=None)),
        ('import_target', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='import_target', default=None)),
        ('export_target_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='export_target_id', default=None)),
        ('export_target', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='export_target', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('rd__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__n', default=None)),
        ('rd__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__ic', default=None)),
        ('rd__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__nic', default=None)),
        ('rd__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__iew', default=None)),
        ('rd__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__niew', default=None)),
        ('rd__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__isw', default=None)),
        ('rd__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__nisw', default=None)),
        ('rd__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__ie', default=None)),
        ('rd__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__nie', default=None)),
        ('rd__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('import_target_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='import_target_id__n', default=None)),
        ('import_target__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='import_target__n', default=None)),
        ('export_target_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='export_target_id__n', default=None)),
        ('export_target__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='export_target__n', default=None)),
))
    )
    exporting_vrfs = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('VRFType'))), graphql_name='exporting_vrfs', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('rd', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd', default=None)),
        ('enforce_unique', sgqlc.types.Arg(Boolean, graphql_name='enforce_unique', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('import_target_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='import_target_id', default=None)),
        ('import_target', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='import_target', default=None)),
        ('export_target_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='export_target_id', default=None)),
        ('export_target', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='export_target', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('rd__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__n', default=None)),
        ('rd__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__ic', default=None)),
        ('rd__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__nic', default=None)),
        ('rd__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__iew', default=None)),
        ('rd__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__niew', default=None)),
        ('rd__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__isw', default=None)),
        ('rd__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__nisw', default=None)),
        ('rd__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__ie', default=None)),
        ('rd__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__nie', default=None)),
        ('rd__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('import_target_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='import_target_id__n', default=None)),
        ('import_target__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='import_target__n', default=None)),
        ('export_target_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='export_target_id__n', default=None)),
        ('export_target__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='export_target__n', default=None)),
))
    )
    importing_l2vpns = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(L2VPNType))), graphql_name='importing_l2vpns')
    exporting_l2vpns = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(L2VPNType))), graphql_name='exporting_l2vpns')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of(JournalEntryType), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class SavedFilterType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'name', 'slug', 'description', 'user', 'weight', 'enabled', 'shared', 'parameters', 'display', 'class_type', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    slug = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='slug')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    user = sgqlc.types.Field('UserType', graphql_name='user')
    weight = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='weight')
    enabled = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='enabled')
    shared = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='shared')
    parameters = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='parameters')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class ServiceTemplateType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'protocol', 'ports', 'name', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    protocol = sgqlc.types.Field(sgqlc.types.non_null(IpamServiceTemplateProtocolChoices), graphql_name='protocol')
    ports = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(Int))), graphql_name='ports')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of(JournalEntryType), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class ServiceType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'protocol', 'ports', 'device', 'virtual_machine', 'name', 'ipaddresses', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    protocol = sgqlc.types.Field(sgqlc.types.non_null(IpamServiceProtocolChoices), graphql_name='protocol')
    ports = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(Int))), graphql_name='ports')
    device = sgqlc.types.Field(DeviceType, graphql_name='device')
    virtual_machine = sgqlc.types.Field('VirtualMachineType', graphql_name='virtual_machine')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    ipaddresses = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(IPAddressType))), graphql_name='ipaddresses', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('dns_name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('family', sgqlc.types.Arg(Int, graphql_name='family', default=None)),
        ('parent', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent', default=None)),
        ('address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address', default=None)),
        ('mask_length', sgqlc.types.Arg(Int, graphql_name='mask_length', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('present_in_vrf_id', sgqlc.types.Arg(String, graphql_name='present_in_vrf_id', default=None)),
        ('present_in_vrf', sgqlc.types.Arg(String, graphql_name='present_in_vrf', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('virtual_machine', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine', default=None)),
        ('virtual_machine_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id', default=None)),
        ('interface', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface', default=None)),
        ('interface_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_id', default=None)),
        ('vminterface', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vminterface', default=None)),
        ('vminterface_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vminterface_id', default=None)),
        ('fhrpgroup_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='fhrpgroup_id', default=None)),
        ('assigned_to_interface', sgqlc.types.Arg(Boolean, graphql_name='assigned_to_interface', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('dns_name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__n', default=None)),
        ('dns_name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__ic', default=None)),
        ('dns_name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__nic', default=None)),
        ('dns_name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__iew', default=None)),
        ('dns_name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__niew', default=None)),
        ('dns_name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__isw', default=None)),
        ('dns_name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__nisw', default=None)),
        ('dns_name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__ie', default=None)),
        ('dns_name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__nie', default=None)),
        ('dns_name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('interface__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface__n', default=None)),
        ('interface_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_id__n', default=None)),
        ('vminterface__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vminterface__n', default=None)),
        ('vminterface_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vminterface_id__n', default=None)),
        ('fhrpgroup_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='fhrpgroup_id__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of(JournalEntryType), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class SiteGroupType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'parent', 'name', 'slug', 'description', 'lft', 'rght', 'tree_id', 'level', 'children', 'sites', 'display', 'class_type', 'custom_fields', 'changelog', 'contacts', 'vlan_groups')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    parent = sgqlc.types.Field('SiteGroupType', graphql_name='parent')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    slug = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='slug')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    lft = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='lft')
    rght = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='rght')
    tree_id = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='tree_id')
    level = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='level')
    children = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('SiteGroupType'))), graphql_name='children', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('parent', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('parent__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent__n', default=None)),
))
    )
    sites = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('SiteType'))), graphql_name='sites', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('facility', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility', default=None)),
        ('latitude', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude', default=None)),
        ('longitude', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('asn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn', default=None)),
        ('asn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('facility__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__n', default=None)),
        ('facility__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__ic', default=None)),
        ('facility__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__nic', default=None)),
        ('facility__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__iew', default=None)),
        ('facility__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__niew', default=None)),
        ('facility__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__isw', default=None)),
        ('facility__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__nisw', default=None)),
        ('facility__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__ie', default=None)),
        ('facility__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__nie', default=None)),
        ('facility__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__empty', default=None)),
        ('latitude__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__n', default=None)),
        ('latitude__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__lte', default=None)),
        ('latitude__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__lt', default=None)),
        ('latitude__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__gte', default=None)),
        ('latitude__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__gt', default=None)),
        ('longitude__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__n', default=None)),
        ('longitude__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__lte', default=None)),
        ('longitude__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__lt', default=None)),
        ('longitude__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__gte', default=None)),
        ('longitude__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
        ('asn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__n', default=None)),
        ('asn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn_id__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')
    contacts = sgqlc.types.Field(sgqlc.types.list_of(ContactAssignmentType), graphql_name='contacts')
    vlan_groups = sgqlc.types.Field(sgqlc.types.list_of('VLANGroupType'), graphql_name='vlan_groups')


class SiteType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'name', '_name', 'slug', 'status', 'region', 'group', 'tenant', 'facility', 'asns', 'time_zone', 'physical_address', 'shipping_address', 'latitude', 'longitude', 'cabletermination_set', 'devices', 'powerpanel_set', 'racks', 'locations', 'circuit_terminations', 'prefixes', 'clusters', 'virtual_machines', 'vlans', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog', 'contacts', 'image_attachments', 'vlan_groups', 'asn')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of('TagType'), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    _name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='_name')
    slug = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='slug')
    status = sgqlc.types.Field(sgqlc.types.non_null(DcimSiteStatusChoices), graphql_name='status')
    region = sgqlc.types.Field(RegionType, graphql_name='region')
    group = sgqlc.types.Field(SiteGroupType, graphql_name='group')
    tenant = sgqlc.types.Field('TenantType', graphql_name='tenant')
    facility = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='facility')
    asns = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ASNType))), graphql_name='asns', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('asn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('rir_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir_id', default=None)),
        ('rir', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('asn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__n', default=None)),
        ('asn__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__lte', default=None)),
        ('asn__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__lt', default=None)),
        ('asn__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__gte', default=None)),
        ('asn__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('rir_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir_id__n', default=None)),
        ('rir__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
))
    )
    time_zone = sgqlc.types.Field(DcimSiteTimeZoneChoices, graphql_name='time_zone')
    physical_address = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='physical_address')
    shipping_address = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='shipping_address')
    latitude = sgqlc.types.Field(Decimal, graphql_name='latitude')
    longitude = sgqlc.types.Field(Decimal, graphql_name='longitude')
    cabletermination_set = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(CableTerminationType))), graphql_name='cabletermination_set', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('cable', sgqlc.types.Arg(String, graphql_name='cable', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('termination_type', sgqlc.types.Arg(String, graphql_name='termination_type', default=None)),
        ('termination_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('cable__n', sgqlc.types.Arg(String, graphql_name='cable__n', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('termination_type__n', sgqlc.types.Arg(String, graphql_name='termination_type__n', default=None)),
        ('termination_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id__n', default=None)),
        ('termination_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id__lte', default=None)),
        ('termination_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id__lt', default=None)),
        ('termination_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id__gte', default=None)),
        ('termination_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_id__gt', default=None)),
))
    )
    devices = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(DeviceType))), graphql_name='devices', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('asset_tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag', default=None)),
        ('face', sgqlc.types.Arg(String, graphql_name='face', default=None)),
        ('position', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position', default=None)),
        ('airflow', sgqlc.types.Arg(String, graphql_name='airflow', default=None)),
        ('vc_position', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position', default=None)),
        ('vc_priority', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('local_context_data', sgqlc.types.Arg(Boolean, graphql_name='local_context_data', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('device_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type', default=None)),
        ('device_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type_id', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('parent_device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_device_id', default=None)),
        ('platform_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id', default=None)),
        ('platform', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('cluster_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id', default=None)),
        ('model', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('is_full_depth', sgqlc.types.Arg(Boolean, graphql_name='is_full_depth', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('serial', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial', default=None)),
        ('has_primary_ip', sgqlc.types.Arg(Boolean, graphql_name='has_primary_ip', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis_member', sgqlc.types.Arg(Boolean, graphql_name='virtual_chassis_member', default=None)),
        ('console_ports', sgqlc.types.Arg(Boolean, graphql_name='console_ports', default=None)),
        ('console_server_ports', sgqlc.types.Arg(Boolean, graphql_name='console_server_ports', default=None)),
        ('power_ports', sgqlc.types.Arg(Boolean, graphql_name='power_ports', default=None)),
        ('power_outlets', sgqlc.types.Arg(Boolean, graphql_name='power_outlets', default=None)),
        ('interfaces', sgqlc.types.Arg(Boolean, graphql_name='interfaces', default=None)),
        ('pass_through_ports', sgqlc.types.Arg(Boolean, graphql_name='pass_through_ports', default=None)),
        ('module_bays', sgqlc.types.Arg(Boolean, graphql_name='module_bays', default=None)),
        ('device_bays', sgqlc.types.Arg(Boolean, graphql_name='device_bays', default=None)),
        ('primary_ip4_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip4_id', default=None)),
        ('primary_ip6_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip6_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('asset_tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__n', default=None)),
        ('asset_tag__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ic', default=None)),
        ('asset_tag__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nic', default=None)),
        ('asset_tag__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__iew', default=None)),
        ('asset_tag__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__niew', default=None)),
        ('asset_tag__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__isw', default=None)),
        ('asset_tag__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nisw', default=None)),
        ('asset_tag__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ie', default=None)),
        ('asset_tag__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nie', default=None)),
        ('asset_tag__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__empty', default=None)),
        ('face__n', sgqlc.types.Arg(String, graphql_name='face__n', default=None)),
        ('position__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__n', default=None)),
        ('position__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__lte', default=None)),
        ('position__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__lt', default=None)),
        ('position__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__gte', default=None)),
        ('position__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__gt', default=None)),
        ('airflow__n', sgqlc.types.Arg(String, graphql_name='airflow__n', default=None)),
        ('vc_position__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__n', default=None)),
        ('vc_position__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__lte', default=None)),
        ('vc_position__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__lt', default=None)),
        ('vc_position__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__gte', default=None)),
        ('vc_position__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__gt', default=None)),
        ('vc_priority__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__n', default=None)),
        ('vc_priority__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__lte', default=None)),
        ('vc_priority__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__lt', default=None)),
        ('vc_priority__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__gte', default=None)),
        ('vc_priority__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
        ('device_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type__n', default=None)),
        ('device_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type_id__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('parent_device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_device_id__n', default=None)),
        ('platform_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id__n', default=None)),
        ('platform__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('cluster_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id__n', default=None)),
        ('model__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__n', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('serial__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__n', default=None)),
        ('serial__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ic', default=None)),
        ('serial__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nic', default=None)),
        ('serial__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__iew', default=None)),
        ('serial__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__niew', default=None)),
        ('serial__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__isw', default=None)),
        ('serial__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nisw', default=None)),
        ('serial__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ie', default=None)),
        ('serial__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nie', default=None)),
        ('serial__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__empty', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('primary_ip4_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip4_id__n', default=None)),
        ('primary_ip6_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip6_id__n', default=None)),
))
    )
    powerpanel_set = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(PowerPanelType))), graphql_name='powerpanel_set', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
))
    )
    racks = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(RackType))), graphql_name='racks', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('facility_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id', default=None)),
        ('asset_tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag', default=None)),
        ('u_height', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height', default=None)),
        ('desc_units', sgqlc.types.Arg(Boolean, graphql_name='desc_units', default=None)),
        ('outer_width', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width', default=None)),
        ('outer_depth', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth', default=None)),
        ('outer_unit', sgqlc.types.Arg(String, graphql_name='outer_unit', default=None)),
        ('mounting_depth', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth', default=None)),
        ('weight', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight', default=None)),
        ('max_weight', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight', default=None)),
        ('weight_unit', sgqlc.types.Arg(String, graphql_name='weight_unit', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('width', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='width', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('serial', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('facility_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__n', default=None)),
        ('facility_id__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__ic', default=None)),
        ('facility_id__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__nic', default=None)),
        ('facility_id__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__iew', default=None)),
        ('facility_id__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__niew', default=None)),
        ('facility_id__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__isw', default=None)),
        ('facility_id__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__nisw', default=None)),
        ('facility_id__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__ie', default=None)),
        ('facility_id__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__nie', default=None)),
        ('facility_id__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__empty', default=None)),
        ('asset_tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__n', default=None)),
        ('asset_tag__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ic', default=None)),
        ('asset_tag__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nic', default=None)),
        ('asset_tag__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__iew', default=None)),
        ('asset_tag__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__niew', default=None)),
        ('asset_tag__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__isw', default=None)),
        ('asset_tag__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nisw', default=None)),
        ('asset_tag__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ie', default=None)),
        ('asset_tag__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nie', default=None)),
        ('asset_tag__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__empty', default=None)),
        ('u_height__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__n', default=None)),
        ('u_height__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__lte', default=None)),
        ('u_height__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__lt', default=None)),
        ('u_height__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__gte', default=None)),
        ('u_height__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__gt', default=None)),
        ('outer_width__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width__n', default=None)),
        ('outer_width__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width__lte', default=None)),
        ('outer_width__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width__lt', default=None)),
        ('outer_width__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width__gte', default=None)),
        ('outer_width__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width__gt', default=None)),
        ('outer_depth__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth__n', default=None)),
        ('outer_depth__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth__lte', default=None)),
        ('outer_depth__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth__lt', default=None)),
        ('outer_depth__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth__gte', default=None)),
        ('outer_depth__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth__gt', default=None)),
        ('outer_unit__n', sgqlc.types.Arg(String, graphql_name='outer_unit__n', default=None)),
        ('mounting_depth__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth__n', default=None)),
        ('mounting_depth__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth__lte', default=None)),
        ('mounting_depth__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth__lt', default=None)),
        ('mounting_depth__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth__gte', default=None)),
        ('mounting_depth__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth__gt', default=None)),
        ('weight__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__n', default=None)),
        ('weight__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__lte', default=None)),
        ('weight__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__lt', default=None)),
        ('weight__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__gte', default=None)),
        ('weight__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__gt', default=None)),
        ('max_weight__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight__n', default=None)),
        ('max_weight__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight__lte', default=None)),
        ('max_weight__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight__lt', default=None)),
        ('max_weight__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight__gte', default=None)),
        ('max_weight__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight__gt', default=None)),
        ('weight_unit__n', sgqlc.types.Arg(String, graphql_name='weight_unit__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('width__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='width__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('serial__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__n', default=None)),
        ('serial__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ic', default=None)),
        ('serial__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nic', default=None)),
        ('serial__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__iew', default=None)),
        ('serial__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__niew', default=None)),
        ('serial__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__isw', default=None)),
        ('serial__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nisw', default=None)),
        ('serial__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ie', default=None)),
        ('serial__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nie', default=None)),
        ('serial__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__empty', default=None)),
))
    )
    locations = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(LocationType))), graphql_name='locations', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('parent', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('parent__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent__n', default=None)),
))
    )
    circuit_terminations = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(CircuitTerminationType))), graphql_name='circuit_terminations', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('term_side', sgqlc.types.Arg(String, graphql_name='term_side', default=None)),
        ('port_speed', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='port_speed', default=None)),
        ('upstream_speed', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='upstream_speed', default=None)),
        ('xconnect_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('circuit_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='circuit_id', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('provider_network_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_network_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('term_side__n', sgqlc.types.Arg(String, graphql_name='term_side__n', default=None)),
        ('port_speed__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='port_speed__n', default=None)),
        ('port_speed__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='port_speed__lte', default=None)),
        ('port_speed__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='port_speed__lt', default=None)),
        ('port_speed__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='port_speed__gte', default=None)),
        ('port_speed__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='port_speed__gt', default=None)),
        ('upstream_speed__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='upstream_speed__n', default=None)),
        ('upstream_speed__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='upstream_speed__lte', default=None)),
        ('upstream_speed__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='upstream_speed__lt', default=None)),
        ('upstream_speed__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='upstream_speed__gte', default=None)),
        ('upstream_speed__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='upstream_speed__gt', default=None)),
        ('xconnect_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__n', default=None)),
        ('xconnect_id__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__ic', default=None)),
        ('xconnect_id__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__nic', default=None)),
        ('xconnect_id__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__iew', default=None)),
        ('xconnect_id__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__niew', default=None)),
        ('xconnect_id__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__isw', default=None)),
        ('xconnect_id__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__nisw', default=None)),
        ('xconnect_id__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__ie', default=None)),
        ('xconnect_id__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__nie', default=None)),
        ('xconnect_id__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='xconnect_id__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('circuit_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='circuit_id__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('provider_network_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_network_id__n', default=None)),
))
    )
    prefixes = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(PrefixType))), graphql_name='prefixes', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('is_pool', sgqlc.types.Arg(Boolean, graphql_name='is_pool', default=None)),
        ('mark_utilized', sgqlc.types.Arg(Boolean, graphql_name='mark_utilized', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('family', sgqlc.types.Arg(Int, graphql_name='family', default=None)),
        ('prefix', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='prefix', default=None)),
        ('within', sgqlc.types.Arg(String, graphql_name='within', default=None)),
        ('within_include', sgqlc.types.Arg(String, graphql_name='within_include', default=None)),
        ('contains', sgqlc.types.Arg(String, graphql_name='contains', default=None)),
        ('depth', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth', default=None)),
        ('children', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children', default=None)),
        ('mask_length', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mask_length', default=None)),
        ('mask_length__gte', sgqlc.types.Arg(Int, graphql_name='mask_length__gte', default=None)),
        ('mask_length__lte', sgqlc.types.Arg(Int, graphql_name='mask_length__lte', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('present_in_vrf_id', sgqlc.types.Arg(String, graphql_name='present_in_vrf_id', default=None)),
        ('present_in_vrf', sgqlc.types.Arg(String, graphql_name='present_in_vrf', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('vlan_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vlan_id', default=None)),
        ('vlan_vid', sgqlc.types.Arg(Int, graphql_name='vlan_vid', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('depth__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__n', default=None)),
        ('depth__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__lte', default=None)),
        ('depth__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__lt', default=None)),
        ('depth__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__gte', default=None)),
        ('depth__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__gt', default=None)),
        ('children__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__n', default=None)),
        ('children__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__lte', default=None)),
        ('children__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__lt', default=None)),
        ('children__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__gte', default=None)),
        ('children__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__gt', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('vlan_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vlan_id__n', default=None)),
        ('vlan_vid__n', sgqlc.types.Arg(Int, graphql_name='vlan_vid__n', default=None)),
        ('vlan_vid__lte', sgqlc.types.Arg(Int, graphql_name='vlan_vid__lte', default=None)),
        ('vlan_vid__lt', sgqlc.types.Arg(Int, graphql_name='vlan_vid__lt', default=None)),
        ('vlan_vid__gte', sgqlc.types.Arg(Int, graphql_name='vlan_vid__gte', default=None)),
        ('vlan_vid__gt', sgqlc.types.Arg(Int, graphql_name='vlan_vid__gt', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
))
    )
    clusters = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ClusterType))), graphql_name='clusters', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type_id', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
        ('type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type_id__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
))
    )
    virtual_machines = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('VirtualMachineType'))), graphql_name='virtual_machines', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('cluster', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster', default=None)),
        ('vcpus', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus', default=None)),
        ('memory', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory', default=None)),
        ('disk', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('local_context_data', sgqlc.types.Arg(Boolean, graphql_name='local_context_data', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('cluster_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group_id', default=None)),
        ('cluster_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group', default=None)),
        ('cluster_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type_id', default=None)),
        ('cluster_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type', default=None)),
        ('cluster_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('platform_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id', default=None)),
        ('platform', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('has_primary_ip', sgqlc.types.Arg(Boolean, graphql_name='has_primary_ip', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('cluster__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster__n', default=None)),
        ('vcpus__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__n', default=None)),
        ('vcpus__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__lte', default=None)),
        ('vcpus__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__lt', default=None)),
        ('vcpus__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__gte', default=None)),
        ('vcpus__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__gt', default=None)),
        ('memory__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__n', default=None)),
        ('memory__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__lte', default=None)),
        ('memory__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__lt', default=None)),
        ('memory__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__gte', default=None)),
        ('memory__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__gt', default=None)),
        ('disk__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__n', default=None)),
        ('disk__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__lte', default=None)),
        ('disk__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__lt', default=None)),
        ('disk__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__gte', default=None)),
        ('disk__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('cluster_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group_id__n', default=None)),
        ('cluster_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group__n', default=None)),
        ('cluster_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type_id__n', default=None)),
        ('cluster_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type__n', default=None)),
        ('cluster_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('platform_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id__n', default=None)),
        ('platform__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
))
    )
    vlans = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('VLANType'))), graphql_name='vlans', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('vid', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('available_on_device', sgqlc.types.Arg(String, graphql_name='available_on_device', default=None)),
        ('available_on_virtualmachine', sgqlc.types.Arg(String, graphql_name='available_on_virtualmachine', default=None)),
        ('l2vpn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id', default=None)),
        ('l2vpn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('vid__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__n', default=None)),
        ('vid__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__lte', default=None)),
        ('vid__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__lt', default=None)),
        ('vid__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__gte', default=None)),
        ('vid__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('l2vpn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id__n', default=None)),
        ('l2vpn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of(JournalEntryType), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')
    contacts = sgqlc.types.Field(sgqlc.types.list_of(ContactAssignmentType), graphql_name='contacts')
    image_attachments = sgqlc.types.Field(sgqlc.types.list_of(ImageAttachmentType), graphql_name='image_attachments')
    vlan_groups = sgqlc.types.Field(sgqlc.types.list_of('VLANGroupType'), graphql_name='vlan_groups')
    asn = sgqlc.types.Field(BigInt, graphql_name='asn')


class TagType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('name', 'slug', 'created', 'last_updated', 'id', 'color', 'description', 'display', 'class_type', 'changelog')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    slug = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='slug')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    color = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='color')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class TenantGroupType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'parent', 'description', 'name', 'slug', 'lft', 'rght', 'tree_id', 'level', 'children', 'tenants', 'display', 'class_type', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of(TagType), graphql_name='tags')
    parent = sgqlc.types.Field('TenantGroupType', graphql_name='parent')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    slug = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='slug')
    lft = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='lft')
    rght = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='rght')
    tree_id = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='tree_id')
    level = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='level')
    children = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('TenantGroupType'))), graphql_name='children', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('parent', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('parent__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent__n', default=None)),
))
    )
    tenants = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('TenantType'))), graphql_name='tenants', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class TenantType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'name', 'slug', 'group', 'wireless_lans', 'wireless_links', 'cables', 'devices', 'vdcs', 'racks', 'rackreservations', 'sites', 'locations', 'circuits', 'vrfs', 'route_targets', 'asns', 'aggregates', 'prefixes', 'ip_ranges', 'ip_addresses', 'l2vpns', 'clusters', 'virtual_machines', 'vlans', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of(TagType), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    slug = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='slug')
    group = sgqlc.types.Field(TenantGroupType, graphql_name='group')
    wireless_lans = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('WirelessLANType'))), graphql_name='wireless_lans', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('ssid', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid', default=None)),
        ('auth_psk', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('vlan_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vlan_id', default=None)),
        ('auth_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_type', default=None)),
        ('auth_cipher', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_cipher', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('ssid__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__n', default=None)),
        ('ssid__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__ic', default=None)),
        ('ssid__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__nic', default=None)),
        ('ssid__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__iew', default=None)),
        ('ssid__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__niew', default=None)),
        ('ssid__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__isw', default=None)),
        ('ssid__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__nisw', default=None)),
        ('ssid__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__ie', default=None)),
        ('ssid__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__nie', default=None)),
        ('ssid__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__empty', default=None)),
        ('auth_psk__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__n', default=None)),
        ('auth_psk__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__ic', default=None)),
        ('auth_psk__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__nic', default=None)),
        ('auth_psk__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__iew', default=None)),
        ('auth_psk__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__niew', default=None)),
        ('auth_psk__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__isw', default=None)),
        ('auth_psk__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__nisw', default=None)),
        ('auth_psk__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__ie', default=None)),
        ('auth_psk__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__nie', default=None)),
        ('auth_psk__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('vlan_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vlan_id__n', default=None)),
        ('auth_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_type__n', default=None)),
        ('auth_cipher__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_cipher__n', default=None)),
))
    )
    wireless_links = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('WirelessLinkType'))), graphql_name='wireless_links', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('ssid', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid', default=None)),
        ('auth_psk', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('interface_a_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_a_id', default=None)),
        ('interface_b_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_b_id', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('auth_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_type', default=None)),
        ('auth_cipher', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_cipher', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('ssid__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__n', default=None)),
        ('ssid__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__ic', default=None)),
        ('ssid__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__nic', default=None)),
        ('ssid__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__iew', default=None)),
        ('ssid__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__niew', default=None)),
        ('ssid__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__isw', default=None)),
        ('ssid__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__nisw', default=None)),
        ('ssid__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__ie', default=None)),
        ('ssid__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__nie', default=None)),
        ('ssid__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__empty', default=None)),
        ('auth_psk__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__n', default=None)),
        ('auth_psk__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__ic', default=None)),
        ('auth_psk__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__nic', default=None)),
        ('auth_psk__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__iew', default=None)),
        ('auth_psk__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__niew', default=None)),
        ('auth_psk__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__isw', default=None)),
        ('auth_psk__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__nisw', default=None)),
        ('auth_psk__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__ie', default=None)),
        ('auth_psk__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__nie', default=None)),
        ('auth_psk__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('interface_a_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_a_id__n', default=None)),
        ('interface_a_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_a_id__lte', default=None)),
        ('interface_a_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_a_id__lt', default=None)),
        ('interface_a_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_a_id__gte', default=None)),
        ('interface_a_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_a_id__gt', default=None)),
        ('interface_b_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_b_id__n', default=None)),
        ('interface_b_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_b_id__lte', default=None)),
        ('interface_b_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_b_id__lt', default=None)),
        ('interface_b_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_b_id__gte', default=None)),
        ('interface_b_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_b_id__gt', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('auth_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_type__n', default=None)),
        ('auth_cipher__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_cipher__n', default=None)),
))
    )
    cables = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(CableType))), graphql_name='cables', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('length', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='length', default=None)),
        ('length_unit', sgqlc.types.Arg(String, graphql_name='length_unit', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('termination_a_type', sgqlc.types.Arg(String, graphql_name='termination_a_type', default=None)),
        ('termination_a_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_a_id', default=None)),
        ('termination_b_type', sgqlc.types.Arg(String, graphql_name='termination_b_type', default=None)),
        ('termination_b_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_b_id', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('color', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('length__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='length__n', default=None)),
        ('length__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='length__lte', default=None)),
        ('length__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='length__lt', default=None)),
        ('length__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='length__gte', default=None)),
        ('length__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='length__gt', default=None)),
        ('length_unit__n', sgqlc.types.Arg(String, graphql_name='length_unit__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('termination_a_type__n', sgqlc.types.Arg(String, graphql_name='termination_a_type__n', default=None)),
        ('termination_a_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_a_id__n', default=None)),
        ('termination_a_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_a_id__lte', default=None)),
        ('termination_a_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_a_id__lt', default=None)),
        ('termination_a_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_a_id__gte', default=None)),
        ('termination_a_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_a_id__gt', default=None)),
        ('termination_b_type__n', sgqlc.types.Arg(String, graphql_name='termination_b_type__n', default=None)),
        ('termination_b_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_b_id__n', default=None)),
        ('termination_b_id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_b_id__lte', default=None)),
        ('termination_b_id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_b_id__lt', default=None)),
        ('termination_b_id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_b_id__gte', default=None)),
        ('termination_b_id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_b_id__gt', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('color__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='color__n', default=None)),
))
    )
    devices = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(DeviceType))), graphql_name='devices', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('asset_tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag', default=None)),
        ('face', sgqlc.types.Arg(String, graphql_name='face', default=None)),
        ('position', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position', default=None)),
        ('airflow', sgqlc.types.Arg(String, graphql_name='airflow', default=None)),
        ('vc_position', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position', default=None)),
        ('vc_priority', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('local_context_data', sgqlc.types.Arg(Boolean, graphql_name='local_context_data', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('device_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type', default=None)),
        ('device_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type_id', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('parent_device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_device_id', default=None)),
        ('platform_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id', default=None)),
        ('platform', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('cluster_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id', default=None)),
        ('model', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('is_full_depth', sgqlc.types.Arg(Boolean, graphql_name='is_full_depth', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('serial', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial', default=None)),
        ('has_primary_ip', sgqlc.types.Arg(Boolean, graphql_name='has_primary_ip', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis_member', sgqlc.types.Arg(Boolean, graphql_name='virtual_chassis_member', default=None)),
        ('console_ports', sgqlc.types.Arg(Boolean, graphql_name='console_ports', default=None)),
        ('console_server_ports', sgqlc.types.Arg(Boolean, graphql_name='console_server_ports', default=None)),
        ('power_ports', sgqlc.types.Arg(Boolean, graphql_name='power_ports', default=None)),
        ('power_outlets', sgqlc.types.Arg(Boolean, graphql_name='power_outlets', default=None)),
        ('interfaces', sgqlc.types.Arg(Boolean, graphql_name='interfaces', default=None)),
        ('pass_through_ports', sgqlc.types.Arg(Boolean, graphql_name='pass_through_ports', default=None)),
        ('module_bays', sgqlc.types.Arg(Boolean, graphql_name='module_bays', default=None)),
        ('device_bays', sgqlc.types.Arg(Boolean, graphql_name='device_bays', default=None)),
        ('primary_ip4_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip4_id', default=None)),
        ('primary_ip6_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip6_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('asset_tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__n', default=None)),
        ('asset_tag__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ic', default=None)),
        ('asset_tag__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nic', default=None)),
        ('asset_tag__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__iew', default=None)),
        ('asset_tag__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__niew', default=None)),
        ('asset_tag__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__isw', default=None)),
        ('asset_tag__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nisw', default=None)),
        ('asset_tag__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ie', default=None)),
        ('asset_tag__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nie', default=None)),
        ('asset_tag__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__empty', default=None)),
        ('face__n', sgqlc.types.Arg(String, graphql_name='face__n', default=None)),
        ('position__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__n', default=None)),
        ('position__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__lte', default=None)),
        ('position__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__lt', default=None)),
        ('position__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__gte', default=None)),
        ('position__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__gt', default=None)),
        ('airflow__n', sgqlc.types.Arg(String, graphql_name='airflow__n', default=None)),
        ('vc_position__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__n', default=None)),
        ('vc_position__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__lte', default=None)),
        ('vc_position__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__lt', default=None)),
        ('vc_position__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__gte', default=None)),
        ('vc_position__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__gt', default=None)),
        ('vc_priority__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__n', default=None)),
        ('vc_priority__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__lte', default=None)),
        ('vc_priority__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__lt', default=None)),
        ('vc_priority__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__gte', default=None)),
        ('vc_priority__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
        ('device_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type__n', default=None)),
        ('device_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type_id__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('parent_device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_device_id__n', default=None)),
        ('platform_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id__n', default=None)),
        ('platform__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('cluster_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id__n', default=None)),
        ('model__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__n', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('serial__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__n', default=None)),
        ('serial__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ic', default=None)),
        ('serial__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nic', default=None)),
        ('serial__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__iew', default=None)),
        ('serial__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__niew', default=None)),
        ('serial__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__isw', default=None)),
        ('serial__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nisw', default=None)),
        ('serial__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ie', default=None)),
        ('serial__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nie', default=None)),
        ('serial__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__empty', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('primary_ip4_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip4_id__n', default=None)),
        ('primary_ip6_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip6_id__n', default=None)),
))
    )
    vdcs = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('VirtualDeviceContextType'))), graphql_name='vdcs', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('has_primary_ip', sgqlc.types.Arg(Boolean, graphql_name='has_primary_ip', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
))
    )
    racks = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(RackType))), graphql_name='racks', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('facility_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id', default=None)),
        ('asset_tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag', default=None)),
        ('u_height', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height', default=None)),
        ('desc_units', sgqlc.types.Arg(Boolean, graphql_name='desc_units', default=None)),
        ('outer_width', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width', default=None)),
        ('outer_depth', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth', default=None)),
        ('outer_unit', sgqlc.types.Arg(String, graphql_name='outer_unit', default=None)),
        ('mounting_depth', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth', default=None)),
        ('weight', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight', default=None)),
        ('max_weight', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight', default=None)),
        ('weight_unit', sgqlc.types.Arg(String, graphql_name='weight_unit', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('width', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='width', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('serial', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('facility_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__n', default=None)),
        ('facility_id__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__ic', default=None)),
        ('facility_id__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__nic', default=None)),
        ('facility_id__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__iew', default=None)),
        ('facility_id__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__niew', default=None)),
        ('facility_id__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__isw', default=None)),
        ('facility_id__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__nisw', default=None)),
        ('facility_id__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__ie', default=None)),
        ('facility_id__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__nie', default=None)),
        ('facility_id__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility_id__empty', default=None)),
        ('asset_tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__n', default=None)),
        ('asset_tag__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ic', default=None)),
        ('asset_tag__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nic', default=None)),
        ('asset_tag__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__iew', default=None)),
        ('asset_tag__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__niew', default=None)),
        ('asset_tag__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__isw', default=None)),
        ('asset_tag__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nisw', default=None)),
        ('asset_tag__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ie', default=None)),
        ('asset_tag__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nie', default=None)),
        ('asset_tag__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__empty', default=None)),
        ('u_height__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__n', default=None)),
        ('u_height__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__lte', default=None)),
        ('u_height__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__lt', default=None)),
        ('u_height__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__gte', default=None)),
        ('u_height__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='u_height__gt', default=None)),
        ('outer_width__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width__n', default=None)),
        ('outer_width__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width__lte', default=None)),
        ('outer_width__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width__lt', default=None)),
        ('outer_width__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width__gte', default=None)),
        ('outer_width__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_width__gt', default=None)),
        ('outer_depth__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth__n', default=None)),
        ('outer_depth__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth__lte', default=None)),
        ('outer_depth__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth__lt', default=None)),
        ('outer_depth__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth__gte', default=None)),
        ('outer_depth__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='outer_depth__gt', default=None)),
        ('outer_unit__n', sgqlc.types.Arg(String, graphql_name='outer_unit__n', default=None)),
        ('mounting_depth__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth__n', default=None)),
        ('mounting_depth__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth__lte', default=None)),
        ('mounting_depth__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth__lt', default=None)),
        ('mounting_depth__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth__gte', default=None)),
        ('mounting_depth__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mounting_depth__gt', default=None)),
        ('weight__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__n', default=None)),
        ('weight__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__lte', default=None)),
        ('weight__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__lt', default=None)),
        ('weight__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__gte', default=None)),
        ('weight__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='weight__gt', default=None)),
        ('max_weight__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight__n', default=None)),
        ('max_weight__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight__lte', default=None)),
        ('max_weight__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight__lt', default=None)),
        ('max_weight__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight__gte', default=None)),
        ('max_weight__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='max_weight__gt', default=None)),
        ('weight_unit__n', sgqlc.types.Arg(String, graphql_name='weight_unit__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('width__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='width__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('serial__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__n', default=None)),
        ('serial__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ic', default=None)),
        ('serial__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nic', default=None)),
        ('serial__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__iew', default=None)),
        ('serial__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__niew', default=None)),
        ('serial__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__isw', default=None)),
        ('serial__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nisw', default=None)),
        ('serial__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ie', default=None)),
        ('serial__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nie', default=None)),
        ('serial__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__empty', default=None)),
))
    )
    rackreservations = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(RackReservationType))), graphql_name='rackreservations', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('user_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='user_id', default=None)),
        ('user', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='user', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('user_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='user_id__n', default=None)),
        ('user__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='user__n', default=None)),
))
    )
    sites = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(SiteType))), graphql_name='sites', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('facility', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility', default=None)),
        ('latitude', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude', default=None)),
        ('longitude', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('asn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn', default=None)),
        ('asn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('facility__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__n', default=None)),
        ('facility__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__ic', default=None)),
        ('facility__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__nic', default=None)),
        ('facility__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__iew', default=None)),
        ('facility__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__niew', default=None)),
        ('facility__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__isw', default=None)),
        ('facility__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__nisw', default=None)),
        ('facility__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__ie', default=None)),
        ('facility__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__nie', default=None)),
        ('facility__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='facility__empty', default=None)),
        ('latitude__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__n', default=None)),
        ('latitude__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__lte', default=None)),
        ('latitude__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__lt', default=None)),
        ('latitude__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__gte', default=None)),
        ('latitude__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='latitude__gt', default=None)),
        ('longitude__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__n', default=None)),
        ('longitude__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__lte', default=None)),
        ('longitude__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__lt', default=None)),
        ('longitude__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__gte', default=None)),
        ('longitude__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='longitude__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
        ('asn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__n', default=None)),
        ('asn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn_id__n', default=None)),
))
    )
    locations = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(LocationType))), graphql_name='locations', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('parent', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('parent__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent__n', default=None)),
))
    )
    circuits = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(CircuitType))), graphql_name='circuits', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('cid', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('install_date', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='install_date', default=None)),
        ('termination_date', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_date', default=None)),
        ('commit_rate', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='commit_rate', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('provider_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_id', default=None)),
        ('provider', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider', default=None)),
        ('provider_network_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_network_id', default=None)),
        ('type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type_id', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('cid__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__n', default=None)),
        ('cid__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__ic', default=None)),
        ('cid__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__nic', default=None)),
        ('cid__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__iew', default=None)),
        ('cid__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__niew', default=None)),
        ('cid__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__isw', default=None)),
        ('cid__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__nisw', default=None)),
        ('cid__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__ie', default=None)),
        ('cid__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__nie', default=None)),
        ('cid__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cid__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('install_date__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='install_date__n', default=None)),
        ('install_date__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='install_date__lte', default=None)),
        ('install_date__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='install_date__lt', default=None)),
        ('install_date__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='install_date__gte', default=None)),
        ('install_date__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='install_date__gt', default=None)),
        ('termination_date__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_date__n', default=None)),
        ('termination_date__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_date__lte', default=None)),
        ('termination_date__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_date__lt', default=None)),
        ('termination_date__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_date__gte', default=None)),
        ('termination_date__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='termination_date__gt', default=None)),
        ('commit_rate__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='commit_rate__n', default=None)),
        ('commit_rate__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='commit_rate__lte', default=None)),
        ('commit_rate__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='commit_rate__lt', default=None)),
        ('commit_rate__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='commit_rate__gte', default=None)),
        ('commit_rate__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='commit_rate__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('provider_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_id__n', default=None)),
        ('provider__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider__n', default=None)),
        ('provider_network_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='provider_network_id__n', default=None)),
        ('type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type_id__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
))
    )
    vrfs = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('VRFType'))), graphql_name='vrfs', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('rd', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd', default=None)),
        ('enforce_unique', sgqlc.types.Arg(Boolean, graphql_name='enforce_unique', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('import_target_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='import_target_id', default=None)),
        ('import_target', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='import_target', default=None)),
        ('export_target_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='export_target_id', default=None)),
        ('export_target', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='export_target', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('rd__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__n', default=None)),
        ('rd__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__ic', default=None)),
        ('rd__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__nic', default=None)),
        ('rd__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__iew', default=None)),
        ('rd__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__niew', default=None)),
        ('rd__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__isw', default=None)),
        ('rd__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__nisw', default=None)),
        ('rd__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__ie', default=None)),
        ('rd__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__nie', default=None)),
        ('rd__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rd__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('import_target_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='import_target_id__n', default=None)),
        ('import_target__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='import_target__n', default=None)),
        ('export_target_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='export_target_id__n', default=None)),
        ('export_target__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='export_target__n', default=None)),
))
    )
    route_targets = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(RouteTargetType))), graphql_name='route_targets', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('importing_vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='importing_vrf_id', default=None)),
        ('importing_vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='importing_vrf', default=None)),
        ('exporting_vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='exporting_vrf_id', default=None)),
        ('exporting_vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='exporting_vrf', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('importing_vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='importing_vrf_id__n', default=None)),
        ('importing_vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='importing_vrf__n', default=None)),
        ('exporting_vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='exporting_vrf_id__n', default=None)),
        ('exporting_vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='exporting_vrf__n', default=None)),
))
    )
    asns = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ASNType))), graphql_name='asns', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('asn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('rir_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir_id', default=None)),
        ('rir', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('asn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__n', default=None)),
        ('asn__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__lte', default=None)),
        ('asn__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__lt', default=None)),
        ('asn__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__gte', default=None)),
        ('asn__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asn__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('rir_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir_id__n', default=None)),
        ('rir__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
))
    )
    aggregates = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(AggregateType))), graphql_name='aggregates', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('date_added', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='date_added', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('family', sgqlc.types.Arg(Int, graphql_name='family', default=None)),
        ('prefix', sgqlc.types.Arg(String, graphql_name='prefix', default=None)),
        ('rir_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir_id', default=None)),
        ('rir', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('date_added__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='date_added__n', default=None)),
        ('date_added__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='date_added__lte', default=None)),
        ('date_added__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='date_added__lt', default=None)),
        ('date_added__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='date_added__gte', default=None)),
        ('date_added__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='date_added__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('rir_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir_id__n', default=None)),
        ('rir__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rir__n', default=None)),
))
    )
    prefixes = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(PrefixType))), graphql_name='prefixes', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('is_pool', sgqlc.types.Arg(Boolean, graphql_name='is_pool', default=None)),
        ('mark_utilized', sgqlc.types.Arg(Boolean, graphql_name='mark_utilized', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('family', sgqlc.types.Arg(Int, graphql_name='family', default=None)),
        ('prefix', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='prefix', default=None)),
        ('within', sgqlc.types.Arg(String, graphql_name='within', default=None)),
        ('within_include', sgqlc.types.Arg(String, graphql_name='within_include', default=None)),
        ('contains', sgqlc.types.Arg(String, graphql_name='contains', default=None)),
        ('depth', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth', default=None)),
        ('children', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children', default=None)),
        ('mask_length', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mask_length', default=None)),
        ('mask_length__gte', sgqlc.types.Arg(Int, graphql_name='mask_length__gte', default=None)),
        ('mask_length__lte', sgqlc.types.Arg(Int, graphql_name='mask_length__lte', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('present_in_vrf_id', sgqlc.types.Arg(String, graphql_name='present_in_vrf_id', default=None)),
        ('present_in_vrf', sgqlc.types.Arg(String, graphql_name='present_in_vrf', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('vlan_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vlan_id', default=None)),
        ('vlan_vid', sgqlc.types.Arg(Int, graphql_name='vlan_vid', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('depth__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__n', default=None)),
        ('depth__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__lte', default=None)),
        ('depth__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__lt', default=None)),
        ('depth__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__gte', default=None)),
        ('depth__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__gt', default=None)),
        ('children__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__n', default=None)),
        ('children__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__lte', default=None)),
        ('children__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__lt', default=None)),
        ('children__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__gte', default=None)),
        ('children__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__gt', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('vlan_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vlan_id__n', default=None)),
        ('vlan_vid__n', sgqlc.types.Arg(Int, graphql_name='vlan_vid__n', default=None)),
        ('vlan_vid__lte', sgqlc.types.Arg(Int, graphql_name='vlan_vid__lte', default=None)),
        ('vlan_vid__lt', sgqlc.types.Arg(Int, graphql_name='vlan_vid__lt', default=None)),
        ('vlan_vid__gte', sgqlc.types.Arg(Int, graphql_name='vlan_vid__gte', default=None)),
        ('vlan_vid__gt', sgqlc.types.Arg(Int, graphql_name='vlan_vid__gt', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
))
    )
    ip_ranges = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(IPRangeType))), graphql_name='ip_ranges', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('family', sgqlc.types.Arg(Int, graphql_name='family', default=None)),
        ('start_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='start_address', default=None)),
        ('end_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='end_address', default=None)),
        ('contains', sgqlc.types.Arg(String, graphql_name='contains', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
))
    )
    ip_addresses = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(IPAddressType))), graphql_name='ip_addresses', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('dns_name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('family', sgqlc.types.Arg(Int, graphql_name='family', default=None)),
        ('parent', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent', default=None)),
        ('address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address', default=None)),
        ('mask_length', sgqlc.types.Arg(Int, graphql_name='mask_length', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('present_in_vrf_id', sgqlc.types.Arg(String, graphql_name='present_in_vrf_id', default=None)),
        ('present_in_vrf', sgqlc.types.Arg(String, graphql_name='present_in_vrf', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('virtual_machine', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine', default=None)),
        ('virtual_machine_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id', default=None)),
        ('interface', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface', default=None)),
        ('interface_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_id', default=None)),
        ('vminterface', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vminterface', default=None)),
        ('vminterface_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vminterface_id', default=None)),
        ('fhrpgroup_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='fhrpgroup_id', default=None)),
        ('assigned_to_interface', sgqlc.types.Arg(Boolean, graphql_name='assigned_to_interface', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('dns_name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__n', default=None)),
        ('dns_name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__ic', default=None)),
        ('dns_name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__nic', default=None)),
        ('dns_name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__iew', default=None)),
        ('dns_name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__niew', default=None)),
        ('dns_name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__isw', default=None)),
        ('dns_name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__nisw', default=None)),
        ('dns_name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__ie', default=None)),
        ('dns_name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__nie', default=None)),
        ('dns_name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('interface__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface__n', default=None)),
        ('interface_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_id__n', default=None)),
        ('vminterface__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vminterface__n', default=None)),
        ('vminterface_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vminterface_id__n', default=None)),
        ('fhrpgroup_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='fhrpgroup_id__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
))
    )
    l2vpns = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(L2VPNType))), graphql_name='l2vpns')
    clusters = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ClusterType))), graphql_name='clusters', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type_id', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
        ('type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type_id__n', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
))
    )
    virtual_machines = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('VirtualMachineType'))), graphql_name='virtual_machines', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('cluster', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster', default=None)),
        ('vcpus', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus', default=None)),
        ('memory', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory', default=None)),
        ('disk', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('local_context_data', sgqlc.types.Arg(Boolean, graphql_name='local_context_data', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('cluster_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group_id', default=None)),
        ('cluster_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group', default=None)),
        ('cluster_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type_id', default=None)),
        ('cluster_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type', default=None)),
        ('cluster_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('platform_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id', default=None)),
        ('platform', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('has_primary_ip', sgqlc.types.Arg(Boolean, graphql_name='has_primary_ip', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('cluster__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster__n', default=None)),
        ('vcpus__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__n', default=None)),
        ('vcpus__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__lte', default=None)),
        ('vcpus__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__lt', default=None)),
        ('vcpus__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__gte', default=None)),
        ('vcpus__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vcpus__gt', default=None)),
        ('memory__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__n', default=None)),
        ('memory__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__lte', default=None)),
        ('memory__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__lt', default=None)),
        ('memory__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__gte', default=None)),
        ('memory__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='memory__gt', default=None)),
        ('disk__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__n', default=None)),
        ('disk__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__lte', default=None)),
        ('disk__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__lt', default=None)),
        ('disk__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__gte', default=None)),
        ('disk__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='disk__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('cluster_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group_id__n', default=None)),
        ('cluster_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_group__n', default=None)),
        ('cluster_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type_id__n', default=None)),
        ('cluster_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_type__n', default=None)),
        ('cluster_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('platform_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id__n', default=None)),
        ('platform__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
))
    )
    vlans = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('VLANType'))), graphql_name='vlans', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('vid', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('available_on_device', sgqlc.types.Arg(String, graphql_name='available_on_device', default=None)),
        ('available_on_virtualmachine', sgqlc.types.Arg(String, graphql_name='available_on_virtualmachine', default=None)),
        ('l2vpn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id', default=None)),
        ('l2vpn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('vid__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__n', default=None)),
        ('vid__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__lte', default=None)),
        ('vid__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__lt', default=None)),
        ('vid__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__gte', default=None)),
        ('vid__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('l2vpn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id__n', default=None)),
        ('l2vpn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of(JournalEntryType), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class UserType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'password', 'groups', 'username', 'first_name', 'last_name', 'email', 'is_staff', 'is_active', 'date_joined')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    password = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='password')
    groups = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(GroupType))), graphql_name='groups', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
))
    )
    username = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='username')
    first_name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='first_name')
    last_name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='last_name')
    email = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='email')
    is_staff = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='is_staff')
    is_active = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='is_active')
    date_joined = sgqlc.types.Field(sgqlc.types.non_null(DateTime), graphql_name='date_joined')


class VLANGroupType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'name', 'slug', 'min_vid', 'max_vid', 'vlans', 'display', 'class_type', 'custom_fields', 'changelog', 'scope')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of(TagType), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    slug = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='slug')
    min_vid = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='min_vid')
    max_vid = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='max_vid')
    vlans = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('VLANType'))), graphql_name='vlans', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('vid', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('available_on_device', sgqlc.types.Arg(String, graphql_name='available_on_device', default=None)),
        ('available_on_virtualmachine', sgqlc.types.Arg(String, graphql_name='available_on_virtualmachine', default=None)),
        ('l2vpn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id', default=None)),
        ('l2vpn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('vid__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__n', default=None)),
        ('vid__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__lte', default=None)),
        ('vid__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__lt', default=None)),
        ('vid__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__gte', default=None)),
        ('vid__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('l2vpn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id__n', default=None)),
        ('l2vpn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')
    scope = sgqlc.types.Field('VLANGroupScopeType', graphql_name='scope')


class VLANType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'site', 'group', 'vid', 'name', 'tenant', 'status', 'role', 'wirelesslan_set', 'interfaces_as_untagged', 'interfaces_as_tagged', 'prefixes', 'vminterfaces_as_untagged', 'vminterfaces_as_tagged', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of(TagType), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    site = sgqlc.types.Field(SiteType, graphql_name='site')
    group = sgqlc.types.Field(VLANGroupType, graphql_name='group')
    vid = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='vid')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    tenant = sgqlc.types.Field(TenantType, graphql_name='tenant')
    status = sgqlc.types.Field(sgqlc.types.non_null(IpamVLANStatusChoices), graphql_name='status')
    role = sgqlc.types.Field(RoleType, graphql_name='role')
    wirelesslan_set = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('WirelessLANType'))), graphql_name='wirelesslan_set', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('ssid', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid', default=None)),
        ('auth_psk', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('vlan_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vlan_id', default=None)),
        ('auth_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_type', default=None)),
        ('auth_cipher', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_cipher', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('ssid__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__n', default=None)),
        ('ssid__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__ic', default=None)),
        ('ssid__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__nic', default=None)),
        ('ssid__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__iew', default=None)),
        ('ssid__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__niew', default=None)),
        ('ssid__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__isw', default=None)),
        ('ssid__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__nisw', default=None)),
        ('ssid__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__ie', default=None)),
        ('ssid__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__nie', default=None)),
        ('ssid__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__empty', default=None)),
        ('auth_psk__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__n', default=None)),
        ('auth_psk__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__ic', default=None)),
        ('auth_psk__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__nic', default=None)),
        ('auth_psk__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__iew', default=None)),
        ('auth_psk__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__niew', default=None)),
        ('auth_psk__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__isw', default=None)),
        ('auth_psk__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__nisw', default=None)),
        ('auth_psk__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__ie', default=None)),
        ('auth_psk__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__nie', default=None)),
        ('auth_psk__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('vlan_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vlan_id__n', default=None)),
        ('auth_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_type__n', default=None)),
        ('auth_cipher__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_cipher__n', default=None)),
))
    )
    interfaces_as_untagged = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(InterfaceType))), graphql_name='interfaces_as_untagged', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('enabled', sgqlc.types.Arg(Boolean, graphql_name='enabled', default=None)),
        ('mtu', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu', default=None)),
        ('mgmt_only', sgqlc.types.Arg(Boolean, graphql_name='mgmt_only', default=None)),
        ('poe_mode', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode', default=None)),
        ('poe_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type', default=None)),
        ('mode', sgqlc.types.Arg(String, graphql_name='mode', default=None)),
        ('rf_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_role', default=None)),
        ('rf_channel', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel', default=None)),
        ('rf_channel_frequency', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency', default=None)),
        ('rf_channel_width', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width', default=None)),
        ('tx_power', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('kind', sgqlc.types.Arg(String, graphql_name='kind', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('bridge_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id', default=None)),
        ('lag_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='lag_id', default=None)),
        ('speed', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed', default=None)),
        ('duplex', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='duplex', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('wwn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn', default=None)),
        ('vlan_id', sgqlc.types.Arg(String, graphql_name='vlan_id', default=None)),
        ('vlan', sgqlc.types.Arg(String, graphql_name='vlan', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('vdc_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_id', default=None)),
        ('vdc_identifier', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_identifier', default=None)),
        ('vdc', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc', default=None)),
        ('l2vpn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id', default=None)),
        ('l2vpn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('mtu__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__n', default=None)),
        ('mtu__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lte', default=None)),
        ('mtu__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lt', default=None)),
        ('mtu__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gte', default=None)),
        ('mtu__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gt', default=None)),
        ('poe_mode__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode__n', default=None)),
        ('poe_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type__n', default=None)),
        ('mode__n', sgqlc.types.Arg(String, graphql_name='mode__n', default=None)),
        ('rf_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_role__n', default=None)),
        ('rf_channel__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel__n', default=None)),
        ('rf_channel_frequency__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__n', default=None)),
        ('rf_channel_frequency__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__lte', default=None)),
        ('rf_channel_frequency__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__lt', default=None)),
        ('rf_channel_frequency__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__gte', default=None)),
        ('rf_channel_frequency__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__gt', default=None)),
        ('rf_channel_width__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__n', default=None)),
        ('rf_channel_width__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__lte', default=None)),
        ('rf_channel_width__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__lt', default=None)),
        ('rf_channel_width__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__gte', default=None)),
        ('rf_channel_width__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__gt', default=None)),
        ('tx_power__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__n', default=None)),
        ('tx_power__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__lte', default=None)),
        ('tx_power__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__lt', default=None)),
        ('tx_power__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__gte', default=None)),
        ('tx_power__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('bridge_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id__n', default=None)),
        ('lag_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='lag_id__n', default=None)),
        ('speed__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__n', default=None)),
        ('speed__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__lte', default=None)),
        ('speed__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__lt', default=None)),
        ('speed__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__gte', default=None)),
        ('speed__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__gt', default=None)),
        ('duplex__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='duplex__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('wwn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__n', default=None)),
        ('wwn__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__ic', default=None)),
        ('wwn__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nic', default=None)),
        ('wwn__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__iew', default=None)),
        ('wwn__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__niew', default=None)),
        ('wwn__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__isw', default=None)),
        ('wwn__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nisw', default=None)),
        ('wwn__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__ie', default=None)),
        ('wwn__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nie', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('vdc_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_id__n', default=None)),
        ('vdc_identifier__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_identifier__n', default=None)),
        ('vdc__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc__n', default=None)),
        ('l2vpn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id__n', default=None)),
        ('l2vpn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn__n', default=None)),
))
    )
    interfaces_as_tagged = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(InterfaceType))), graphql_name='interfaces_as_tagged', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('enabled', sgqlc.types.Arg(Boolean, graphql_name='enabled', default=None)),
        ('mtu', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu', default=None)),
        ('mgmt_only', sgqlc.types.Arg(Boolean, graphql_name='mgmt_only', default=None)),
        ('poe_mode', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode', default=None)),
        ('poe_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type', default=None)),
        ('mode', sgqlc.types.Arg(String, graphql_name='mode', default=None)),
        ('rf_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_role', default=None)),
        ('rf_channel', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel', default=None)),
        ('rf_channel_frequency', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency', default=None)),
        ('rf_channel_width', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width', default=None)),
        ('tx_power', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('kind', sgqlc.types.Arg(String, graphql_name='kind', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('bridge_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id', default=None)),
        ('lag_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='lag_id', default=None)),
        ('speed', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed', default=None)),
        ('duplex', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='duplex', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('wwn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn', default=None)),
        ('vlan_id', sgqlc.types.Arg(String, graphql_name='vlan_id', default=None)),
        ('vlan', sgqlc.types.Arg(String, graphql_name='vlan', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('vdc_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_id', default=None)),
        ('vdc_identifier', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_identifier', default=None)),
        ('vdc', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc', default=None)),
        ('l2vpn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id', default=None)),
        ('l2vpn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('mtu__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__n', default=None)),
        ('mtu__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lte', default=None)),
        ('mtu__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lt', default=None)),
        ('mtu__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gte', default=None)),
        ('mtu__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gt', default=None)),
        ('poe_mode__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode__n', default=None)),
        ('poe_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type__n', default=None)),
        ('mode__n', sgqlc.types.Arg(String, graphql_name='mode__n', default=None)),
        ('rf_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_role__n', default=None)),
        ('rf_channel__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel__n', default=None)),
        ('rf_channel_frequency__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__n', default=None)),
        ('rf_channel_frequency__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__lte', default=None)),
        ('rf_channel_frequency__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__lt', default=None)),
        ('rf_channel_frequency__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__gte', default=None)),
        ('rf_channel_frequency__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__gt', default=None)),
        ('rf_channel_width__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__n', default=None)),
        ('rf_channel_width__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__lte', default=None)),
        ('rf_channel_width__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__lt', default=None)),
        ('rf_channel_width__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__gte', default=None)),
        ('rf_channel_width__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__gt', default=None)),
        ('tx_power__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__n', default=None)),
        ('tx_power__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__lte', default=None)),
        ('tx_power__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__lt', default=None)),
        ('tx_power__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__gte', default=None)),
        ('tx_power__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('bridge_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id__n', default=None)),
        ('lag_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='lag_id__n', default=None)),
        ('speed__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__n', default=None)),
        ('speed__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__lte', default=None)),
        ('speed__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__lt', default=None)),
        ('speed__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__gte', default=None)),
        ('speed__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__gt', default=None)),
        ('duplex__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='duplex__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('wwn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__n', default=None)),
        ('wwn__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__ic', default=None)),
        ('wwn__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nic', default=None)),
        ('wwn__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__iew', default=None)),
        ('wwn__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__niew', default=None)),
        ('wwn__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__isw', default=None)),
        ('wwn__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nisw', default=None)),
        ('wwn__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__ie', default=None)),
        ('wwn__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nie', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('vdc_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_id__n', default=None)),
        ('vdc_identifier__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_identifier__n', default=None)),
        ('vdc__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc__n', default=None)),
        ('l2vpn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id__n', default=None)),
        ('l2vpn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn__n', default=None)),
))
    )
    prefixes = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(PrefixType))), graphql_name='prefixes', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('is_pool', sgqlc.types.Arg(Boolean, graphql_name='is_pool', default=None)),
        ('mark_utilized', sgqlc.types.Arg(Boolean, graphql_name='mark_utilized', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('family', sgqlc.types.Arg(Int, graphql_name='family', default=None)),
        ('prefix', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='prefix', default=None)),
        ('within', sgqlc.types.Arg(String, graphql_name='within', default=None)),
        ('within_include', sgqlc.types.Arg(String, graphql_name='within_include', default=None)),
        ('contains', sgqlc.types.Arg(String, graphql_name='contains', default=None)),
        ('depth', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth', default=None)),
        ('children', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children', default=None)),
        ('mask_length', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mask_length', default=None)),
        ('mask_length__gte', sgqlc.types.Arg(Int, graphql_name='mask_length__gte', default=None)),
        ('mask_length__lte', sgqlc.types.Arg(Int, graphql_name='mask_length__lte', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('present_in_vrf_id', sgqlc.types.Arg(String, graphql_name='present_in_vrf_id', default=None)),
        ('present_in_vrf', sgqlc.types.Arg(String, graphql_name='present_in_vrf', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('vlan_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vlan_id', default=None)),
        ('vlan_vid', sgqlc.types.Arg(Int, graphql_name='vlan_vid', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('depth__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__n', default=None)),
        ('depth__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__lte', default=None)),
        ('depth__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__lt', default=None)),
        ('depth__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__gte', default=None)),
        ('depth__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__gt', default=None)),
        ('children__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__n', default=None)),
        ('children__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__lte', default=None)),
        ('children__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__lt', default=None)),
        ('children__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__gte', default=None)),
        ('children__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__gt', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('vlan_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vlan_id__n', default=None)),
        ('vlan_vid__n', sgqlc.types.Arg(Int, graphql_name='vlan_vid__n', default=None)),
        ('vlan_vid__lte', sgqlc.types.Arg(Int, graphql_name='vlan_vid__lte', default=None)),
        ('vlan_vid__lt', sgqlc.types.Arg(Int, graphql_name='vlan_vid__lt', default=None)),
        ('vlan_vid__gte', sgqlc.types.Arg(Int, graphql_name='vlan_vid__gte', default=None)),
        ('vlan_vid__gt', sgqlc.types.Arg(Int, graphql_name='vlan_vid__gt', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
))
    )
    vminterfaces_as_untagged = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('VMInterfaceType'))), graphql_name='vminterfaces_as_untagged', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('enabled', sgqlc.types.Arg(Boolean, graphql_name='enabled', default=None)),
        ('mtu', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cluster_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id', default=None)),
        ('cluster', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster', default=None)),
        ('virtual_machine_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id', default=None)),
        ('virtual_machine', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('bridge_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('l2vpn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id', default=None)),
        ('l2vpn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('mtu__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__n', default=None)),
        ('mtu__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lte', default=None)),
        ('mtu__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lt', default=None)),
        ('mtu__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gte', default=None)),
        ('mtu__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('cluster_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id__n', default=None)),
        ('cluster__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster__n', default=None)),
        ('virtual_machine_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id__n', default=None)),
        ('virtual_machine__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('bridge_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('l2vpn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id__n', default=None)),
        ('l2vpn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn__n', default=None)),
))
    )
    vminterfaces_as_tagged = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('VMInterfaceType'))), graphql_name='vminterfaces_as_tagged', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('enabled', sgqlc.types.Arg(Boolean, graphql_name='enabled', default=None)),
        ('mtu', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cluster_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id', default=None)),
        ('cluster', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster', default=None)),
        ('virtual_machine_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id', default=None)),
        ('virtual_machine', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('bridge_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('l2vpn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id', default=None)),
        ('l2vpn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('mtu__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__n', default=None)),
        ('mtu__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lte', default=None)),
        ('mtu__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lt', default=None)),
        ('mtu__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gte', default=None)),
        ('mtu__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('cluster_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id__n', default=None)),
        ('cluster__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster__n', default=None)),
        ('virtual_machine_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id__n', default=None)),
        ('virtual_machine__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('bridge_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('l2vpn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id__n', default=None)),
        ('l2vpn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of(JournalEntryType), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class VMInterfaceType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'enabled', 'mac_address', 'mtu', 'mode', 'parent', 'bridge', 'virtual_machine', 'name', '_name', 'description', 'untagged_vlan', 'tagged_vlans', 'vrf', 'child_interfaces', 'bridge_interfaces', 'display', 'class_type', 'custom_fields', 'changelog', 'ip_addresses')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of(TagType), graphql_name='tags')
    enabled = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='enabled')
    mac_address = sgqlc.types.Field(String, graphql_name='mac_address')
    mtu = sgqlc.types.Field(Int, graphql_name='mtu')
    mode = sgqlc.types.Field(VirtualizationVMInterfaceModeChoices, graphql_name='mode')
    parent = sgqlc.types.Field('VMInterfaceType', graphql_name='parent')
    bridge = sgqlc.types.Field('VMInterfaceType', graphql_name='bridge')
    virtual_machine = sgqlc.types.Field(sgqlc.types.non_null('VirtualMachineType'), graphql_name='virtual_machine')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    _name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='_name')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    untagged_vlan = sgqlc.types.Field(VLANType, graphql_name='untagged_vlan')
    tagged_vlans = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(VLANType))), graphql_name='tagged_vlans', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('vid', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('available_on_device', sgqlc.types.Arg(String, graphql_name='available_on_device', default=None)),
        ('available_on_virtualmachine', sgqlc.types.Arg(String, graphql_name='available_on_virtualmachine', default=None)),
        ('l2vpn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id', default=None)),
        ('l2vpn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('vid__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__n', default=None)),
        ('vid__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__lte', default=None)),
        ('vid__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__lt', default=None)),
        ('vid__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__gte', default=None)),
        ('vid__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vid__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('l2vpn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id__n', default=None)),
        ('l2vpn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn__n', default=None)),
))
    )
    vrf = sgqlc.types.Field('VRFType', graphql_name='vrf')
    child_interfaces = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('VMInterfaceType'))), graphql_name='child_interfaces', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('enabled', sgqlc.types.Arg(Boolean, graphql_name='enabled', default=None)),
        ('mtu', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cluster_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id', default=None)),
        ('cluster', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster', default=None)),
        ('virtual_machine_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id', default=None)),
        ('virtual_machine', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('bridge_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('l2vpn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id', default=None)),
        ('l2vpn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('mtu__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__n', default=None)),
        ('mtu__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lte', default=None)),
        ('mtu__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lt', default=None)),
        ('mtu__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gte', default=None)),
        ('mtu__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('cluster_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id__n', default=None)),
        ('cluster__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster__n', default=None)),
        ('virtual_machine_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id__n', default=None)),
        ('virtual_machine__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('bridge_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('l2vpn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id__n', default=None)),
        ('l2vpn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn__n', default=None)),
))
    )
    bridge_interfaces = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('VMInterfaceType'))), graphql_name='bridge_interfaces', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('enabled', sgqlc.types.Arg(Boolean, graphql_name='enabled', default=None)),
        ('mtu', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cluster_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id', default=None)),
        ('cluster', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster', default=None)),
        ('virtual_machine_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id', default=None)),
        ('virtual_machine', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('bridge_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('l2vpn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id', default=None)),
        ('l2vpn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('mtu__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__n', default=None)),
        ('mtu__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lte', default=None)),
        ('mtu__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lt', default=None)),
        ('mtu__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gte', default=None)),
        ('mtu__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('cluster_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id__n', default=None)),
        ('cluster__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster__n', default=None)),
        ('virtual_machine_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id__n', default=None)),
        ('virtual_machine__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('bridge_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('l2vpn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id__n', default=None)),
        ('l2vpn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')
    ip_addresses = sgqlc.types.Field(sgqlc.types.list_of(IPAddressType), graphql_name='ip_addresses')


class VRFType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'name', 'rd', 'tenant', 'enforce_unique', 'import_targets', 'export_targets', 'interfaces', 'prefixes', 'ip_ranges', 'ip_addresses', 'vminterfaces', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of(TagType), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    rd = sgqlc.types.Field(String, graphql_name='rd')
    tenant = sgqlc.types.Field(TenantType, graphql_name='tenant')
    enforce_unique = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='enforce_unique')
    import_targets = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(RouteTargetType))), graphql_name='import_targets', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('importing_vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='importing_vrf_id', default=None)),
        ('importing_vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='importing_vrf', default=None)),
        ('exporting_vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='exporting_vrf_id', default=None)),
        ('exporting_vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='exporting_vrf', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('importing_vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='importing_vrf_id__n', default=None)),
        ('importing_vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='importing_vrf__n', default=None)),
        ('exporting_vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='exporting_vrf_id__n', default=None)),
        ('exporting_vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='exporting_vrf__n', default=None)),
))
    )
    export_targets = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(RouteTargetType))), graphql_name='export_targets', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('importing_vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='importing_vrf_id', default=None)),
        ('importing_vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='importing_vrf', default=None)),
        ('exporting_vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='exporting_vrf_id', default=None)),
        ('exporting_vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='exporting_vrf', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('importing_vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='importing_vrf_id__n', default=None)),
        ('importing_vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='importing_vrf__n', default=None)),
        ('exporting_vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='exporting_vrf_id__n', default=None)),
        ('exporting_vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='exporting_vrf__n', default=None)),
))
    )
    interfaces = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(InterfaceType))), graphql_name='interfaces', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('enabled', sgqlc.types.Arg(Boolean, graphql_name='enabled', default=None)),
        ('mtu', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu', default=None)),
        ('mgmt_only', sgqlc.types.Arg(Boolean, graphql_name='mgmt_only', default=None)),
        ('poe_mode', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode', default=None)),
        ('poe_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type', default=None)),
        ('mode', sgqlc.types.Arg(String, graphql_name='mode', default=None)),
        ('rf_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_role', default=None)),
        ('rf_channel', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel', default=None)),
        ('rf_channel_frequency', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency', default=None)),
        ('rf_channel_width', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width', default=None)),
        ('tx_power', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('kind', sgqlc.types.Arg(String, graphql_name='kind', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('bridge_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id', default=None)),
        ('lag_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='lag_id', default=None)),
        ('speed', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed', default=None)),
        ('duplex', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='duplex', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('wwn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn', default=None)),
        ('vlan_id', sgqlc.types.Arg(String, graphql_name='vlan_id', default=None)),
        ('vlan', sgqlc.types.Arg(String, graphql_name='vlan', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('vdc_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_id', default=None)),
        ('vdc_identifier', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_identifier', default=None)),
        ('vdc', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc', default=None)),
        ('l2vpn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id', default=None)),
        ('l2vpn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('mtu__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__n', default=None)),
        ('mtu__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lte', default=None)),
        ('mtu__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lt', default=None)),
        ('mtu__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gte', default=None)),
        ('mtu__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gt', default=None)),
        ('poe_mode__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode__n', default=None)),
        ('poe_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type__n', default=None)),
        ('mode__n', sgqlc.types.Arg(String, graphql_name='mode__n', default=None)),
        ('rf_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_role__n', default=None)),
        ('rf_channel__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel__n', default=None)),
        ('rf_channel_frequency__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__n', default=None)),
        ('rf_channel_frequency__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__lte', default=None)),
        ('rf_channel_frequency__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__lt', default=None)),
        ('rf_channel_frequency__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__gte', default=None)),
        ('rf_channel_frequency__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__gt', default=None)),
        ('rf_channel_width__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__n', default=None)),
        ('rf_channel_width__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__lte', default=None)),
        ('rf_channel_width__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__lt', default=None)),
        ('rf_channel_width__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__gte', default=None)),
        ('rf_channel_width__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__gt', default=None)),
        ('tx_power__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__n', default=None)),
        ('tx_power__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__lte', default=None)),
        ('tx_power__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__lt', default=None)),
        ('tx_power__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__gte', default=None)),
        ('tx_power__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('bridge_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id__n', default=None)),
        ('lag_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='lag_id__n', default=None)),
        ('speed__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__n', default=None)),
        ('speed__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__lte', default=None)),
        ('speed__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__lt', default=None)),
        ('speed__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__gte', default=None)),
        ('speed__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__gt', default=None)),
        ('duplex__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='duplex__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('wwn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__n', default=None)),
        ('wwn__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__ic', default=None)),
        ('wwn__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nic', default=None)),
        ('wwn__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__iew', default=None)),
        ('wwn__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__niew', default=None)),
        ('wwn__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__isw', default=None)),
        ('wwn__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nisw', default=None)),
        ('wwn__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__ie', default=None)),
        ('wwn__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nie', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('vdc_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_id__n', default=None)),
        ('vdc_identifier__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_identifier__n', default=None)),
        ('vdc__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc__n', default=None)),
        ('l2vpn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id__n', default=None)),
        ('l2vpn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn__n', default=None)),
))
    )
    prefixes = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(PrefixType))), graphql_name='prefixes', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('is_pool', sgqlc.types.Arg(Boolean, graphql_name='is_pool', default=None)),
        ('mark_utilized', sgqlc.types.Arg(Boolean, graphql_name='mark_utilized', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('family', sgqlc.types.Arg(Int, graphql_name='family', default=None)),
        ('prefix', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='prefix', default=None)),
        ('within', sgqlc.types.Arg(String, graphql_name='within', default=None)),
        ('within_include', sgqlc.types.Arg(String, graphql_name='within_include', default=None)),
        ('contains', sgqlc.types.Arg(String, graphql_name='contains', default=None)),
        ('depth', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth', default=None)),
        ('children', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children', default=None)),
        ('mask_length', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mask_length', default=None)),
        ('mask_length__gte', sgqlc.types.Arg(Int, graphql_name='mask_length__gte', default=None)),
        ('mask_length__lte', sgqlc.types.Arg(Int, graphql_name='mask_length__lte', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('present_in_vrf_id', sgqlc.types.Arg(String, graphql_name='present_in_vrf_id', default=None)),
        ('present_in_vrf', sgqlc.types.Arg(String, graphql_name='present_in_vrf', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('vlan_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vlan_id', default=None)),
        ('vlan_vid', sgqlc.types.Arg(Int, graphql_name='vlan_vid', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('depth__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__n', default=None)),
        ('depth__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__lte', default=None)),
        ('depth__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__lt', default=None)),
        ('depth__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__gte', default=None)),
        ('depth__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='depth__gt', default=None)),
        ('children__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__n', default=None)),
        ('children__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__lte', default=None)),
        ('children__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__lt', default=None)),
        ('children__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__gte', default=None)),
        ('children__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='children__gt', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('vlan_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vlan_id__n', default=None)),
        ('vlan_vid__n', sgqlc.types.Arg(Int, graphql_name='vlan_vid__n', default=None)),
        ('vlan_vid__lte', sgqlc.types.Arg(Int, graphql_name='vlan_vid__lte', default=None)),
        ('vlan_vid__lt', sgqlc.types.Arg(Int, graphql_name='vlan_vid__lt', default=None)),
        ('vlan_vid__gte', sgqlc.types.Arg(Int, graphql_name='vlan_vid__gte', default=None)),
        ('vlan_vid__gt', sgqlc.types.Arg(Int, graphql_name='vlan_vid__gt', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
))
    )
    ip_ranges = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(IPRangeType))), graphql_name='ip_ranges', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('family', sgqlc.types.Arg(Int, graphql_name='family', default=None)),
        ('start_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='start_address', default=None)),
        ('end_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='end_address', default=None)),
        ('contains', sgqlc.types.Arg(String, graphql_name='contains', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
))
    )
    ip_addresses = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(IPAddressType))), graphql_name='ip_addresses', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('dns_name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('family', sgqlc.types.Arg(Int, graphql_name='family', default=None)),
        ('parent', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent', default=None)),
        ('address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='address', default=None)),
        ('mask_length', sgqlc.types.Arg(Int, graphql_name='mask_length', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('present_in_vrf_id', sgqlc.types.Arg(String, graphql_name='present_in_vrf_id', default=None)),
        ('present_in_vrf', sgqlc.types.Arg(String, graphql_name='present_in_vrf', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('virtual_machine', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine', default=None)),
        ('virtual_machine_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id', default=None)),
        ('interface', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface', default=None)),
        ('interface_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_id', default=None)),
        ('vminterface', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vminterface', default=None)),
        ('vminterface_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vminterface_id', default=None)),
        ('fhrpgroup_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='fhrpgroup_id', default=None)),
        ('assigned_to_interface', sgqlc.types.Arg(Boolean, graphql_name='assigned_to_interface', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('dns_name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__n', default=None)),
        ('dns_name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__ic', default=None)),
        ('dns_name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__nic', default=None)),
        ('dns_name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__iew', default=None)),
        ('dns_name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__niew', default=None)),
        ('dns_name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__isw', default=None)),
        ('dns_name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__nisw', default=None)),
        ('dns_name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__ie', default=None)),
        ('dns_name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__nie', default=None)),
        ('dns_name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='dns_name__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('interface__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface__n', default=None)),
        ('interface_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='interface_id__n', default=None)),
        ('vminterface__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vminterface__n', default=None)),
        ('vminterface_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vminterface_id__n', default=None)),
        ('fhrpgroup_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='fhrpgroup_id__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
))
    )
    vminterfaces = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(VMInterfaceType))), graphql_name='vminterfaces', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('enabled', sgqlc.types.Arg(Boolean, graphql_name='enabled', default=None)),
        ('mtu', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cluster_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id', default=None)),
        ('cluster', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster', default=None)),
        ('virtual_machine_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id', default=None)),
        ('virtual_machine', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('bridge_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('l2vpn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id', default=None)),
        ('l2vpn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('mtu__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__n', default=None)),
        ('mtu__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lte', default=None)),
        ('mtu__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lt', default=None)),
        ('mtu__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gte', default=None)),
        ('mtu__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('cluster_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id__n', default=None)),
        ('cluster__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster__n', default=None)),
        ('virtual_machine_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id__n', default=None)),
        ('virtual_machine__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('bridge_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('l2vpn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id__n', default=None)),
        ('l2vpn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of(JournalEntryType), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class VirtualChassisType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'master', 'name', 'domain', 'members', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of(TagType), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    master = sgqlc.types.Field(DeviceType, graphql_name='master')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    domain = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='domain')
    members = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(DeviceType))), graphql_name='members', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('asset_tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag', default=None)),
        ('face', sgqlc.types.Arg(String, graphql_name='face', default=None)),
        ('position', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position', default=None)),
        ('airflow', sgqlc.types.Arg(String, graphql_name='airflow', default=None)),
        ('vc_position', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position', default=None)),
        ('vc_priority', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('contact', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact', default=None)),
        ('contact_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role', default=None)),
        ('contact_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group', default=None)),
        ('local_context_data', sgqlc.types.Arg(Boolean, graphql_name='local_context_data', default=None)),
        ('manufacturer_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id', default=None)),
        ('manufacturer', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer', default=None)),
        ('device_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type', default=None)),
        ('device_type_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type_id', default=None)),
        ('role_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id', default=None)),
        ('role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role', default=None)),
        ('parent_device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_device_id', default=None)),
        ('platform_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id', default=None)),
        ('platform', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('cluster_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id', default=None)),
        ('model', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('is_full_depth', sgqlc.types.Arg(Boolean, graphql_name='is_full_depth', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('serial', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial', default=None)),
        ('has_primary_ip', sgqlc.types.Arg(Boolean, graphql_name='has_primary_ip', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis_member', sgqlc.types.Arg(Boolean, graphql_name='virtual_chassis_member', default=None)),
        ('console_ports', sgqlc.types.Arg(Boolean, graphql_name='console_ports', default=None)),
        ('console_server_ports', sgqlc.types.Arg(Boolean, graphql_name='console_server_ports', default=None)),
        ('power_ports', sgqlc.types.Arg(Boolean, graphql_name='power_ports', default=None)),
        ('power_outlets', sgqlc.types.Arg(Boolean, graphql_name='power_outlets', default=None)),
        ('interfaces', sgqlc.types.Arg(Boolean, graphql_name='interfaces', default=None)),
        ('pass_through_ports', sgqlc.types.Arg(Boolean, graphql_name='pass_through_ports', default=None)),
        ('module_bays', sgqlc.types.Arg(Boolean, graphql_name='module_bays', default=None)),
        ('device_bays', sgqlc.types.Arg(Boolean, graphql_name='device_bays', default=None)),
        ('primary_ip4_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip4_id', default=None)),
        ('primary_ip6_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip6_id', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('asset_tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__n', default=None)),
        ('asset_tag__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ic', default=None)),
        ('asset_tag__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nic', default=None)),
        ('asset_tag__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__iew', default=None)),
        ('asset_tag__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__niew', default=None)),
        ('asset_tag__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__isw', default=None)),
        ('asset_tag__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nisw', default=None)),
        ('asset_tag__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__ie', default=None)),
        ('asset_tag__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__nie', default=None)),
        ('asset_tag__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='asset_tag__empty', default=None)),
        ('face__n', sgqlc.types.Arg(String, graphql_name='face__n', default=None)),
        ('position__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__n', default=None)),
        ('position__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__lte', default=None)),
        ('position__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__lt', default=None)),
        ('position__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__gte', default=None)),
        ('position__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='position__gt', default=None)),
        ('airflow__n', sgqlc.types.Arg(String, graphql_name='airflow__n', default=None)),
        ('vc_position__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__n', default=None)),
        ('vc_position__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__lte', default=None)),
        ('vc_position__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__lt', default=None)),
        ('vc_position__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__gte', default=None)),
        ('vc_position__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_position__gt', default=None)),
        ('vc_priority__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__n', default=None)),
        ('vc_priority__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__lte', default=None)),
        ('vc_priority__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__lt', default=None)),
        ('vc_priority__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__gte', default=None)),
        ('vc_priority__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vc_priority__gt', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('contact__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact__n', default=None)),
        ('contact_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_role__n', default=None)),
        ('contact_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='contact_group__n', default=None)),
        ('manufacturer_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer_id__n', default=None)),
        ('manufacturer__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='manufacturer__n', default=None)),
        ('device_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type__n', default=None)),
        ('device_type_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_type_id__n', default=None)),
        ('role_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role_id__n', default=None)),
        ('role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='role__n', default=None)),
        ('parent_device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_device_id__n', default=None)),
        ('platform_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform_id__n', default=None)),
        ('platform__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='platform__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('cluster_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id__n', default=None)),
        ('model__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='model__n', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('serial__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__n', default=None)),
        ('serial__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ic', default=None)),
        ('serial__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nic', default=None)),
        ('serial__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__iew', default=None)),
        ('serial__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__niew', default=None)),
        ('serial__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__isw', default=None)),
        ('serial__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nisw', default=None)),
        ('serial__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__ie', default=None)),
        ('serial__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__nie', default=None)),
        ('serial__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='serial__empty', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('primary_ip4_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip4_id__n', default=None)),
        ('primary_ip6_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='primary_ip6_id__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of(JournalEntryType), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class VirtualDeviceContextType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'device', 'name', 'status', 'identifier', 'primary_ip4', 'primary_ip6', 'tenant', 'comments', 'interfaces', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of(TagType), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    device = sgqlc.types.Field(DeviceType, graphql_name='device')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    status = sgqlc.types.Field(sgqlc.types.non_null(DcimVirtualDeviceContextStatusChoices), graphql_name='status')
    identifier = sgqlc.types.Field(Int, graphql_name='identifier')
    primary_ip4 = sgqlc.types.Field(IPAddressType, graphql_name='primary_ip4')
    primary_ip6 = sgqlc.types.Field(IPAddressType, graphql_name='primary_ip6')
    tenant = sgqlc.types.Field(TenantType, graphql_name='tenant')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    interfaces = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(InterfaceType))), graphql_name='interfaces', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('enabled', sgqlc.types.Arg(Boolean, graphql_name='enabled', default=None)),
        ('mtu', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu', default=None)),
        ('mgmt_only', sgqlc.types.Arg(Boolean, graphql_name='mgmt_only', default=None)),
        ('poe_mode', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode', default=None)),
        ('poe_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type', default=None)),
        ('mode', sgqlc.types.Arg(String, graphql_name='mode', default=None)),
        ('rf_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_role', default=None)),
        ('rf_channel', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel', default=None)),
        ('rf_channel_frequency', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency', default=None)),
        ('rf_channel_width', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width', default=None)),
        ('tx_power', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('kind', sgqlc.types.Arg(String, graphql_name='kind', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('bridge_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id', default=None)),
        ('lag_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='lag_id', default=None)),
        ('speed', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed', default=None)),
        ('duplex', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='duplex', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('wwn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn', default=None)),
        ('vlan_id', sgqlc.types.Arg(String, graphql_name='vlan_id', default=None)),
        ('vlan', sgqlc.types.Arg(String, graphql_name='vlan', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('vdc_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_id', default=None)),
        ('vdc_identifier', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_identifier', default=None)),
        ('vdc', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc', default=None)),
        ('l2vpn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id', default=None)),
        ('l2vpn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('mtu__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__n', default=None)),
        ('mtu__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lte', default=None)),
        ('mtu__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lt', default=None)),
        ('mtu__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gte', default=None)),
        ('mtu__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gt', default=None)),
        ('poe_mode__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode__n', default=None)),
        ('poe_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type__n', default=None)),
        ('mode__n', sgqlc.types.Arg(String, graphql_name='mode__n', default=None)),
        ('rf_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_role__n', default=None)),
        ('rf_channel__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel__n', default=None)),
        ('rf_channel_frequency__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__n', default=None)),
        ('rf_channel_frequency__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__lte', default=None)),
        ('rf_channel_frequency__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__lt', default=None)),
        ('rf_channel_frequency__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__gte', default=None)),
        ('rf_channel_frequency__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__gt', default=None)),
        ('rf_channel_width__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__n', default=None)),
        ('rf_channel_width__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__lte', default=None)),
        ('rf_channel_width__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__lt', default=None)),
        ('rf_channel_width__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__gte', default=None)),
        ('rf_channel_width__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__gt', default=None)),
        ('tx_power__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__n', default=None)),
        ('tx_power__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__lte', default=None)),
        ('tx_power__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__lt', default=None)),
        ('tx_power__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__gte', default=None)),
        ('tx_power__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('bridge_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id__n', default=None)),
        ('lag_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='lag_id__n', default=None)),
        ('speed__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__n', default=None)),
        ('speed__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__lte', default=None)),
        ('speed__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__lt', default=None)),
        ('speed__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__gte', default=None)),
        ('speed__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__gt', default=None)),
        ('duplex__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='duplex__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('wwn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__n', default=None)),
        ('wwn__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__ic', default=None)),
        ('wwn__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nic', default=None)),
        ('wwn__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__iew', default=None)),
        ('wwn__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__niew', default=None)),
        ('wwn__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__isw', default=None)),
        ('wwn__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nisw', default=None)),
        ('wwn__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__ie', default=None)),
        ('wwn__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nie', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('vdc_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_id__n', default=None)),
        ('vdc_identifier__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_identifier__n', default=None)),
        ('vdc__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc__n', default=None)),
        ('l2vpn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id__n', default=None)),
        ('l2vpn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of(JournalEntryType), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class VirtualMachineType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'local_context_data', 'site', 'cluster', 'device', 'tenant', 'platform', 'name', '_name', 'status', 'role', 'primary_ip4', 'primary_ip6', 'vcpus', 'memory', 'disk', 'services', 'interfaces', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog', 'config_context')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of(TagType), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    local_context_data = sgqlc.types.Field(JSONString, graphql_name='local_context_data')
    site = sgqlc.types.Field(SiteType, graphql_name='site')
    cluster = sgqlc.types.Field(ClusterType, graphql_name='cluster')
    device = sgqlc.types.Field(DeviceType, graphql_name='device')
    tenant = sgqlc.types.Field(TenantType, graphql_name='tenant')
    platform = sgqlc.types.Field(PlatformType, graphql_name='platform')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    _name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='_name')
    status = sgqlc.types.Field(sgqlc.types.non_null(VirtualizationVirtualMachineStatusChoices), graphql_name='status')
    role = sgqlc.types.Field(DeviceRoleType, graphql_name='role')
    primary_ip4 = sgqlc.types.Field(IPAddressType, graphql_name='primary_ip4')
    primary_ip6 = sgqlc.types.Field(IPAddressType, graphql_name='primary_ip6')
    vcpus = sgqlc.types.Field(Decimal, graphql_name='vcpus')
    memory = sgqlc.types.Field(Int, graphql_name='memory')
    disk = sgqlc.types.Field(Int, graphql_name='disk')
    services = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(ServiceType))), graphql_name='services', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('protocol', sgqlc.types.Arg(String, graphql_name='protocol', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_machine_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id', default=None)),
        ('virtual_machine', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine', default=None)),
        ('ipaddress_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ipaddress_id', default=None)),
        ('ipaddress', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ipaddress', default=None)),
        ('port', sgqlc.types.Arg(Int, graphql_name='port', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('protocol__n', sgqlc.types.Arg(String, graphql_name='protocol__n', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('device_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id__n', default=None)),
        ('device__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device__n', default=None)),
        ('virtual_machine_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id__n', default=None)),
        ('virtual_machine__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine__n', default=None)),
        ('ipaddress_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ipaddress_id__n', default=None)),
        ('ipaddress__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ipaddress__n', default=None)),
))
    )
    interfaces = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(VMInterfaceType))), graphql_name='interfaces', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('enabled', sgqlc.types.Arg(Boolean, graphql_name='enabled', default=None)),
        ('mtu', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cluster_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id', default=None)),
        ('cluster', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster', default=None)),
        ('virtual_machine_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id', default=None)),
        ('virtual_machine', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('bridge_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('l2vpn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id', default=None)),
        ('l2vpn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('mtu__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__n', default=None)),
        ('mtu__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lte', default=None)),
        ('mtu__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lt', default=None)),
        ('mtu__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gte', default=None)),
        ('mtu__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('cluster_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster_id__n', default=None)),
        ('cluster__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='cluster__n', default=None)),
        ('virtual_machine_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine_id__n', default=None)),
        ('virtual_machine__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_machine__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('bridge_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('l2vpn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id__n', default=None)),
        ('l2vpn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of(JournalEntryType), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')
    config_context = sgqlc.types.Field(GenericScalar, graphql_name='config_context')


class WebhookType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'name', 'type_create', 'type_update', 'type_delete', 'payload_url', 'enabled', 'http_method', 'http_content_type', 'additional_headers', 'body_template', 'secret', 'conditions', 'ssl_verification', 'ca_file_path', 'display', 'class_type', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    type_create = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='type_create')
    type_update = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='type_update')
    type_delete = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='type_delete')
    payload_url = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='payload_url')
    enabled = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='enabled')
    http_method = sgqlc.types.Field(sgqlc.types.non_null(ExtrasWebhookHttpMethodChoices), graphql_name='http_method')
    http_content_type = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='http_content_type')
    additional_headers = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='additional_headers')
    body_template = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='body_template')
    secret = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='secret')
    conditions = sgqlc.types.Field(JSONString, graphql_name='conditions')
    ssl_verification = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='ssl_verification')
    ca_file_path = sgqlc.types.Field(String, graphql_name='ca_file_path')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class WirelessLANGroupType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'parent', 'description', 'name', 'slug', 'lft', 'rght', 'tree_id', 'level', 'children', 'wireless_lans', 'display', 'class_type', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of(TagType), graphql_name='tags')
    parent = sgqlc.types.Field('WirelessLANGroupType', graphql_name='parent')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')
    slug = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='slug')
    lft = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='lft')
    rght = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='rght')
    tree_id = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='tree_id')
    level = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='level')
    children = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('WirelessLANGroupType'))), graphql_name='children', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('slug', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('parent', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('slug__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__n', default=None)),
        ('slug__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ic', default=None)),
        ('slug__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nic', default=None)),
        ('slug__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__iew', default=None)),
        ('slug__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__niew', default=None)),
        ('slug__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__isw', default=None)),
        ('slug__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nisw', default=None)),
        ('slug__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__ie', default=None)),
        ('slug__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__nie', default=None)),
        ('slug__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='slug__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('parent__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent__n', default=None)),
))
    )
    wireless_lans = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('WirelessLANType'))), graphql_name='wireless_lans', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('ssid', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid', default=None)),
        ('auth_psk', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('tenant_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id', default=None)),
        ('tenant_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group', default=None)),
        ('tenant_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id', default=None)),
        ('tenant', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant', default=None)),
        ('group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id', default=None)),
        ('group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group', default=None)),
        ('status', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status', default=None)),
        ('vlan_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vlan_id', default=None)),
        ('auth_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_type', default=None)),
        ('auth_cipher', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_cipher', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('ssid__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__n', default=None)),
        ('ssid__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__ic', default=None)),
        ('ssid__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__nic', default=None)),
        ('ssid__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__iew', default=None)),
        ('ssid__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__niew', default=None)),
        ('ssid__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__isw', default=None)),
        ('ssid__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__nisw', default=None)),
        ('ssid__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__ie', default=None)),
        ('ssid__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__nie', default=None)),
        ('ssid__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='ssid__empty', default=None)),
        ('auth_psk__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__n', default=None)),
        ('auth_psk__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__ic', default=None)),
        ('auth_psk__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__nic', default=None)),
        ('auth_psk__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__iew', default=None)),
        ('auth_psk__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__niew', default=None)),
        ('auth_psk__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__isw', default=None)),
        ('auth_psk__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__nisw', default=None)),
        ('auth_psk__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__ie', default=None)),
        ('auth_psk__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__nie', default=None)),
        ('auth_psk__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_psk__empty', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('tenant_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group_id__n', default=None)),
        ('tenant_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_group__n', default=None)),
        ('tenant_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant_id__n', default=None)),
        ('tenant__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tenant__n', default=None)),
        ('group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group_id__n', default=None)),
        ('group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='group__n', default=None)),
        ('status__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='status__n', default=None)),
        ('vlan_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vlan_id__n', default=None)),
        ('auth_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_type__n', default=None)),
        ('auth_cipher__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='auth_cipher__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class WirelessLANType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'auth_type', 'auth_cipher', 'auth_psk', 'ssid', 'group', 'status', 'vlan', 'tenant', 'interfaces', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of(TagType), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    auth_type = sgqlc.types.Field(WirelessWirelessLANAuthTypeChoices, graphql_name='auth_type')
    auth_cipher = sgqlc.types.Field(WirelessWirelessLANAuthCipherChoices, graphql_name='auth_cipher')
    auth_psk = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='auth_psk')
    ssid = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='ssid')
    group = sgqlc.types.Field(WirelessLANGroupType, graphql_name='group')
    status = sgqlc.types.Field(sgqlc.types.non_null(WirelessWirelessLANStatusChoices), graphql_name='status')
    vlan = sgqlc.types.Field(VLANType, graphql_name='vlan')
    tenant = sgqlc.types.Field(TenantType, graphql_name='tenant')
    interfaces = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(InterfaceType))), graphql_name='interfaces', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id', default=None)),
        ('name', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name', default=None)),
        ('label', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label', default=None)),
        ('type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type', default=None)),
        ('enabled', sgqlc.types.Arg(Boolean, graphql_name='enabled', default=None)),
        ('mtu', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu', default=None)),
        ('mgmt_only', sgqlc.types.Arg(Boolean, graphql_name='mgmt_only', default=None)),
        ('poe_mode', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode', default=None)),
        ('poe_type', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type', default=None)),
        ('mode', sgqlc.types.Arg(String, graphql_name='mode', default=None)),
        ('rf_role', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_role', default=None)),
        ('rf_channel', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel', default=None)),
        ('rf_channel_frequency', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency', default=None)),
        ('rf_channel_width', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width', default=None)),
        ('tx_power', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power', default=None)),
        ('description', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description', default=None)),
        ('cable_end', sgqlc.types.Arg(String, graphql_name='cable_end', default=None)),
        ('q', sgqlc.types.Arg(String, graphql_name='q', default=None)),
        ('region_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id', default=None)),
        ('region', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region', default=None)),
        ('site_group_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id', default=None)),
        ('site_group', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group', default=None)),
        ('site_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id', default=None)),
        ('site', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site', default=None)),
        ('location_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id', default=None)),
        ('location', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location', default=None)),
        ('rack_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id', default=None)),
        ('rack', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack', default=None)),
        ('device_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device_id', default=None)),
        ('device', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='device', default=None)),
        ('virtual_chassis_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id', default=None)),
        ('virtual_chassis', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis', default=None)),
        ('module_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id', default=None)),
        ('created', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created', default=None)),
        ('last_updated', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated', default=None)),
        ('tag', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag', default=None)),
        ('cabled', sgqlc.types.Arg(Boolean, graphql_name='cabled', default=None)),
        ('occupied', sgqlc.types.Arg(Boolean, graphql_name='occupied', default=None)),
        ('connected', sgqlc.types.Arg(Boolean, graphql_name='connected', default=None)),
        ('kind', sgqlc.types.Arg(String, graphql_name='kind', default=None)),
        ('parent_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id', default=None)),
        ('bridge_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id', default=None)),
        ('lag_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='lag_id', default=None)),
        ('speed', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed', default=None)),
        ('duplex', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='duplex', default=None)),
        ('mac_address', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address', default=None)),
        ('wwn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn', default=None)),
        ('vlan_id', sgqlc.types.Arg(String, graphql_name='vlan_id', default=None)),
        ('vlan', sgqlc.types.Arg(String, graphql_name='vlan', default=None)),
        ('vrf_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id', default=None)),
        ('vrf', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf', default=None)),
        ('vdc_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_id', default=None)),
        ('vdc_identifier', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_identifier', default=None)),
        ('vdc', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc', default=None)),
        ('l2vpn_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id', default=None)),
        ('l2vpn', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn', default=None)),
        ('id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__n', default=None)),
        ('id__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lte', default=None)),
        ('id__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__lt', default=None)),
        ('id__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gte', default=None)),
        ('id__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='id__gt', default=None)),
        ('name__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__n', default=None)),
        ('name__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ic', default=None)),
        ('name__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nic', default=None)),
        ('name__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__iew', default=None)),
        ('name__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__niew', default=None)),
        ('name__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__isw', default=None)),
        ('name__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nisw', default=None)),
        ('name__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__ie', default=None)),
        ('name__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__nie', default=None)),
        ('name__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='name__empty', default=None)),
        ('label__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__n', default=None)),
        ('label__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ic', default=None)),
        ('label__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nic', default=None)),
        ('label__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__iew', default=None)),
        ('label__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__niew', default=None)),
        ('label__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__isw', default=None)),
        ('label__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nisw', default=None)),
        ('label__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__ie', default=None)),
        ('label__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__nie', default=None)),
        ('label__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='label__empty', default=None)),
        ('type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='type__n', default=None)),
        ('mtu__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__n', default=None)),
        ('mtu__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lte', default=None)),
        ('mtu__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__lt', default=None)),
        ('mtu__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gte', default=None)),
        ('mtu__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mtu__gt', default=None)),
        ('poe_mode__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_mode__n', default=None)),
        ('poe_type__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='poe_type__n', default=None)),
        ('mode__n', sgqlc.types.Arg(String, graphql_name='mode__n', default=None)),
        ('rf_role__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_role__n', default=None)),
        ('rf_channel__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel__n', default=None)),
        ('rf_channel_frequency__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__n', default=None)),
        ('rf_channel_frequency__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__lte', default=None)),
        ('rf_channel_frequency__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__lt', default=None)),
        ('rf_channel_frequency__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__gte', default=None)),
        ('rf_channel_frequency__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_frequency__gt', default=None)),
        ('rf_channel_width__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__n', default=None)),
        ('rf_channel_width__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__lte', default=None)),
        ('rf_channel_width__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__lt', default=None)),
        ('rf_channel_width__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__gte', default=None)),
        ('rf_channel_width__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rf_channel_width__gt', default=None)),
        ('tx_power__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__n', default=None)),
        ('tx_power__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__lte', default=None)),
        ('tx_power__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__lt', default=None)),
        ('tx_power__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__gte', default=None)),
        ('tx_power__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tx_power__gt', default=None)),
        ('description__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__n', default=None)),
        ('description__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ic', default=None)),
        ('description__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nic', default=None)),
        ('description__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__iew', default=None)),
        ('description__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__niew', default=None)),
        ('description__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__isw', default=None)),
        ('description__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nisw', default=None)),
        ('description__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__ie', default=None)),
        ('description__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__nie', default=None)),
        ('description__empty', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='description__empty', default=None)),
        ('cable_end__n', sgqlc.types.Arg(String, graphql_name='cable_end__n', default=None)),
        ('region_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region_id__n', default=None)),
        ('region__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='region__n', default=None)),
        ('site_group_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group_id__n', default=None)),
        ('site_group__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_group__n', default=None)),
        ('site_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site_id__n', default=None)),
        ('site__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='site__n', default=None)),
        ('location_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location_id__n', default=None)),
        ('location__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='location__n', default=None)),
        ('rack_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack_id__n', default=None)),
        ('rack__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='rack__n', default=None)),
        ('virtual_chassis_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis_id__n', default=None)),
        ('virtual_chassis__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='virtual_chassis__n', default=None)),
        ('module_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='module_id__n', default=None)),
        ('created__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__n', default=None)),
        ('created__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lte', default=None)),
        ('created__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__lt', default=None)),
        ('created__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gte', default=None)),
        ('created__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='created__gt', default=None)),
        ('last_updated__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__n', default=None)),
        ('last_updated__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lte', default=None)),
        ('last_updated__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__lt', default=None)),
        ('last_updated__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gte', default=None)),
        ('last_updated__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='last_updated__gt', default=None)),
        ('tag__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='tag__n', default=None)),
        ('parent_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='parent_id__n', default=None)),
        ('bridge_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='bridge_id__n', default=None)),
        ('lag_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='lag_id__n', default=None)),
        ('speed__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__n', default=None)),
        ('speed__lte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__lte', default=None)),
        ('speed__lt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__lt', default=None)),
        ('speed__gte', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__gte', default=None)),
        ('speed__gt', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='speed__gt', default=None)),
        ('duplex__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='duplex__n', default=None)),
        ('mac_address__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__n', default=None)),
        ('mac_address__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ic', default=None)),
        ('mac_address__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nic', default=None)),
        ('mac_address__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__iew', default=None)),
        ('mac_address__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__niew', default=None)),
        ('mac_address__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__isw', default=None)),
        ('mac_address__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nisw', default=None)),
        ('mac_address__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__ie', default=None)),
        ('mac_address__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='mac_address__nie', default=None)),
        ('wwn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__n', default=None)),
        ('wwn__ic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__ic', default=None)),
        ('wwn__nic', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nic', default=None)),
        ('wwn__iew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__iew', default=None)),
        ('wwn__niew', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__niew', default=None)),
        ('wwn__isw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__isw', default=None)),
        ('wwn__nisw', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nisw', default=None)),
        ('wwn__ie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__ie', default=None)),
        ('wwn__nie', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='wwn__nie', default=None)),
        ('vrf_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf_id__n', default=None)),
        ('vrf__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vrf__n', default=None)),
        ('vdc_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_id__n', default=None)),
        ('vdc_identifier__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc_identifier__n', default=None)),
        ('vdc__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='vdc__n', default=None)),
        ('l2vpn_id__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn_id__n', default=None)),
        ('l2vpn__n', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='l2vpn__n', default=None)),
))
    )
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of(JournalEntryType), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')


class WirelessLinkType(sgqlc.types.Type):
    __schema__ = schema
    __field_names__ = ('id', 'created', 'last_updated', 'custom_field_data', 'tags', 'description', 'comments', 'auth_type', 'auth_cipher', 'auth_psk', 'interface_a', 'interface_b', 'ssid', 'status', 'tenant', '_interface_a_device', '_interface_b_device', 'display', 'class_type', 'journal_entries', 'custom_fields', 'changelog')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    created = sgqlc.types.Field(DateTime, graphql_name='created')
    last_updated = sgqlc.types.Field(DateTime, graphql_name='last_updated')
    custom_field_data = sgqlc.types.Field(sgqlc.types.non_null(JSONString), graphql_name='custom_field_data')
    tags = sgqlc.types.Field(sgqlc.types.list_of(TagType), graphql_name='tags')
    description = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='description')
    comments = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='comments')
    auth_type = sgqlc.types.Field(WirelessWirelessLinkAuthTypeChoices, graphql_name='auth_type')
    auth_cipher = sgqlc.types.Field(WirelessWirelessLinkAuthCipherChoices, graphql_name='auth_cipher')
    auth_psk = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='auth_psk')
    interface_a = sgqlc.types.Field(sgqlc.types.non_null(InterfaceType), graphql_name='interface_a')
    interface_b = sgqlc.types.Field(sgqlc.types.non_null(InterfaceType), graphql_name='interface_b')
    ssid = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='ssid')
    status = sgqlc.types.Field(sgqlc.types.non_null(WirelessWirelessLinkStatusChoices), graphql_name='status')
    tenant = sgqlc.types.Field(TenantType, graphql_name='tenant')
    _interface_a_device = sgqlc.types.Field(DeviceType, graphql_name='_interface_a_device')
    _interface_b_device = sgqlc.types.Field(DeviceType, graphql_name='_interface_b_device')
    display = sgqlc.types.Field(String, graphql_name='display')
    class_type = sgqlc.types.Field(String, graphql_name='class_type')
    journal_entries = sgqlc.types.Field(sgqlc.types.list_of(JournalEntryType), graphql_name='journal_entries')
    custom_fields = sgqlc.types.Field(GenericScalar, graphql_name='custom_fields')
    changelog = sgqlc.types.Field(sgqlc.types.list_of(ObjectChangeType), graphql_name='changelog')



########################################################################
# Unions
########################################################################
class CableTerminationTerminationType(sgqlc.types.Union):
    __schema__ = schema
    __types__ = (CircuitTerminationType, ConsolePortType, ConsoleServerPortType, FrontPortType, InterfaceType, PowerFeedType, PowerOutletType, PowerPortType, RearPortType)


class FHRPGroupInterfaceType(sgqlc.types.Union):
    __schema__ = schema
    __types__ = (InterfaceType, VMInterfaceType)


class IPAddressAssignmentType(sgqlc.types.Union):
    __schema__ = schema
    __types__ = (InterfaceType, FHRPGroupType, VMInterfaceType)


class InventoryItemComponentType(sgqlc.types.Union):
    __schema__ = schema
    __types__ = (ConsolePortType, ConsoleServerPortType, FrontPortType, InterfaceType, PowerOutletType, PowerPortType, RearPortType)


class InventoryItemTemplateComponentType(sgqlc.types.Union):
    __schema__ = schema
    __types__ = (ConsolePortTemplateType, ConsoleServerPortTemplateType, FrontPortTemplateType, InterfaceTemplateType, PowerOutletTemplateType, PowerPortTemplateType, RearPortTemplateType)


class L2VPNAssignmentType(sgqlc.types.Union):
    __schema__ = schema
    __types__ = (InterfaceType, VLANType, VMInterfaceType)


class LinkPeerType(sgqlc.types.Union):
    __schema__ = schema
    __types__ = (CircuitTerminationType, ConsolePortType, ConsoleServerPortType, FrontPortType, InterfaceType, PowerFeedType, PowerOutletType, PowerPortType, RearPortType)


class VLANGroupScopeType(sgqlc.types.Union):
    __schema__ = schema
    __types__ = (ClusterType, ClusterGroupType, LocationType, RackType, RegionType, SiteType, SiteGroupType)



########################################################################
# Schema Entry Points
########################################################################
schema.query_type = Query
schema.mutation_type = None
schema.subscription_type = None

